/**
 * @license
 * Copyright Akveo. All Rights Reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 */
import { BrowserModule } from '@angular/platform-browser';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { APP_INITIALIZER, LOCALE_ID, NgModule } from '@angular/core';
import { HTTP_INTERCEPTORS, HttpClientModule } from '@angular/common/http';
import { AppComponent } from './app.component';
import { AppRoutingModule } from './app-routing.module';
import {
    NbAlertModule,
    NbDatepickerModule,
    NbDialogModule,
    NbMenuModule,
    NbSidebarModule,
    NbTimepickerModule,
    NbToastrModule,
    NbWindowModule,
} from '@nebular/theme';

import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import { environment } from 'environments/environment';

import { SharedModule } from './shared/shared.module';
import { KeycloakAngularModule, KeycloakService } from 'keycloak-angular';
import { LayoutModule } from './layout/layout.module';
import { RouteReuseStrategy } from '@angular/router';
import { CustomReuseStrategy } from './shared/route-reuse/custom-reuse-strategy';
import { HTTP_CACHE_CONFIG, HttpCacheInterceptorModule } from '@ngneat/cashew';
import { KatexModule } from '@sls/ng-katex';
import { CoursesModule } from './courses/courses.module';
import { ExercisesModule } from './exercises/exercises.module';
import { TasksModule } from './tasks/tasks.module';
import { MisconceptionsModule } from './misconceptions/misconceptions.module';
import { ModulesModule } from './modules/modules.module';
import { SearchModule } from './search/search.module';
import { AuthModule } from './auth/auth.module';
import { StoreModule } from '@ngrx/store';
import { EffectsModule } from '@ngrx/effects';
import { initialState, logger, rootReducer } from './store/app.reducers';
import { AppEffects } from './store/app.effects';
import { StoreDevtoolsModule } from '@ngrx/store-devtools';
import { LandingPageModule } from 'app/landing/landing.page.module';
import { DashboardPageModule } from 'app/dashboard/dashboard.page.module';
import { AssessmentModule } from 'app/assessment/assessment.module';
import { SolutionsModule } from 'app/solutions/solutions.module';
import { AttendancesModule } from 'app/attendances/attendances.module';
import { ProfilePageModule } from 'app/profile/profile.page.module';
import { ApiVersionInterceptor } from 'app/shared/misc/api-version-interceptor';

registerLocaleData(localeDe);

function initializeKeycloak(keycloak: KeycloakService) {
    return () =>
        keycloak.init({
            config: {
                url: environment.keycloakUrl,
                realm: environment.keycloakRealm,
                clientId: 'subato',
            },
            initOptions: {
                onLoad: 'check-sso',
                // disable silent check in production because the SBI proxy is blocking iframes (X-Frame-Options: DENY)
                silentCheckSsoRedirectUri: environment.production
                    ? undefined
                    : window.location.origin + '/assets/silent-check-sso.html',
            },
            shouldAddToken: (request) => {
                return request.url.startsWith(environment.backendUrl);
            },
        });
}

function cashewConfig() {
    return {
        strategy: 'implicit',
        ttl: 0,
    };
}

let ngrxDevToolsConfig = !environment.production
    ? [
          StoreDevtoolsModule.instrument({
              maxAge: 25, // Retains last 25 states
              autoPause: true, // Pauses recording actions and state changes when the extension window is not open
              //trace: false, //  If set to true, will include stack trace for every dispatched action, so you can see it in trace tab jumping directly to that part of code
              //traceLimit: 75, // maximum stack trace frames to be stored (in case trace option was provided as true)
          }),
      ]
    : [];

@NgModule({
    declarations: [AppComponent],
    imports: [
        BrowserModule,
        BrowserAnimationsModule,
        HttpClientModule,
        KeycloakAngularModule,
        AppRoutingModule,
        KatexModule,
        NbSidebarModule.forRoot(),
        NbMenuModule.forRoot(),
        NbDatepickerModule.forRoot(),
        NbTimepickerModule.forRoot(),
        NbDialogModule.forRoot(),
        NbWindowModule.forRoot(),
        NbToastrModule.forRoot(),
        LayoutModule.forRoot(),
        SharedModule,
        NbAlertModule,
        NbToastrModule.forRoot(),
        HttpCacheInterceptorModule.forRoot(),
        StoreModule.forRoot(rootReducer, {
            initialState: initialState,
            metaReducers: environment.production ? [] : [logger],
        }),
        EffectsModule.forRoot([AppEffects]),
        ...ngrxDevToolsConfig,
        AuthModule,
        CoursesModule,
        AttendancesModule,
        AssessmentModule,
        ExercisesModule,
        TasksModule,
        MisconceptionsModule,
        ModulesModule,
        DashboardPageModule,
        ProfilePageModule,
        LandingPageModule,
        SolutionsModule,
        SearchModule,
    ],
    bootstrap: [AppComponent],
    providers: [
        { provide: LOCALE_ID, useValue: 'de-DE' },
        {
            provide: APP_INITIALIZER,
            useFactory: initializeKeycloak,
            multi: true,
            deps: [KeycloakService],
        },
        { provide: RouteReuseStrategy, useClass: CustomReuseStrategy },
        { provide: HTTP_CACHE_CONFIG, useValue: cashewConfig() },
        {
            provide: HTTP_INTERCEPTORS,
            useClass: ApiVersionInterceptor,
            multi: true,
        },
        /*
        {
            provide: APP_INITIALIZER,
            useFactory: (s: StompService) => () => {},
            deps: [StompService],
            multi: true,
        },*/
    ],
})
export class AppModule {}
