import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { AuthService } from 'app/auth/auth.service';
import { BaseComponent } from 'app/shared/misc/base.component';
import { EMPTY } from 'rxjs';
import { catchError, map, takeUntil } from 'rxjs/operators';
import { ParamHelper } from 'app/shared/misc/helper/param.helper';
import { ErrorContainerConfig } from 'app/shared/error/error-container-config';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { ExerciseAssessmentPageStore } from 'app/assessment/pages/exercise-assessment/exercise-assessment.page.store';
import { ifAttached } from 'app/shared/route-reuse/route-reuse-life.cycle.directive';
import { PageHeaderAction } from 'app/shared/page-header/page-header/page-header.component';
import { Exercise } from 'app/exercises/models/exercise';

@Component({
    templateUrl: './exercise-assessment.page.component.html',
    styleUrls: ['./exercise-assessment.page.component.scss'],
    providers: [ExerciseAssessmentPageStore],
})
export class ExerciseAssessmentPageComponent
    extends BaseComponent
    implements OnInit
{
    exerciseState$ = this.exerciseAssessmentStore.exerciseState$;

    errorConfig: ErrorContainerConfig = {
        notFound: {
            message: 'Diese Übung konnten wir leider nicht finden.',
        },
    };

    actions$ = this.exerciseState$.pipe(
        takeUntil(this.destroy$),
        ifAttached(this),
        map((exerciseState) => this.buildActions(exerciseState.exercise))
    );

    constructor(
        private exerciseAssessmentStore: ExerciseAssessmentPageStore,
        private store: Store<SubatoState>,
        private authService: AuthService,
        private globalErrorHandler: GlobalErrorHandler,
        private route: ActivatedRoute,
        private router: Router
    ) {
        super();
    }

    ngOnInit(): void {
        this.route.paramMap
            .pipe(
                takeUntil(this.destroy$),
                ifAttached(this),
                map((params) =>
                    ParamHelper.parseIdOrThrow(params, 'exercise_id')
                ),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.exerciseAssessmentStore.patchExercise({ error: err });
                    return EMPTY;
                })
            )
            .subscribe((exerciseId) =>
                this.exerciseAssessmentStore.loadExercise(exerciseId)
            );
    }

    onAttach() {
        super.onAttach();
        this.exerciseAssessmentStore.onAttach();
    }

    onDetach() {
        super.onDetach();
        this.exerciseAssessmentStore.onDetach();
    }

    buildActions(exercise: Exercise): PageHeaderAction[] {
        let actions: PageHeaderAction[] = [];
        if (exercise != null) {
            actions.push({
                name: 'Abnahmetests',
                icon: 'play-circle-outline',
                link: {
                    commands: ['/', 'exercises', exercise.id, 'acceptance'],
                },
                status: 'primary',
            });
        }

        return actions;
    }
}
