import { Component, OnInit } from '@angular/core';
import { Breadcrumb } from 'app/shared/breadcrumbs/breadcrumb';
import { Submission } from 'app/assessment/models/submission';
import { TaskInstance } from 'app/exercises/models/task-instance';
import { Origin } from 'app/assessment/models/origin';
import { combineLatest, Observable } from 'rxjs';
import { filter, map, takeUntil } from 'rxjs/operators';
import { SubmissionPageStore } from 'app/assessment/pages/submission/submission.page.store';
import { BaseComponent } from 'app/shared/misc/base.component';

export interface SubmissionBreadcrumbsParams {
    submission: Submission;
    taskInstance: TaskInstance;
    taskId: number;
    origin: Origin;
}

@Component({
    selector: 'sb-submission-breadcrumbs',
    templateUrl: './breadcrumbs.component.html',
    styleUrls: ['./breadcrumbs.component.scss'],
})
export class BreadcrumbsComponent extends BaseComponent implements OnInit {
    breadcrumbs: Breadcrumb[] = [];

    origin$ = this.submissionStore.origin$;
    currentTaskInstance$ = this.submissionStore.currentTaskInstance$;
    submission$ = this.submissionStore.submission$;

    breadcrumbsParams$: Observable<SubmissionBreadcrumbsParams> = combineLatest(
        this.submission$,
        this.currentTaskInstance$,
        this.origin$
    ).pipe(
        takeUntil(this.destroy$),
        filter(
            ([submission, taskInstance, origin]) =>
                submission != null && taskInstance != null && origin != null
        ),
        map(([submission, taskInstance, origin]) => ({
            submission: submission,
            taskInstance: taskInstance,
            taskId: taskInstance.task.id,
            origin: origin,
        }))
    );

    constructor(private submissionStore: SubmissionPageStore) {
        super();
    }

    ngOnInit() {
        this.breadcrumbsParams$.subscribe((params) =>
            this.buildBreadcrumbs(params)
        );
    }

    buildBreadcrumbs(params: SubmissionBreadcrumbsParams) {
        let commonPrefix = [
            {
                id: 'exercises',
                params: {
                    course_id: params.submission.exercise.course.id,
                },
            },
            {
                id: 'exercise',
                params: {
                    exercise_id: params.submission.exercise.id,
                },
                ctx: params.submission.exercise,
            },
        ];

        if (params.origin == Origin.Task) {
            let originTaskSuffix = [
                {
                    id: 'submitSolution',
                    params: {
                        exercise_id: params.submission.exercise.id,
                        task_id: params.taskId,
                    },
                    ctx: params.taskInstance,
                },
                {
                    id: 'taskInstanceAssessment',
                    params: {
                        exercise_id: params.submission.exercise.id,
                        task_id: params.taskId,
                    },
                },
                {
                    id: 'submission',
                    params: {
                        submission_id: params.submission.id,
                    },
                    ctx: params.submission,
                },
            ];

            this.breadcrumbs = [...commonPrefix, ...originTaskSuffix];
        } else {
            let originExerciseSuffix = [
                {
                    id: 'exerciseAssessment',
                    params: {
                        exercise_id: params.submission.exercise.id,
                    },
                },
                {
                    id: 'submission',
                    params: {
                        submission_id: params.submission.id,
                    },
                    ctx: params.submission,
                },
            ];

            this.breadcrumbs = [...commonPrefix, ...originExerciseSuffix];
        }
    }
}
