import {
    AfterViewInit,
    ChangeDetectorRef,
    Component,
    OnInit,
    QueryList,
    ViewChild,
    ViewChildren,
} from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { BaseComponent } from 'app/shared/misc/base.component';
import { combineLatest, EMPTY } from 'rxjs';
import { catchError, map, take, takeUntil } from 'rxjs/operators';
import { TaskInstance } from 'app/exercises/models/task-instance';
import { NbTabsetComponent } from '@nebular/theme';
import { SubmissionListComponent } from 'app/assessment/pages/submission/components/submission-list/submission-list.component';
import { CodemirrorComponent } from '@ctrl/ngx-codemirror';
import { Solution } from 'app/solutions/models/solution';
import { Origin } from 'app/assessment/models/origin';
import { CodemirrorHelper } from 'app/shared/codemirror/helper/codemirror.helper';
import { ParamHelper } from 'app/shared/misc/helper/param.helper';
import { ErrorContainerConfig } from 'app/shared/error/error-container-config';
import { SubatoState } from 'app/store/app.reducers';
import { Store } from '@ngrx/store';
import { SubmissionPageStore } from 'app/assessment/pages/submission/submission.page.store';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';

@Component({
    templateUrl: './submission.page.component.html',
    styleUrls: ['./submission.page.component.scss'],
    providers: [SubmissionPageStore],
})
export class SubmissionPageComponent
    extends BaseComponent
    implements OnInit, AfterViewInit
{
    submissionSelection: SubmissionListComponent;
    @ViewChildren(SubmissionListComponent)
    submissionSelectionQuery: QueryList<SubmissionListComponent>;
    @ViewChild('solutionTabset') solutionTabset: NbTabsetComponent;
    @ViewChild('evalResultsTabset') evalResultsTabset: NbTabsetComponent;

    @ViewChildren(CodemirrorComponent)
    codemirrorComponents: QueryList<CodemirrorComponent>;

    errorConfig: ErrorContainerConfig = {
        notFound: {
            message: 'Diese Abgabe konnten wir leider nicht finden.',
        },
    };

    taskContentState$ = this.submissionStore.taskContentState$;
    solutionState$ = this.submissionStore.solutionState$;
    solution$ = this.submissionStore.solution$;
    currentTaskInstance$ = this.submissionStore.currentTaskInstance$;
    progress$ = this.submissionStore.progress$;
    taskInstances$ = this.submissionStore.taskInstances$;
    unit$ = this.submissionStore.unit$;
    submissionState$ = this.submissionStore.submissionState$;
    isLecturerOrAdmin$ = this.submissionStore.isLecturerOrAdmin$;

    submissionSelectionTitle$ = this.submissionStore.select(
        this.submissionStore.unit$,
        (unit) => unit?.shortName ?? 'Teilnehmer'
    );

    hasValidSolution$ = this.submissionStore.select(
        this.submissionStore.solution$,
        (solution) => solution != null
    );

    showHistoryDot$ = this.submissionStore.select(
        this.submissionStore.historySolutions$,
        (historySolutions) =>
            historySolutions.length != 0 &&
            (historySolutions.some((s) => s.commentsCount != 0) ||
                historySolutions.some((s) => s.delayed || s.attemptsExceeded))
    );

    showRegularResultsTab$ = this.solution$.pipe(
        map((solution) => solution?.hasResult ?? false)
    );

    showAcceptanceResultsTab$ = combineLatest([
        this.solution$,
        this.currentTaskInstance$,
        this.isLecturerOrAdmin$,
    ]).pipe(
        map(
            ([solution, taskInstance, isLecturerOrAdmin]) =>
                solution != null &&
                isLecturerOrAdmin &&
                (taskInstance?.task?.acceptanceTestAvailable ?? false)
        )
    );

    constructor(
        private submissionStore: SubmissionPageStore,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler,
        private route: ActivatedRoute,
        private cd: ChangeDetectorRef
    ) {
        super();
    }

    ngAfterViewInit() {
        this.submissionSelectionQuery.changes
            .pipe(takeUntil(this.destroy$))
            .subscribe(
                (comps: QueryList<SubmissionListComponent>) =>
                    (this.submissionSelection = comps.first)
            );

        combineLatest([this.solution$, this.codemirrorComponents.changes])
            .pipe(takeUntil(this.destroy$))
            .subscribe(
                ([_, codemirrorComponents]: [
                    Solution,
                    CodemirrorComponent[]
                ]) => this.refreshEditors(codemirrorComponents)
            );
    }

    ngOnInit(): void {
        this.route.paramMap
            .pipe(
                takeUntil(this.destroy$),
                map((params) =>
                    ParamHelper.parseIdOrThrow(params, 'submission_id')
                ),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.submissionStore.patchSubmission({ error: err });
                    return EMPTY;
                })
            )
            .subscribe((submissionId) =>
                this.submissionStore.loadSubmission(submissionId)
            );

        this.route.queryParamMap
            .pipe(
                takeUntil(this.destroy$),
                map((queryParams) =>
                    queryParams.has('unitId')
                        ? ParamHelper.parseIdOrThrow(queryParams, 'unitId')
                        : null
                ),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.submissionStore.patchUnit({ error: err });
                    return EMPTY;
                })
            )
            .subscribe((unitId) => this.submissionStore.loadUnit(unitId));

        this.route.queryParamMap
            .pipe(
                takeUntil(this.destroy$),
                map((queryParams) =>
                    queryParams.has('taskId')
                        ? ParamHelper.parseIdOrThrow(queryParams, 'taskId')
                        : null
                )
                /*
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.submissionStore.patchUnit({ error: err });
                    return EMPTY;
                })*/
            )
            .subscribe((taskId) =>
                this.submissionStore.patchState({ taskId: taskId })
            );

        this.route.queryParamMap
            .pipe(
                takeUntil(this.destroy$),
                map(
                    (queryParams) =>
                        <Origin>(
                            (queryParams.has('origin')
                                ? ParamHelper.parseIdOrThrow(
                                      queryParams,
                                      'origin'
                                  )
                                : Origin.Exercise)
                        )
                )
                /*
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.submissionStore.patchUnit({ error: err });
                    return EMPTY;
                })*/
            )
            .subscribe((origin) => this.submissionStore.setOrigin(origin));

        combineLatest([
            this.showRegularResultsTab$,
            this.showAcceptanceResultsTab$,
        ])
            .pipe(takeUntil(this.destroy$))
            .subscribe(([showRegularResultsTab, showAcceptanceResultsTab]) => {
                this.fixSelectedEvalResultsTab(
                    showRegularResultsTab,
                    showAcceptanceResultsTab
                );
            });
    }

    onTasksLoaded(taskInstances: TaskInstance[]) {
        this.submissionStore.patchState({ taskInstances: taskInstances });
        this.cd.detectChanges();
    }

    onSolutionHistoryLoaded(solutions: Solution[]) {
        this.submissionStore.patchState({ historySolutions: solutions });
        this.cd.detectChanges();
    }

    onTaskSelected(taskInstance: TaskInstance) {
        this.submissionStore.navigateToTaskInstance(taskInstance);
    }

    refreshEditors(codemirrorComponents: CodemirrorComponent[]) {
        codemirrorComponents.forEach((component) => {
            CodemirrorHelper.waitForEditor(component)
                .pipe(take(1))
                .subscribe(() => component.codeMirror.refresh());
        });
    }

    selectSolutionTab() {
        let tab = this.solutionTabset?.tabs?.find((t) => t.tabId == 'solution');
        if (tab) {
            this.solutionTabset.selectTab(tab);
        }
    }

    // bug in nebular: if a tab that was previously selected is hidden, the
    // state of the tabset is broken (still shows the content from the hidden tab)
    // so we fix that here
    fixSelectedEvalResultsTab(
        showRegularResultsTab: boolean,
        showAcceptanceResultsTab: boolean
    ) {
        let activeTab = this.evalResultsTabset?.tabs?.find((t) => t.active);

        if (
            activeTab != null &&
            activeTab.tabId == 'regular' &&
            !showRegularResultsTab
        ) {
            let acceptanceTab = this.evalResultsTabset?.tabs?.find(
                (t) => t.tabId == 'acceptance'
            );
            if (acceptanceTab != null) {
                this.evalResultsTabset.selectTab(acceptanceTab);
            }
        }

        if (
            activeTab != null &&
            activeTab.tabId == 'acceptance' &&
            !showAcceptanceResultsTab
        ) {
            let regularTab = this.evalResultsTabset?.tabs?.find(
                (t) => t.tabId == 'regular'
            );
            if (regularTab != null) {
                this.evalResultsTabset.selectTab(regularTab);
            }
        }
    }

    onSolutionSelected(solution: Solution) {
        this.selectSolutionTab();
        this.submissionStore.selectSolution(solution);
    }
}
