import { Injectable } from '@angular/core';
import { combineLatest, EMPTY, Observable, of } from 'rxjs';
import { Submission } from 'app/assessment/models/submission';
import { TaskInstance } from 'app/exercises/models/task-instance';
import { Origin } from 'app/assessment/models/origin';
import { AssessmentProgress } from 'app/assessment/models/assessment-progress';
import { Solution } from 'app/solutions/models/solution';
import {
    catchError,
    distinctUntilChanged,
    filter,
    map,
    switchMap,
    take,
    tap,
    withLatestFrom,
} from 'rxjs/operators';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { ChangeCourse } from 'app/courses/store/courses.actions';
import { ExerciseService } from 'app/exercises/services/exercise.service';
import { SubmissionService } from 'app/assessment/service/submission.service';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { CustomRouterLink } from 'app/shared/misc/directives/custom-link/custom-router-link.directive';
import {
    DetailParams,
    SubmissionLinkProvider,
} from 'app/assessment/routing/submission-link-provider';
import { Router } from '@angular/router';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import {
    loadSubmission,
    loadSubmissions,
    SubmissionsState,
    SubmissionState,
} from 'app/exercises/shared/operators/submission.operators';
import {
    CourseUnitState,
    loadUnit,
} from 'app/courses/shared/operators/course-unit.operators';
import {
    loadLatestSolution,
    loadSolution,
    SolutionState,
} from 'app/solutions/shared/operators/solution.operators';
import {
    loadTaskContent,
    TaskContentState,
} from 'app/exercises/shared/operators/task-content.operators';
import {
    loadTaskInstance,
    TaskInstanceState,
} from 'app/exercises/shared/operators/task-instance.operators';

export interface SubmissionPageState {
    submissionState: SubmissionState;
    nextSubmission: Submission;
    submissionsState: SubmissionsState;
    unitState: CourseUnitState;
    taskInstances: TaskInstance[];
    origin: Origin;
    progress: AssessmentProgress;
    currentTaskInstanceState: TaskInstanceState;
    solutionState: SolutionState;
    historySolutions: Solution[];
    taskContentState: TaskContentState;
    savingPoints: boolean;
    savingPassed: boolean;
    taskId: number;
}

export const initialState: SubmissionPageState = {
    submissionsState: { submissions: [], error: null, loading: true },
    submissionState: { submission: null, error: null, loading: true },
    currentTaskInstanceState: {
        taskInstance: null,
        error: null,
        loading: true,
    },
    historySolutions: [],
    nextSubmission: null,
    origin: null,
    progress: null,
    unitState: { unit: null, error: null, loading: false },
    taskInstances: [],
    taskContentState: { content: null, error: null, loading: true },
    solutionState: { solution: null, error: null, loading: false },
    savingPoints: false,
    savingPassed: false,
    taskId: null,
};

interface SavePointsParams {
    points: number;
    done: () => void;
}

@Injectable()
export class SubmissionPageStore extends BaseComponentStore<SubmissionPageState> {
    public readonly principal$ = this.store.select((s) => s.auth.principal);

    public readonly submissions$ = this.select(
        (state) => state.submissionsState.submissions
    );
    public readonly submissionsState$ = this.select(
        (state) => state.submissionsState
    );

    public readonly submission$ = this.select(
        (state) => state.submissionState.submission
    );
    public readonly submissionState$ = this.select(
        (state) => state.submissionState
    );
    public readonly origin$ = this.select((state) => state.origin);
    public readonly taskId$ = this.select((state) => state.taskId);
    public readonly unit$ = this.select((state) => state.unitState.unit);
    public readonly unitState$ = this.select((state) => state.unitState);
    public readonly nextSubmission$ = this.select(
        (state) => state.nextSubmission
    );
    public readonly progress$ = this.select((state) => state.progress);
    public readonly currentTaskInstance$ = this.select(
        (state) => state.currentTaskInstanceState.taskInstance
    );
    public readonly taskInstances$ = this.select(
        (state) => state.taskInstances
    );
    public readonly taskContentState$ = this.select(
        (state) => state.taskContentState
    );
    public readonly savingPoints$ = this.select((state) => state.savingPoints);
    public readonly savingPassed$ = this.select((state) => state.savingPassed);
    public readonly solution$ = this.select(
        (state) => state.solutionState.solution
    );
    public readonly solutionState$ = this.select(
        (state) => state.solutionState
    );

    public readonly historySolutions$ = this.select(
        (state) => state.historySolutions
    );

    prevTaskInstance$ = this.select(
        this.currentTaskInstance$,
        this.taskInstances$,
        (taskInstance, taskInstances) =>
            taskInstance != null
                ? taskInstance.getPrevTaskInstance(taskInstances)
                : null
    );

    nextTaskInstance$ = this.select(
        this.currentTaskInstance$,
        this.taskInstances$,
        (taskInstance, taskInstances) =>
            taskInstance != null
                ? taskInstance.getNextTaskInstance(taskInstances)
                : null
    );

    public readonly isStaff$ = this.select(
        this.submission$,
        this.principal$,
        (submission, principal) =>
            principal != null &&
            submission != null &&
            principal.hasAnyGroup(submission.exercise.course.id)
    );

    public readonly isLecturerOrAdmin$ = this.select(
        this.submission$,
        this.principal$,
        (submission, principal) =>
            submission != null &&
            principal != null &&
            principal.isLecturerOrAdmin(submission.exercise.course.id)
    );

    constructor(
        private store: Store<SubatoState>,
        private submissionService: SubmissionService,
        private exerciseService: ExerciseService,
        private notificationService: NotificationService,
        private linkProvider: SubmissionLinkProvider,
        private router: Router,
        private globalErrorHandler: GlobalErrorHandler
    ) {
        super(initialState);
    }

    readonly patchUnit = this.updaterFor('unitState');
    readonly patchSubmission = this.updaterFor('submissionState');
    readonly patchSubmissions = this.updaterFor('submissionsState');
    readonly patchSolution = this.updaterFor('solutionState');
    readonly patchTaskContent = this.updaterFor('taskContentState');
    readonly patchCurrentTaskInstance = this.updaterFor(
        'currentTaskInstanceState'
    );

    readonly loadUnit = this.effect((unitId$: Observable<number>) =>
        unitId$.pipe(
            filter((unitId) => unitId != null),
            loadUnit((s) => this.patchUnit(s))
        )
    );

    readonly loadSubmission = this.effect((submissionId$: Observable<number>) =>
        submissionId$.pipe(loadSubmission((s) => this.patchSubmission(s)))
    );

    getDetailLink(
        submission: Submission,
        taskInstance?: TaskInstance
    ): Observable<CustomRouterLink> {
        return of(submission).pipe(
            take(1),
            withLatestFrom(this.unit$, this.origin$),
            map(([submission, unit, origin]) => {
                let params: DetailParams = {
                    submissionId: submission.id,
                    unitId: unit?.id,
                    origin: origin,
                };

                if (taskInstance != null) {
                    params = { ...params, taskId: taskInstance.task.id };
                }

                return this.linkProvider.detail(params);
            })
        );
    }

    readonly selectSolution = this.effect((solution$: Observable<Solution>) =>
        solution$.pipe(
            map((solution) => solution.id),
            loadSolution((s) => this.patchSolution(s))
        )
    );

    readonly navigateToTaskInstance = this.effect(
        (taskInstance$: Observable<TaskInstance>) =>
            taskInstance$.pipe(
                withLatestFrom(this.submission$),
                filter(([ti, submission]) => ti != null),
                switchMap(([ti, submission]) =>
                    this.getDetailLink(submission, ti).pipe(
                        tap((link) =>
                            this.router.navigate(link.commands, {
                                queryParams: link.queryParams,
                            })
                        )
                    )
                )
            )
    );

    readonly navigateToNextTaskInstance = this.effect(
        (trigger$: Observable<void>) =>
            trigger$.pipe(
                withLatestFrom(this.nextTaskInstance$),
                tap(([_, ti]) => this.navigateToTaskInstance(ti))
            )
    );

    readonly navigateToPrevTaskInstance = this.effect(
        (trigger$: Observable<void>) =>
            trigger$.pipe(
                withLatestFrom(this.prevTaskInstance$),
                tap(([_, ti]) => this.navigateToTaskInstance(ti))
            )
    );

    readonly navigateToNextSubmission = this.effect(
        (trigger$: Observable<void>) =>
            trigger$.pipe(
                withLatestFrom(this.nextSubmission$),
                filter(([_, submission]) => submission != null),
                switchMap(([_, submission]) =>
                    this.getDetailLink(submission).pipe(
                        tap((link) =>
                            this.router.navigate(link.commands, {
                                queryParams: link.queryParams,
                            })
                        )
                    )
                )
            )
    );

    readonly savePoints = this.effect((params$: Observable<SavePointsParams>) =>
        params$.pipe(
            withLatestFrom(this.submission$, this.currentTaskInstance$),
            tap(() => this.patchState({ savingPoints: true })),
            switchMap(([params, submission, taskInstance]) =>
                this.submissionService
                    .updatePoints(submission.id, {
                        points: params.points,
                        taskId: taskInstance.task.id,
                    })
                    .pipe(
                        tap((submission) => {
                            this.patchState({
                                savingPoints: false,
                            });

                            this.patchSubmission({
                                submission: submission,
                            });

                            if (
                                params.points < 0 ||
                                params.points > taskInstance.reachablePoints
                            ) {
                                this.notificationService.showWarning(
                                    'Punktzahl gespeichert aber hast du sie richtig eingegeben?'
                                );
                            } else {
                                this.notificationService.showSuccess(
                                    'Punktzahl gespeichert'
                                );
                            }

                            params.done();
                        }),
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((error) => {
                            this.patchState({ savingPoints: false });
                            this.notificationService.showError(
                                'Fehler beim Speichern'
                            );
                            return EMPTY;
                        })
                    )
            )
        )
    );

    readonly updatePassed = this.effect((passed$: Observable<boolean | null>) =>
        passed$.pipe(
            withLatestFrom(this.submission$),
            tap(() => this.patchState({ savingPassed: true })),
            switchMap(([passed, submission]) =>
                this.submissionService
                    .updatePassed(submission.id, { passed: passed })
                    .pipe(
                        tap((submission) => {
                            this.patchState({
                                savingPassed: false,
                            });

                            this.patchSubmission({
                                submission: submission,
                            });

                            this.notificationService.showSuccess(
                                'Bewertung erfolgreich gespeichert'
                            );
                        }),
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((error) => {
                            this.patchState({ savingPassed: false });
                            this.notificationService.showError(
                                'Fehler beim Speichern'
                            );
                            return EMPTY;
                        })
                    )
            )
        )
    );

    readonly setOrigin = this.effect((origin$: Observable<Origin>) =>
        origin$.pipe(tap((origin) => this.patchState({ origin: origin })))
    );

    private readonly changeCourse$ = this.effect(() =>
        this.submission$.pipe(
            filter((submission) => submission != null),
            tap((submission) =>
                this.store.dispatch(
                    new ChangeCourse({ course: submission.exercise.course })
                )
            )
        )
    );

    readonly loadTaskInstance$ = this.effect(() =>
        combineLatest(
            this.taskInstances$,
            this.taskId$,
            this.submission$,
            this.unit$
        ).pipe(
            map(([taskInstances, taskId]) => {
                return taskId
                    ? taskInstances.find((ti) => ti.task.id == taskId)
                    : taskInstances[0];
            }),
            filter((ti) => ti != null),
            map((ti) => ({ taskId: ti.task.id, exerciseId: ti.exercise.id })),
            loadTaskInstance((s) => this.patchCurrentTaskInstance(s))
        )
    );

    /*
    readonly reset$ =         combineLatest(this.submissionId$, taskId$)
        .subscribe(([submissionId, taskId]) =>
            this.submissionStore.patchState({
                currentTaskInstance: null,
                historySolutions: [],
                solution: null,
            })
        );

     */

    readonly correctOrigin$ = this.effect(() =>
        this.origin$.pipe(
            distinctUntilChanged(),
            withLatestFrom(this.taskId$),
            map(([origin, taskId]) =>
                origin == Origin.Task && taskId == null
                    ? Origin.Exercise
                    : origin
            ),
            tap((origin) => this.setOrigin(origin))
        )
    );

    readonly updateNextSubmission$ = this.effect(() =>
        combineLatest(this.submissions$, this.submission$).pipe(
            filter(([submissions, submission]) => submission != null),
            map(([submissions, submission]) =>
                submission.getNextSubmission(submissions)
            ),
            tap((nextSubmission) =>
                this.patchState({ nextSubmission: nextSubmission })
            )
        )
    );

    readonly updateSubmissionAfterChange$ = this.effect(() =>
        this.submission$.pipe(
            filter((submission) => submission != null),
            withLatestFrom(this.submissions$),
            map(([submission, submissions]) => [
                submission,
                submissions.find((s) => s.id == submission.id),
            ]),
            filter(
                ([newSubmission, submissionInList]) => submissionInList != null
            ),
            tap(([newSubmission, submissionInList]) => {
                submissionInList.checkedBy = newSubmission.checkedBy;
                submissionInList.passed = newSubmission.passed;
                submissionInList.totalPoints = newSubmission.totalPoints;
                submissionInList.pointRatingCount =
                    newSubmission.pointRatings.length;
            })
        )
    );

    // todo: progressState
    readonly updateInitialProgress$ = this.effect(() =>
        combineLatest(this.submission$, this.unit$).pipe(
            switchMap(([submission, unit]) =>
                submission != null &&
                submission.exercise.usingAssessmentMode &&
                unit != null
                    ? this.exerciseService
                          .getAssessmentProgress(
                              submission.exercise.id,
                              unit.id
                          )
                          .pipe(
                              catchError((err) =>
                                  this.globalErrorHandler.handleError(err)
                              ),
                              catchError((err) => {
                                  return [];
                              })
                          )
                    : of([])
            ),
            tap((unitProgresses) =>
                this.patchState({ progress: unitProgresses[0]?.progress })
            )
        )
    );

    readonly updateTaskContent$ = this.effect(() =>
        this.currentTaskInstance$.pipe(
            distinctUntilChanged(
                (a, b) => a == b || (a != null && a.equals(b))
            ),
            filter(
                (taskInstance) =>
                    taskInstance != null && !taskInstance.task.archived
            ),
            loadTaskContent((s) => this.patchTaskContent(s))
        )
    );

    readonly updateSolution$ = this.effect(() =>
        combineLatest(this.submission$, this.currentTaskInstance$).pipe(
            filter(([submission, ti]) => ti != null && submission != null),
            distinctUntilChanged(
                ([submissionA, taskInstanceA], [submissionB, taskInstanceB]) =>
                    (taskInstanceA == taskInstanceB ||
                        taskInstanceA.equals(taskInstanceB)) &&
                    (submissionA == submissionB ||
                        submissionA.id == submissionB.id)
            ),
            map(([submission, ti]) => ({
                taskId: ti.task.id,
                submissionId: submission.id,
                valid: true,
            })),
            loadLatestSolution((s) => this.patchSolution(s))
        )
    );

    readonly updateSubmissions$ = this.effect(() =>
        combineLatest(
            this.submission$.pipe(
                distinctUntilChanged(
                    (a, b) => a?.exercise?.id == b?.exercise?.id
                )
            ),
            this.unit$
        ).pipe(
            filter(([submission, unit]) => submission != null && unit != null),
            map(([submission, unit]) => ({
                exerciseId: submission.exercise.id,
                unitId: unit.id,
            })),
            loadSubmissions((s) => this.patchSubmissions(s))
        )
    );
}
