import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { BaseComponent } from 'app/shared/misc/base.component';
import { ParamHelper } from 'app/shared/misc/helper/param.helper';
import { EMPTY } from 'rxjs';
import { catchError, map, takeUntil } from 'rxjs/operators';
import { Origin } from 'app/assessment/models/origin';
import { ErrorContainerConfig } from 'app/shared/error/error-container-config';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { TaskInstanceAssessmentPageStore } from 'app/assessment/pages/task-instance-assessment/task-instance-assessment.page.store';
import { ifAttached } from 'app/shared/route-reuse/route-reuse-life.cycle.directive';
import { PageHeaderAction } from 'app/shared/page-header/page-header/page-header.component';
import { TaskInstance } from 'app/exercises/models/task-instance';

@Component({
    templateUrl: './task-instance-assessment.page.component.html',
    styleUrls: ['./task-instance-assessment.page.component.scss'],
    providers: [TaskInstanceAssessmentPageStore],
})
export class TaskInstanceAssessmentPageComponent
    extends BaseComponent
    implements OnInit
{
    taskInstanceState$ = this.taskInstanceAssessmentStore.taskInstanceState$;

    errorConfig: ErrorContainerConfig = {
        notFound: {
            message: 'Diese Übung konnten wir leider nicht finden.',
        },
    };

    actions$ = this.taskInstanceState$.pipe(
        takeUntil(this.destroy$),
        ifAttached(this),
        map((taskInstanceState) =>
            this.buildActions(taskInstanceState.taskInstance)
        )
    );

    Origin = Origin;

    constructor(
        private taskInstanceAssessmentStore: TaskInstanceAssessmentPageStore,
        private globalErrorHandler: GlobalErrorHandler,
        private route: ActivatedRoute
    ) {
        super();
    }

    ngOnInit(): void {
        this.route.paramMap
            .pipe(
                takeUntil(this.destroy$),
                ifAttached(this),
                map((params) => {
                    let exerciseId = ParamHelper.parseIdOrThrow(
                        params,
                        'exercise_id'
                    );
                    let taskId = ParamHelper.parseIdOrThrow(params, 'task_id');
                    return [exerciseId, taskId];
                }),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.taskInstanceAssessmentStore.patchTaskInstance({
                        error: err,
                    });
                    return EMPTY;
                })
            )
            .subscribe(([exerciseId, taskId]) => {
                this.taskInstanceAssessmentStore.loadTaskInstance({
                    exerciseId,
                    taskId,
                });
            });
    }

    onAttach() {
        super.onAttach();
        this.taskInstanceAssessmentStore.onAttach();
    }

    onDetach() {
        super.onDetach();
        this.taskInstanceAssessmentStore.onDetach();
    }

    buildActions(taskInstance: TaskInstance): PageHeaderAction[] {
        let actions: PageHeaderAction[] = [];
        if (taskInstance != null) {
            actions.push({
                name: 'Abnahmetests',
                icon: 'play-circle-outline',
                link: {
                    commands: [
                        '/',
                        'exercises',
                        taskInstance.exercise.id,
                        'acceptance',
                    ],
                },
                status: 'primary',
            });
        }

        return actions;
    }
}
