import { Injectable } from '@angular/core';
import { catchError, filter, switchMap, tap } from 'rxjs/operators';
import { EMPTY, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { SubatoError } from 'app/shared/error/error';
import { SubatoPrincipal } from 'app/auth/models/principal/subato-principal';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { Exercise } from 'app/exercises/models/exercise';
import { UnitAssessmentProgress } from 'app/assessment/models/unit-assessment-progress';
import { ExerciseService } from 'app/exercises/services/exercise.service';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';

interface UnitAssessmentProgressesState {
    progresses: UnitAssessmentProgress[];
    error: SubatoError;
    loading: boolean;
}

interface SubmissionListState {
    exercise: Exercise;
    progressesState: UnitAssessmentProgressesState;
    onlyOwnUnits: boolean;
}

export const initialState: SubmissionListState = {
    exercise: null,
    progressesState: { progresses: [], error: null, loading: true },
    onlyOwnUnits: true,
};

@Injectable()
export class SubmissionListStore extends BaseComponentStore<SubmissionListState> {
    public readonly exercise$ = this.select((state) => state.exercise);
    public readonly progresses$ = this.select(
        (state) => state.progressesState.progresses
    );
    public readonly progressesState$ = this.select(
        (state) => state.progressesState
    );

    public readonly onlyOwnUnits$ = this.select((state) => state.onlyOwnUnits);

    principal$: Observable<SubatoPrincipal> = this.store.select(
        (s) => s.auth.principal
    );

    public readonly filteredProgresses$ = this.select(
        this.progresses$,
        this.onlyOwnUnits$,
        this.principal$,
        (progresses, onlyOwnUnits, principal) =>
            progresses.filter(
                (p) =>
                    !onlyOwnUnits ||
                    p.unit.lecturers.some(
                        (l) => l.loginName == principal.loginName
                    )
            )
    );

    constructor(
        private store: Store<SubatoState>,
        private exerciseService: ExerciseService,
        private globalErrorHandler: GlobalErrorHandler
    ) {
        super(initialState);
    }

    patchProgresses = this.updaterFor('progressesState');

    readonly loadProgresses$ = this.effect(() =>
        this.exercise$.pipe(
            filter((exercise) => exercise != null),
            tap(() =>
                this.patchProgresses({
                    loading: true,
                })
            ),
            switchMap((exercise) =>
                this.exerciseService.getAssessmentProgress(exercise.id).pipe(
                    tap((progresses) =>
                        this.patchProgresses({
                            progresses: progresses,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        this.patchProgresses({
                            progresses: [],
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        )
    );
}
