import { Component, Input, OnInit } from '@angular/core';
import { AttendanceService } from 'app/attendances/services/attendance.service';
import { CourseUnitEvent } from 'app/courses/models/course-unit-event';
import { BaseComponent } from 'app/shared/misc/base.component';
import { catchError, map, takeUntil } from 'rxjs/operators';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { ParticipantAttendance } from 'app/attendances/dto/participant-attendance.dto';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { Observable } from 'rxjs';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { SubatoError } from 'app/shared/error/error';

@Component({
    selector: 'sb-participant-attendance-list',
    templateUrl: './participant-attendance-list.component.html',
    styleUrls: ['./participant-attendance-list.component.scss'],
})
export class ParticipantAttendanceListComponent
    extends BaseComponent
    implements OnInit
{
    @Input() participantAttendance: ParticipantAttendance;
    @Input() writable: boolean = false;
    principal$ = this.store.select((s) => s.auth.principal);
    isLecturerOrAdmin$ = this.principal$.pipe(
        takeUntil(this.destroy$),
        map(
            (principal) =>
                principal != null &&
                this.participantAttendance != null &&
                principal.isLecturerOrAdmin(
                    this.participantAttendance.courseUnit.course.id
                )
        )
    );

    processingEvent: CourseUnitEvent | null = null;

    constructor(
        private attendanceService: AttendanceService,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService
    ) {
        super();
    }

    ngOnInit(): void {}

    get events() {
        return (
            this.participantAttendance.events
                //.filter(e => moment().isSameOrAfter(e.takesPlaceAt))
                .sort((a, b) => a.takesPlaceAt.diff(b.takesPlaceAt))
        );
    }

    isPresent(event: CourseUnitEvent): boolean {
        return (
            this.participantAttendance.eventAttendances.find(
                (eA) => eA.event.id === event.id
            ) != null
        );
    }

    createAttendance(event: CourseUnitEvent) {
        this.processingEvent = event;
        this.attendanceService
            .createAttendance({
                userId: this.participantAttendance.user.id,
                eventId: event.id,
            })
            .pipe(
                takeUntil(this.destroy$),
                catchError((err) => this.globalErrorHandler.handleError(err))
            )
            .subscribe(
                (attendance) => {
                    let eventAttendance =
                        this.participantAttendance.eventAttendances.find(
                            (eA) => eA.event.id === event.id
                        );
                    if (eventAttendance == null) {
                        this.participantAttendance.eventAttendances.push(
                            attendance
                        );
                        this.participantAttendance.presentSum += 1;
                    }
                    this.processingEvent = null;
                    this.notificationService.showSuccess(
                        'Anwesenheit erfolgreich gesetzt'
                    );
                },
                (error: SubatoError) => {
                    this.processingEvent = null;

                    this.notificationService.showError(
                        this.globalErrorHandler.format(
                            'Fehler beim Speichern der Anwesenheit',
                            error
                        )
                    );
                }
            );
    }

    deleteAttendance(event: CourseUnitEvent) {
        this.processingEvent = event;
        this.attendanceService
            .deleteAttendance(this.participantAttendance.user.id, event.id)
            .pipe(
                takeUntil(this.destroy$),
                catchError((err) => this.globalErrorHandler.handleError(err))
            )
            .subscribe(
                () => {
                    let idx =
                        this.participantAttendance.eventAttendances.findIndex(
                            (eA) => eA.event.id === event.id
                        );
                    if (idx != -1) {
                        this.participantAttendance.eventAttendances.splice(
                            idx,
                            1
                        );
                        this.participantAttendance.presentSum -= 1;
                    }
                    this.processingEvent = null;
                    this.notificationService.showSuccess(
                        'Anwesenheit erfolgreich entfernt'
                    );
                },
                (error: SubatoError) => {
                    this.processingEvent = null;
                    this.notificationService.showError(
                        this.globalErrorHandler.format(
                            'Fehler beim Speichern der Anwesenheit',
                            error
                        )
                    );
                }
            );
    }
}
