import { SubatoError } from 'app/shared/error/error';
import { Course } from 'app/courses/models/course';
import { EMPTY, Observable } from 'rxjs';
import { catchError, filter, switchMap, tap } from 'rxjs/operators';
import { inject, Injectable } from '@angular/core';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { ChangeCourse } from 'app/courses/store/courses.actions';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { User } from 'app/shared/user/user';
import { UserService } from 'app/shared/user/user.service';

export interface UserState {
    user: User | null;
    error: SubatoError;
    loading: boolean;
}

export function loadUser(updaterFn: (state: Partial<UserState>) => void) {
    let op = inject(UserOperators);
    return (userId: Observable<number>) => op.loadUser(userId, updaterFn);
}

@Injectable({ providedIn: 'root' })
export class UserOperators {
    constructor(
        private userService: UserService,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler
    ) {}

    public dispatchCourseChange(course: Observable<Course>) {
        return course.pipe(
            filter((course) => course != null),
            tap((course) =>
                this.store.dispatch(new ChangeCourse({ course: course }))
            )
        );
    }

    public loadUser(
        userId$: Observable<number>,
        updaterFn: (state: Partial<UserState>) => void
    ) {
        return userId$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((userId) =>
                this.userService.getById(userId).pipe(
                    tap((user: User) =>
                        updaterFn({
                            user: user,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            user: null,
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }
}
