import { Injectable } from '@angular/core';
import {
    catchError,
    filter,
    map,
    switchMap,
    tap,
    withLatestFrom,
} from 'rxjs/operators';
import { NEVER, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { CourseService } from 'app/courses/services/course.service';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { CourseFile } from 'app/courses/models/course-file';
import { CourseFileService } from 'app/courses/services/course-file.service';
import { SubatoPrincipal } from 'app/auth/models/principal/subato-principal';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import {
    CourseState,
    dispatchCourseChange,
    loadCourse,
} from 'app/courses/shared/operators/course.operators';
import {
    CourseFilesState,
    loadFiles,
} from 'app/courses/shared/operators/course-file.operators';

interface CourseFilesPageState {
    courseState: CourseState;
    filesState: CourseFilesState;
    deletingFile: CourseFile;
    uploading: boolean;
}

export const initialState: CourseFilesPageState = {
    courseState: { course: null, error: null, loading: true },
    filesState: { files: [], error: null, loading: true },
    deletingFile: null,
    uploading: false,
};

export interface CreateCourseFileParams {
    description: string;
    file: File;
}

@Injectable()
export class CourseFilesPageStore extends BaseComponentStore<CourseFilesPageState> {
    principal$: Observable<SubatoPrincipal> = this.store.select(
        (s) => s.auth.principal
    );

    public readonly courseState$ = this.select((state) => state.courseState);
    public readonly course$ = this.select((state) => state.courseState.course);
    public readonly filesState$ = this.select((state) => state.filesState);
    public readonly files$ = this.select((state) => state.filesState.files);
    public readonly deletingFile$ = this.select((state) => state.deletingFile);
    public readonly uploading$ = this.select((state) => state.uploading);

    isAdmin$ = this.select(
        this.course$,
        this.principal$,
        (course, principal) =>
            course != null && principal != null && principal.isAdmin(course.id)
    );

    constructor(
        private store: Store<SubatoState>,
        private courseService: CourseService,
        private courseFileService: CourseFileService,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService
    ) {
        super(initialState);
    }

    readonly patchCourse = this.updaterFor('courseState');
    readonly patchFiles = this.updaterFor('filesState');

    readonly loadCourse = this.effect((courseId$: Observable<number>) =>
        courseId$.pipe(loadCourse((s) => this.patchCourse(s)))
    );

    readonly dispatchCourseChange$ = this.effect(() =>
        this.course$.pipe(dispatchCourseChange)
    );

    readonly loadFiles$ = this.effect(() =>
        this.course$.pipe(
            filter((course) => course != null),
            map((course) => course.id),
            loadFiles((s) => this.patchFiles(s))
        )
    );

    readonly deleteFile = this.effect((file: Observable<CourseFile>) =>
        file.pipe(
            tap((file) => this.patchState({ deletingFile: file })),
            switchMap((file) =>
                this.courseFileService.delete(file.id).pipe(
                    tap(() => {
                        this.patchStateFor('filesState', (s) => ({
                            files: s.files.filter((m) => m != file),
                        }));

                        this.patchState((s) => ({
                            deletingFile: null,
                        }));
                    }),
                    tap((membership) => {
                        this.notificationService.showSuccess(
                            'Datei erfolgreich gelöscht'
                        );
                    }),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.notificationService.showError(
                            this.globalErrorHandler.format(
                                'Fehler beim Löschen der Datei',
                                error
                            )
                        );

                        this.patchState({ deletingFile: null });
                        return NEVER;
                    })
                )
            )
        )
    );

    readonly uploadFile = this.effect(
        (params: Observable<CreateCourseFileParams>) =>
            params.pipe(
                tap(() => this.patchState({ uploading: true })),
                withLatestFrom(this.course$),
                switchMap(([params, course]) =>
                    this.courseFileService
                        .create(
                            {
                                description: params.description,
                                courseId: course.id,
                            },
                            params.file
                        )
                        .pipe(
                            tap((file) => {
                                this.patchStateFor('filesState', (s) => ({
                                    files: [file, ...s.files],
                                }));

                                this.patchState((s) => ({
                                    uploading: false,
                                }));
                            }),
                            tap((file) => {
                                this.notificationService.showSuccess(
                                    'Datei erfolgreich hochgeladen'
                                );
                            }),
                            catchError((err) =>
                                this.globalErrorHandler.handleError(err)
                            ),
                            catchError((error) => {
                                this.notificationService.showError(
                                    this.globalErrorHandler.format(
                                        'Fehler beim Hochladen der Datei',
                                        error
                                    )
                                );

                                this.patchState({ uploading: false });
                                return NEVER;
                            })
                        )
                )
            )
    );
}
