import { Injectable } from '@angular/core';
import { filter, map } from 'rxjs/operators';
import { combineLatest, Observable } from 'rxjs';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import {
    CourseState,
    dispatchCourseChange,
    loadCourse,
} from 'app/courses/shared/operators/course.operators';
import {
    CourseParticipantsState,
    loadParticipants,
} from 'app/courses/shared/operators/course-participant.operators';
import { ifAttached } from 'app/shared/route-reuse/route-reuse-life.cycle.directive';
import { debounceWithInitialValue } from 'app/shared/misc/rxjs-operators';
import { environment } from 'environments/environment';

interface CourseParticipantsPage {
    courseState: CourseState;
    participantsState: CourseParticipantsState;
    query: string;
}

export const initialState: CourseParticipantsPage = {
    courseState: { course: null, error: null, loading: true },
    participantsState: { participants: [], error: null, loading: true },
    query: '',
};

@Injectable()
export class CourseParticipantsPageStore extends BaseComponentStore<CourseParticipantsPage> {
    public readonly course$ = this.select((state) => state.courseState.course);
    public readonly courseState$ = this.select((state) => state.courseState);

    public readonly participants$ = this.select(
        (state) => state.participantsState.participants
    );
    public readonly participantsState$ = this.select(
        (state) => state.participantsState
    );

    public readonly query$ = this.select((state) => state.query);
    public readonly debouncedQuery$ = this.query$.pipe(
        debounceWithInitialValue(environment.debounceTimeMs, initialState.query)
    );

    constructor() {
        super(initialState);
    }

    readonly patchCourse = this.updaterFor('courseState');
    readonly patchParticipants = this.updaterFor('participantsState');

    readonly loadCourse = this.effect((courseId$: Observable<number>) =>
        courseId$.pipe(loadCourse((s) => this.patchCourse(s)))
    );

    readonly dispatchCourseChange$ = this.effect(() =>
        combineLatest([this.course$, this.detached$], (course) => course).pipe(
            ifAttached(this),
            dispatchCourseChange
        )
    );

    readonly loadParticipants$ = this.effect(() =>
        combineLatest([this.course$, this.debouncedQuery$]).pipe(
            filter(([course]) => course != null),
            map(([course, query]) => ({ courseId: course.id, query: query })),
            loadParticipants((s) => this.patchParticipants(s))
        )
    );
}
