import { Injectable } from '@angular/core';
import { ComponentStore } from '@ngrx/component-store';
import { catchError, filter, switchMap, tap } from 'rxjs/operators';
import { EMPTY, NEVER, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { SubatoError } from 'app/shared/error/error';
import { CourseStaffMembership } from 'app/courses/models/course-staff-membership';
import { CourseService } from 'app/courses/services/course.service';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { Course } from 'app/courses/models/course';
import { StaffGroup } from 'app/courses/models/staff-group';

interface CourseStaffMembershipState {
    memberships: CourseStaffMembership[];
    loading: boolean;
    error: SubatoError;
}

export const initialState: CourseStaffMembershipState = {
    memberships: [],
    loading: true,
    error: null,
};

export interface AddStaffMembershipParams {
    userId: number;
    courseId: number;
    group: StaffGroup;
}

@Injectable()
export class CourseStaffMembershipStore extends ComponentStore<CourseStaffMembershipState> {
    public readonly memberships$ = this.select((state) => state.memberships);
    public readonly loading$ = this.select((state) => state.loading);
    public readonly error$ = this.select((state) => state.error);

    constructor(
        private store: Store<SubatoState>,
        private courseService: CourseService,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService
    ) {
        super(initialState);
    }

    readonly loadStaff = this.effect((course$: Observable<Course>) =>
        course$.pipe(
            filter((course) => course != null),
            switchMap((course) =>
                this.courseService.getStaff(course.id).pipe(
                    tap((memberships) =>
                        this.patchState({
                            memberships: memberships,
                            error: null,
                            loading: false,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.patchState({ error: error, loading: false });
                        return EMPTY;
                    })
                )
            )
        )
    );

    readonly removeStaff = this.effect(
        (membership: Observable<CourseStaffMembership>) =>
            membership.pipe(
                switchMap((membership) =>
                    this.courseService
                        .removeStaff(
                            membership.course.id,
                            membership.user.id,
                            membership.group
                        )
                        .pipe(
                            tap(() => {
                                this.patchState((s) => ({
                                    memberships: s.memberships.filter(
                                        (m) => m != membership
                                    ),
                                }));
                            }),
                            tap((membership) => {
                                this.notificationService.showSuccess(
                                    'Berechtigung erfolgreich entfernt'
                                );
                            }),
                            catchError((err) =>
                                this.globalErrorHandler.handleError(err)
                            ),
                            catchError((error) => {
                                this.notificationService.showError(
                                    this.globalErrorHandler.format(
                                        'Fehler bei Entfernung der Berechtigung',
                                        error
                                    )
                                );

                                return NEVER;
                            })
                        )
                )
            )
    );

    readonly addStaff = this.effect(
        (params: Observable<AddStaffMembershipParams>) =>
            params.pipe(
                switchMap((params) =>
                    this.courseService
                        .addStaff(params.courseId, params.userId, params.group)
                        .pipe(
                            tap((membership) => {
                                this.patchState((s) => ({
                                    memberships: [membership, ...s.memberships],
                                }));
                            }),
                            tap((membership) => {
                                this.notificationService.showSuccess(
                                    'Berechtigung erfolgreich hinzugefügt'
                                );
                            }),
                            catchError((err) =>
                                this.globalErrorHandler.handleError(err)
                            ),
                            catchError((error) => {
                                this.notificationService.showError(
                                    this.globalErrorHandler.format(
                                        'Fehler bei Erstellung der Berechtigung',
                                        error
                                    )
                                );

                                return NEVER;
                            })
                        )
                )
            )
    );
}
