import { Component, OnInit } from '@angular/core';
import { Course } from '../../../../models/course';
import {
    UntypedFormBuilder,
    UntypedFormGroup,
    Validators,
} from '@angular/forms';
import { CourseService } from '../../../../services/course.service';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { ActivatedRoute, Router } from '@angular/router';
import { AuthService } from 'app/auth/auth.service';
import { NbDialogService } from '@nebular/theme';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { UpdateCourseDto } from '../../../../dto/update-course.dto';
import { filter, map, switchMap, take, takeUntil } from 'rxjs/operators';
import { FormComponent } from 'app/shared/validation/form.component';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { CourseSettingsPageStore } from 'app/courses/pages/course-settings/course-settings.page.store';
import { combineLatest } from 'rxjs';
import { ExplicitConfirmationDialogComponent } from 'app/shared/dialogs/explicit-confirmation-dialog/explicit-confirmation-dialog.component';

@Component({
    selector: 'sb-course-settings-form',
    templateUrl: './course-settings-form.component.html',
    styleUrls: ['./course-settings-form.component.scss'],
})
export class CourseSettingsFormComponent
    extends FormComponent
    implements OnInit
{
    courseForm: UntypedFormGroup;

    courseState$ = this.courseSettingsStore.courseState$;
    deletingCourse$ = this.courseSettingsStore.deletingCourse$;
    canFinish$ = this.courseSettingsStore.canFinish$;
    finishingCourse$ = this.courseSettingsStore.finishingCourse$;

    descriptionState$ = this.courseSettingsStore.descriptionState$;
    validationError$ = this.courseSettingsStore.validationError$;

    constructor(
        private courseService: CourseService,
        private store: Store<SubatoState>,
        private router: Router,
        private courseSettingsStore: CourseSettingsPageStore,
        private authService: AuthService,
        private globalErrorHandler: GlobalErrorHandler,
        private dialogService: NbDialogService,
        private notificationService: NotificationService,
        private fb: UntypedFormBuilder,
        private route: ActivatedRoute
    ) {
        super();
    }

    ngOnInit(): void {
        this.validationError$
            .pipe(
                takeUntil(this.destroy$),
                filter((error) => error != null),
                map((error) => this.getValidationErrors(error))
            )
            .subscribe((errors) => (this.errors = errors));

        combineLatest([
            this.courseState$.pipe(map((courseState) => courseState.course)),
            this.descriptionState$.pipe(
                map((descriptionState) => descriptionState.description)
            ),
        ])
            .pipe(
                filter(
                    ([course, description]) =>
                        course != null && description != null
                )
            )
            .subscribe(
                ([course, description]) =>
                    (this.courseForm = this.createForm(course, description))
            );
    }

    createForm(course?: Course, description?: string) {
        return this.fb.group({
            name: [course?.name ?? '', [Validators.required]],
            shortName: [course?.shortName ?? '', [Validators.required]],
            description: [description ?? ''],
            motd: [course?.motd ?? ''],
            showEmails: [course?.showEmails ?? false],
            allowJoiningUnits: [course?.allowJoiningUnits ?? false],
            disableEditor: [course?.disableEditor ?? false],
            visibility: [course?.visibility ?? null, [Validators.required]],
            submitPolicy: [course?.submitPolicy ?? null, [Validators.required]],
        });
    }

    save() {
        let dto: UpdateCourseDto = {
            name: this.courseForm.get('name').value,
            shortName: this.courseForm.get('shortName').value,
            description: this.courseForm.get('description').value,
            motd: this.courseForm.get('motd').value,
            showEmails: this.courseForm.get('showEmails').value,
            allowJoiningUnits: this.courseForm.get('allowJoiningUnits').value,
            disableEditor: this.courseForm.get('disableEditor').value,
            visibility: this.courseForm.get('visibility').value,
            submitPolicy: this.courseForm.get('submitPolicy').value,
        };
        this.courseSettingsStore.updateCourse(dto);
    }

    delete() {
        this.courseState$
            .pipe(
                take(1),
                switchMap((courseState) =>
                    this.dialogService
                        .open(ExplicitConfirmationDialogComponent, {
                            context: {
                                title: `Löschung von Kurs: ${courseState.course.name}`,
                                text: `Bist du wirklich sicher, dass du den Kurs '${courseState.course.name}' löschen willst? Alle verknüpften Übungen, Abgaben, Lösungen etc. werden damit permanent gelöscht.`,
                                actionText: 'Löschen',
                                expectedText: courseState.course.name,
                            },
                        })
                        .onClose.pipe(
                            takeUntil(this.destroy$),
                            filter((result) => result && result.confirmed)
                        )
                )
            )
            .subscribe(() => this.courseSettingsStore.deleteCourse());
    }

    finish(finished: boolean) {
        this.courseSettingsStore.updateFinished(finished);
    }
}
