import { Injectable } from '@angular/core';
import {
    catchError,
    map,
    switchMap,
    tap,
    withLatestFrom,
} from 'rxjs/operators';
import { EMPTY, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { UpdateCourseDto } from 'app/courses/dto/update-course.dto';
import { CourseService } from 'app/courses/services/course.service';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { Router } from '@angular/router';
import { isValidationError, ValidationError } from 'app/shared/error/error';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import {
    CourseState,
    dispatchCourseChange,
    loadCourse,
} from 'app/courses/shared/operators/course.operators';
import {
    DescriptionState,
    loadDescription,
} from 'app/courses/shared/operators/description.operators';

interface CourseSettingsPageState {
    courseState: CourseState;
    descriptionState: DescriptionState;
    validationError: ValidationError | null;
    deletingCourse: boolean;
    reimportingCourse: boolean;
    finishingCourse: boolean;
}

export const initialState: CourseSettingsPageState = {
    courseState: { course: null, error: null, loading: true },
    descriptionState: { description: null, error: null, loading: true },
    validationError: null,
    deletingCourse: false,
    reimportingCourse: false,
    finishingCourse: false,
};

@Injectable()
export class CourseSettingsPageStore extends BaseComponentStore<CourseSettingsPageState> {
    principal$ = this.store.select((s) => s.auth.principal);
    isSuperAdmin$ = this.select(this.principal$, (principal) =>
        principal.hasSuperAdminRole()
    );

    public readonly courseState$ = this.select((state) => state.courseState);
    public readonly course$ = this.select((state) => state.courseState.course);
    public readonly descriptionState$ = this.select(
        (state) => state.descriptionState
    );

    public readonly validationError$ = this.select(
        (state) => state.validationError
    );
    public readonly deletingCourse$ = this.select(
        (state) => state.deletingCourse
    );
    public readonly reimportingCourse$ = this.select(
        (state) => state.reimportingCourse
    );
    public readonly finishingCourse$ = this.select(
        (state) => state.finishingCourse
    );

    isAdmin$ = this.select(
        this.course$,
        this.principal$,
        (course, principal) =>
            course != null && principal != null && principal.isAdmin(course.id)
    );

    canReimport$ = this.select(
        this.course$,
        this.isSuperAdmin$,
        (course, isAdmin) =>
            course != null && course.modules.length != 0 && isAdmin
    );

    canFinish$ = this.select(
        this.course$,
        this.isAdmin$,
        (course, isAdmin) => course != null && !course.finished && isAdmin
    );

    constructor(
        private store: Store<SubatoState>,
        private courseService: CourseService,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService,
        private router: Router
    ) {
        super(initialState);
    }

    readonly patchCourse = this.updaterFor('courseState');
    readonly patchDescription = this.updaterFor('descriptionState');

    readonly loadCourse = this.effect((courseId$: Observable<number>) =>
        courseId$.pipe(loadCourse((s) => this.patchCourse(s)))
    );

    readonly dispatchCourseChange$ = this.effect(() =>
        this.course$.pipe(dispatchCourseChange)
    );

    readonly updateCourse = this.effect((params: Observable<UpdateCourseDto>) =>
        params.pipe(
            withLatestFrom(this.course$),
            switchMap(([dto, course]) =>
                this.courseService.update(course.id, dto).pipe(
                    tap((course) => this.patchCourse({ course: course })),
                    tap((course) => {
                        this.notificationService.showSuccess(
                            'Kurs erfolgreich aktualisiert'
                        );
                        this.router.navigate(['/', 'course', course.id]);
                    }),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        if (isValidationError(error)) {
                            this.patchState({ validationError: error });
                        }

                        this.notificationService.showError(
                            this.globalErrorHandler.format(
                                'Fehler bei Aktualisierung des Kurses',
                                error
                            )
                        );

                        return EMPTY;
                    })
                )
            )
        )
    );

    readonly deleteCourse = this.effect((trigger$: Observable<void>) =>
        trigger$.pipe(
            withLatestFrom(this.course$),
            map(([_, course]) => course),
            tap(() => this.patchState({ deletingCourse: true })),
            switchMap((course) =>
                this.courseService.delete(course.id).pipe(
                    tap(() => this.patchState({ deletingCourse: false })),
                    tap(() => {
                        this.notificationService.showSuccess(
                            'Kurs erfolgreich gelöscht'
                        );
                        this.router.navigate(['course']);
                    }),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.patchState({ deletingCourse: false });

                        this.notificationService.showError(
                            this.globalErrorHandler.format(
                                'Fehler beim Löschen des Kurses',
                                error
                            )
                        );

                        return EMPTY;
                    })
                )
            )
        )
    );

    readonly reimportCourse = this.effect((trigger$: Observable<void>) =>
        trigger$.pipe(
            withLatestFrom(this.course$),
            map(([_, course]) => course),
            tap(() => this.patchState({ reimportingCourse: true })),
            switchMap((course) =>
                this.courseService.reimport(course.id).pipe(
                    tap(() => this.patchState({ reimportingCourse: false })),
                    tap(() => {
                        this.notificationService.showSuccess(
                            'Reimport erfolgreich'
                        );
                    }),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.patchState({ reimportingCourse: false });

                        this.notificationService.showError(
                            this.globalErrorHandler.format(
                                'Fehler bei Reimport',
                                error
                            )
                        );

                        return EMPTY;
                    })
                )
            )
        )
    );

    readonly updateFinished = this.effect((finished$: Observable<boolean>) =>
        finished$.pipe(
            withLatestFrom(this.course$),
            tap(() => this.patchState({ finishingCourse: true })),
            switchMap(([finished, course]) =>
                this.courseService
                    .updateFinished(course.id, { finished: finished })
                    .pipe(
                        tap(() => this.patchState({ finishingCourse: false })),
                        tap((course) => this.patchCourse({ course: course })),
                        tap(() => {
                            this.notificationService.showSuccess(
                                finished
                                    ? 'Abschluss erfolgreich'
                                    : 'Abschluss erfolgreich aufgehoben'
                            );
                        }),
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((error) => {
                            this.patchState({ finishingCourse: false });

                            this.notificationService.showError(
                                this.globalErrorHandler.format(
                                    finished
                                        ? 'Fehler bei Abschluss'
                                        : 'Fehler bei Aufhebung von Abschluss',
                                    error
                                )
                            );

                            return EMPTY;
                        })
                    )
            )
        )
    );

    readonly loadDescription$ = this.effect(() =>
        this.course$.pipe(
            loadDescription(true, (s) => this.patchDescription(s))
        )
    );
}
