import { Injectable } from '@angular/core';
import { ComponentStore } from '@ngrx/component-store';
import { catchError, filter, switchMap, tap } from 'rxjs/operators';
import { EMPTY, NEVER, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { SubatoError } from 'app/shared/error/error';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { CourseUnitService } from 'app/courses/services/course-unit.service';
import { CourseUnitEvent } from 'app/courses/models/course-unit-event';
import { CourseUnitEventService } from 'app/courses/services/course-unit-event.service';
import { CourseUnit } from 'app/courses/models/course-unit';

interface CourseUnitEventsState {
    unit: CourseUnit;
    loading: boolean;
    error: SubatoError;
    events: CourseUnitEvent[];
    processingEvent: CourseUnitEvent;
}

export const initialState: CourseUnitEventsState = {
    unit: null,
    loading: true,
    error: null,
    events: [],
    processingEvent: null,
};

export interface UpdateCancellationParams {
    event: CourseUnitEvent;
    cancel: boolean;
}

@Injectable()
export class CourseUnitEventsStore extends ComponentStore<CourseUnitEventsState> {
    public readonly unit$ = this.select((state) => state.unit);
    public readonly events$ = this.select((state) => state.events);
    public readonly loading$ = this.select((state) => state.loading);
    public readonly error$ = this.select((state) => state.error);
    public readonly processingEvent$ = this.select(
        (state) => state.processingEvent
    );

    constructor(
        private store: Store<SubatoState>,
        private courseUnitEventService: CourseUnitEventService,
        private courseUnitService: CourseUnitService,
        private notificationService: NotificationService,
        private globalErrorHandler: GlobalErrorHandler
    ) {
        super(initialState);
    }

    readonly setUnit = this.updater((state, unit: CourseUnit) => ({
        ...state,
        unit: unit,
    }));

    readonly updateCancellation = this.effect(
        (params$: Observable<UpdateCancellationParams>) =>
            params$.pipe(
                tap((params) =>
                    this.patchState({ processingEvent: params.event })
                ),
                switchMap((params) =>
                    this.courseUnitEventService
                        .updateCancellation(params.event.id, {
                            canceled: params.cancel,
                        })
                        .pipe(
                            tap((event) => {
                                this.patchState((s) => ({
                                    events: s.events.map((e) =>
                                        e != params.event ? e : event
                                    ),
                                    processingEvent: null,
                                }));
                            }),
                            tap((participant) => {
                                this.notificationService.showSuccess(
                                    'Termin erfolgreich geändert'
                                );
                            }),
                            catchError((err) =>
                                this.globalErrorHandler.handleError(err)
                            ),
                            catchError((error) => {
                                this.notificationService.showError(
                                    this.globalErrorHandler.format(
                                        'Fehler bei Änderung des Termins',
                                        error
                                    )
                                );

                                this.patchState({ processingEvent: null });
                                return NEVER;
                            })
                        )
                )
            )
    );

    readonly loadEvents = this.effect(() =>
        this.unit$.pipe(
            filter((unit) => unit != null),
            tap(() => this.patchState({ loading: true })),
            switchMap((unit) =>
                this.courseUnitService.getEvents(unit.id).pipe(
                    tap((events) =>
                        this.patchState({
                            events: events,
                            error: null,
                            loading: false,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.patchState({ error: error, loading: false });

                        this.notificationService.showError(
                            this.globalErrorHandler.format(
                                'Fehler beim Laden der Termine',
                                error
                            )
                        );

                        return EMPTY;
                    })
                )
            )
        )
    );
}
