import { Injectable } from '@angular/core';
import {
    catchError,
    filter,
    map,
    switchMap,
    tap,
    withLatestFrom,
} from 'rxjs/operators';
import { combineLatest, NEVER, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { User } from 'app/shared/user/user';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { CourseUnitService } from 'app/courses/services/course-unit.service';
import { CourseUnit } from 'app/courses/models/course-unit';
import {
    CourseUnitParticipantsState,
    loadParticipants,
} from 'app/courses/shared/operators/course-unit-participant.operators';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { debounceWithInitialValue } from 'app/shared/misc/rxjs-operators';
import { environment } from 'environments/environment';

interface CourseUnitParticipantListState {
    unit: CourseUnit;
    participantsState: CourseUnitParticipantsState;
    query: string;
    removingParticipant: User;
    addingParticipant: boolean;
}

export const initialState: CourseUnitParticipantListState = {
    unit: null,
    participantsState: { participants: [], error: null, loading: true },
    query: '',
    removingParticipant: null,
    addingParticipant: false,
};

@Injectable()
export class ParticipantListStore extends BaseComponentStore<CourseUnitParticipantListState> {
    public readonly unit$ = this.select((state) => state.unit);
    public readonly participants$ = this.select(
        (state) => state.participantsState.participants
    );
    public readonly participantsState$ = this.select(
        (state) => state.participantsState
    );
    public readonly query$ = this.select((state) => state.query);
    public readonly debouncedQuery$ = this.query$.pipe(
        debounceWithInitialValue(environment.debounceTimeMs, initialState.query)
    );
    public readonly removingParticipant$ = this.select(
        (state) => state.removingParticipant
    );
    public readonly addingParticipant$ = this.select(
        (state) => state.addingParticipant
    );

    constructor(
        private store: Store<SubatoState>,
        private courseUnitService: CourseUnitService,
        private notificationService: NotificationService,
        private globalErrorHandler: GlobalErrorHandler
    ) {
        super(initialState);
    }

    readonly patchParticipants = this.updaterFor('participantsState');

    readonly setUnit = this.updater((state, unit: CourseUnit) => ({
        ...state,
        unit: unit,
    }));

    readonly removeParticipant = this.effect((participant$: Observable<User>) =>
        participant$.pipe(
            withLatestFrom(this.unit$),
            tap(([participant, unit]) =>
                this.patchState({ removingParticipant: participant })
            ),
            switchMap(([participant, unit]) =>
                this.courseUnitService
                    .removeParticipant(unit.id, participant.id)
                    .pipe(
                        tap(() => {
                            this.patchStateFor('participantsState', (s) => ({
                                participants: s.participants.filter(
                                    (p) => p != participant
                                ),
                            }));
                        }),
                        tap((participant) => {
                            this.patchState({ removingParticipant: null });
                            this.notificationService.showSuccess(
                                'Teilnehmer erfolgreich entfernt'
                            );
                        }),
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((error) => {
                            this.patchState({ removingParticipant: null });

                            this.notificationService.showError(
                                this.globalErrorHandler.format(
                                    'Fehler bei Entfernung des Teilnehmers',
                                    error
                                )
                            );

                            return NEVER;
                        })
                    )
            )
        )
    );

    readonly addParticipant = this.effect((participant$: Observable<User>) =>
        participant$.pipe(
            withLatestFrom(this.unit$),
            tap(([participant, unit]) =>
                this.patchState({ addingParticipant: true })
            ),
            switchMap(([participant, unit]) =>
                this.courseUnitService
                    .addParticipant(unit.id, participant.id)
                    .pipe(map(() => participant))
                    .pipe(
                        tap((participant) => {
                            this.patchStateFor('participantsState', (s) => ({
                                participants: [participant, ...s.participants],
                            }));
                        }),
                        tap((membership) => {
                            this.patchState({ addingParticipant: false });

                            this.notificationService.showSuccess(
                                'Teilnehmer erfolgreich hinzugefügt'
                            );
                        }),
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((error) => {
                            this.patchState({ addingParticipant: false });

                            this.notificationService.showError(
                                this.globalErrorHandler.format(
                                    'Fehler beim Hinzufügen des Teilnehmers',
                                    error
                                )
                            );

                            return NEVER;
                        })
                    )
            )
        )
    );

    readonly loadParticipants$ = this.effect(() =>
        combineLatest([this.unit$, this.debouncedQuery$]).pipe(
            filter(([unit]) => unit != null),
            map(([unit, query]) => ({ unitId: unit.id, query: query })),
            loadParticipants((s) => this.patchParticipants(s))
        )
    );
}
