import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { User } from 'app/shared/user/user';
import { AuthService } from 'app/auth/auth.service';
import { AttendanceService } from 'app/attendances/services/attendance.service';
import { BaseComponent } from 'app/shared/misc/base.component';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { catchError, map, takeUntil } from 'rxjs/operators';
import { CourseUnitService } from 'app/courses/services/course-unit.service';
import { ifAttached } from 'app/shared/route-reuse/route-reuse-life.cycle.directive';
import { CourseService } from 'app/courses/services/course.service';
import { ParamHelper } from 'app/shared/misc/helper/param.helper';
import { ErrorContainerConfig } from 'app/shared/error/error-container-config';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { CourseUnitPageStore } from 'app/courses/pages/course-unit/course-unit.page.store';
import { combineLatest, EMPTY } from 'rxjs';
import { PageHeaderAction } from 'app/shared/page-header/page-header/page-header.component';
import { CourseUnit } from 'app/courses/models/course-unit';

@Component({
    selector: 'ngx-course-unit',
    templateUrl: './course-unit.page.component.html',
    styleUrls: ['./course-unit.page.component.scss'],
    providers: [CourseUnitPageStore],
})
export class CourseUnitPageComponent extends BaseComponent implements OnInit {
    unitState$ = this.courseUnitDetailStore.unitState$;
    isLecturerOrAdmin$ = this.courseUnitDetailStore.isLecturerOrAdmin$;
    attendanceState$ = this.courseUnitDetailStore.attendanceState$;
    selectedUser$ = this.courseUnitDetailStore.selectedUser$;

    errorConfig: ErrorContainerConfig = {
        notFound: {
            message: 'Diese Veranstaltung konnten wir leider nicht finden.',
        },
    };

    actions$ = combineLatest([this.unitState$, this.isLecturerOrAdmin$]).pipe(
        takeUntil(this.destroy$),
        ifAttached(this),
        map(([unitState, isLecturerOrAdmin$]) =>
            this.buildActions(unitState.unit, isLecturerOrAdmin$)
        )
    );

    constructor(
        private courseUnitDetailStore: CourseUnitPageStore,
        private store: Store<SubatoState>,
        private courseService: CourseService,
        private globalErrorHandler: GlobalErrorHandler,
        private courseUnitService: CourseUnitService,
        private attendanceService: AttendanceService,
        private notificationService: NotificationService,
        private authService: AuthService,
        private route: ActivatedRoute,
        private router: Router
    ) {
        super();
    }

    public onAttach() {
        super.onAttach();
        this.courseUnitDetailStore.onAttach();
    }

    public onDetach() {
        super.onDetach();
        this.courseUnitDetailStore.onDetach();
    }

    ngOnInit(): void {
        this.route.paramMap
            .pipe(
                takeUntil(this.destroy$),
                ifAttached(this),
                map((params) => ParamHelper.parseIdOrThrow(params, 'unit_id')),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.courseUnitDetailStore.patchUnit({ error: err });
                    return EMPTY;
                })
            )
            .subscribe((unitId) => this.courseUnitDetailStore.loadUnit(unitId));
    }

    selectUser(user: User) {
        this.courseUnitDetailStore.patchState({ selectedUser: user });
    }

    buildActions(
        unit: CourseUnit,
        isLecturerOrAdmin$: boolean
    ): PageHeaderAction[] {
        let actions: PageHeaderAction[] = [];
        if (isLecturerOrAdmin$ && unit != null) {
            actions.push({
                name: 'Bearbeiten',
                icon: 'edit-2-outline',
                link: {
                    commands: ['/', 'unit', unit.id, 'admin'],
                },
                disabled: unit.course.finished,
                status: 'primary',
            });
        }

        return actions;
    }
}
