import { Injectable } from '@angular/core';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { combineLatest, Observable } from 'rxjs';
import { filter, map } from 'rxjs/operators';
import { SubatoPrincipal } from 'app/auth/models/principal/subato-principal';
import { User } from 'app/shared/user/user';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { dispatchCourseChange } from 'app/courses/shared/operators/course.operators';
import {
    CourseUnitState,
    loadUnit,
} from 'app/courses/shared/operators/course-unit.operators';
import {
    loadAttendance,
    ParticipantAttendanceState,
} from 'app/courses/shared/operators/participant-attendance.operators';
import { ifAttached } from 'app/shared/route-reuse/route-reuse-life.cycle.directive';

interface CourseUnitPageState {
    unitState: CourseUnitState;
    attendanceState: ParticipantAttendanceState;
    selectedUser: User;
}

export const initialState: CourseUnitPageState = {
    unitState: { unit: null, error: null, loading: true },
    attendanceState: { attendance: null, error: null, loading: true },
    selectedUser: null,
};

@Injectable()
export class CourseUnitPageStore extends BaseComponentStore<CourseUnitPageState> {
    principal$: Observable<SubatoPrincipal> = this.store.select(
        (s) => s.auth.principal
    );
    public readonly unit$ = this.select((state) => state.unitState.unit);
    public readonly unitState$ = this.select((state) => state.unitState);
    attendanceState$ = this.select((s) => s.attendanceState);

    selectedUser$ = this.select((s) => s.selectedUser);

    public readonly isLecturerOrAdmin$ = this.select(
        this.unit$,
        this.principal$,
        (unit, principal) =>
            unit != null &&
            principal != null &&
            principal.isLecturerOrAdmin(unit.course.id)
    );

    constructor(private store: Store<SubatoState>) {
        super(initialState);
    }

    readonly patchUnit = this.updaterFor('unitState');
    readonly patchAttendance = this.updaterFor('attendanceState');

    readonly loadUnit = this.effect((unitId: Observable<number>) =>
        unitId.pipe(loadUnit((s) => this.patchUnit(s)))
    );

    readonly dispatchCourseChange$ = this.effect(() =>
        combineLatest([this.unit$, this.detached$], (unit) => unit).pipe(
            map((unit) => unit?.course),
            ifAttached(this),
            dispatchCourseChange
        )
    );

    readonly loadAttendance$ = this.effect(() =>
        combineLatest([this.selectedUser$, this.unit$]).pipe(
            filter(([user, unit]) => user != null && unit != null),
            map(([user, unit]) => ({ unitId: unit.id, userId: user.id })),
            loadAttendance((s) => this.patchAttendance(s))
        )
    );
}
