import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { BaseComponent } from 'app/shared/misc/base.component';
import { catchError, map, takeUntil } from 'rxjs/operators';
import { ErrorContainerConfig } from 'app/shared/error/error-container-config';
import { SubatoState } from 'app/store/app.reducers';
import { Store } from '@ngrx/store';
import { ParamHelper } from 'app/shared/misc/helper/param.helper';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { EMPTY } from 'rxjs';
import { CourseUnitsPageStore } from 'app/courses/pages/course-units/course-units.page.store';
import { CourseUnit } from 'app/courses/models/course-unit';
import { SubatoPrincipal } from 'app/auth/models/principal/subato-principal';
import { CourseParticipantProfile } from 'app/auth/models/principal/course-participant-profile';
import { PublicConfig } from 'app/shared/config/public-config';
import { ifAttached } from 'app/shared/route-reuse/route-reuse-life.cycle.directive';

@Component({
    selector: 'ngx-course-units',
    templateUrl: './course-units.page.component.html',
    styleUrls: ['./course-units.page.component.scss'],
    providers: [CourseUnitsPageStore],
})
export class CourseUnitsPageComponent extends BaseComponent implements OnInit {
    errorConfig: ErrorContainerConfig = {
        notFound: {
            message: 'Diesen Kurs konnten wir leider nicht finden.',
        },
    };

    hasAnyGroup$ = this.courseUnitListStore.hasAnyGroup$;
    isLecturerOrAdmin$ = this.courseUnitListStore.isLecturerOrAdmin$;
    onlyOwnUnits$ = this.courseUnitListStore.onlyOwnUnits$;
    units$ = this.courseUnitListStore.units$;
    filteredUnits$ = this.courseUnitListStore.filteredUnits$;
    courseState$ = this.courseUnitListStore.courseState$;
    loading$ = this.courseUnitListStore.loading$;
    error$ = this.courseUnitListStore.error$;
    joiningUnit$ = this.courseUnitListStore.joiningUnit$;
    principal$ = this.courseUnitListStore.principal$;
    config$ = this.courseUnitListStore.config$;
    participantProfile$ = this.courseUnitListStore.participantProfile$;
    showLabChangeWarning$ = this.courseUnitListStore.showLabChangeWarning$;

    constructor(
        private courseUnitListStore: CourseUnitsPageStore,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler,
        private route: ActivatedRoute
    ) {
        super();
    }

    ngOnInit(): void {
        this.route.paramMap
            .pipe(
                takeUntil(this.destroy$),
                ifAttached(this),
                map((params) =>
                    ParamHelper.parseIdOrThrow(params, 'course_id')
                ),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.courseUnitListStore.patchCourse({ error: err });
                    return EMPTY;
                })
            )
            .subscribe((courseId) =>
                this.courseUnitListStore.loadCourse(courseId)
            );
    }

    onAttach() {
        super.onAttach();
        this.courseUnitListStore.onAttach();
    }

    onDetach() {
        super.onDetach();
        this.courseUnitListStore.onDetach();
    }

    onlyOwnUnitsChanged(onlyOwnUnits: boolean) {
        this.courseUnitListStore.patchState({ onlyOwnUnits: onlyOwnUnits });
    }

    getUnitLink() {
        return (unit) =>
            this.hasAnyGroup$.pipe(
                map((hasAnyGroup) => {
                    if (hasAnyGroup) {
                        return ['/', 'unit', unit.id];
                    }

                    return null;
                })
            );
    }

    canJoinUnit(
        unit: CourseUnit,
        principal: SubatoPrincipal,
        profile: CourseParticipantProfile,
        config: PublicConfig
    ) {
        return (
            principal != null &&
            !principal.isParticipantOfUnit(unit.id) &&
            (profile == null ||
                profile.labChangedCounter < config.maxLabChanges)
        );
    }

    joinUnit(unit: CourseUnit) {
        this.courseUnitListStore.joinUnit(unit);
    }
}
