import { Injectable } from '@angular/core';
import { catchError, filter, map, switchMap, tap } from 'rxjs/operators';
import { combineLatest, EMPTY, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { SubatoPrincipal } from 'app/auth/models/principal/subato-principal';
import {
    CourseState,
    dispatchCourseChange,
    loadCourse,
} from 'app/courses/shared/operators/course.operators';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import {
    CourseUnitsState,
    loadUnits,
} from 'app/courses/shared/operators/course-unit.operators';
import { ifAttached } from 'app/shared/route-reuse/route-reuse-life.cycle.directive';
import { CourseUnit } from 'app/courses/models/course-unit';
import { CourseUnitService } from 'app/courses/services/course-unit.service';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { LoadPrincipal } from 'app/auth/store/auth.actions';

interface CourseUnitsPageState {
    courseState: CourseState;
    unitsState: CourseUnitsState;
    onlyOwnUnits: boolean;
    joiningUnit: CourseUnit;
}

export const initialState: CourseUnitsPageState = {
    courseState: { course: null, error: null, loading: true },
    unitsState: { units: [], error: null, loading: true },
    onlyOwnUnits: false,
    joiningUnit: null,
};

@Injectable()
export class CourseUnitsPageStore extends BaseComponentStore<CourseUnitsPageState> {
    public readonly course$ = this.select((state) => state.courseState.course);
    public readonly courseState$ = this.select((state) => state.courseState);

    public readonly units$ = this.select((state) => state.unitsState.units);
    public readonly unitsState$ = this.select((state) => state.unitsState);

    public readonly onlyOwnUnits$ = this.select((state) => state.onlyOwnUnits);
    public readonly joiningUnit$ = this.select((state) => state.joiningUnit);

    principal$: Observable<SubatoPrincipal> = this.store.select(
        (s) => s.auth.principal
    );

    config$ = this.store.select((s) => s.config.config);

    public readonly participantProfile$ = this.select(
        this.course$,
        this.principal$,
        (course, principal) =>
            principal != null && course != null
                ? principal.getProfile(course.id)
                : null
    );

    public readonly showLabChangeWarning$ = this.select(
        this.participantProfile$,
        this.course$,
        this.config$,
        (profile, course, config) =>
            course != null &&
            course.allowJoiningUnits &&
            profile != null &&
            profile.labChangedCounter >= config.maxLabChanges - 1 // if there is 1 (or no) change left
    );

    public readonly loading$ = this.select(
        this.select((s) => s.courseState.loading),
        this.select((s) => s.unitsState.loading),
        (l1, l2) => l1 || l2
    );

    public readonly error$ = this.select(
        this.select((s) => s.courseState.error),
        this.select((s) => s.unitsState.error),
        (e1, e2) => e1 ?? e2
    );

    public readonly filteredUnits$ = this.select(
        this.units$,
        this.onlyOwnUnits$,
        this.principal$,
        (units, onlyOwnUnits, principal) =>
            units.filter(
                (unit) =>
                    unit.isLab &&
                    (!onlyOwnUnits ||
                        unit.lecturers.some(
                            (l) => l.loginName == principal.loginName
                        ))
            )
    );

    public readonly isLecturerOrAdmin$ = this.select(
        this.course$,
        this.principal$,
        (course, principal) =>
            course != null &&
            principal != null &&
            principal.isLecturerOrAdmin(course.id)
    );

    public readonly hasAnyGroup$ = this.select(
        this.course$,
        this.principal$,
        (course, principal) =>
            course != null &&
            principal != null &&
            principal.hasAnyGroup(course.id)
    );

    constructor(
        private store: Store<SubatoState>,
        private unitService: CourseUnitService,
        private notificationService: NotificationService,
        private globalErrorHandler: GlobalErrorHandler
    ) {
        super(initialState);
    }

    patchCourse = this.updaterFor('courseState');
    patchUnits = this.updaterFor('unitsState');

    readonly loadCourse = this.effect((courseId$: Observable<number>) =>
        courseId$.pipe(loadCourse((s) => this.patchCourse(s)))
    );

    readonly dispatchCourseChange$ = this.effect(() =>
        combineLatest([this.course$, this.detached$], (course) => course).pipe(
            ifAttached(this),
            dispatchCourseChange
        )
    );

    readonly loadUnits$ = this.effect(() =>
        combineLatest([this.course$, this.detached$], (course) => course).pipe(
            filter((course) => course != null),
            ifAttached(this),
            map((course) => course.id),
            loadUnits((s) => this.patchUnits(s))
        )
    );

    setInitialOnlyOwnUnits$ = this.effect(() =>
        this.isLecturerOrAdmin$.pipe(
            tap((isLecturerOrAdmin) =>
                this.patchState({ onlyOwnUnits: isLecturerOrAdmin })
            )
        )
    );

    readonly joinUnit = this.effect((unit$: Observable<CourseUnit>) =>
        unit$.pipe(
            tap((unit) => this.patchState({ joiningUnit: unit })),
            switchMap((unit) =>
                this.unitService.joinUnit(unit.id).pipe(
                    tap(() => {
                        this.store.dispatch(new LoadPrincipal());
                        this.patchState({ joiningUnit: null });

                        this.notificationService.showSuccess(
                            'Veranstaltung erfolgreich beigetreten'
                        );
                    }),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.patchState({ joiningUnit: null });

                        this.notificationService.showError(
                            this.globalErrorHandler.format(
                                'Fehler bei Beitritt der Veranstaltung',
                                error
                            )
                        );

                        return EMPTY;
                    })
                )
            )
        )
    );
}
