import { Injectable } from '@angular/core';
import { catchError, map, switchMap, takeUntil, tap } from 'rxjs/operators';
import { combineLatest, EMPTY, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { SubatoError } from 'app/shared/error/error';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { ProgramGroupService } from 'app/courses/pages/courses/services/program-group.service';
import { Term } from 'app/modules/models/term';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { ProgramGroup } from 'app/courses/pages/courses/models/program-group';
import { ifAttached } from 'app/shared/route-reuse/route-reuse-life.cycle.directive';
import { debounceWithInitialValue } from 'app/shared/misc/rxjs-operators';
import { environment } from 'environments/environment';

interface ProgramGroupsState {
    programGroups: ProgramGroup[];
    loading: boolean;
    error: SubatoError;
}

interface CoursesPageState {
    programGroupsState: ProgramGroupsState;
    query: string;
}

export const initialState: CoursesPageState = {
    programGroupsState: { programGroups: [], loading: true, error: null },
    query: '',
};

@Injectable()
export class CoursesPageStore extends BaseComponentStore<CoursesPageState> {
    principal$ = this.store.select((s) => s.auth.principal);
    public readonly term$: Observable<Term | null> = this.store.select(
        (s) => s.modules.currentTerm
    );
    public readonly query$ = this.select((state) => state.query);
    public readonly debouncedQuery$ = this.query$.pipe(
        debounceWithInitialValue(environment.debounceTimeMs, initialState.query)
    );
    public readonly programGroups$ = this.select(
        (state) => state.programGroupsState.programGroups
    );
    public readonly programGroupsState$ = this.select(
        (state) => state.programGroupsState
    );

    canCreateCourse$ = this.principal$.pipe(
        takeUntil(this.destroy$),
        map(
            (principal) =>
                principal &&
                (principal.hasStaffRole() || principal.hasSuperAdminRole())
        )
    );

    public readonly expand$ = this.select(
        this.programGroups$,
        this.debouncedQuery$,
        (programGroups, query) => programGroups && query.length > 0
    );

    constructor(
        private store: Store<SubatoState>,
        private programGroupService: ProgramGroupService,
        private globalErrorHandler: GlobalErrorHandler
    ) {
        super(initialState);
    }

    readonly patchProgramGroups = this.updaterFor('programGroupsState');

    readonly loadProgramGroups$ = this.effect(() =>
        combineLatest([this.term$, this.debouncedQuery$]).pipe(
            ifAttached(this),
            tap(() => this.patchProgramGroups({ loading: true })),
            switchMap(([term, query]) =>
                this.programGroupService
                    .getAll({
                        termId: term?.id,
                        query: query,
                    })
                    .pipe(
                        tap((programGroups) =>
                            this.patchProgramGroups({
                                programGroups: programGroups,
                                error: null,
                                loading: false,
                            })
                        ),
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((error) => {
                            this.patchProgramGroups({
                                error: error,
                                loading: false,
                                programGroups: [],
                            });
                            return EMPTY;
                        })
                    )
            )
        )
    );
}
