import { Course } from 'app/courses/models/course';
import { SubatoError } from 'app/shared/error/error';
import { Injectable } from '@angular/core';
import { ComponentStore } from '@ngrx/component-store';
import { DataQueryParams } from 'app/shared/paging/data-query-params';
import { CourseService } from 'app/courses/services/course.service';
import { EMPTY } from 'rxjs';
import {
    catchError,
    debounceTime,
    switchMap,
    tap,
    withLatestFrom,
} from 'rxjs/operators';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { environment } from 'environments/environment';

export interface CourseTemplateListState {
    error: SubatoError;
    loading: boolean;
    selectedCourse: Course | null;
    courses: Course[];
    query: string;
    projection: DataQueryParams;
}

export const initialState: CourseTemplateListState = {
    error: null,
    loading: true,
    selectedCourse: null,
    courses: [],
    query: '',
    projection: {
        sortBy: 'term.id',
        sortOrder: 'desc',
    },
};

@Injectable()
export class CourseTemplateListStore extends ComponentStore<CourseTemplateListState> {
    public readonly loading$ = this.select((state) => state.loading);
    public readonly error$ = this.select((state) => state.error);
    public readonly courses$ = this.select((state) => state.courses);
    public readonly query$ = this.select((state) => state.query);
    public readonly projection$ = this.select((state) => state.projection);
    public readonly selectedCourse$ = this.select(
        (state) => state.selectedCourse
    );

    constructor(
        private courseService: CourseService,
        private globalErrorHandler: GlobalErrorHandler
    ) {
        super(initialState);
    }

    readonly loadCourses$ = this.effect(() =>
        this.query$.pipe(
            debounceTime(environment.debounceTimeMs),
            withLatestFrom(this.projection$),
            tap(() => this.patchState({ loading: true })),
            switchMap(([query, projection]) =>
                this.courseService.getAll({ query: query }, projection).pipe(
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.patchState({
                            courses: [],
                            error: error,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            ),
            tap((courses) =>
                this.patchState({
                    courses: courses,
                    error: null,
                    loading: false,
                })
            )
        )
    );
}
