import { SubatoError } from 'app/shared/error/error';
import { Injectable } from '@angular/core';
import { ComponentStore } from '@ngrx/component-store';
import { combineLatest, EMPTY, Observable } from 'rxjs';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { Term } from 'app/modules/models/term';
import { ProgramModule } from 'app/modules/models/program-module';
import { ProgramModuleService } from 'app/modules/services/program-module.service';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { debounceWithInitialValue } from 'app/shared/misc/rxjs-operators';
import { environment } from 'environments/environment';

export interface ModuleMultiselectState {
    error: SubatoError;
    loading: boolean;
    modules: ProgramModule[];
    selectedModules: ProgramModule[];
    query: string;
}

export const initialState: ModuleMultiselectState = {
    error: null,
    loading: true,
    modules: [],
    selectedModules: [],
    query: '',
};

@Injectable()
export class ModuleMultiselectStore extends ComponentStore<ModuleMultiselectState> {
    public readonly loading$ = this.select((state) => state.loading);
    public readonly error$ = this.select((state) => state.error);
    public readonly modules$ = this.select((state) => state.modules);
    public readonly selectedModules$ = this.select(
        (state) => state.selectedModules
    );
    public readonly query$ = this.select((state) => state.query);
    public readonly debouncedQuery$ = this.query$.pipe(
        debounceWithInitialValue(environment.debounceTimeMs, initialState.query)
    );

    term$: Observable<Term | null> = this.store.select(
        (s) => s.modules.currentTerm
    );

    constructor(
        private store: Store<SubatoState>,
        private programModuleService: ProgramModuleService,
        private globalErrorHandler: GlobalErrorHandler
    ) {
        super(initialState);
    }

    containsModule(modules: ProgramModule[], module: ProgramModule) {
        return modules.map((m) => m.id).indexOf(module.id) != -1;
    }

    readonly addModules = this.updater((state, modules: ProgramModule[]) => ({
        ...state,
        selectedModules: [
            ...state.selectedModules,
            ...modules.filter(
                (m) => !this.containsModule(state.selectedModules, m)
            ),
        ],
    }));

    readonly removeModule = this.updater((state, module: ProgramModule) => ({
        ...state,
        selectedModules: state.selectedModules.filter((m) => m != module),
    }));

    readonly loadModules$ = this.effect(() =>
        combineLatest([this.term$, this.debouncedQuery$]).pipe(
            tap(() => this.patchState({ loading: true })),
            switchMap(([term, query]) =>
                this.programModuleService.getAllForTerm(term?.id, query).pipe(
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.patchState({
                            modules: [],
                            error: error,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            ),
            tap((modules) =>
                this.patchState({
                    modules: modules,
                    error: null,
                    loading: false,
                })
            )
        )
    );
}
