import { Component, OnInit } from '@angular/core';
import {
    UntypedFormBuilder,
    UntypedFormGroup,
    Validators,
} from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { FormComponent } from 'app/shared/validation/form.component';
import { map, take } from 'rxjs/operators';
import { CreateCourseDto } from 'app/courses/dto/create-course.dto';
import { Observable } from 'rxjs';
import { Term } from 'app/modules/models/term';
import { SubatoState } from 'app/store/app.reducers';
import { Store } from '@ngrx/store';
import { templateFormValidator } from 'app/courses/pages/new-course/validators';
import { NewCoursePageStore } from 'app/courses/pages/new-course/new-course.page.store';

@Component({
    templateUrl: './new-course.page.component.html',
    styleUrls: ['./new-course.page.component.scss'],
    providers: [NewCoursePageStore],
})
export class NewCoursePageComponent extends FormComponent implements OnInit {
    term$: Observable<Term | null> = this.store.select(
        (s) => s.modules.currentTerm
    );

    generalForm: UntypedFormGroup;
    templateForm: UntypedFormGroup;
    modulesForm: UntypedFormGroup;

    saving$ = this.newCoursePageStore.saving$;

    constructor(
        private route: ActivatedRoute,
        private router: Router,
        private newCoursePageStore: NewCoursePageStore,
        private store: Store<SubatoState>,
        private fb: UntypedFormBuilder
    ) {
        super();
    }

    ngOnInit(): void {
        this.createForms();
    }

    createForms() {
        this.generalForm = this.fb.group({
            name: ['', Validators.required],
            shortName: ['', Validators.required],
            permanent: [false, Validators.required],
        });

        this.templateForm = this.fb.group(
            {
                useTemplate: [false],
                course: [null],
            },
            { validators: templateFormValidator }
        );

        this.modulesForm = this.fb.group({
            selectedModules: [[]],
        });

        this.templateForm.get('course').valueChanges.subscribe((course) => {
            this.generalForm.patchValue({
                name: course?.name ?? '',
                shortName: course?.shortName ?? '',
            });
        });

        this.templateForm.get('course').valueChanges.subscribe((course) => {
            let selectedModules = course?.modules ?? [];
            this.modulesForm.patchValue({ selectedModules: selectedModules });
        });
    }

    buildDto() {
        return this.term$.pipe(
            take(1),
            map((term) => {
                let dto: CreateCourseDto = {
                    name: this.generalForm.get('name').value,
                    shortName: this.generalForm.get('shortName').value,
                    courseId: this.templateForm.get('course').value?.id,
                    moduleIds: this.modulesForm
                        .get('selectedModules')
                        .value.map((m) => m.id),
                    termId: this.generalForm.get('permanent').value
                        ? null
                        : term?.id,
                };

                return dto;
            })
        );
    }

    save() {
        this.buildDto()
            .pipe(take(1))
            .subscribe((dto) => this.newCoursePageStore.createCourse(dto));
    }
}
