import { Component, OnInit } from '@angular/core';
import { BaseComponent } from 'app/shared/misc/base.component';
import { ActivatedRoute, Router } from '@angular/router';
import { combineLatest, EMPTY, Observable } from 'rxjs';
import { catchError, map, takeUntil, tap } from 'rxjs/operators';
import { CourseService } from 'app/courses/services/course.service';
import { SubatoPrincipal } from 'app/auth/models/principal/subato-principal';
import { ifAttached } from 'app/shared/route-reuse/route-reuse-life.cycle.directive';
import { ParamHelper } from 'app/shared/misc/helper/param.helper';
import { ErrorContainerConfig } from 'app/shared/error/error-container-config';
import { SubatoState } from 'app/store/app.reducers';
import { Store } from '@ngrx/store';
import { AccessDeniedError } from 'app/shared/error/error';
import { ParticipantProgressPageStore } from 'app/courses/pages/participant-progress/participant-progress.page.store';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';

@Component({
    templateUrl: './participant-progress.page.component.html',
    styleUrls: ['./participant-progress.page.component.scss'],
    providers: [ParticipantProgressPageStore],
})
export class ParticipantProgressPageComponent
    extends BaseComponent
    implements OnInit
{
    principal$: Observable<SubatoPrincipal> = this.store.select(
        (s) => s.auth.principal
    );

    courseState$ = this.participantProgressStore.courseState$;
    userState$ = this.participantProgressStore.userState$;
    attendancesState$ = this.participantProgressStore.attendancesState$;
    canSeeFilter$ = this.participantProgressStore.hasAnyGroup$;

    errorConfig: ErrorContainerConfig = {
        notFound: {
            message: 'Diesen Kurs konnten wir leider nicht finden.',
        },
    };

    constructor(
        private courseService: CourseService,
        private participantProgressStore: ParticipantProgressPageStore,
        private store: Store<SubatoState>,
        private route: ActivatedRoute,
        private globalErrorHandler: GlobalErrorHandler,
        private router: Router
    ) {
        super();
    }

    public onAttach() {
        super.onAttach();
        this.participantProgressStore.onAttach();
    }

    public onDetach() {
        super.onDetach();
        this.participantProgressStore.onDetach();
    }

    ngOnInit(): void {
        this.route.paramMap
            .pipe(
                takeUntil(this.destroy$),
                ifAttached(this),
                map((params) =>
                    ParamHelper.parseIdOrThrow(params, 'course_id')
                ),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.participantProgressStore.patchCourse({ error: err });
                    return EMPTY;
                })
            )
            .subscribe((courseId) =>
                this.participantProgressStore.loadCourse(courseId)
            );

        combineLatest([this.route.paramMap, this.principal$])
            .pipe(
                takeUntil(this.destroy$),
                ifAttached(this),
                tap(([params, principal]) => {
                    if (principal == null) {
                        throw new AccessDeniedError('');
                    }
                }),
                map(([params, principal]) =>
                    params.has('user_id')
                        ? ParamHelper.parseIdOrThrow(params, 'user_id')
                        : principal.id
                ),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.participantProgressStore.patchUser({ error: err });
                    return EMPTY;
                })
            )
            .subscribe((userId) =>
                this.participantProgressStore.loadUser(userId)
            );
    }
}
