import { Injectable } from '@angular/core';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { combineLatest, Observable } from 'rxjs';
import { filter, map } from 'rxjs/operators';
import { SubatoPrincipal } from 'app/auth/models/principal/subato-principal';
import {
    CourseState,
    dispatchCourseChange,
    loadCourse,
} from 'app/courses/shared/operators/course.operators';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { loadUser, UserState } from 'app/auth/shared/store/user.operators';
import {
    loadAttendances,
    ParticipantAttendancesState,
} from 'app/courses/shared/operators/participant-attendance.operators';
import { ifAttached } from 'app/shared/route-reuse/route-reuse-life.cycle.directive';

interface ParticipantProgressPageState {
    courseState: CourseState;
    userState: UserState;
    attendancesState: ParticipantAttendancesState;
}

export const initialState: ParticipantProgressPageState = {
    courseState: { course: null, error: null, loading: true },
    userState: { user: null, error: null, loading: true },
    attendancesState: { attendances: [], error: null, loading: true },
};

@Injectable()
export class ParticipantProgressPageStore extends BaseComponentStore<ParticipantProgressPageState> {
    public readonly course$ = this.select((state) => state.courseState.course);
    public readonly courseState$ = this.select((state) => state.courseState);
    public readonly user$ = this.select((state) => state.userState.user);
    public readonly userState$ = this.select((state) => state.userState);

    public readonly attendancesState$ = this.select(
        (state) => state.attendancesState
    );

    principal$: Observable<SubatoPrincipal> = this.store.select(
        (s) => s.auth.principal
    );

    public readonly hasAnyGroup$ = this.select(
        this.course$,
        this.principal$,
        (course, principal) =>
            course != null &&
            principal != null &&
            principal.hasAnyGroup(course.id)
    );

    constructor(private store: Store<SubatoState>) {
        super(initialState);
    }

    readonly patchCourse = this.updaterFor('courseState');
    readonly patchUser = this.updaterFor('userState');
    readonly patchAttendances = this.updaterFor('attendancesState');

    readonly loadCourse = this.effect((courseId$: Observable<number>) =>
        courseId$.pipe(loadCourse((s) => this.patchCourse(s)))
    );

    readonly loadUser = this.effect((userId$: Observable<number>) =>
        userId$.pipe(loadUser((s) => this.patchUser(s)))
    );

    readonly dispatchCourseChange$ = this.effect(() =>
        combineLatest([this.course$, this.detached$], (course) => course).pipe(
            ifAttached(this),
            dispatchCourseChange
        )
    );

    readonly loadAttendances$ = this.effect(() =>
        combineLatest([this.user$, this.course$]).pipe(
            filter(([user, course]) => user != null && course != null),
            map(([user, course]) => ({ courseId: course.id, userId: user.id })),
            loadAttendances((s) => this.patchAttendances(s))
        )
    );
}
