import { HttpClient, HttpParams } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { HttpCacheManager } from '@ngneat/cashew';
import { User } from 'app/shared/user/user';
import { ParticipantAttendance } from 'app/attendances/dto/participant-attendance.dto';
import { BaseService } from 'app/shared/misc/base.service';
import { environment } from 'environments/environment';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import { CourseUnit } from '../models/course-unit';
import { CourseUnitEvent } from '../models/course-unit-event';

@Injectable({
    providedIn: 'root',
})
export class CourseUnitService extends BaseService {
    constructor(
        private http: HttpClient,
        private cacheManager: HttpCacheManager
    ) {
        super();
    }

    getById(id: number): Observable<CourseUnit> {
        return this.http
            .get<CourseUnit>(`${environment.backendUrl}/course_unit/${id}`)
            .pipe(map((course) => new CourseUnit(course)));
    }

    getEvents(id: number): Observable<CourseUnitEvent[]> {
        return this.http
            .get<CourseUnitEvent[]>(
                `${environment.backendUrl}/course_unit/${id}/events`
            )
            .pipe(
                map((events) =>
                    events.map((event) => new CourseUnitEvent(event))
                )
            );
    }

    getParticipants(id: number, query?: string): Observable<User[]> {
        let params = new HttpParams();

        if (query != null) {
            params = params.set('q', query);
        }

        return this.http
            .get<User[]>(
                `${environment.backendUrl}/course_unit/${id}/participants`,
                { params: params }
            )
            .pipe(map((users) => users.map((u) => new User(u))));
    }

    joinUnit(id: number): Observable<void> {
        return this.http
            .post<void>(`${environment.backendUrl}/course_unit/${id}/join`, {})
            .pipe(map(() => this.cacheManager.clear()));
    }

    addParticipant(id: number, userId: number): Observable<void> {
        return this.http
            .put<void>(
                `${environment.backendUrl}/course_unit/${id}/participants/${userId}`,
                {}
            )
            .pipe(map(() => this.cacheManager.clear()));
    }

    removeParticipant(id: number, userId: number): Observable<void> {
        return this.http
            .delete<void>(
                `${environment.backendUrl}/course_unit/${id}/participants/${userId}`
            )
            .pipe(map(() => this.cacheManager.clear()));
    }

    getAttendancesForUserAndUnit(
        courseUnitId: number,
        userId: number
    ): Observable<ParticipantAttendance> {
        let url = `${environment.backendUrl}/course_unit/${courseUnitId}/attendance/${userId}`;
        return this.http
            .get<ParticipantAttendance>(url)
            .pipe(map((dto) => new ParticipantAttendance(dto)));
    }
}
