import { HttpClient, HttpParams } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { BaseService } from 'app/shared/misc/base.service';
import { environment } from 'environments/environment';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import { Course } from '../models/course';
import { UpdateCourseDto } from '../dto/update-course.dto';
import { CreateCourseDto } from '../dto/create-course.dto';
import { CourseStaffMembership } from '../models/course-staff-membership';
import { StaffGroup } from '../models/staff-group';
import { CourseUnit } from '../models/course-unit';
import { CourseFile } from '../models/course-file';
import { User } from 'app/shared/user/user';
import { DataQueryParams } from 'app/shared/paging/data-query-params';
import { ParticipantAttendance } from 'app/attendances/dto/participant-attendance.dto';
import { CourseResult } from '../models/course-result';
import { UpdateFinishedDto } from 'app/courses/dto/update-finished.dto';

export interface CourseFilter {
    termId?: number;
    query?: string;
}

export interface CourseResultFilter {
    query: string;
    showInvisible: boolean;
}

@Injectable({
    providedIn: 'root',
})
export class CourseService extends BaseService {
    constructor(private http: HttpClient) {
        super();
    }

    getById(id: number): Observable<Course> {
        return this.http
            .get<Course>(`${environment.backendUrl}/course/${id}`)
            .pipe(map((course) => new Course(course)));
    }

    getDescription(courseId: number, raw?: boolean): Observable<string> {
        let params = new HttpParams();
        if (raw != null) {
            params = params.set('raw', raw);
        }

        return this.http.get(
            `${environment.backendUrl}/course/${courseId}/description`,
            {
                responseType: 'text',
                params: params,
            }
        );
    }

    getAll(
        filter?: CourseFilter,
        projection?: DataQueryParams
    ): Observable<Course[]> {
        let params = DataQueryParams.toHttpParams(projection);

        if (filter?.termId != null) {
            params = params.set('termId', filter.termId);
        }

        if (filter?.query != null) {
            params = params.set('q', filter.query);
        }

        return this.http
            .get<Course[]>(`${environment.backendUrl}/course`, {
                params: params,
            })
            .pipe(
                map((courses) => courses.map((course) => new Course(course)))
            );
    }

    create(dto: CreateCourseDto): Observable<Course> {
        return this.http
            .post<Course>(`${environment.backendUrl}/course`, dto)
            .pipe(map((course) => new Course(course)));
    }

    update(courseId: number, dto: UpdateCourseDto): Observable<Course> {
        return this.http
            .put<Course>(`${environment.backendUrl}/course/${courseId}`, dto)
            .pipe(
                map((course) => {
                    return new Course(course);
                })
            );
    }

    delete(id: number): Observable<void> {
        return this.http.delete<void>(`${environment.backendUrl}/course/${id}`);
    }

    getUnits(id: number): Observable<CourseUnit[]> {
        return this.http
            .get<CourseUnit[]>(`${environment.backendUrl}/course/${id}/units`)
            .pipe(map((units) => units.map((unit) => new CourseUnit(unit))));
    }

    getStaff(id: number): Observable<CourseStaffMembership[]> {
        return this.http
            .get<CourseStaffMembership[]>(
                `${environment.backendUrl}/course/${id}/staff`
            )
            .pipe(
                map((memberships) =>
                    memberships.map(
                        (membership) => new CourseStaffMembership(membership)
                    )
                )
            );
    }

    getParticipants(id: number, query?: string): Observable<User[]> {
        let params = new HttpParams();

        if (query != null) {
            params = params.set('q', query);
        }

        return this.http
            .get<User[]>(
                `${environment.backendUrl}/course/${id}/participants`,
                { params: params }
            )
            .pipe(map((users) => users.map((u) => new User(u))));
    }

    addStaff(
        id: number,
        userId: number,
        group: StaffGroup
    ): Observable<CourseStaffMembership> {
        return this.http
            .put<CourseStaffMembership>(
                `${environment.backendUrl}/course/${id}/staff/${userId}/group/${group}`,
                {}
            )
            .pipe(
                map((membership) => {
                    return new CourseStaffMembership(membership);
                })
            );
    }

    removeStaff(
        id: number,
        userId: number,
        group: StaffGroup
    ): Observable<void> {
        return this.http.delete<void>(
            `${environment.backendUrl}/course/${id}/staff/${userId}/group/${group}`
        );
    }

    reimport(id: number): Observable<Course> {
        return this.http
            .post<Course>(`${environment.backendUrl}/course/${id}/reimport`, {})
            .pipe(map((course) => new Course(course)));
    }

    updateFinished(id: number, dto: UpdateFinishedDto): Observable<Course> {
        return this.http
            .put<Course>(`${environment.backendUrl}/course/${id}/finished`, dto)
            .pipe(map((course) => new Course(course)));
    }

    getFiles(id: number): Observable<CourseFile[]> {
        return this.http
            .get<CourseFile[]>(`${environment.backendUrl}/course/${id}/files`)
            .pipe(map((files) => files.map((file) => new CourseFile(file))));
    }

    getProgressForUser(
        courseId: number,
        userId: number,
        filter?: CourseResultFilter
    ): Observable<CourseResult> {
        let params = new HttpParams();
        if (filter?.showInvisible != null) {
            params = params.set('showInvisibleExercises', filter.showInvisible);
        }

        if (filter?.query != null) {
            params = params.set('q', filter.query);
        }

        return this.http
            .get<CourseResult>(
                `${environment.backendUrl}/course/${courseId}/result/${userId}`,
                {
                    params: params,
                }
            )
            .pipe(map((result) => new CourseResult(result)));
    }

    getAttendancesForUser(
        courseId: number,
        userId: number
    ): Observable<ParticipantAttendance[]> {
        let url = `${environment.backendUrl}/course/${courseId}/attendance/${userId}`;
        return this.http
            .get<ParticipantAttendance[]>(url)
            .pipe(map((dto) => dto.map((d) => new ParticipantAttendance(d))));
    }
}
