import {
    CourseResultFilter,
    CourseService,
} from 'app/courses/services/course.service';
import { CourseResult } from 'app/courses/models/course-result';
import { Injectable } from '@angular/core';
import { ComponentStore } from '@ngrx/component-store';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { CourseUnitService } from 'app/courses/services/course-unit.service';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { combineLatest, EMPTY, Observable } from 'rxjs';
import { catchError, filter, switchMap, tap } from 'rxjs/operators';
import { SubatoPrincipal } from 'app/auth/models/principal/subato-principal';
import { Course } from 'app/courses/models/course';
import { User } from 'app/shared/user/user';

interface CourseResultState {
    course: Course;
    user: User;
    courseResultFilter: CourseResultFilter;
    courseResult: CourseResult;
    loadingResults: boolean;
}

export const initialState: CourseResultState = {
    course: null,
    user: null,
    courseResultFilter: {
        query: '',
        showInvisible: true,
    },
    courseResult: null,
    loadingResults: false,
};

@Injectable()
export class CourseResultStore extends ComponentStore<CourseResultState> {
    public readonly courseResult$ = this.select((state) => state.courseResult);
    public readonly course$ = this.select((state) => state.course);
    public readonly user$ = this.select((state) => state.user);

    public readonly loadingResults$ = this.select(
        (state) => state.loadingResults
    );
    public readonly courseResultFilter$ = this.select(
        (state) => state.courseResultFilter
    );
    principal$: Observable<SubatoPrincipal> = this.store.select(
        (s) => s.auth.principal
    );

    constructor(
        private store: Store<SubatoState>,
        private courseUnitService: CourseUnitService,
        private courseService: CourseService,
        private notificationService: NotificationService,
        private globalErrorHandler: GlobalErrorHandler
    ) {
        super(initialState);
    }

    readonly setCourse = this.updater((state, course: Course) => ({
        ...state,
        course: course,
    }));

    readonly setUser = this.updater((state, user: User) => ({
        ...state,
        user: user,
    }));

    readonly setFilter = this.updater(
        (state, filter: Partial<CourseResultFilter>) => ({
            ...state,
            courseResultFilter: { ...state.courseResultFilter, ...filter },
        })
    );

    readonly updateCourseResult$ = this.effect(() =>
        combineLatest([
            this.user$,
            this.course$,
            this.courseResultFilter$,
        ]).pipe(
            filter(
                ([user, course, filter]) =>
                    user != null && course != null && filter != null
            ),
            tap(() => this.patchState({ loadingResults: true })),
            switchMap(([user, course, filter]) =>
                this.courseService
                    .getProgressForUser(course.id, user.id, filter)
                    .pipe(
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((err) => {
                            this.patchState({
                                courseResult: null,
                                loadingResults: false,
                            });
                            return EMPTY;
                        })
                    )
            ),
            tap((courseResult) =>
                this.patchState({
                    courseResult: courseResult,
                    loadingResults: false,
                })
            )
        )
    );
}
