import { SubatoError } from 'app/shared/error/error';
import { EMPTY, Observable } from 'rxjs';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { CourseService } from 'app/courses/services/course.service';
import { inject, Injectable } from '@angular/core';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { CourseUnitService } from 'app/courses/services/course-unit.service';
import { CourseFile } from 'app/courses/models/course-file';

export interface CourseFilesState {
    files: CourseFile[];
    error: SubatoError;
    loading: boolean;
}

export function loadFiles(
    updaterFn: (state: Partial<CourseFilesState>) => void
) {
    let op = inject(CourseFileOperators);
    return (courseId: Observable<number>) => op.loadFiles(courseId, updaterFn);
}

@Injectable({ providedIn: 'root' })
export class CourseFileOperators {
    constructor(
        private courseService: CourseService,
        private courseUnitService: CourseUnitService,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler
    ) {}

    public loadFiles(
        courseId$: Observable<number>,
        updaterFn: (state: Partial<CourseFilesState>) => void
    ) {
        return courseId$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((courseId) =>
                this.courseService.getFiles(courseId).pipe(
                    tap((files) =>
                        updaterFn({
                            files: files,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            files: [],
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }
}
