import { SubatoError } from 'app/shared/error/error';
import { EMPTY, Observable } from 'rxjs';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { inject, Injectable } from '@angular/core';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { CourseUnitService } from 'app/courses/services/course-unit.service';
import { User } from 'app/shared/user/user';

export interface CourseUnitParticipantsState {
    participants: User[];
    loading: boolean;
    error: SubatoError;
}

export interface LoadCourseUnitParticipantsParams {
    unitId: number;
    query?: string;
}

export function loadParticipants(
    updaterFn: (state: Partial<CourseUnitParticipantsState>) => void
) {
    let op = inject(CourseUnitParticipantOperators);
    return (params$: Observable<LoadCourseUnitParticipantsParams>) =>
        op.loadParticipants(params$, updaterFn);
}

@Injectable({ providedIn: 'root' })
export class CourseUnitParticipantOperators {
    constructor(
        private courseUnitService: CourseUnitService,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler
    ) {}

    public loadParticipants(
        params$: Observable<LoadCourseUnitParticipantsParams>,
        updaterFn: (state: Partial<CourseUnitParticipantsState>) => void
    ) {
        return params$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((params) =>
                this.courseUnitService
                    .getParticipants(params.unitId, params.query)
                    .pipe(
                        tap((participants) =>
                            updaterFn({
                                participants: participants,
                                loading: false,
                                error: null,
                            })
                        ),
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((err) => {
                            updaterFn({
                                participants: [],
                                error: err,
                                loading: false,
                            });
                            return EMPTY;
                        })
                    )
            )
        );
    }
}
