import { SubatoError } from 'app/shared/error/error';
import { EMPTY, Observable } from 'rxjs';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { CourseService } from 'app/courses/services/course.service';
import { inject, Injectable } from '@angular/core';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { CourseUnit } from 'app/courses/models/course-unit';
import { CourseUnitService } from 'app/courses/services/course-unit.service';

export interface CourseUnitState {
    unit: CourseUnit | null;
    error: SubatoError;
    loading: boolean;
}

export interface CourseUnitsState {
    units: CourseUnit[];
    error: SubatoError;
    loading: boolean;
}

export function loadUnit(updaterFn: (state: Partial<CourseUnitState>) => void) {
    let op = inject(CourseUnitOperators);
    return (unitId: Observable<number>) => op.loadUnit(unitId, updaterFn);
}

export function loadUnits(
    updaterFn: (state: Partial<CourseUnitsState>) => void
) {
    let op = inject(CourseUnitOperators);
    return (courseId: Observable<number>) => op.loadUnits(courseId, updaterFn);
}

@Injectable({ providedIn: 'root' })
export class CourseUnitOperators {
    constructor(
        private courseService: CourseService,
        private courseUnitService: CourseUnitService,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler
    ) {}

    public loadUnit(
        unitId$: Observable<number>,
        updaterFn: (state: Partial<CourseUnitState>) => void
    ) {
        return unitId$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((unitId) =>
                this.courseUnitService.getById(unitId).pipe(
                    tap((unit: CourseUnit) =>
                        updaterFn({
                            unit: unit,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            unit: null,
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }

    public loadUnits(
        courseId$: Observable<number>,
        updaterFn: (state: Partial<CourseUnitsState>) => void
    ) {
        return courseId$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((courseId) =>
                this.courseService.getUnits(courseId).pipe(
                    tap((units) =>
                        updaterFn({
                            units: units,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            units: [],
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }
}
