import { SubatoError } from 'app/shared/error/error';
import { Course } from 'app/courses/models/course';
import { EMPTY, Observable } from 'rxjs';
import { catchError, filter, switchMap, tap } from 'rxjs/operators';
import { CourseService } from 'app/courses/services/course.service';
import { inject, Injectable } from '@angular/core';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { ChangeCourse } from 'app/courses/store/courses.actions';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';

export interface CourseState {
    course: Course | null;
    error: SubatoError;
    loading: boolean;
}

export function dispatchCourseChange(course: Observable<Course>) {
    let courseOp = inject(CourseOperators);
    return courseOp.dispatchCourseChange(course);
}

export function loadCourse(updaterFn: (state: Partial<CourseState>) => void) {
    let op = inject(CourseOperators);
    return (courseId: Observable<number>) => op.loadCourse(courseId, updaterFn);
}

@Injectable({ providedIn: 'root' })
export class CourseOperators {
    constructor(
        private courseService: CourseService,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler
    ) {}

    public dispatchCourseChange(course: Observable<Course>) {
        return course.pipe(
            filter((course) => course != null),
            tap((course) =>
                this.store.dispatch(new ChangeCourse({ course: course }))
            )
        );
    }

    public loadCourse(
        courseId$: Observable<number>,
        updaterFn: (state: Partial<CourseState>) => void
    ) {
        return courseId$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((courseId) =>
                this.courseService.getById(courseId).pipe(
                    tap((course: Course) =>
                        updaterFn({
                            course: course,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            course: null,
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }
}
