import { SubatoError } from 'app/shared/error/error';
import { EMPTY, Observable } from 'rxjs';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { CourseService } from 'app/courses/services/course.service';
import { inject, Injectable } from '@angular/core';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { ParticipantAttendance } from 'app/attendances/dto/participant-attendance.dto';
import { CourseUnitService } from 'app/courses/services/course-unit.service';

export interface ParticipantAttendancesState {
    attendances: ParticipantAttendance[];
    loading: boolean;
    error: SubatoError;
}

export interface ParticipantAttendanceState {
    attendance: ParticipantAttendance;
    loading: boolean;
    error: SubatoError;
}

export interface LoadAttendancesParams {
    courseId: number;
    userId: number;
}

export interface LoadAttendanceParams {
    unitId: number;
    userId: number;
}

export function loadAttendances(
    updaterFn: (state: Partial<ParticipantAttendancesState>) => void
) {
    let op = inject(ParticipantAttendanceOperators);
    return (params: Observable<LoadAttendancesParams>) =>
        op.loadAttendances(params, updaterFn);
}

export function loadAttendance(
    updaterFn: (state: Partial<ParticipantAttendanceState>) => void
) {
    let op = inject(ParticipantAttendanceOperators);
    return (params: Observable<LoadAttendanceParams>) =>
        op.loadAttendance(params, updaterFn);
}

@Injectable({ providedIn: 'root' })
export class ParticipantAttendanceOperators {
    constructor(
        private courseService: CourseService,
        private courseUnitService: CourseUnitService,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler
    ) {}

    public loadAttendances(
        params$: Observable<LoadAttendancesParams>,
        updaterFn: (state: Partial<ParticipantAttendancesState>) => void
    ) {
        return params$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((params) =>
                this.courseService
                    .getAttendancesForUser(params.courseId, params.userId)
                    .pipe(
                        tap((attendances: ParticipantAttendance[]) =>
                            updaterFn({
                                attendances: attendances,
                                loading: false,
                                error: null,
                            })
                        ),
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((err) => {
                            updaterFn({
                                attendances: [],
                                error: err,
                                loading: false,
                            });
                            return EMPTY;
                        })
                    )
            )
        );
    }

    public loadAttendance(
        params$: Observable<LoadAttendanceParams>,
        updaterFn: (state: Partial<ParticipantAttendanceState>) => void
    ) {
        return params$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((params) =>
                this.courseUnitService
                    .getAttendancesForUserAndUnit(params.unitId, params.userId)
                    .pipe(
                        tap((attendance: ParticipantAttendance) =>
                            updaterFn({
                                attendance: attendance,
                                loading: false,
                                error: null,
                            })
                        ),
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((err) => {
                            updaterFn({
                                attendance: null,
                                error: err,
                                loading: false,
                            });
                            return EMPTY;
                        })
                    )
            )
        );
    }
}
