import { Injectable } from '@angular/core';
import { ComponentStore } from '@ngrx/component-store';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { EMPTY, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { Dashboard } from 'app/dashboard/models/dashboard';
import { Term } from 'app/modules/models/term';
import { DashboardService } from 'app/dashboard/services/dashboard.service';
import { SubatoError } from 'app/shared/error/error';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';

interface DashboardState {
    dashboard: Dashboard;
    loading: boolean;
    error: SubatoError;
}

export const initialState: DashboardState = {
    dashboard: null,
    loading: true,
    error: null,
};

@Injectable()
export class DashboardStore extends ComponentStore<DashboardState> {
    public readonly dashboard$ = this.select((state) => state.dashboard);
    public readonly loading$ = this.select((state) => state.loading);
    public readonly error$ = this.select((state) => state.error);

    term$: Observable<Term | null> = this.store.select(
        (s) => s.modules.currentTerm
    );

    constructor(
        private store: Store<SubatoState>,
        private dashboardService: DashboardService,
        private globalErrorHandler: GlobalErrorHandler
    ) {
        super(initialState);
    }

    readonly loadDashboard$ = this.effect(() =>
        this.term$.pipe(
            tap(() => this.patchState({ loading: true })),
            switchMap((term) =>
                this.dashboardService.getDashboard(term?.id).pipe(
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        this.patchState({
                            dashboard: null,
                            loading: false,
                            error: err,
                        });
                        return EMPTY;
                    })
                )
            ),
            tap((dashboard) =>
                this.patchState({
                    dashboard: dashboard,
                    loading: false,
                    error: null,
                })
            )
        )
    );
}
