import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { BaseComponent } from 'app/shared/misc/base.component';
import { ParamHelper } from 'app/shared/misc/helper/param.helper';
import {
    catchError,
    filter,
    map,
    switchMap,
    take,
    takeUntil,
} from 'rxjs/operators';
import { ErrorContainerConfig } from 'app/shared/error/error-container-config';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { CreateExerciseDto } from 'app/exercises/dto/create-exercise.dto';
import { EMPTY } from 'rxjs';
import { EditExercisePageStore } from 'app/exercises/pages/exercise-admin/edit-exercise/edit-exercise.page.store';
import { ExplicitConfirmationDialogComponent } from 'app/shared/dialogs/explicit-confirmation-dialog/explicit-confirmation-dialog.component';
import { NbDialogService } from '@nebular/theme';

@Component({
    templateUrl: './edit-exercise.page.component.html',
    styleUrls: ['./edit-exercise.page.component.scss'],
    providers: [EditExercisePageStore],
})
export class EditExercisePageComponent extends BaseComponent implements OnInit {
    errorConfig: ErrorContainerConfig = {
        notFound: {
            message: 'Diese Übung konnten wir leider nicht finden.',
        },
    };

    exercise$ = this.editExerciseStore.exercise$;
    saving$ = this.editExerciseStore.saving$;
    deleting$ = this.editExerciseStore.deleting$;
    loading$ = this.editExerciseStore.loading$;
    error$ = this.editExerciseStore.error$;
    exerciseState$ = this.editExerciseStore.exerciseState$;
    validationError$ = this.editExerciseStore.validationError$;
    descriptionState$ = this.editExerciseStore.descriptionState$;
    taskInstancesState$ = this.editExerciseStore.taskInstancesState$;

    constructor(
        private store: Store<SubatoState>,
        private editExerciseStore: EditExercisePageStore,
        private dialogService: NbDialogService,
        private globalErrorHandler: GlobalErrorHandler,
        private route: ActivatedRoute
    ) {
        super();
    }

    ngOnInit(): void {
        this.route.paramMap
            .pipe(
                takeUntil(this.destroy$),
                map((params) =>
                    ParamHelper.parseIdOrThrow(params, 'exercise_id')
                ),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.editExerciseStore.patchExercise({ error: err });
                    return EMPTY;
                })
            )
            .subscribe((exerciseId) =>
                this.editExerciseStore.loadExercise(exerciseId)
            );
    }

    saveExercise(dto: CreateExerciseDto) {
        this.editExerciseStore.updateExercise(dto);
    }

    deleteExercise() {
        this.exercise$
            .pipe(
                take(1),
                switchMap((exercise) =>
                    this.dialogService
                        .open(ExplicitConfirmationDialogComponent, {
                            context: {
                                title: `Löschung von Übung: ${exercise.name}`,
                                text: `Bist du wirklich sicher, dass du die Übung '${exercise.name}' löschen willst? Alle verknüpften Lösungen, Abgaben etc. werden damit permanent gelöscht.`,
                                actionText: 'Löschen',
                                expectedText: exercise.name,
                            },
                        })
                        .onClose.pipe(
                            filter((result) => result && result.confirmed),
                            map(() => exercise)
                        )
                )
            )
            .subscribe((exercise) =>
                this.editExerciseStore.deleteExercise(exercise)
            );
    }
}
