import { Injectable } from '@angular/core';
import {
    catchError,
    filter,
    map,
    switchMap,
    tap,
    withLatestFrom,
} from 'rxjs/operators';
import { EMPTY, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { Router } from '@angular/router';
import {
    DangerousActionError,
    isDangerousAction,
    isValidationError,
    ValidationError,
} from 'app/shared/error/error';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { CreateExerciseDto } from 'app/exercises/dto/create-exercise.dto';
import { ExerciseService } from 'app/exercises/services/exercise.service';
import { ConfirmationDialogComponent } from 'app/shared/dialogs/confirmation-dialog/confirmation-dialog.component';
import { NbDialogService } from '@nebular/theme';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import {
    DescriptionState,
    loadDescription,
} from 'app/exercises/shared/operators/description.operators';
import {
    ExerciseState,
    loadExercise,
} from 'app/exercises/shared/operators/exercise.operators';
import { dispatchCourseChange } from 'app/courses/shared/operators/course.operators';
import {
    loadTaskInstances,
    TaskInstancesState,
} from 'app/exercises/shared/operators/task-instance.operators';
import { Exercise } from 'app/exercises/models/exercise';

interface EditExerciseState {
    exerciseState: ExerciseState;
    descriptionState: DescriptionState;
    taskInstancesState: TaskInstancesState;
    validationError: ValidationError | null;
    saving: boolean;
    confirm: boolean;
    dto: CreateExerciseDto;
    dangerousAction: DangerousActionError | null;
    deleting: boolean;
}

export const initialState: EditExerciseState = {
    exerciseState: { exercise: null, error: null, loading: true },
    descriptionState: { description: null, error: null, loading: true },
    taskInstancesState: { taskInstances: [], error: null, loading: true },
    validationError: null,
    saving: false,
    confirm: false,
    dto: null,
    dangerousAction: null,
    deleting: false,
};

@Injectable()
export class EditExercisePageStore extends BaseComponentStore<EditExerciseState> {
    principal$ = this.store.select((s) => s.auth.principal);

    public readonly confirm$ = this.select((s) => s.confirm);
    public readonly dto$ = this.select((s) => s.dto);
    public readonly saving$ = this.select((s) => s.saving);
    public readonly dangerousAction$ = this.select((s) => s.dangerousAction);
    public readonly deleting$ = this.select((s) => s.deleting);

    public readonly exercise$ = this.select((s) => s.exerciseState.exercise);
    public readonly exerciseState$ = this.select((s) => s.exerciseState);
    public readonly descriptionState$ = this.select((s) => s.descriptionState);
    public readonly taskInstancesState$ = this.select(
        (s) => s.taskInstancesState
    );

    public readonly validationError$ = this.select(
        (state) => state.validationError
    );

    public readonly loading$ = this.select(
        this.select((s) => s.exerciseState.loading),
        this.select((s) => s.descriptionState.loading),
        this.select((s) => s.taskInstancesState.loading),
        (l1, l2, l3) => l1 || l2 || l3
    );

    public readonly error$ = this.select(
        this.select((s) => s.exerciseState.error),
        this.select((s) => s.descriptionState.error),
        this.select((s) => s.taskInstancesState.error),
        (e1, e2, e3) => e1 ?? e2 ?? e3
    );

    public readonly isAdmin$ = this.select(
        this.exercise$,
        this.principal$,
        (exercise, principal) => principal?.isAdmin(exercise.course.id) ?? false
    );

    constructor(
        private store: Store<SubatoState>,
        private exerciseService: ExerciseService,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService,
        private dialogService: NbDialogService,
        private router: Router
    ) {
        super(initialState);
    }

    readonly patchExercise = this.updaterFor('exerciseState');
    readonly patchDescription = this.updaterFor('descriptionState');
    readonly patchTaskInstances = this.updaterFor('taskInstancesState');

    readonly loadExercise = this.effect((exerciseId$: Observable<number>) =>
        exerciseId$.pipe(loadExercise((s) => this.patchExercise(s)))
    );

    readonly dispatchCourseChange$ = this.effect(() =>
        this.exerciseState$.pipe(
            map((state) => state.exercise),
            filter((exercise) => exercise != null),
            map((exercise) => exercise?.course),
            dispatchCourseChange
        )
    );

    readonly updateExercise = this.effect(
        (params: Observable<CreateExerciseDto>) =>
            params.pipe(
                tap((dto) => this.patchState({ saving: true, dto: dto })),
                withLatestFrom(this.exerciseState$, this.confirm$),
                switchMap(([dto, exerciseState, confirm]) =>
                    this.exerciseService
                        .update(exerciseState.exercise.id, dto, confirm)
                        .pipe(
                            tap((exercise) => {
                                this.notificationService.showSuccess(
                                    'Übung erfolgreich aktualisiert'
                                );

                                this.router.navigate([
                                    '/',
                                    'exercises',
                                    exercise.id,
                                ]);
                            }),
                            catchError((err) =>
                                this.globalErrorHandler.handleError(err)
                            ),
                            catchError((error) => {
                                this.patchState({ saving: false });

                                if (isValidationError(error)) {
                                    this.patchState({ validationError: error });
                                }

                                if (isDangerousAction(error)) {
                                    this.patchState({ dangerousAction: error });
                                    return EMPTY;
                                }

                                this.notificationService.showError(
                                    this.globalErrorHandler.format(
                                        'Fehler bei Aktualisierung der Übung',
                                        error
                                    )
                                );

                                return EMPTY;
                            })
                        )
                )
            )
    );

    readonly showDangerousActionDialog$ = this.effect(() =>
        this.dangerousAction$.pipe(
            filter((dangerousAction) => dangerousAction != null),
            switchMap(
                (error) =>
                    this.dialogService.open(ConfirmationDialogComponent, {
                        context: {
                            title: `Achtung`,
                            text: `${error.description}. ${error.suggestion}. Trotzdem speichern?`,
                            actionText: 'Speichern',
                        },
                    }).onClose
            ),
            tap(() => this.patchState({ dangerousAction: null })),
            filter((result) => result && result.confirmed),
            tap(() => this.patchState({ confirm: true })),
            withLatestFrom(this.dto$),
            tap(([result, dto]) => this.updateExercise(dto))
        )
    );

    readonly loadDescription$ = this.effect(() =>
        this.exerciseState$.pipe(
            map((exerciseState) => exerciseState.exercise),
            loadDescription(true, (s) => this.patchDescription(s))
        )
    );

    readonly loadTaskInstances$ = this.effect(() =>
        this.exerciseState$.pipe(
            map((exerciseState) => exerciseState.exercise),
            filter((exercise) => exercise != null),
            map((exercise) => exercise.id),
            loadTaskInstances((s) => this.patchTaskInstances(s))
        )
    );

    readonly deleteExercise = this.effect((exercise$: Observable<Exercise>) =>
        exercise$.pipe(
            tap(() => this.patchState({ deleting: true })),
            switchMap((exercise) =>
                this.exerciseService.delete(exercise.id).pipe(
                    tap(() => {
                        this.router.navigate([
                            '/',
                            'course',
                            exercise.course.id,
                            'exercises',
                        ]);
                        this.notificationService.showSuccess(
                            'Übung erfolgreich gelöscht.'
                        );
                    }),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.patchState({ deleting: false });
                        this.notificationService.showError(
                            this.globalErrorHandler.format(
                                'Fehler beim Löschen der Übung',
                                error
                            )
                        );

                        return EMPTY;
                    })
                )
            )
        )
    );
}
