import {
    Component,
    EventEmitter,
    Input,
    OnChanges,
    OnInit,
    Output,
    SimpleChanges,
} from '@angular/core';
import {
    UntypedFormArray,
    UntypedFormBuilder,
    UntypedFormGroup,
    Validators,
} from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import {
    CreateExerciseDto,
    CreateTaskInstanceDto,
} from 'app/exercises/dto/create-exercise.dto';
import { Exercise } from 'app/exercises/models/exercise';
import { ExerciseService } from 'app/exercises/services/exercise.service';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { FormComponent } from 'app/shared/validation/form.component';
import { Course } from 'app/courses/models/course';
import { Task } from 'app/tasks/models/task';
import { TaskInstance } from 'app/exercises/models/task-instance';
import { NbDialogService } from '@nebular/theme';
import { Iso8601Helper } from 'app/shared/misc/helper/iso8601.helper';
import { SubatoState } from 'app/store/app.reducers';
import { Store } from '@ngrx/store';
import { reachablePointsTaskInstancesValidator } from 'app/exercises/pages/exercise-admin/exercise-form/validators';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { ValidationError } from 'app/shared/error/error';
import { takeUntil } from 'rxjs/operators';

@Component({
    selector: 'sb-exercise-form',
    templateUrl: './exercise-form.component.html',
    styleUrls: ['./exercise-form.component.scss'],
})
export class ExerciseFormComponent
    extends FormComponent
    implements OnInit, OnChanges
{
    @Input() exercise: Exercise | null;
    @Input() description: string | null;
    @Input() taskInstances: TaskInstance[] = [];

    @Input() saving: boolean = false;
    @Input() deleting: boolean = false;
    @Input() showDelete: boolean = false;
    @Input() course: Course | null;
    @Input() validationError: ValidationError;

    @Output() onSave = new EventEmitter<CreateExerciseDto>();
    @Output() onDelete = new EventEmitter<void>();

    excerciseForm: UntypedFormGroup;
    selectedTasks: Task[] = [];

    constructor(
        private exerciseService: ExerciseService,
        private dialogService: NbDialogService,
        private globalErrorHandler: GlobalErrorHandler,
        private router: Router,
        private store: Store<SubatoState>,
        private route: ActivatedRoute,
        private notificationService: NotificationService,
        private fb: UntypedFormBuilder
    ) {
        super();
    }

    ngOnChanges(changes: SimpleChanges): void {
        if (
            changes.hasOwnProperty('taskInstances') ||
            changes.hasOwnProperty('exercise') ||
            changes.hasOwnProperty('description')
        ) {
            this.createForm(
                this.exercise,
                this.taskInstances,
                this.description
            );

            this.selectedTasks = this.taskInstances?.map((ti) => ti.task) ?? [];
        }

        if (changes.hasOwnProperty('validationError')) {
            this.errors = this.getValidationErrors(this.validationError);
        }
    }

    ngOnInit(): void {
        this.createForm(this.exercise, this.taskInstances, this.description);
    }

    get taskInstancesForm() {
        return this.excerciseForm.controls['taskInstances'] as UntypedFormArray;
    }

    createForm(
        exercise?: Exercise,
        taskInstances?: TaskInstance[],
        description?: string
    ) {
        this.excerciseForm = this.fb.group(
            {
                name: [exercise?.name ?? '', Validators.required],
                assessmentMode: [exercise?.assessmentMode],
                orderNumber: [exercise?.orderNumber ?? 0, Validators.required],
                description: [description ?? ''],
                summary: [exercise?.summary ?? ''],
                dueTo: [exercise?.dueTo ?? ''],
                visibleFrom: [exercise?.visibleFrom ?? ''],
                taskInstances: this.fb.array(
                    taskInstances?.map((ti) =>
                        this.createTaskInstanceForm(ti)
                    ) ?? []
                ),
            },
            { validators: reachablePointsTaskInstancesValidator }
        );

        this.excerciseForm
            .get('taskInstances')
            .valueChanges.pipe(takeUntil(this.destroy$))
            .subscribe((f) => {
                this.selectedTasks = this.taskInstancesForm.controls.map(
                    (group) => group.get('task').value
                );
            });
    }

    createTaskInstanceForm(taskInstance: TaskInstance) {
        return this.fb.group({
            reachablePoints: [
                taskInstance.reachablePoints,
                [Validators.pattern('^[0-9]*$')],
            ],
            task: taskInstance.task,
        });
    }

    buildDto() {
        let taskInstances = this.taskInstancesForm.controls.map((c, i) => {
            let values = c.value;
            return <CreateTaskInstanceDto>{
                orderNumber: i,
                reachablePoints: values.reachablePoints
                    ? parseInt(values.reachablePoints)
                    : null,
                taskId: values.task.id,
            };
        });

        let dto: CreateExerciseDto = {
            courseId: this.course.id,
            name: this.excerciseForm.get('name').value,
            orderNumber: this.excerciseForm.get('orderNumber').value,
            assessmentMode: this.excerciseForm.get('assessmentMode').value,
            description: this.excerciseForm.get('description').value,
            summary: this.excerciseForm.get('summary').value,
            dueTo: Iso8601Helper.toString(
                this.excerciseForm.get('dueTo').value
            ),
            visibleFrom: Iso8601Helper.toString(
                this.excerciseForm.get('visibleFrom').value
            ),
            taskInstances: taskInstances,
        };

        return dto;
    }

    save() {
        let dto = this.buildDto();
        this.onSave.emit(dto);
    }

    delete() {
        this.onDelete.emit();
    }

    selectTask(task: Task) {
        let taskInstance: TaskInstance = new TaskInstance({
            task: task,
            reachablePoints: null,
        });
        let taskInstanceForm = this.createTaskInstanceForm(taskInstance);
        this.taskInstancesForm.push(taskInstanceForm);
    }
}
