import { Injectable } from '@angular/core';
import { catchError, switchMap, tap, withLatestFrom } from 'rxjs/operators';
import { EMPTY, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';

import { Task } from 'app/tasks/models/task';
import { TaskPool } from 'app/tasks/models/task-pool';
import { TaskFilter, TaskService } from 'app/tasks/services/task.service';
import { TaskPoolService } from 'app/tasks/services/task-pool.service';
import { environment } from 'environments/environment';
import { debounceWithInitialValue } from 'app/shared/misc/rxjs-operators';
import {
    createInitialState,
    PagingStore,
} from 'app/shared/paging/paging.store';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import {
    loadPools,
    TaskPoolsState,
} from 'app/tasks/shared/operators/task-pool.operators';

interface TaskSelectState {
    filter: TaskFilter;
    selectedTasks: Task[];
    poolsState: TaskPoolsState;
    selectedPool: TaskPool;
}

export const initialState: TaskSelectState = {
    filter: { poolId: null, q: '', showArchived: false },
    poolsState: { pools: [], error: null, loading: true },
    selectedPool: null,
    selectedTasks: [],
};

@Injectable()
export class TaskSelectStore extends BaseComponentStore<TaskSelectState> {
    public readonly filter$ = this.select((state) => state.filter);
    public readonly debouncedFilter$ = this.filter$.pipe(
        debounceWithInitialValue(
            environment.debounceTimeMs,
            initialState.filter
        )
    );
    public readonly query$ = this.select((state) => state.filter.q);
    public readonly showArchived$ = this.select(
        (state) => state.filter.showArchived
    );

    public readonly pools$ = this.select((state) => state.poolsState.pools);
    public readonly selectedPool$ = this.select((state) => state.selectedPool);

    private _paging: PagingStore<Task> = new PagingStore<Task>({
        ...createInitialState(),
    });

    get paging() {
        return this._paging;
    }

    constructor(
        private store: Store<SubatoState>,
        private taskService: TaskService,
        private taskPoolService: TaskPoolService,
        private globalErrorHandler: GlobalErrorHandler
    ) {
        super(initialState);
    }

    readonly patchPools = this.updaterFor('poolsState');

    readonly setPool = this.updater((state, pool: TaskPool) => ({
        ...state,
        selectedPool: pool,
        filter: { ...state.filter, poolId: pool?.id },
    }));

    readonly setQuery = this.updater((state, query: string) => ({
        ...state,
        filter: { ...state.filter, q: query },
    }));

    readonly setShowArchived = this.updater((state, showArchived: boolean) => ({
        ...state,
        filter: { ...state.filter, showArchived: showArchived },
    }));

    containsTask(tasks: Task[], task: Task) {
        return tasks.map((m) => m.id).indexOf(task.id) != -1;
    }

    readonly selectTasks = this.updater((state, tasks: Task[]) => ({
        ...state,
        selectedTasks: [
            ...state.selectedTasks,
            ...tasks.filter((m) => !this.containsTask(state.selectedTasks, m)),
        ],
    }));

    readonly loadPools = this.effect((trigger$: Observable<void>) =>
        trigger$.pipe(loadPools((s) => this.patchPools(s)))
    );

    readonly filterChange$ = this.effect(() =>
        this.debouncedFilter$.pipe(
            tap(() => this.paging.patchState({ loading: true, items: [] })),
            withLatestFrom(this.paging.projection$),
            switchMap(([filter, projection]) =>
                this.taskService
                    .getAll(filter, { ...projection, pageNumber: null })
                    .pipe(
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((error) => {
                            this.paging.patchError(error);
                            return EMPTY;
                        })
                    )
            ),
            tap((page) => this.paging.patchPage(page))
        )
    );

    readonly projectionChange$ = this.effect(() =>
        this.paging.projection$.pipe(
            tap(() => this.paging.patchState({ loading: true })),
            withLatestFrom(this.debouncedFilter$),
            switchMap(([projection, filter]) =>
                this.taskService.getAll(filter, projection).pipe(
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.paging.patchError(error);
                        return EMPTY;
                    })
                )
            ),
            tap((page) => this.paging.patchPage(page))
        )
    );
}
