import { Injectable } from '@angular/core';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { EMPTY, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { isValidationError, ValidationError } from 'app/shared/error/error';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { CreateExerciseDto } from 'app/exercises/dto/create-exercise.dto';
import { ExerciseService } from 'app/exercises/services/exercise.service';
import { CourseService } from 'app/courses/services/course.service';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import {
    CourseState,
    dispatchCourseChange,
    loadCourse,
} from 'app/courses/shared/operators/course.operators';
import { Router } from '@angular/router';

interface NewExerciseState {
    courseState: CourseState;
    validationError: ValidationError | null;
    saving: boolean;
}

export const initialState: NewExerciseState = {
    courseState: { course: null, error: null, loading: true },
    validationError: null,
    saving: false,
};

@Injectable()
export class NewExercisePageStore extends BaseComponentStore<NewExerciseState> {
    public readonly courseState$ = this.select((s) => s.courseState);
    public readonly course$ = this.select((s) => s.courseState.course);
    public readonly saving$ = this.select((s) => s.saving);
    public readonly validationError$ = this.select(
        (state) => state.validationError
    );

    constructor(
        private store: Store<SubatoState>,
        private courseService: CourseService,
        private exerciseService: ExerciseService,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService,
        private router: Router
    ) {
        super(initialState);
    }

    readonly patchCourse = this.updaterFor('courseState');

    readonly loadCourse = this.effect((courseId$: Observable<number>) =>
        courseId$.pipe(loadCourse((s) => this.patchCourse(s)))
    );

    readonly dispatchCourseChange$ = this.effect(() =>
        this.course$.pipe(dispatchCourseChange)
    );

    readonly createExercise = this.effect(
        (params: Observable<CreateExerciseDto>) =>
            params.pipe(
                tap((dto) => this.patchState({ saving: true })),
                switchMap((dto) =>
                    this.exerciseService.create(dto).pipe(
                        tap((exercise) => {
                            this.notificationService.showSuccess(
                                'Übung erfolgreich erstellt'
                            );

                            this.router.navigate([
                                '/',
                                'exercises',
                                exercise.id,
                            ]);
                        }),
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((error) => {
                            this.patchState({ saving: false });

                            if (isValidationError(error)) {
                                this.patchState({ validationError: error });
                            }

                            this.notificationService.showError(
                                this.globalErrorHandler.format(
                                    'Fehler bei Erstellung der Übung',
                                    error
                                )
                            );

                            return EMPTY;
                        })
                    )
                )
            )
    );
}
