import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { BaseComponent } from 'app/shared/misc/base.component';
import { catchError, map, takeUntil } from 'rxjs/operators';
import { TaskInstance } from 'app/exercises/models/task-instance';
import { ParamHelper } from 'app/shared/misc/helper/param.helper';
import { ErrorContainerConfig } from 'app/shared/error/error-container-config';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { combineLatest, EMPTY } from 'rxjs';
import { ExercisePageStore } from 'app/exercises/pages/exercise/exercise.page.store';
import { Exercise } from 'app/exercises/models/exercise';
import { Submission } from 'app/assessment/models/submission';
import { PageHeaderAction } from 'app/shared/page-header/page-header/page-header.component';
import { SubmissionLinkProvider } from 'app/assessment/routing/submission-link-provider';

@Component({
    templateUrl: './exercise.page.component.html',
    styleUrls: ['./exercise.page.component.scss'],
    providers: [ExercisePageStore],
})
export class ExercisePageComponent extends BaseComponent implements OnInit {
    exerciseState$ = this.exerciseDetailStore.exerciseState$;
    descriptionState$ = this.exerciseDetailStore.descriptionState$;
    submission$ = this.exerciseDetailStore.submission$;
    isAdmin$ = this.exerciseDetailStore.isAdmin$;
    hasAnyGroup$ = this.exerciseDetailStore.hasAnyGroup$;

    errorConfig: ErrorContainerConfig = {
        notFound: {
            message: 'Diese Übung konnten wir leider nicht finden.',
        },
    };

    actions$ = combineLatest([
        this.exerciseState$,
        this.submission$,
        this.isAdmin$,
        this.hasAnyGroup$,
    ]).pipe(
        takeUntil(this.destroy$),
        map(([exerciseState, submission, isAdmin, hasAnyGroup]) =>
            this.buildActions(
                exerciseState.exercise,
                submission,
                isAdmin,
                hasAnyGroup
            )
        )
    );

    constructor(
        private globalErrorHandler: GlobalErrorHandler,
        private store: Store<SubatoState>,
        private exerciseDetailStore: ExercisePageStore,
        private router: Router,
        private route: ActivatedRoute,
        private linkProvider: SubmissionLinkProvider
    ) {
        super();
    }

    ngOnInit(): void {
        this.route.paramMap
            .pipe(
                takeUntil(this.destroy$),
                map((params) =>
                    ParamHelper.parseIdOrThrow(params, 'exercise_id')
                ),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.exerciseDetailStore.patchExercise({ error: err });
                    return EMPTY;
                })
            )
            .subscribe((exerciseId) =>
                this.exerciseDetailStore.loadExercise(exerciseId)
            );
    }

    getTaskInstanceLink(taskInstance: TaskInstance) {
        return [
            '/',
            'exercises',
            taskInstance.exercise.id,
            'tasks',
            taskInstance.task.id,
        ];
    }

    buildActions(
        exercise: Exercise,
        submission: Submission,
        isAdmin: boolean,
        hasAnyGroup: boolean
    ): PageHeaderAction[] {
        let actions: PageHeaderAction[] = [];

        if (exercise != null) {
            if (hasAnyGroup) {
                actions.push({
                    icon: 'checkmark-square-outline',
                    name: 'Bewerten',
                    status: 'primary',
                    link: { commands: ['assessment'] },
                });
            }

            if (isAdmin) {
                actions.push({
                    icon: 'edit-2-outline',
                    name: 'Bearbeiten',
                    disabled: exercise.course.finished,
                    link: { commands: ['edit'] },
                });
            }

            if (submission != null) {
                actions.push({
                    icon: 'info-outline',
                    name: 'Abgabe einsehen',
                    link: this.linkProvider.detail({
                        submissionId: submission.id,
                    }),
                });
            }
        }

        return actions;
    }
}
