import { Injectable } from '@angular/core';
import { filter, map } from 'rxjs/operators';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { combineLatest, Observable } from 'rxjs';
import { ExerciseService } from 'app/exercises/services/exercise.service';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { Router } from '@angular/router';
import { SubmissionService } from 'app/assessment/service/submission.service';
import {
    DescriptionState,
    loadDescription,
} from 'app/exercises/shared/operators/description.operators';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import {
    loadLatestSubmission,
    SubmissionState,
} from 'app/exercises/shared/operators/submission.operators';
import {
    ExerciseState,
    loadExercise,
} from 'app/exercises/shared/operators/exercise.operators';
import { dispatchCourseChange } from 'app/courses/shared/operators/course.operators';

interface ExercisePageState {
    exerciseState: ExerciseState;
    descriptionState: DescriptionState;
    submissionState: SubmissionState;
}

export const initialState: ExercisePageState = {
    exerciseState: { exercise: null, error: null, loading: true },
    descriptionState: { description: null, error: null, loading: true },
    submissionState: { submission: null, error: null, loading: true },
};

@Injectable()
export class ExercisePageStore extends BaseComponentStore<ExercisePageState> {
    principal$ = this.store.select((s) => s.auth.principal);
    public readonly exercise$ = this.select((s) => s.exerciseState.exercise);
    public readonly exerciseState$ = this.select((s) => s.exerciseState);

    public readonly submission$ = this.select(
        (state) => state.submissionState.submission
    );
    public readonly descriptionState$ = this.select((s) => s.descriptionState);

    public readonly isAdmin$ = this.select(
        this.exercise$,
        this.principal$,
        (exercise, principal) =>
            exercise != null &&
            principal != null &&
            principal.isAdmin(exercise.course.id)
    );

    public readonly hasAnyGroup$ = this.select(
        this.exercise$,
        this.principal$,
        (exercise, principal) =>
            exercise != null &&
            principal != null &&
            principal.hasAnyGroup(exercise.course.id)
    );

    constructor(
        private store: Store<SubatoState>,
        private exerciseService: ExerciseService,
        private submissionService: SubmissionService,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService,
        private router: Router
    ) {
        super(initialState);
    }

    readonly patchDescription = this.updaterFor('descriptionState');
    readonly patchSubmission = this.updaterFor('submissionState');
    readonly patchExercise = this.updaterFor('exerciseState');

    readonly loadExercise = this.effect((exerciseId$: Observable<number>) =>
        exerciseId$.pipe(loadExercise((s) => this.patchExercise(s)))
    );

    readonly dispatchCourseChange$ = this.effect(() =>
        this.exercise$.pipe(
            filter((exercise) => exercise != null),
            map((exercise) => exercise?.course),
            dispatchCourseChange
        )
    );

    readonly loadDescription$ = this.effect(() =>
        this.exercise$.pipe(
            loadDescription(false, (s) => this.patchDescription(s))
        )
    );

    readonly loadSubmission$ = this.effect(() =>
        combineLatest(this.principal$, this.exercise$).pipe(
            filter(
                ([principal, exercise]) => principal != null && exercise != null
            ),
            map(([principal, exercise]) => ({
                exerciseId: exercise.id,
                userId: principal.id,
            })),
            loadLatestSubmission((s) => this.patchSubmission(s))
        )
    );
}
