import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { BaseComponent } from 'app/shared/misc/base.component';
import { catchError, map, takeUntil } from 'rxjs/operators';
import { ifAttached } from 'app/shared/route-reuse/route-reuse-life.cycle.directive';
import { ParamHelper } from 'app/shared/misc/helper/param.helper';
import { ErrorContainerConfig } from 'app/shared/error/error-container-config';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { combineLatest, EMPTY } from 'rxjs';
import { ExercisesPageStore } from 'app/exercises/pages/exercises/exercises.page.store';
import { PageHeaderAction } from 'app/shared/page-header/page-header/page-header.component';
import { Course } from 'app/courses/models/course';

@Component({
    templateUrl: './exercises.page.component.html',
    styleUrls: ['./exercises.page.component.scss'],
    providers: [ExercisesPageStore],
})
export class ExercisesPageComponent extends BaseComponent implements OnInit {
    errorConfig: ErrorContainerConfig = {
        notFound: {
            message: 'Diesen Kurs konnten wir leider nicht finden.',
        },
    };

    courseState$ = this.exercisePageStore.courseState$;
    isAdmin$ = this.exercisePageStore.isAdmin$;

    actions$ = combineLatest([this.courseState$, this.isAdmin$]).pipe(
        takeUntil(this.destroy$),
        ifAttached(this),
        map(([courseState, isAdmin]) =>
            this.buildActions(courseState.course, isAdmin)
        )
    );

    constructor(
        private store: Store<SubatoState>,
        private exercisePageStore: ExercisesPageStore,
        private globalErrorHandler: GlobalErrorHandler,
        private route: ActivatedRoute
    ) {
        super();
    }

    onAttach() {
        super.onAttach();
        this.exercisePageStore.onAttach();
    }

    onDetach() {
        super.onDetach();
        this.exercisePageStore.onDetach();
    }

    ngOnInit(): void {
        this.route.paramMap
            .pipe(
                takeUntil(this.destroy$),
                ifAttached(this),
                map((params) =>
                    ParamHelper.parseIdOrThrow(params, 'course_id')
                ),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.exercisePageStore.patchCourse({ error: err });
                    return EMPTY;
                })
            )
            .subscribe((courseId) =>
                this.exercisePageStore.loadCourse(courseId)
            );
    }

    buildActions(course: Course, isAdmin: boolean): PageHeaderAction[] {
        let actions: PageHeaderAction[] = [];
        if (isAdmin && course != null) {
            actions.push({
                name: 'Neue Übung',
                icon: 'plus-outline',
                link: {
                    commands: ['new'],
                },
                disabled: course.finished,
                status: 'primary',
            });
        }

        return actions;
    }
}
