import { Injectable } from '@angular/core';
import { filter, map } from 'rxjs/operators';
import { combineLatest, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import {
    CourseState,
    dispatchCourseChange,
    loadCourse,
} from 'app/courses/shared/operators/course.operators';
import {
    ExercisesState,
    loadExercises,
} from 'app/exercises/shared/operators/exercise.operators';
import { ifAttached } from 'app/shared/route-reuse/route-reuse-life.cycle.directive';
import { debounceWithInitialValue } from 'app/shared/misc/rxjs-operators';
import { environment } from 'environments/environment';

interface ExercisesPageState {
    courseState: CourseState;
    exercisesState: ExercisesState;
    query: string;
    exercisesDirty: boolean;
}

export const initialState: ExercisesPageState = {
    courseState: { course: null, error: null, loading: true },
    exercisesState: { exercises: [], error: null, loading: true },
    query: '',
    exercisesDirty: true,
};

@Injectable()
export class ExercisesPageStore extends BaseComponentStore<ExercisesPageState> {
    principal$ = this.store.select((s) => s.auth.principal);

    public readonly courseState$ = this.select((state) => state.courseState);
    public readonly course$ = this.select((state) => state.courseState.course);

    public readonly exercisesState$ = this.select(
        (state) => state.exercisesState
    );
    public readonly exercises$ = this.select(
        (state) => state.exercisesState.exercises
    );

    public readonly query$ = this.select((state) => state.query);
    public readonly debouncedQuery$ = this.query$.pipe(
        debounceWithInitialValue(environment.debounceTimeMs, initialState.query)
    );
    public readonly exercisesDirty$ = this.select(
        (state) => state.exercisesDirty
    );

    public readonly isAdmin$ = this.select(
        this.course$,
        this.principal$,
        (course, principal) =>
            course != null && principal != null && principal.isAdmin(course.id)
    );

    public readonly hasAnyGroup$ = this.select(
        this.course$,
        this.principal$,
        (course, principal) =>
            course != null &&
            principal != null &&
            principal.hasAnyGroup(course.id)
    );

    constructor(private store: Store<SubatoState>) {
        super(initialState);
    }

    readonly patchExercises = this.updaterFor('exercisesState');
    readonly patchCourse = this.updaterFor('courseState');

    readonly loadCourse = this.effect((courseId$: Observable<number>) =>
        courseId$.pipe(loadCourse((s) => this.patchCourse(s)))
    );

    readonly dispatchCourseChange$ = this.effect(() =>
        combineLatest([this.course$, this.detached$], (course) => course).pipe(
            ifAttached(this),
            dispatchCourseChange
        )
    );

    readonly loadExercises$ = this.effect(() =>
        combineLatest([
            this.course$,
            this.debouncedQuery$,
            this.detached$,
        ]).pipe(
            ifAttached(this),
            filter(([course, query, detached]) => course != null),
            map(([course, query]) => ({ courseId: course.id, query: query })),
            loadExercises((s) => this.patchExercises(s))
        )
    );
}
