import { ChangeDetectorRef, Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { BaseComponent } from 'app/shared/misc/base.component';
import { combineLatest, EMPTY, Observable } from 'rxjs';
import { catchError, filter, map, takeUntil } from 'rxjs/operators';

import { TaskInstance } from 'app/exercises/models/task-instance';
import { ExerciseService } from 'app/exercises/services/exercise.service';
import { SubatoPrincipal } from 'app/auth/models/principal/subato-principal';
import { ParamHelper } from 'app/shared/misc/helper/param.helper';
import { ErrorContainerConfig } from 'app/shared/error/error-container-config';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { SubmitSolutionPageStore } from 'app/exercises/pages/submit-solution/submit-solution.page.store';
import { PageHeaderAction } from 'app/shared/page-header/page-header/page-header.component';
import { TaskInstanceState } from 'app/exercises/shared/operators/task-instance.operators';
import { ConfirmationDialogComponent } from 'app/shared/dialogs/confirmation-dialog/confirmation-dialog.component';
import { NbDialogService } from '@nebular/theme';

@Component({
    templateUrl: './submit-solution.page.component.html',
    styleUrls: ['./submit-solution.page.component.scss'],
    providers: [SubmitSolutionPageStore],
})
export class SubmitSolutionPageComponent
    extends BaseComponent
    implements OnInit
{
    principal$: Observable<SubatoPrincipal> = this.store.select(
        (s) => s.auth.principal
    );

    taskInstanceState$ = this.submitSolutionStore.taskInstanceState$;
    taskInstancesState$ = this.submitSolutionStore.taskInstancesState$;
    submissionState$ = this.submitSolutionStore.submissionState$;

    canPublish$ = this.submitSolutionStore.canPublish$;
    hasAnyGroup$ = this.submitSolutionStore.hasAnyGroup$;
    publishingSolution$ = this.submitSolutionStore.publishingSolution$;
    canUnpublish$ = this.submitSolutionStore.canUnpublish$;
    taskContent$ = this.submitSolutionStore.taskContent$;

    isStaff$ = this.principal$.pipe(
        takeUntil(this.destroy$),
        map((principal) => principal && principal.hasStaffRole())
    );

    errorConfig: ErrorContainerConfig = {
        notFound: {
            message:
                'Diese Aufgabe konnten wir in diesem Übungsblatt leider nicht finden.',
        },
    };

    actions$ = combineLatest([
        this.taskInstanceState$,
        this.canUnpublish$,
        this.canPublish$,
        this.hasAnyGroup$,
        this.publishingSolution$,
    ]).pipe(
        takeUntil(this.destroy$),
        map(
            ([
                taskInstanceState,
                canUnpublish,
                canPublish,
                hasAnyGroup,
                publishingSolution,
            ]) =>
                this.buildActions(
                    taskInstanceState,
                    canUnpublish,
                    canPublish,
                    hasAnyGroup,
                    publishingSolution
                )
        )
    );

    constructor(
        private exerciseService: ExerciseService,
        private submitSolutionStore: SubmitSolutionPageStore,
        private store: Store<SubatoState>,
        private router: Router,
        private globalErrorHandler: GlobalErrorHandler,
        private dialogService: NbDialogService,
        private route: ActivatedRoute,
        private cd: ChangeDetectorRef
    ) {
        super();
    }

    onTasksLoaded(taskInstances: TaskInstance[]) {
        this.submitSolutionStore.patchTaskInstances({
            taskInstances: taskInstances,
        });

        this.cd.detectChanges();
    }

    getTaskInstanceLink(taskInstance: TaskInstance) {
        return [
            '/',
            'exercises',
            taskInstance.exercise.id,
            'tasks',
            taskInstance.task.id,
        ];
    }

    ngOnInit(): void {
        this.route.paramMap
            .pipe(
                takeUntil(this.destroy$),
                map((params) => {
                    let exerciseId = ParamHelper.parseIdOrThrow(
                        params,
                        'exercise_id'
                    );
                    let taskId = ParamHelper.parseIdOrThrow(params, 'task_id');
                    return [exerciseId, taskId];
                }),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.submitSolutionStore.patchTaskInstance({ error: err });
                    return EMPTY;
                })
            )
            .subscribe(([exerciseId, taskId]) => {
                this.submitSolutionStore.loadTaskInstance({
                    exerciseId,
                    taskId,
                });
            });
    }

    publishSolution(publish: boolean) {
        if (publish) {
            this.dialogService
                .open(ConfirmationDialogComponent, {
                    context: {
                        title: `Veröffentlichung der Lösung`,
                        text: `Bist du wirklich sicher, dass du die Lösung für alle Teilnehmer veröffentlichen willst?`,
                        actionText: 'Veröffentlichen',
                        confirmStatus: 'warning',
                    },
                })
                .onClose.pipe(
                    takeUntil(this.destroy$),
                    filter((result) => result && result.confirmed)
                )
                .subscribe(() =>
                    this.submitSolutionStore.publishSolution(true)
                );
        } else {
            this.submitSolutionStore.publishSolution(false);
        }
    }

    buildActions(
        taskInstanceState: TaskInstanceState,
        canUnpublish: boolean,
        canPublish: boolean,
        hasAnyGroup: boolean,
        publishingSolution: boolean
    ): PageHeaderAction[] {
        let actions: PageHeaderAction[] = [];

        if (taskInstanceState.taskInstance != null) {
            if (hasAnyGroup) {
                actions.push({
                    icon: 'checkmark-square-outline',
                    name: 'Bewerten',
                    status: 'primary',
                    link: { commands: ['assessment'] },
                });
            }

            if (canPublish) {
                actions.push({
                    icon: 'eye-outline',
                    name: 'Lösung veröffentlichen',
                    loading: publishingSolution,
                    pos: 'right',
                    status: 'warning',
                    disabled:
                        taskInstanceState.taskInstance.exercise.course
                            .finished || publishingSolution,
                    click: () => this.publishSolution(true),
                });
            }

            if (canUnpublish) {
                actions.push({
                    icon: 'eye-outline',
                    name: 'Lösung öffentlich',
                    status: 'success',
                    loading: publishingSolution,
                    pos: 'right',
                    disabled:
                        taskInstanceState.taskInstance.exercise.course
                            .finished || publishingSolution,
                    click: () => this.publishSolution(false),
                });
            }
        }

        return actions;
    }
}
