import { Injectable } from '@angular/core';
import { SubatoError } from 'app/shared/error/error';
import {
    catchError,
    filter,
    map,
    switchMap,
    tap,
    withLatestFrom,
} from 'rxjs/operators';
import { combineLatest, EMPTY, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { ExerciseService } from 'app/exercises/services/exercise.service';
import { TaskInstance } from 'app/exercises/models/task-instance';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { SubatoPrincipal } from 'app/auth/models/principal/subato-principal';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import {
    loadLatestSubmission,
    SubmissionState,
} from 'app/exercises/shared/operators/submission.operators';
import {
    loadTaskInstance,
    LoadTaskInstanceParams,
    TaskInstanceState,
} from 'app/exercises/shared/operators/task-instance.operators';
import { dispatchCourseChange } from 'app/courses/shared/operators/course.operators';
import {
    loadTaskContent,
    TaskContentState,
} from 'app/exercises/shared/operators/task-content.operators';
import { ifAttached } from 'app/shared/route-reuse/route-reuse-life.cycle.directive';

interface TaskInstancesState {
    taskInstances: TaskInstance[];
    error: SubatoError;
    loading: boolean;
}

export interface SubmitSolutionState {
    taskInstanceState: TaskInstanceState;
    taskInstancesState: TaskInstancesState;
    submissionState: SubmissionState;
    publishingSolution: boolean;
    taskContentState: TaskContentState;
}

export const initialState: SubmitSolutionState = {
    taskInstanceState: { taskInstance: null, error: null, loading: true },
    taskInstancesState: { taskInstances: [], error: null, loading: true },
    submissionState: { submission: null, error: null, loading: true },
    publishingSolution: false,
    taskContentState: { content: null, error: null, loading: true },
};

@Injectable()
export class SubmitSolutionPageStore extends BaseComponentStore<SubmitSolutionState> {
    public readonly taskInstanceState$ = this.select(
        (s) => s.taskInstanceState
    );
    public readonly taskInstance$ = this.select(
        (s) => s.taskInstanceState.taskInstance
    );

    public readonly taskInstancesState$ = this.select(
        (s) => s.taskInstancesState
    );

    public readonly submissionState$ = this.select((s) => s.submissionState);

    taskContent$ = this.select((s) => s.taskContentState.content);
    publishingSolution$ = this.select((s) => s.publishingSolution);

    principal$: Observable<SubatoPrincipal> = this.store.select(
        (s) => s.auth.principal
    );

    isAdmin$ = this.select(
        this.taskInstance$,
        this.principal$,
        (taskInstance, principal) =>
            taskInstance != null &&
            principal &&
            principal.isAdmin(taskInstance.exercise.course.id)
    );

    hasAnyGroup$ = this.select(
        this.taskInstance$,
        this.principal$,
        (taskInstance, principal) =>
            taskInstance != null &&
            principal &&
            principal.hasAnyGroup(taskInstance.exercise.course.id)
    );

    canPublish$ = this.select(
        this.taskInstance$,
        this.isAdmin$,
        (taskInstance, isAdmin) =>
            taskInstance != null &&
            isAdmin &&
            !taskInstance.task.archived &&
            !taskInstance.solutionPublic
    );

    canUnpublish$ = this.select(
        this.taskInstance$,
        this.isAdmin$,
        (taskInstance, isAdmin) =>
            taskInstance != null &&
            isAdmin &&
            !taskInstance.task.archived &&
            taskInstance.solutionPublic
    );

    constructor(
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler,
        private exerciseService: ExerciseService,
        private notificationService: NotificationService
    ) {
        super(initialState);
    }

    readonly patchTaskInstance = this.updaterFor('taskInstanceState');
    readonly patchTaskInstances = this.updaterFor('taskInstancesState');
    readonly patchSubmission = this.updaterFor('submissionState');
    readonly patchTaskContent = this.updaterFor('taskContentState');

    readonly loadTaskInstance = this.effect(
        (params$: Observable<LoadTaskInstanceParams>) =>
            params$.pipe(loadTaskInstance((s) => this.patchTaskInstance(s)))
    );

    publishSolution = this.effect((publish$: Observable<boolean>) =>
        publish$.pipe(
            tap(() => this.patchState({ publishingSolution: true })),
            withLatestFrom(this.taskInstance$),
            switchMap(([publish, taskInstance]) =>
                this.exerciseService
                    .publishSolution(
                        taskInstance.exercise.id,
                        taskInstance.task.id,
                        publish
                    )
                    .pipe(
                        tap(() =>
                            this.patchState({
                                publishingSolution: false,
                            })
                        ),
                        tap(() => {
                            // todo
                            let ti = new TaskInstance({});
                            ti.solutionPublic = publish;
                            ti.orderNumber = taskInstance.orderNumber;
                            ti.exercise = taskInstance.exercise;
                            ti.reachablePoints = taskInstance.reachablePoints;
                            ti.task = taskInstance.task;
                            ti.downloadBundleUrl =
                                taskInstance.downloadBundleUrl;
                            this.patchTaskInstance({
                                taskInstance: ti,
                            });
                        }),
                        tap(() =>
                            this.notificationService.showSuccess(
                                publish
                                    ? 'Musterlösung erfolgreich veröffentlicht'
                                    : 'Musterlösungen sind nun nicht mehr öffentlich'
                            )
                        ),
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((error) => {
                            this.patchState({
                                publishingSolution: false,
                            });

                            this.notificationService.showError(
                                this.globalErrorHandler.format(
                                    'Fehler bei Veröffentlichung der Musterlösung',
                                    error
                                )
                            );

                            return EMPTY;
                        })
                    )
            )
        )
    );

    readonly dispatchCourseChange$ = this.effect(() =>
        combineLatest(
            [this.taskInstance$, this.detached$],
            (taskInstance) => taskInstance
        ).pipe(
            map((taskInstance) => taskInstance?.exercise?.course),
            ifAttached(this),
            dispatchCourseChange
        )
    );

    readonly loadSubmission$ = this.effect(() =>
        combineLatest(this.principal$, this.taskInstance$).pipe(
            filter(
                ([principal, taskInstance]) =>
                    principal != null && taskInstance != null
            ),
            map(([principal, taskInstance]) => ({
                exerciseId: taskInstance.exercise.id,
                userId: principal.id,
            })),
            loadLatestSubmission((s) => this.patchSubmission(s))
        )
    );

    loadTaskContent$: Observable<string> = this.effect(() =>
        this.taskInstance$.pipe(
            filter((taskInstance) => taskInstance != null),
            loadTaskContent((s) => this.patchTaskContent(s))
        )
    );
}
