import { HttpClient, HttpParams } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { BaseService } from 'app/shared/misc/base.service';
import { environment } from 'environments/environment';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import { CreateExerciseDto } from '../dto/create-exercise.dto';
import { Exercise } from '../models/exercise';
import { TaskInstance } from '../models/task-instance';
import { FileRef } from 'app/tasks/models/file-ref';
import { DownloadService } from 'app/shared/download/download.service';
import { UnitAssessmentProgress } from 'app/assessment/models/unit-assessment-progress';

@Injectable({
    providedIn: 'root',
})
export class ExerciseService extends BaseService {
    constructor(
        private http: HttpClient,
        private downloadService: DownloadService
    ) {
        super();
    }

    getById(id: number): Observable<Exercise> {
        return this.http
            .get<Exercise>(`${environment.backendUrl}/exercise/${id}`)
            .pipe(map((exercise) => new Exercise(exercise)));
    }

    getDescription(exerciseId: number, raw?: boolean): Observable<string> {
        let params = new HttpParams();
        if (raw != null) {
            params = params.set('raw', raw);
        }

        return this.http.get(
            `${environment.backendUrl}/exercise/${exerciseId}/description`,
            {
                responseType: 'text',
                params: params,
            }
        );
    }

    getTaskInstance(
        exerciseId: number,
        taskId: number
    ): Observable<TaskInstance> {
        return this.http
            .get<TaskInstance>(
                `${environment.backendUrl}/exercise/${exerciseId}/task/${taskId}`
            )
            .pipe(map((taskInstance) => new TaskInstance(taskInstance)));
    }

    getTaskContent(exerciseId: number, taskId: number): Observable<string> {
        return this.http.get(
            `${environment.backendUrl}/exercise/${exerciseId}/task/${taskId}/content`,
            { responseType: 'text' }
        );
    }

    create(dto: CreateExerciseDto): Observable<Exercise> {
        return this.http
            .post<Exercise>(`${environment.backendUrl}/exercise`, dto)
            .pipe(
                map((exercise) => {
                    return new Exercise(exercise);
                })
            );
    }

    update(
        exerciseId: number,
        dto: CreateExerciseDto,
        confirmDangerousAction?: boolean
    ): Observable<Exercise> {
        let headers = {};
        if (confirmDangerousAction) {
            headers['ConfirmDangerousAction'] = 'true';
        }

        return this.http
            .put<Exercise>(
                `${environment.backendUrl}/exercise/${exerciseId}`,
                dto,
                {
                    headers: headers,
                }
            )
            .pipe(
                map((exercise) => {
                    return new Exercise(exercise);
                })
            );
    }

    getForCourse(courseId: number, query?: string): Observable<Exercise[]> {
        let params = new HttpParams();
        params = params.set('courseId', courseId);
        if (query) {
            params = params.set('query', query);
        }

        return this.http
            .get<Exercise[]>(`${environment.backendUrl}/exercise`, {
                params: params,
            })
            .pipe(
                map((exercises) =>
                    exercises.map((exercise) => new Exercise(exercise))
                )
            );
    }

    getTaskInstanceFiles(
        exerciseId: number,
        taskId: number
    ): Observable<FileRef[]> {
        return this.http
            .get<FileRef[]>(
                `${environment.backendUrl}/exercise/${exerciseId}/task/${taskId}/files`
            )
            .pipe(map((files) => files.map((f) => new FileRef(f))));
    }

    getFileUrl(exerciseId: number, taskId: number, ref: FileRef): string {
        return `${environment.backendUrl}/exercise/${exerciseId}/task/${taskId}/files/${ref.file.uuid}/download`;
    }

    getBundleUrl(exerciseId: number, taskId: number): string {
        return `${environment.backendUrl}/exercise/${exerciseId}/task/${taskId}/bundle`;
    }

    getFile(
        exerciseId: number,
        taskId: number,
        ref: FileRef
    ): Observable<string> {
        return this.http
            .get(this.getFileUrl(exerciseId, taskId, ref), {
                responseType: 'text',
            })
            .pipe(map((data: any) => data));
    }

    getTaskInstances(exerciseId: number): Observable<TaskInstance[]> {
        return this.http
            .get<TaskInstance[]>(
                `${environment.backendUrl}/exercise/${exerciseId}/task`
            )
            .pipe(
                map((taskInstances) =>
                    taskInstances.map((ti) => new TaskInstance(ti))
                )
            );
    }

    delete(id: number): Observable<void> {
        return this.http.delete<void>(
            `${environment.backendUrl}/exercise/${id}`
        );
    }

    publishSolution(
        exerciseId: number,
        taskId: number,
        publish: boolean
    ): Observable<void> {
        return this.http.put<void>(
            `${environment.backendUrl}/exercise/${exerciseId}/task/${taskId}/public`,
            {
                pub: publish,
            }
        );
    }

    getAssessmentProgress(
        exerciseId: number,
        unitId?: number
    ): Observable<UnitAssessmentProgress[]> {
        let params = new HttpParams();
        if (unitId != null) {
            params = params.set('unitId', unitId);
        }

        return this.http
            .get<UnitAssessmentProgress[]>(
                `${environment.backendUrl}/exercise/${exerciseId}/assessment`,
                {
                    params: params,
                }
            )
            .pipe(
                map((assessmentProgresses) =>
                    assessmentProgresses.map(
                        (progress) => new UnitAssessmentProgress(progress)
                    )
                )
            );
    }
}
