import { EMPTY, Observable } from 'rxjs';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { inject, Injectable } from '@angular/core';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { SubatoError } from 'app/shared/error/error';
import { ExerciseService } from 'app/exercises/services/exercise.service';
import { Exercise } from 'app/exercises/models/exercise';
import { SubmissionService } from 'app/assessment/service/submission.service';

export interface ExerciseState {
    exercise: Exercise | null;
    error: SubatoError;
    loading: boolean;
}

export interface ExercisesState {
    exercises: Exercise[];
    error: SubatoError;
    loading: boolean;
}

export interface LoadExercisesParams {
    courseId: number;
    query: string;
}

export function loadExercise(
    updaterFn: (state: Partial<ExerciseState>) => void
) {
    let op = inject(ExerciseOperators);
    return (exerciseId: Observable<number>) =>
        op.loadExercise(exerciseId, updaterFn);
}

export function loadExercises(
    updaterFn: (state: Partial<ExercisesState>) => void
) {
    let op = inject(ExerciseOperators);
    return (params: Observable<LoadExercisesParams>) =>
        op.loadExercises(params, updaterFn);
}

@Injectable({ providedIn: 'root' })
class ExerciseOperators {
    constructor(
        private exerciseService: ExerciseService,
        private submissionService: SubmissionService,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler
    ) {}

    public loadExercise(
        exerciseId$: Observable<number>,
        updaterFn: (state: Partial<ExerciseState>) => void
    ) {
        return exerciseId$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((exerciseId) =>
                this.exerciseService.getById(exerciseId).pipe(
                    tap((exercise) =>
                        updaterFn({
                            exercise: exercise,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            exercise: null,
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }

    public loadExercises(
        params$: Observable<LoadExercisesParams>,
        updaterFn: (state: Partial<ExercisesState>) => void
    ) {
        return params$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((params) =>
                this.exerciseService
                    .getForCourse(params.courseId, params.query)
                    .pipe(
                        tap((exercises) =>
                            updaterFn({
                                exercises: exercises,
                                loading: false,
                                error: null,
                            })
                        ),
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((err) => {
                            updaterFn({
                                exercises: [],
                                error: err,
                                loading: false,
                            });
                            return EMPTY;
                        })
                    )
            )
        );
    }
}
