import { EMPTY, Observable } from 'rxjs';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { inject, Injectable } from '@angular/core';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { SubatoError } from 'app/shared/error/error';
import { ExerciseService } from 'app/exercises/services/exercise.service';
import { SubmissionFilter } from 'app/assessment/dto/submission-filter';
import { SubmissionService } from 'app/assessment/service/submission.service';
import { Submission } from 'app/assessment/models/submission';

export interface SubmissionState {
    submission: Submission | null;
    error: SubatoError;
    loading: boolean;
}

export interface SubmissionsState {
    submissions: Submission[];
    error: SubatoError;
    loading: boolean;
}

export function loadSubmissions(
    updaterFn: (state: Partial<SubmissionsState>) => void
) {
    let op = inject(SubmissionOperators);
    return (filter$: Observable<SubmissionFilter>) =>
        op.loadSubmissions(filter$, updaterFn);
}

export function loadSubmission(
    updaterFn: (state: Partial<SubmissionState>) => void
) {
    let op = inject(SubmissionOperators);
    return (submissionId$: Observable<number>) =>
        op.loadSubmission(submissionId$, updaterFn);
}

export function loadLatestSubmission(
    updaterFn: (state: Partial<SubmissionState>) => void
) {
    let op = inject(SubmissionOperators);
    return (submissionFilter: Observable<SubmissionFilter>) =>
        op.loadLatestSubmission(submissionFilter, updaterFn);
}

@Injectable({ providedIn: 'root' })
class SubmissionOperators {
    constructor(
        private exerciseService: ExerciseService,
        private submissionService: SubmissionService,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler
    ) {}

    public loadSubmissions(
        filter$: Observable<SubmissionFilter>,
        updaterFn: (state: Partial<SubmissionsState>) => void
    ) {
        return filter$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((filter) =>
                this.submissionService.getSubmissions(filter).pipe(
                    tap((submissions) =>
                        updaterFn({
                            submissions: submissions,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            submissions: [],
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }

    public loadSubmission(
        submissionId$: Observable<number>,
        updaterFn: (state: Partial<SubmissionState>) => void
    ) {
        return submissionId$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((submissionId) =>
                this.submissionService.getById(submissionId).pipe(
                    tap((submission) =>
                        updaterFn({
                            submission: submission,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            submission: null,
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }

    public loadLatestSubmission(
        filter$: Observable<SubmissionFilter>,
        updaterFn: (state: Partial<SubmissionState>) => void
    ) {
        return filter$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((submissionFilter) =>
                this.submissionService
                    .getLatestSubmission(submissionFilter)
                    .pipe(
                        tap((submission) =>
                            updaterFn({
                                submission: submission,
                                loading: false,
                                error: null,
                            })
                        ),
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((err) => {
                            updaterFn({
                                submission: null,
                                error: err,
                                loading: false,
                            });
                            return EMPTY;
                        })
                    )
            )
        );
    }
}
