import { EMPTY, Observable } from 'rxjs';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { inject, Injectable } from '@angular/core';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { SubatoError } from 'app/shared/error/error';
import { ExerciseService } from 'app/exercises/services/exercise.service';
import { SubmissionService } from 'app/assessment/service/submission.service';
import { TaskInstance } from 'app/exercises/models/task-instance';

export interface TaskContentState {
    content: string | null;
    error: SubatoError;
    loading: boolean;
}

export function loadTaskContent(
    updaterFn: (state: Partial<TaskContentState>) => void
) {
    let op = inject(TaskInstanceOperators);
    return (params: Observable<TaskInstance>) =>
        op.loadTaskContent(params, updaterFn);
}

@Injectable({ providedIn: 'root' })
class TaskInstanceOperators {
    constructor(
        private exerciseService: ExerciseService,
        private submissionService: SubmissionService,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler
    ) {}

    public loadTaskContent(
        taskInstance$: Observable<TaskInstance>,
        updaterFn: (state: Partial<TaskContentState>) => void
    ) {
        return taskInstance$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((taskInstance) =>
                this.exerciseService
                    .getTaskContent(
                        taskInstance.exercise.id,
                        taskInstance.task.id
                    )
                    .pipe(
                        tap((content) =>
                            updaterFn({
                                content: content,
                                loading: false,
                                error: null,
                            })
                        ),
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((err) => {
                            updaterFn({
                                content: null,
                                error: err,
                                loading: false,
                            });
                            return EMPTY;
                        })
                    )
            )
        );
    }
}
