import { EMPTY, Observable } from 'rxjs';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { inject, Injectable } from '@angular/core';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { SubatoError } from 'app/shared/error/error';
import { ExerciseService } from 'app/exercises/services/exercise.service';
import { SubmissionService } from 'app/assessment/service/submission.service';
import { TaskInstance } from 'app/exercises/models/task-instance';

export interface TaskInstancesState {
    taskInstances: TaskInstance[];
    error: SubatoError;
    loading: boolean;
}

export interface TaskInstanceState {
    taskInstance: TaskInstance | null;
    error: SubatoError;
    loading: boolean;
}

export interface LoadTaskInstanceParams {
    taskId: number;
    exerciseId: number;
}

export function loadTaskInstance(
    updaterFn: (state: Partial<TaskInstanceState>) => void
) {
    let op = inject(TaskInstanceOperators);
    return (params$: Observable<LoadTaskInstanceParams>) =>
        op.loadTaskInstance(params$, updaterFn);
}

export function loadTaskInstances(
    updaterFn: (state: Partial<TaskInstancesState>) => void
) {
    let op = inject(TaskInstanceOperators);
    return (exerciseId$: Observable<number>) =>
        op.loadTaskInstances(exerciseId$, updaterFn);
}

@Injectable({ providedIn: 'root' })
class TaskInstanceOperators {
    constructor(
        private exerciseService: ExerciseService,
        private submissionService: SubmissionService,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler
    ) {}

    public loadTaskInstance(
        params$: Observable<LoadTaskInstanceParams>,
        updaterFn: (state: Partial<TaskInstanceState>) => void
    ) {
        return params$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((params) =>
                this.exerciseService
                    .getTaskInstance(params.exerciseId, params.taskId)
                    .pipe(
                        tap((taskInstance) =>
                            updaterFn({
                                taskInstance: taskInstance,
                                loading: false,
                                error: null,
                            })
                        ),
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((err) => {
                            updaterFn({
                                taskInstance: null,
                                error: err,
                                loading: false,
                            });
                            return EMPTY;
                        })
                    )
            )
        );
    }

    public loadTaskInstances(
        exerciseId$: Observable<number>,
        updaterFn: (state: Partial<TaskInstancesState>) => void
    ) {
        return exerciseId$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((exerciseId) =>
                this.exerciseService.getTaskInstances(exerciseId).pipe(
                    tap((taskInstances) =>
                        updaterFn({
                            taskInstances: taskInstances,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            taskInstances: [],
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }
}
