import { Injectable } from '@angular/core';
import {
    catchError,
    map,
    switchMap,
    tap,
    withLatestFrom,
} from 'rxjs/operators';
import { EMPTY, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { Router } from '@angular/router';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { ThemeChange } from 'app/layout/store/layout.actions';
import { SubatoPrincipal } from 'app/auth/models/principal/subato-principal';
import {
    NbMediaBreakpointsService,
    NbMenuItem,
    NbThemeService,
} from '@nebular/theme';
import { AuthService } from 'app/auth/auth.service';
import { Course } from 'app/courses/models/course';
import { cloneDeep } from 'lodash';

interface HeaderState {
    userPictureOnly: boolean;
    showMenu: boolean;
}

export const initialState: HeaderState = {
    userPictureOnly: false,
    showMenu: false,
};

@Injectable()
export class HeaderStore extends BaseComponentStore<HeaderState> {
    public readonly showMenu$ = this.select((state) => state.showMenu);

    public readonly userPictureOnly$ = this.select(
        (state) => state.userPictureOnly
    );

    principal$: Observable<SubatoPrincipal> = this.store.select(
        (s) => s.auth.principal
    );
    darkMode$: Observable<boolean> = this.store.select(
        (s) => s.layout.darkMode
    );
    course$: Observable<Course | null> = this.store.select(
        (s) => s.courses.course
    );
    mainMenu$: Observable<NbMenuItem[]> = this.store
        .select((s) => s.layout.menu.mainMenu)
        .pipe(map((items) => cloneDeep(items))); // nebular may change the individual items but the store is immutable

    constructor(
        private store: Store<SubatoState>,
        private themeService: NbThemeService,
        private authService: AuthService,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService,
        private breakpointService: NbMediaBreakpointsService,
        private router: Router
    ) {
        super(initialState);
    }

    getLgBreakpoint() {
        const { lg } = this.breakpointService.getBreakpointsMap();
        return lg;
    }

    readonly toggleTheme = this.effect((trigger$: Observable<void>) =>
        trigger$.pipe(
            withLatestFrom(this.darkMode$),
            map(([, darkMode]) => new ThemeChange({ darkMode: !darkMode })),
            tap((next) => this.store.dispatch(next))
        )
    );

    readonly navigateHome = this.effect((trigger$: Observable<void>) =>
        trigger$.pipe(
            withLatestFrom(this.principal$),
            map(([_, principal]) => principal),
            tap((principal) =>
                this.router.navigate(principal ? ['/dashboard'] : ['/'])
            )
        )
    );

    readonly login = this.effect((trigger$: Observable<void>) =>
        trigger$.pipe(tap(() => this.authService.login()))
    );

    readonly logout = this.effect((trigger$: Observable<void>) =>
        trigger$.pipe(
            switchMap(() =>
                this.authService.logout().pipe(
                    tap(() => this.navigateHome()),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.notificationService.showError(
                            'Fehler beim Logout'
                        );
                        return EMPTY;
                    })
                )
            )
        )
    );

    readonly widthChange$ = this.effect(() =>
        this.themeService.onMediaQueryChange().pipe(
            map(
                ([, currentBreakpoint]) =>
                    currentBreakpoint.width < this.getLgBreakpoint()
            ),
            tap((isLessThan: boolean) =>
                this.patchState({
                    userPictureOnly: isLessThan,
                    showMenu: !isLessThan,
                })
            )
        )
    );
}
