import { Injectable } from '@angular/core';

import { Actions, createEffect, ofType } from '@ngrx/effects';
import { filter, map, tap } from 'rxjs/operators';
import {
    NbMediaBreakpointsService,
    NbMenuItem,
    NbThemeService,
} from '@nebular/theme';
import {
    ChangeCourseMenu,
    ChangeMainMenu,
    CompactMenu,
    LayoutActions,
    ThemeChange,
    UpdateSideNav,
} from './layout.actions';
import { AppActions } from 'app/store/app.actions';
import { combineLatest, Observable } from 'rxjs';
import { NavigationEnd, Router } from '@angular/router';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { SubatoPrincipal } from 'app/auth/models/principal/subato-principal';
import { Term } from 'app/modules/models/term';
import { Course } from 'app/courses/models/course';
import { CourseMenuBuilder } from '../menus/course-menu-builder';
import { MainMenuBuilder } from '../menus/main-menu-builder';
import { environment } from 'environments/environment';

@Injectable()
export class LayoutEffects {
    loadTheme$ = createEffect(() => {
        return this.actions$.pipe(
            ofType(AppActions.Init),
            map(() => {
                let darkMode = localStorage.getItem('darkMode') == 'true';
                return new ThemeChange({ darkMode: darkMode });
            })
        );
    });

    applyTheme$ = createEffect(
        () => {
            return this.actions$.pipe(
                ofType(LayoutActions.ThemeChange),
                tap((a: ThemeChange) => {
                    this.themeService.changeTheme(
                        a.payload.darkMode
                            ? environment.darkTheme
                            : environment.lightTheme
                    );
                })
            );
        },
        { dispatch: false }
    );

    persistTheme$ = createEffect(
        () => {
            return this.actions$.pipe(
                ofType(LayoutActions.ThemeChange),
                tap((a: ThemeChange) => {
                    localStorage.setItem(
                        'darkMode',
                        a.payload.darkMode ? 'true' : 'false'
                    );
                })
            );
        },
        { dispatch: false }
    );

    updateSideNav$ = createEffect(() => {
        return this.actions$.pipe(
            ofType(
                LayoutActions.ChangeCourseMenu,
                LayoutActions.ChangeMainMenu,
                LayoutActions.CompactMenu
            ),
            map(() => {
                return new UpdateSideNav();
            })
        );
    });

    principal$: Observable<SubatoPrincipal | null> = this.store.select(
        (s) => s.auth.principal
    );
    term$: Observable<Term | null> = this.store.select(
        (s) => s.modules.currentTerm
    );
    course$: Observable<Course | null> = this.store.select(
        (s) => s.courses.course
    );

    constructor(
        private actions$: Actions,
        private themeService: NbThemeService,
        private router: Router,
        private breakpointService: NbMediaBreakpointsService,
        private store: Store<SubatoState>
    ) {
        combineLatest([
            router.events,
            this.principal$,
            this.term$,
            this.course$,
        ])
            .pipe(
                filter(
                    ([routerEvents, principal, term, course]) =>
                        (<any>routerEvents).routerEvent instanceof NavigationEnd
                )
            )
            .subscribe(([val, principal, term, course]) => {
                let url = (<any>val).routerEvent.url;
                let courseMenu =
                    course != null
                        ? CourseMenuBuilder.getMenu(url, course, principal)
                        : [];

                let mainMenu: NbMenuItem[] = MainMenuBuilder.getMenu(
                    url,
                    course,
                    principal
                );
                this.store.dispatch(
                    new ChangeCourseMenu({ courseMenu: courseMenu })
                );
                this.store.dispatch(new ChangeMainMenu({ mainMenu: mainMenu }));
            });

        const { lg } = this.breakpointService.getBreakpointsMap();
        this.themeService
            .onMediaQueryChange()
            .pipe(map(([, currentBreakpoint]) => currentBreakpoint.width < lg))
            .subscribe((compact) =>
                this.store.dispatch(new CompactMenu({ compact: compact }))
            );
    }
}
