import { Injectable } from '@angular/core';
import { EMPTY } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { ifAttached } from 'app/shared/route-reuse/route-reuse-life.cycle.directive';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { SubatoError } from 'app/shared/error/error';
import { debounceWithInitialValue } from 'app/shared/misc/rxjs-operators';
import { environment } from 'environments/environment';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { CompetenceGroup } from 'app/misconceptions/models/competence-group';
import { CompetenceGroupService } from 'app/misconceptions/services/competence-group.service';

interface CompetenceGroupsState {
    competenceGroups: CompetenceGroup[];
    loading: boolean;
    error: SubatoError;
}

interface CompetencesPageState {
    competenceGroupsState: CompetenceGroupsState;
    query: string;
}

export const initialState: CompetencesPageState = {
    competenceGroupsState: { competenceGroups: [], error: null, loading: true },
    query: '',
};

@Injectable()
export class CompetencesPageStore extends BaseComponentStore<CompetencesPageState> {
    principal$ = this.store.select((s) => s.auth.principal);

    public readonly query$ = this.select((state) => state.query);
    public readonly debouncedQuery$ = this.query$.pipe(
        debounceWithInitialValue(environment.debounceTimeMs, initialState.query)
    );

    public readonly competenceGroupsState$ = this.select(
        (state) => state.competenceGroupsState
    );

    constructor(
        private store: Store<SubatoState>,
        private competenceGroupService: CompetenceGroupService,
        private globalErrorHandler: GlobalErrorHandler
    ) {
        super(initialState);
    }

    readonly patchCompetenceGroups = this.updaterFor('competenceGroupsState');

    readonly loadCompetenceGroups$ = this.effect(() =>
        this.debouncedQuery$.pipe(
            ifAttached(this),
            tap(() => this.patchCompetenceGroups({ loading: true })),
            switchMap(([term, query]) =>
                this.competenceGroupService
                    .getAll(/*{
                        termId: term?.id,
                        query: query,
                    }*/)
                    .pipe(
                        tap((competenceGroups) =>
                            this.patchCompetenceGroups({
                                competenceGroups: competenceGroups,
                                error: null,
                                loading: false,
                            })
                        ),
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((error) => {
                            this.patchCompetenceGroups({
                                error: error,
                                loading: false,
                                competenceGroups: [],
                            });
                            return EMPTY;
                        })
                    )
            )
        )
    );
}
