import { Injectable } from '@angular/core';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { EMPTY, Observable } from 'rxjs';
import { catchError, map, switchMap, tap } from 'rxjs/operators';
import { SubatoError } from 'app/shared/error/error';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { NotificationService } from 'app/shared/notifications/notification.service';
import {
    EvalResult,
    EvalService,
} from 'app/misconceptions/services/eval.service';

interface EvalState {
    result: EvalResult;
    loading: boolean;
    error: SubatoError;
}

export const initialState: EvalState = {
    result: null,
    loading: false,
    error: null,
};

@Injectable()
export class EvalStore extends BaseComponentStore<EvalState> {
    public readonly result$ = this.select((state) => state.result);
    public readonly loading$ = this.select((state) => state.loading);
    public readonly error$ = this.select((state) => state.error);

    constructor(
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler,
        private evalService: EvalService,
        private notificationService: NotificationService
    ) {
        super(initialState);
    }

    readonly execute = this.effect((code$: Observable<string>) =>
        code$.pipe(
            tap(() => this.patchState({ loading: true })),
            switchMap((code) =>
                this.evalService.eval({ code: code }).pipe(
                    map((result) => {
                        result.out = result.out?.trim();
                        return result;
                    }),
                    tap((result) =>
                        this.patchState({
                            result: result,
                            error: null,
                            loading: false,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.patchState({
                            error: error,
                            loading: false,
                            result: null,
                        });

                        this.notificationService.showError(
                            this.globalErrorHandler.format(
                                'Fehler bei der Ausführung',
                                error
                            )
                        );

                        return EMPTY;
                    })
                )
            )
        )
    );
}
