import { EMPTY, Observable } from 'rxjs';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { inject, Injectable } from '@angular/core';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { SubatoError } from 'app/shared/error/error';
import { Misconception } from 'app/misconceptions/models/misconception';
import { MisconceptionService } from 'app/misconceptions/services/misconception.service';

export interface MisconceptionState {
    misconception: Misconception | null;
    error: SubatoError;
    loading: boolean;
}

export interface MisconceptionsState {
    misconceptions: Misconception[];
    error: SubatoError;
    loading: boolean;
}

export function loadMisconception(
    updaterFn: (state: Partial<MisconceptionState>) => void
) {
    let op = inject(MisconceptionOperators);
    return (misconceptionId$: Observable<string>) =>
        op.loadMisconception(misconceptionId$, updaterFn);
}

export function loadMisconceptions(
    updaterFn: (state: Partial<MisconceptionsState>) => void
) {
    let op = inject(MisconceptionOperators);
    return (trigger$: Observable<void>) =>
        op.loadMisconceptions(trigger$, updaterFn);
}

@Injectable({ providedIn: 'root' })
class MisconceptionOperators {
    constructor(
        private misconceptionService: MisconceptionService,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler
    ) {}

    public loadMisconception(
        misconceptionId$: Observable<string>,
        updaterFn: (state: Partial<MisconceptionState>) => void
    ) {
        return misconceptionId$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((misconceptionId) =>
                this.misconceptionService.getById(misconceptionId).pipe(
                    tap((misconception) =>
                        updaterFn({
                            misconception: misconception,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            misconception: null,
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }

    public loadMisconceptions(
        trigger$: Observable<void>,
        updaterFn: (state: Partial<MisconceptionsState>) => void
    ) {
        return trigger$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((_) =>
                this.misconceptionService.getAll().pipe(
                    tap((misconceptions) =>
                        updaterFn({
                            misconceptions: misconceptions,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            misconceptions: [],
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }
}
