import { Injectable } from '@angular/core';
import { catchError, switchMap, tap, withLatestFrom } from 'rxjs/operators';
import { EMPTY, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { Router } from '@angular/router';
import { isValidationError, ValidationError } from 'app/shared/error/error';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { TermService } from 'app/modules/services/term.service';
import { CreateTermDto } from 'app/modules/dto/create-term.dto';
import {
    loadTerm,
    TermState,
} from 'app/modules/shared/operators/term.operators';

interface EditTermPageState {
    termState: TermState;
    validationError: ValidationError | null;
    saving: boolean;
}

export const initialState: EditTermPageState = {
    termState: { term: null, error: null, loading: true },
    validationError: null,
    saving: false,
};

@Injectable()
export class EditTermPageStore extends BaseComponentStore<EditTermPageState> {
    public readonly saving$ = this.select((s) => s.saving);
    public readonly termState$ = this.select((s) => s.termState);
    public readonly validationError$ = this.select(
        (state) => state.validationError
    );

    constructor(
        private store: Store<SubatoState>,
        private termService: TermService,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService,
        private router: Router
    ) {
        super(initialState);
    }

    readonly patchTerm = this.updaterFor('termState');

    readonly loadTerm = this.effect((termId$: Observable<number>) =>
        termId$.pipe(loadTerm((s) => this.patchTerm(s)))
    );

    readonly updateTerm = this.effect((params: Observable<CreateTermDto>) =>
        params.pipe(
            tap((dto) => this.patchState({ saving: true })),
            withLatestFrom(this.termState$),
            switchMap(([dto, termState]) =>
                this.termService.update(termState.term.id, dto).pipe(
                    tap((pool) => {
                        this.patchState({ saving: false });
                        this.notificationService.showSuccess(
                            'Semester erfolgreich aktualisiert'
                        );
                        this.router.navigate(['/', 'term']);
                    }),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.patchState({ saving: false });

                        if (isValidationError(error)) {
                            this.patchState({ validationError: error });
                        }

                        this.notificationService.showError(
                            this.globalErrorHandler.format(
                                'Fehler bei Aktualisierung des Semesters',
                                error
                            )
                        );

                        return EMPTY;
                    })
                )
            )
        )
    );
}
