import { Injectable } from '@angular/core';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { EMPTY, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { isValidationError, ValidationError } from 'app/shared/error/error';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { Router } from '@angular/router';
import { TermService } from 'app/modules/services/term.service';
import { CreateTermDto } from 'app/modules/dto/create-term.dto';

interface NewTermPageState {
    validationError: ValidationError | null;
    saving: boolean;
}

export const initialState: NewTermPageState = {
    validationError: null,
    saving: false,
};

@Injectable()
export class NewTermPageStore extends BaseComponentStore<NewTermPageState> {
    public readonly saving$ = this.select((s) => s.saving);
    public readonly validationError$ = this.select(
        (state) => state.validationError
    );

    constructor(
        private store: Store<SubatoState>,
        private termService: TermService,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService,
        private router: Router
    ) {
        super(initialState);
    }

    readonly createTerm = this.effect((params: Observable<CreateTermDto>) =>
        params.pipe(
            tap((dto) => this.patchState({ saving: true })),
            switchMap((dto) =>
                this.termService.create(dto).pipe(
                    tap((language) => {
                        this.patchState({ saving: false });
                        this.notificationService.showSuccess(
                            'Semester erfolgreich erstellt'
                        );
                        this.router.navigate(['/', 'term']);
                    }),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.patchState({ saving: false });

                        if (isValidationError(error)) {
                            this.patchState({ validationError: error });
                        }

                        this.notificationService.showError(
                            this.globalErrorHandler.format(
                                'Fehler bei Erstellung des Semesters',
                                error
                            )
                        );

                        return EMPTY;
                    })
                )
            )
        )
    );
}
