import { Injectable } from '@angular/core';
import { EMPTY, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { SubatoPrincipal } from 'app/auth/models/principal/subato-principal';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import {
    loadTerms,
    TermsState,
} from 'app/modules/shared/operators/term.operators';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { Term } from 'app/modules/models/term';
import { TermService } from 'app/modules/services/term.service';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { NotificationService } from 'app/shared/notifications/notification.service';

interface TermListState {
    termsState: TermsState;
    finishingTerm: boolean;
}

export const initialState: TermListState = {
    termsState: { terms: [], error: null, loading: true },
    finishingTerm: false,
};

@Injectable()
export class TermListStore extends BaseComponentStore<TermListState> {
    principal$: Observable<SubatoPrincipal> = this.store.select(
        (s) => s.auth.principal
    );

    public readonly termsState$ = this.select((state) => state.termsState);
    public readonly finishingTerm$ = this.select(
        (state) => state.finishingTerm
    );
    public readonly terms$ = this.select((state) => state.termsState.terms);

    isSuperAdmin$ = this.select(this.principal$, (principal) =>
        principal.hasSuperAdminRole()
    );

    constructor(
        private store: Store<SubatoState>,
        private termService: TermService,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService
    ) {
        super(initialState);
    }

    readonly patchTerms = this.updaterFor('termsState');

    readonly loadTerms = this.effect((trigger$: Observable<void>) =>
        trigger$.pipe(loadTerms((s) => this.patchTerms(s)))
    );

    readonly finishTerm = this.effect((term$: Observable<Term>) =>
        term$.pipe(
            tap(() => this.patchState({ finishingTerm: true })),
            switchMap((term) =>
                this.termService.finishTerm(term.id).pipe(
                    tap(() => this.patchState({ finishingTerm: false })),
                    tap(() => {
                        this.notificationService.showSuccess(
                            'Abschluss erfolgreich'
                        );
                    }),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.patchState({ finishingTerm: false });

                        this.notificationService.showError(
                            this.globalErrorHandler.format(
                                'Fehler bei Abschluss',
                                error
                            )
                        );

                        return EMPTY;
                    })
                )
            )
        )
    );
}
