import { EMPTY, Observable } from 'rxjs';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { inject, Injectable } from '@angular/core';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { SubatoError } from 'app/shared/error/error';
import { Term } from 'app/modules/models/term';
import { TermService } from 'app/modules/services/term.service';

export interface TermsState {
    terms: Term[];
    error: SubatoError;
    loading: boolean;
}

export interface TermState {
    term: Term | null;
    error: SubatoError;
    loading: boolean;
}

export function loadTerms(updaterFn: (state: Partial<TermsState>) => void) {
    let op = inject(TermOperators);
    return (trigger$: Observable<void>) => op.loadTerms(trigger$, updaterFn);
}

export function loadTerm(updaterFn: (state: Partial<TermState>) => void) {
    let op = inject(TermOperators);
    return (termId$: Observable<number>) => op.loadTerm(termId$, updaterFn);
}

@Injectable({ providedIn: 'root' })
class TermOperators {
    constructor(
        private termService: TermService,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler
    ) {}

    public loadTerms(
        trigger$: Observable<void>,
        updaterFn: (state: Partial<TermsState>) => void
    ) {
        return trigger$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((languageId) =>
                this.termService.getAll().pipe(
                    tap((terms) =>
                        updaterFn({
                            terms: terms,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            terms: [],
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }

    public loadTerm(
        termId$: Observable<number>,
        updaterFn: (state: Partial<TermState>) => void
    ) {
        return termId$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((termId) =>
                this.termService.getById(termId).pipe(
                    tap((term) =>
                        updaterFn({
                            term: term,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            term: null,
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }
}
