import { Injectable } from '@angular/core';
import { ComponentStore } from '@ngrx/component-store';
import { EMPTY, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { Term } from 'app/modules/models/term';
import {
    isValidationError,
    SubatoError,
    ValidationError,
} from 'app/shared/error/error';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { SubatoPrincipal } from 'app/auth/models/principal/subato-principal';
import { catchError, switchMap, tap, withLatestFrom } from 'rxjs/operators';
import { UpdateProfileDto } from 'app/shared/user/update-profile.dto';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { UserService } from 'app/shared/user/user.service';
import { LoadPrincipal } from 'app/auth/store/auth.actions';

interface ProfileState {
    loading: boolean;
    saving: boolean;
    error: SubatoError;
    validationError: ValidationError | null;
}

export const initialState: ProfileState = {
    loading: true,
    saving: false,
    error: null,
    validationError: null,
};

@Injectable()
export class ProfileStore extends ComponentStore<ProfileState> {
    public readonly loading$ = this.select((state) => state.loading);
    public readonly saving$ = this.select((state) => state.saving);
    public readonly error$ = this.select((state) => state.error);

    term$: Observable<Term | null> = this.store.select(
        (s) => s.modules.currentTerm
    );

    principal$: Observable<SubatoPrincipal> = this.store.select(
        (s) => s.auth.principal
    );

    constructor(
        private store: Store<SubatoState>,
        private userService: UserService,
        private notificationService: NotificationService,
        private globalErrorHandler: GlobalErrorHandler
    ) {
        super(initialState);
    }

    readonly updateLoading$ = this.effect(() =>
        this.principal$.pipe(
            tap((principal) => this.patchState({ loading: principal == null }))
        )
    );

    readonly updateProfile = this.effect(
        (params: Observable<UpdateProfileDto>) =>
            params.pipe(
                withLatestFrom(this.principal$),
                tap(() => this.patchState({ saving: true })),
                switchMap(([dto, principal]) =>
                    this.userService.updateProfile(principal.id, dto).pipe(
                        tap(() => {
                            this.notificationService.showSuccess(
                                'Profil erfolgreich aktualisiert'
                            );
                            this.patchState({
                                loading: true,
                                saving: false,
                                validationError: null,
                            });
                            this.store.dispatch(new LoadPrincipal());
                        }),
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((error) => {
                            this.patchState({
                                saving: false,
                                loading: false,
                            });

                            if (isValidationError(error)) {
                                this.patchState({
                                    validationError: error,
                                });
                            }

                            this.notificationService.showError(
                                this.globalErrorHandler.format(
                                    'Fehler bei Aktualisierung des Profils',
                                    error
                                )
                            );

                            return EMPTY;
                        })
                    )
                )
            )
    );
}
