import { Injectable } from '@angular/core';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { EMPTY, Observable, of } from 'rxjs';
import {
    catchError,
    debounceTime,
    filter,
    switchMap,
    tap,
    withLatestFrom,
} from 'rxjs/operators';
import { SearchResult } from 'app/search/models/search-result';
import { SubatoError } from 'app/shared/error/error';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { SearchService } from 'app/search/services/search.service';
import { NotificationService } from 'app/shared/notifications/notification.service';
import * as _ from 'lodash';
import { Item } from 'app/search/models/item';
import { environment } from 'environments/environment';

class SearchItemGroup {
    group: string;
    items: Item[];
}

interface SearchState {
    courseId: number;
    query: string;
    result: SearchResult;
    searching: boolean;
    error: SubatoError;
}

export const initialState: SearchState = {
    courseId: null,
    query: '',
    result: null,
    searching: false,
    error: null,
};

@Injectable()
export class SearchStore extends BaseComponentStore<SearchState> {
    principal$ = this.store.select((s) => s.auth.principal);

    public readonly courseId$ = this.select((state) => state.courseId);
    public readonly result$ = this.select((state) => state.result);
    public readonly searching$ = this.select((state) => state.searching);
    public readonly error$ = this.select((state) => state.error);
    public readonly query$ = this.select((state) => state.query);

    public readonly itemGroups$: Observable<SearchItemGroup[]> = this.select(
        this.result$,
        (result) => {
            let items = result?.items ?? [];
            let grouped = _.groupBy(items, (item) => item.group);
            return Object.keys(grouped).map((group) => ({
                group: group,
                items: grouped[group],
            }));
        }
    );

    constructor(
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler,
        private searchService: SearchService,
        private notificationService: NotificationService
    ) {
        super(initialState);
    }

    readonly search$ = this.effect(() =>
        this.query$.pipe(debounceTime(environment.debounceTimeMs)).pipe(
            withLatestFrom(this.courseId$),
            filter(([, courseId]) => courseId != null),
            tap(() => this.patchState({ searching: true })),
            switchMap(([query, courseId]) =>
                this.search(query, courseId).pipe(
                    tap((result) =>
                        this.patchState({
                            result: result,
                            error: null,
                            searching: false,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.patchState({
                            error: error,
                            searching: false,
                            result: null,
                        });

                        this.notificationService.showError(
                            this.globalErrorHandler.format(
                                'Fehler bei der Suche',
                                error
                            )
                        );

                        return EMPTY;
                    })
                )
            )
        )
    );

    private search(query: string, courseId: number) {
        if (query) {
            return this.searchService
                .search(query, courseId)
                .pipe(
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    )
                );
        } else {
            return of(new SearchResult({ query: query, items: [] }));
        }
    }
}
