import { RxStomp, RxStompConfig } from '@stomp/rx-stomp';
import { Injectable, OnDestroy } from '@angular/core';
import { KeycloakEventType, KeycloakService } from 'keycloak-angular';
import { from, Subject } from 'rxjs';
import { filter, switchMap, take, takeUntil } from 'rxjs/operators';
import { ErrorResponse } from 'app/shared/error/backend-response';
import { IFrame } from '@stomp/stompjs';
import { environment } from 'environments/environment';

@Injectable({
    providedIn: 'root',
})
@Injectable()
export class StompService extends RxStomp implements OnDestroy {
    protected destroy$ = new Subject<void>();

    constructor(private keycloakService: KeycloakService) {
        super();
        this.init();
    }

    ngOnDestroy(): void {
        this.destroy$.next();
        this.destroy$.complete();
    }

    init() {
        from(this.keycloakService.getToken())
            .pipe(filter((token) => token != null))
            .subscribe((token) => {
                this.connect(token);
            });

        this.keycloakService.keycloakEvents$
            .pipe(takeUntil(this.destroy$))
            .subscribe((evt) => {
                if (evt.type == KeycloakEventType.OnAuthSuccess) {
                    from(this.keycloakService.getToken()).subscribe((token) => {
                        this.connect(token);
                    });
                }
            });

        this.destroy$
            .pipe(switchMap(() => from(this.deactivate())))
            .subscribe(() => {});
    }

    private connect(token: string) {
        from(this.deactivate())
            .pipe(take(1))
            .subscribe(() => {
                this.configure(this.createConfig(token));
                this.activate();
            });
    }

    private onError(error: IFrame): string {
        let errorResponse: ErrorResponse = JSON.parse(error.body);

        if (errorResponse != null) {
            // for redirecting the error object (not yet implemented)
            // let subatoError = createSubatoError(errorResponse);

            console.error(
                '[Subato API - STOMP]',
                errorResponse.statusCode,
                errorResponse.reasonPhrase,
                errorResponse.data
            );
        }

        return null;
    }

    private createConfig(token: string): RxStompConfig {
        return {
            // Which server?
            brokerURL: environment.wsUrl,

            // Will log diagnostics on console
            // It can be quite verbose, not recommended in production
            // Skip this key to stop logging to console
            debug: (msg: string): void => {
                if (!environment.production) {
                    console.log('STOMP', msg);
                }
            },

            connectHeaders: {
                token: token,
            },
            correlateErrors: (error) => this.onError(error),
        };
    }
}
