import {DOCUMENT} from '@angular/common';
import {Inject, Injectable} from '@angular/core';
import {
    NbGlobalPhysicalPosition,
    NbToastrConfig,
    NbToastRef,
    NbToastrService,
} from '@nebular/theme';
import {BehaviorSubject, fromEvent, merge, Observable, Subject} from 'rxjs';
import {delay, filter} from 'rxjs/operators';

/**
 * This service represents a state machine for controlling the display of toast notifications.
 *
 * Notifications are in state 'pending' if they are shown.
 * They are shown on the screen until the user performs any action, then they are in state 'scheduled'.
 * After a certain time interval, scheduled notifications are closed
 */
@Injectable({
    providedIn: 'root',
})
export class NotificationService {
    private pendingRefs: NbToastRef[] = [];
    private _refScheduledForClose: Subject<NbToastRef>;
    private refScheduledForClose$: Observable<NbToastRef>;

    private options: Partial<NbToastrConfig> = {
        hasIcon: false,
        icon: null,
        duration: 0, // unlimited
        position: NbGlobalPhysicalPosition.TOP_LEFT,
    };

    SHOW_DURATION = 6000; // how long are notifications shown if in state scheduled

    constructor(
        private toastrService: NbToastrService,
        @Inject(DOCUMENT) private document: Document
    ) {
        this._refScheduledForClose = new BehaviorSubject<NbToastRef>(null);
        this.refScheduledForClose$ = this._refScheduledForClose.asObservable();

        this.refScheduledForClose$
            .pipe(
                filter((r) => r != null),
                //tap(r => console.log("scheduled")),
                delay(this.SHOW_DURATION)
            )
            .subscribe((r) => {
                //console.log("close");
                r.close();
            });

        /**
         * Note: The scroll event is triggered whenever the element's scroll position changes,
         * whether by user interaction or programmatically. On the other hand,
         * the wheel event is only triggered when the user scrolls using a mouse or touchpad.
         */
        let userAction$ = merge(
            fromEvent(document, 'wheel'),
            fromEvent(document, 'mousemove'),
            fromEvent(document, 'keydown')
        );
        userAction$.subscribe(() => this.schedulePendingRefs());
    }

    private schedulePendingRefs() {
        this.pendingRefs.forEach((r) => this._refScheduledForClose.next(r));
        this.pendingRefs = [];
    }

    showSuccess(message: string) {
        let ref = this.toastrService.success(null, message, this.options);
        this.pendingRefs.push(ref);
    }

    showError(message: string) {
        let ref = this.toastrService.danger(null, message, this.options);
        this.pendingRefs.push(ref);
    }

    showWarning(message: string) {
        let ref = this.toastrService.warning(null, message, this.options);
        this.pendingRefs.push(ref);
    }
}
