import { ComponentStore } from '@ngrx/component-store';
import { filter, map, tap } from 'rxjs/operators';
import { SubatoError } from 'app/shared/error/error';
import { Page } from 'app/shared/paging/page';
import { DataQueryParams } from 'app/shared/paging/data-query-params';

export interface PagingState<T> {
    page: Page<T>;
    items: T[];
    error: SubatoError;
    loading: boolean;
    projection: DataQueryParams;
}

export function createInitialState<T>(): PagingState<T> {
    return {
        page: null,
        items: [],
        error: null,
        loading: true,
        projection: null,
    };
}

export class PagingStore<T> extends ComponentStore<PagingState<T>> {
    public readonly page$ = this.select((state) => state.page);
    public readonly items$ = this.select((state) => state.items);
    public readonly projection$ = this.select((state) => state.projection);
    public readonly loading$ = this.select((state) => state.loading);
    public readonly error$ = this.select((state) => state.error);

    hasNext$ = this.page$.pipe(map((page) => page != null && page.hasNext()));

    public readonly vm$ = this.select(
        this.loading$,
        this.error$,
        this.page$,
        this.items$,
        this.hasNext$,
        (loading, error, page, items, hasNext) => ({
            loading: loading,
            error: error,
            page: page,
            items: items,
            hasNext: hasNext,
        })
    );

    public constructor(initialState: PagingState<T>) {
        super(initialState);
    }

    readonly nextPage = this.updater((state) => ({
        ...state,
        projection: DataQueryParams.next(state.page, state.projection),
    }));

    readonly patchPage = this.updater((state, page: Page<T>) => ({
        ...state,
        page: page,
        error: null,
        loading: false,
    }));

    readonly patchError = this.updater((state, error: SubatoError) => ({
        ...state,
        page: null,
        error: error,
        loading: false,
    }));

    readonly appendItems$ = this.effect(() =>
        this.page$.pipe(
            filter((page) => page != null),
            tap((page) =>
                this.patchState((state) => ({
                    items: [...state.items, ...page.content],
                }))
            )
        )
    );

    readonly resetItems$ = this.effect(() =>
        this.error$.pipe(
            filter((error) => error != null),
            tap(() => this.patchState({ items: [] }))
        )
    );
}
