import { Injectable } from '@angular/core';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { EMPTY } from 'rxjs';
import {
    catchError,
    debounceTime,
    filter,
    switchMap,
    tap,
} from 'rxjs/operators';
import { SubatoError } from 'app/shared/error/error';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { User } from 'app/shared/user/user';
import { UserService } from 'app/shared/user/user.service';
import { environment } from 'environments/environment';

interface UserPickerState {
    query: string;
    users: User[];
    searching: boolean;
    error: SubatoError;
}

export const initialState: UserPickerState = {
    query: '',
    users: [],
    searching: false,
    error: null,
};

@Injectable()
export class UserPickerStore extends BaseComponentStore<UserPickerState> {
    principal$ = this.store.select((s) => s.auth.principal);

    public readonly users$ = this.select((state) => state.users);
    public readonly searching$ = this.select((state) => state.searching);
    public readonly error$ = this.select((state) => state.error);
    public readonly query$ = this.select((state) => state.query);

    constructor(
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler,
        private userService: UserService,
        private notificationService: NotificationService
    ) {
        super(initialState);
    }

    readonly search$ = this.effect(() =>
        this.query$.pipe(debounceTime(environment.debounceTimeMs)).pipe(
            filter((query) => query != ''),
            tap(() => this.patchState({ searching: true })),
            switchMap((query) =>
                this.userService.findAll({ q: query }).pipe(
                    tap((page) =>
                        this.patchState({
                            users: page.content,
                            error: null,
                            searching: false,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.patchState({
                            error: error,
                            searching: false,
                            users: [],
                        });

                        this.notificationService.showError(
                            this.globalErrorHandler.format(
                                'Fehler bei der Benutzersuche',
                                error
                            )
                        );

                        return EMPTY;
                    })
                )
            )
        )
    );
}
