import { ChangeDetectorRef, Component, OnInit } from '@angular/core';
import { ExerciseAcceptancePageStore } from 'app/solutions/pages/exercise-acceptance/exercise-acceptance.page.store';
import { BaseComponent } from 'app/shared/misc/base.component';
import { ErrorContainerConfig } from 'app/shared/error/error-container-config';
import {
    catchError,
    filter,
    map,
    switchMap,
    take,
    takeUntil,
} from 'rxjs/operators';
import { ParamHelper } from 'app/shared/misc/helper/param.helper';
import { EMPTY, of } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { ActivatedRoute, Router } from '@angular/router';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { TaskInstance } from 'app/exercises/models/task-instance';
import { NbDialogService } from '@nebular/theme';
import { RestartDialogComponent } from 'app/solutions/pages/exercise-acceptance/components/restart-dialog/restart-dialog.component';
import { ifAttached } from 'app/shared/route-reuse/route-reuse-life.cycle.directive';
import { Exercise } from 'app/exercises/models/exercise';
import { PageHeaderAction } from 'app/shared/page-header/page-header/page-header.component';

@Component({
    templateUrl: './exercise-acceptance.page.component.html',
    styleUrls: ['./exercise-acceptance.page.component.scss'],
    providers: [ExerciseAcceptancePageStore],
})
export class ExerciseAcceptancePageComponent
    extends BaseComponent
    implements OnInit
{
    loading$ = this.exerciseAcceptancePageStore.loading$;
    error$ = this.exerciseAcceptancePageStore.error$;
    exerciseState$ = this.exerciseAcceptancePageStore.exerciseState$;
    canRun$ = this.exerciseAcceptancePageStore.canRun$;
    canCancel$ = this.exerciseAcceptancePageStore.canCancel$;
    canReset$ = this.exerciseAcceptancePageStore.canReset$;
    resetting$ = this.exerciseAcceptancePageStore.resetting$;
    bulkEvalState$ = this.exerciseAcceptancePageStore.bulkEvalState$;

    errorConfig: ErrorContainerConfig = {
        notFound: {
            message:
                'Diese Aufgabe konnten wir in diesem Übungsblatt leider nicht finden.',
        },
    };

    actions$ = this.exerciseState$.pipe(
        takeUntil(this.destroy$),
        ifAttached(this),
        map((exerciseState) => this.buildActions(exerciseState.exercise))
    );

    constructor(
        private exerciseAcceptancePageStore: ExerciseAcceptancePageStore,
        private store: Store<SubatoState>,
        private dialogService: NbDialogService,
        private router: Router,
        private globalErrorHandler: GlobalErrorHandler,
        private route: ActivatedRoute,
        private cd: ChangeDetectorRef
    ) {
        super();
    }

    ngOnInit(): void {
        this.route.paramMap
            .pipe(
                takeUntil(this.destroy$),
                map((params) =>
                    ParamHelper.parseIdOrThrow(params, 'exercise_id')
                ),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.exerciseAcceptancePageStore.patchExercise({
                        error: err,
                    });
                    return EMPTY;
                })
            )
            .subscribe((exerciseId) => {
                this.exerciseAcceptancePageStore.loadExercise(exerciseId);
            });
    }

    getTaskInstanceLink(taskInstance: TaskInstance) {
        return [
            '/',
            'exercises',
            taskInstance.exercise.id,
            'tasks',
            taskInstance.task.id,
        ];
    }

    onTasksLoaded(taskInstances: TaskInstance[]) {
        this.exerciseAcceptancePageStore.patchTaskInstances({
            taskInstances: taskInstances,
        });

        this.cd.detectChanges();
    }

    run() {
        this.bulkEvalState$
            .pipe(
                take(1),
                switchMap((state) =>
                    state != null
                        ? this.dialogService
                              .open(RestartDialogComponent, {})
                              .onClose.pipe(
                                  filter(
                                      (result) =>
                                          result && result.reset != undefined
                                  ),
                                  map((result) => <boolean>result.reset)
                              )
                        : of(true)
                )
            )
            .subscribe((reset) =>
                this.exerciseAcceptancePageStore.run({ reset: reset })
            );
    }

    cancel() {
        this.exerciseAcceptancePageStore.cancel();
    }

    resetState() {
        this.exerciseAcceptancePageStore.reset();
    }

    buildActions(exercise: Exercise): PageHeaderAction[] {
        let actions: PageHeaderAction[] = [];
        if (exercise != null) {
            actions.push({
                name: 'Bewerten',
                icon: 'checkmark-square-outline',
                link: {
                    commands: ['/', 'exercises', exercise.id, 'assessment'],
                },
                status: 'primary',
            });
        }

        return actions;
    }
}
