import { Injectable } from '@angular/core';
import {
    catchError,
    filter,
    map,
    switchMap,
    tap,
    withLatestFrom,
} from 'rxjs/operators';
import { EMPTY, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { dispatchCourseChange } from 'app/courses/shared/operators/course.operators';
import {
    ExerciseState,
    loadExercise,
} from 'app/exercises/shared/operators/exercise.operators';
import { TaskInstancesState } from 'app/exercises/shared/operators/task-instance.operators';
import { AcceptanceService } from 'app/solutions/services/acceptance.service';
import { BulkEvalState } from 'app/solutions/models/bulk-eval-state';
import { RunExerciseAcceptanceDto } from 'app/solutions/dto/run-exercise-acceptance.dto';
import { SubatoError } from 'app/shared/error/error';

export interface InternalBulkEvalState {
    loading: boolean;
    error: SubatoError;
    state: BulkEvalState;
}

export interface ExerciseAcceptancePageState {
    exerciseState: ExerciseState;
    taskInstancesState: TaskInstancesState;
    scheduled: boolean;
    cancelled: boolean;
    bulkEvalState: InternalBulkEvalState;
    resetting: boolean;
}

export const initialState: ExerciseAcceptancePageState = {
    exerciseState: { exercise: null, error: null, loading: true },
    taskInstancesState: { taskInstances: [], error: null, loading: false },
    scheduled: false,
    cancelled: false,
    bulkEvalState: { state: null, error: null, loading: true },
    resetting: false,
};

@Injectable()
export class ExerciseAcceptancePageStore extends BaseComponentStore<ExerciseAcceptancePageState> {
    public readonly exerciseState$ = this.select((s) => s.exerciseState);
    public readonly exercise$ = this.select((s) => s.exerciseState.exercise);
    public readonly scheduled$ = this.select((s) => s.scheduled);
    public readonly cancelled$ = this.select((s) => s.cancelled);
    public readonly resetting$ = this.select((s) => s.resetting);
    public readonly bulkEvalState$ = this.select((s) => s.bulkEvalState.state);

    public readonly canRun$ = this.select(
        this.bulkEvalState$,
        this.scheduled$,
        (state, scheduled) =>
            state != null && state.total != 0 && !scheduled && !state.running
    );

    public readonly canCancel$ = this.select(
        this.bulkEvalState$,
        this.scheduled$,
        this.cancelled$,
        (state, scheduled, cancelled) =>
            state != null && !scheduled && state.running && !cancelled
    );

    public readonly canReset$ = this.select(
        this.bulkEvalState$,
        this.scheduled$,
        this.cancelled$,
        this.resetting$,
        (state, scheduled, cancelled, resetting) =>
            state != null &&
            state.detached &&
            !scheduled &&
            !cancelled &&
            !resetting
    );

    public readonly loading$ = this.select(
        this.select((s) => s.bulkEvalState.loading),
        this.select((s) => s.exerciseState.loading),
        (l1, l2) => l1 || l2
    );

    public readonly error$ = this.select(
        this.select((s) => s.bulkEvalState.error),
        this.select((s) => s.exerciseState.error),
        (e1, e2) => e1 ?? e2
    );

    constructor(
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler,
        private acceptanceService: AcceptanceService,
        private notificationService: NotificationService
    ) {
        super(initialState);
    }

    readonly patchExercise = this.updaterFor('exerciseState');
    readonly patchBulkEvalState = this.updaterFor('bulkEvalState');
    readonly patchTaskInstances = this.updaterFor('taskInstancesState');

    readonly loadExercise = this.effect((exerciseId$: Observable<number>) =>
        exerciseId$.pipe(loadExercise((s) => this.patchExercise(s)))
    );

    readonly run = this.effect(
        (params$: Observable<RunExerciseAcceptanceDto>) =>
            params$.pipe(
                withLatestFrom(this.exercise$),
                tap(() => this.patchState({ scheduled: true })),
                switchMap(([params, exercise]) =>
                    this.acceptanceService.run(exercise.id, params).pipe(
                        tap(() =>
                            this.patchState({
                                scheduled: false,
                            })
                        ),
                        tap((state) =>
                            this.patchBulkEvalState({
                                state: state,
                            })
                        ),
                        tap(() => {
                            this.notificationService.showSuccess(
                                'Ausführung gestartet.'
                            );
                        }),
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((error) => {
                            this.patchState({ scheduled: false });

                            this.notificationService.showError(
                                this.globalErrorHandler.format(
                                    'Fehler bei Ausführung der Abnahmeauswertung',
                                    error
                                )
                            );

                            return EMPTY;
                        })
                    )
                )
            )
    );

    readonly cancel = this.effect((trigger$: Observable<void>) =>
        trigger$.pipe(
            withLatestFrom(this.exercise$),
            tap(() => this.patchState({ cancelled: true })),
            switchMap(([params, exercise]) =>
                this.acceptanceService.cancel(exercise.id).pipe(
                    tap(() =>
                        this.patchState({
                            cancelled: false,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.patchState({ cancelled: false });

                        this.notificationService.showError(
                            this.globalErrorHandler.format(
                                'Fehler bei Abbruch der Abnahmeauswertung',
                                error
                            )
                        );

                        return EMPTY;
                    })
                )
            )
        )
    );

    readonly reset = this.effect((trigger$: Observable<void>) =>
        trigger$.pipe(
            withLatestFrom(this.exercise$),
            tap(() => this.patchState({ resetting: true })),
            switchMap(([, exercise]) =>
                this.acceptanceService.reset(exercise.id).pipe(
                    tap(() =>
                        this.patchState({
                            resetting: false,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.patchState({ resetting: false });

                        this.notificationService.showError(
                            this.globalErrorHandler.format(
                                'Fehler beim Zurücksetzen der Abnahmeauswertung',
                                error
                            )
                        );

                        return EMPTY;
                    })
                )
            )
        )
    );

    readonly dispatchCourseChange$ = this.effect(() =>
        this.exercise$.pipe(
            filter((exercise) => exercise != null),
            map((exercise) => exercise?.course),
            dispatchCourseChange
        )
    );

    readonly loadInitialState$ = this.effect(() =>
        this.exercise$.pipe(
            filter((exercise) => exercise != null),
            switchMap((exercise) =>
                this.acceptanceService.getState(exercise.id).pipe(
                    tap((state) =>
                        this.patchBulkEvalState({
                            state: state,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        this.patchBulkEvalState({
                            error: err,
                            loading: false,
                            state: null,
                        });
                        return EMPTY;
                    })
                )
            )
        )
    );

    readonly onStateUpdate$ = this.effect(() =>
        this.exercise$.pipe(
            filter((exercise) => exercise != null),
            switchMap((exercise) =>
                this.acceptanceService
                    .onStateChange(exercise.id)
                    .pipe(
                        tap((state) =>
                            this.patchBulkEvalState({ state: state })
                        )
                    )
            )
        )
    );
}
