import {
    AfterViewInit,
    Component,
    ElementRef,
    Input,
    OnChanges,
    OnInit,
    QueryList,
    SimpleChanges,
    ViewChildren,
} from '@angular/core';
import { Solution } from '../../../../models/solution';
import party from 'party-js';
import { empty, of } from 'rxjs';
import { delay, expand, takeUntil } from 'rxjs/operators';
import { BaseComponent } from 'app/shared/misc/base.component';
import { TaskInstance } from 'app/exercises/models/task-instance';

interface ProposedAction {
    enabled: boolean;
    status: string;
}

interface SolutionFeedback {
    status: string;
    message: string;
    submessage?: string;
    emoji?: string;
    actions: {
        next?: ProposedAction;
        retry?: ProposedAction;
    };
}

@Component({
    selector: 'sb-solution-feedback',
    templateUrl: './feedback.component.html',
    styleUrls: ['./feedback.component.scss'],
})
export class FeedbackComponent
    extends BaseComponent
    implements OnInit, AfterViewInit, OnChanges
{
    @Input() solution: Solution;
    @Input() justSubmitted: boolean;
    @Input() nextTaskInstance: TaskInstance | null = null;
    feedback: SolutionFeedback;

    @ViewChildren('alertContainer', { read: ElementRef })
    alertContainer: QueryList<ElementRef>;

    constructor() {
        super();
    }

    ngOnChanges(changes: SimpleChanges) {
        var shouldRefresh = changes.hasOwnProperty('solution');

        if (shouldRefresh) {
            this.updateFeedback();
        }
    }

    ngOnInit(): void {
        this.updateFeedback();
    }

    updateFeedback() {
        if (!this.solution.hasResult) {
            this.feedback = this.neutralFeedback(this.solution);
        }

        if (this.solution.evalFailed) {
            this.feedback = this.evalFailedFeedback(this.solution);
        }

        if (this.solution.isGood) {
            this.feedback = this.goodFeedback(this.solution);
        }

        if (this.solution.isOk) {
            this.feedback = this.okFeedback(this.solution);
        }

        if (this.solution.isBad) {
            if (this.solution.evalState.result.compilation != null) {
                this.feedback = this.solution.evalState.result.compilation
                    .compiling
                    ? this.badFeedbackCompiling(this.solution)
                    : this.badFeedbackNotCompiling(this.solution);
            } else {
                this.feedback = this.badFeedback(this.solution);
            }
        }
    }

    ngAfterViewInit(): void {
        if (this.justSubmitted && this.solution.isGood) {
            setTimeout(() => {
                this.alertContainer.forEach((e) => {
                    of(null)
                        .pipe(
                            expand((prev, now) =>
                                now > 20 ? empty() : of(null).pipe(delay(1000))
                            ),
                            takeUntil(this.destroy$)
                        )
                        .subscribe(() => this.shootConfetti(e.nativeElement));
                });
            }, 300);
        }
    }

    emojiClicked(event) {
        if (this.solution.isGood) {
            this.shootConfetti(event);
        }
    }

    shootConfetti(event) {
        party.confetti(event, {
            count: party.variation.range(20, 40),
            spread: party.variation.range(100, 140),
        });
    }

    evalFailedFeedback(solution: Solution): SolutionFeedback {
        return {
            status: 'warning',
            message:
                'Die Tests konnten nicht ausgeführt werden. Deine Lösung wurde jedoch trotzdem gespeichert.',
            actions: {
                retry: {
                    enabled: true,
                    status: 'basic',
                },
                next: {
                    enabled: true,
                    status: 'primary',
                },
            },
        };
    }

    neutralFeedback(solution: Solution): SolutionFeedback {
        return {
            status: 'primary',
            message:
                'Wir haben deine Lösung erhalten und schauen es uns manuell an.',
            actions: {
                retry: {
                    enabled: true,
                    status: 'basic',
                },
                next: {
                    enabled: true,
                    status: 'primary',
                },
            },
        };
    }

    goodFeedback(solution: Solution): SolutionFeedback {
        return {
            status: 'success',
            message:
                'Deine Lösung kompiliert und hat alle Tests erfolgreich bestanden.',
            submessage: 'Sehr gut, weiter so!',
            emoji: '🎉',
            actions: {
                retry: {
                    enabled: true,
                    status: 'basic',
                },
                next: {
                    enabled: true,
                    status: 'success',
                },
            },
        };
    }

    okFeedback(solution: Solution): SolutionFeedback {
        return {
            status: 'warning',
            message: `Deine Lösung kompiliert und hat ${solution.evalState.result.test.passed} von ${solution.evalState.result.test.testcases} Tests bestanden.`,
            submessage:
                'Das ist ein guter Anfang. Du solltest dir deine Lösung und die Testergebnisse unten aber nochmal ansehen und versuchen, die Probleme zu beheben.',
            emoji: '🙂',
            actions: {
                retry: {
                    enabled: true,
                    status: 'success',
                },
                next: {
                    enabled: true,
                    status: 'warning',
                },
            },
        };
    }

    badFeedbackNotCompiling(solution: Solution): SolutionFeedback {
        return {
            status: 'danger',
            message: `Deine Lösung kompiliert leider nicht und es wurden keine Tests ausgeführt.`,
            submessage:
                'Aller Anfang ist schwer, wir wissen das. Du solltest dir <strong>unbedingt</strong> noch einmal ansehen, warum deine Lösung nicht kompiliert. Oft ist es nur eine Kleinigkeit, die daran Schuld ist. Das kann zum Beispiel eine vergessene Klammer oder ein fehlendes Semikolon sein.',
            emoji: '🤨',
            actions: {
                retry: {
                    enabled: true,
                    status: 'success',
                },
            },
        };
    }

    badFeedbackCompiling(solution: Solution): SolutionFeedback {
        let message =
            solution.evalState.result.test.passed == 0
                ? 'Deine Lösung kompiliert. Leider hat sie aber keine Tests bestanden.'
                : `Deine Lösung kompiliert. Leider hat sie aber nur ${solution.evalState.result.test.passed} von ${solution.evalState.result.test.testcases} Tests bestanden.`;

        return {
            status: 'danger',
            message: message,
            submessage: `Die Aufgaben sind vielleicht nicht ganz einfach zu lösen, du solltest dir deine Lösung und die Testergebnisse unten aber <strong>unbedingt</strong> nochmal ansehen und versuchen, die Probleme zu beheben. Beim nächsten Mal klappt's bestimmt.`,
            emoji: '🤨',
            actions: {
                retry: {
                    enabled: true,
                    status: 'success',
                },
            },
        };
    }

    badFeedback(solution: Solution): SolutionFeedback {
        let message =
            solution.evalState.result.test.passed == 0
                ? 'Leider hat deine Lösung keine Tests bestanden.'
                : `Leider hat deine Lösung nur ${solution.evalState.result.test.passed} von ${solution.evalState.result.test.testcases} Tests bestanden.`;

        return {
            status: 'danger',
            message: message,
            submessage: `Die Aufgaben sind vielleicht nicht ganz einfach zu lösen, du solltest dir deine Lösung und die Testergebnisse unten aber <strong>unbedingt</strong> nochmal ansehen und versuchen, die Probleme zu beheben. Beim nächsten Mal klappt's bestimmt.`,
            emoji: '🤨',
            actions: {
                retry: {
                    enabled: true,
                    status: 'success',
                },
            },
        };
    }
}
