import { Injectable } from '@angular/core';
import { filter, map, tap } from 'rxjs/operators';
import { combineLatest, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { TaskInstance } from 'app/exercises/models/task-instance';
import { SubatoPrincipal } from 'app/auth/models/principal/subato-principal';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { Solution } from 'app/solutions/models/solution';
import {
    loadSolution,
    SolutionState,
} from 'app/solutions/shared/operators/solution.operators';
import { ChangeCourse } from 'app/courses/store/courses.actions';
import {
    loadTaskInstances,
    TaskInstancesState,
} from 'app/exercises/shared/operators/task-instance.operators';
import { ifAttached } from 'app/shared/route-reuse/route-reuse-life.cycle.directive';

export interface SolutionPageState {
    solutionState: SolutionState;
    taskInstancesState: TaskInstancesState;
    nextTaskInstance: TaskInstance;
    justSubmitted: boolean;
}

export const initialState: SolutionPageState = {
    solutionState: { solution: null, error: null, loading: true },
    taskInstancesState: { taskInstances: [], error: null, loading: true },
    nextTaskInstance: null,
    justSubmitted: false,
};

@Injectable()
export class SolutionPageStore extends BaseComponentStore<SolutionPageState> {
    public readonly solutionState$ = this.select((s) => s.solutionState);
    public readonly solution$ = this.select((s) => s.solutionState.solution);

    public readonly taskInstancesState$ = this.select(
        (s) => s.taskInstancesState
    );
    public readonly taskInstances$ = this.select(
        (s) => s.taskInstancesState.taskInstances
    );

    public readonly justSubmitted$ = this.select((s) => s.justSubmitted);

    principal$: Observable<SubatoPrincipal> = this.store.select(
        (s) => s.auth.principal
    );

    nextTaskInstance$ = combineLatest(this.solution$, this.taskInstances$).pipe(
        filter(([solution, taskInstances]) => solution != null),
        map(([solution, taskInstances]) => ({
            taskInstance: taskInstances.find(
                (ti) => ti.task.id == solution.task.id
            ),
            taskInstances: taskInstances,
        })),
        map(({ taskInstances, taskInstance }) =>
            taskInstance
                ? taskInstance.getNextTaskInstance(taskInstances)
                : null
        )
    );

    hasAccessToAcceptance$ = combineLatest([
        this.solution$,
        this.principal$,
    ]).pipe(
        filter(([solution, principal]) => solution != null),
        map(([solution, principal]) =>
            solution.exercise
                ? principal != null &&
                  principal.isLecturerOrAdmin(solution.exercise.course.id)
                : principal != null && principal.hasStaffRole()
        )
    );

    constructor(private store: Store<SubatoState>) {
        super(initialState);
    }

    readonly patchSolution = this.updaterFor('solutionState');
    readonly patchTaskInstances = this.updaterFor('taskInstancesState');

    readonly loadSolution = this.effect((solutionId$: Observable<number>) =>
        solutionId$.pipe(loadSolution((s) => this.patchSolution(s)))
    );

    readonly dispatchCourseChange$ = this.effect(() =>
        combineLatest(
            [this.solution$, this.detached$],
            (solution) => solution
        ).pipe(
            ifAttached(this),
            filter((solution) => solution != null),
            tap((solution: Solution) => {
                if (solution.exercise != null) {
                    this.store.dispatch(
                        new ChangeCourse({ course: solution.exercise.course })
                    );
                } else {
                    this.store.dispatch(new ChangeCourse({ course: null }));
                }
            })
        )
    );

    readonly loadTaskInstances$ = this.effect(() =>
        this.solution$.pipe(
            map((solution) => solution?.exercise),
            filter((exercise) => exercise != null),
            map((exercise) => exercise.id),
            loadTaskInstances((s) => this.patchTaskInstances(s))
        )
    );
}
