import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { BaseService } from 'app/shared/misc/base.service';
import { environment } from 'environments/environment';
import { Observable, of } from 'rxjs';
import { map, switchMap } from 'rxjs/operators';
import { Page } from 'app/shared/paging/page';
import { DataQueryParams } from 'app/shared/paging/data-query-params';
import { FileTree } from 'app/shared/fs/file-tree';
import { DownloadService } from 'app/shared/download/download.service';
import { Client } from '../models/client';
import { Solution } from '../models/solution';
import { SignService } from 'app/shared/signing/sign.service';
import { EvalState } from 'app/solutions/models/eval-state';

export interface SolutionFilter {
    exerciseId?: number;
    taskId?: number;
    userId?: number;
    submissionId?: number;
    valid?: boolean;
}

export interface SubmitFileSolution {
    exerciseId: number;
    taskId: number;
    comment: String;
    lateSubmit: boolean;
    submittedWith: Client;
}

export interface SubmitInlineSolution {
    exerciseId: number;
    taskId: number;
    comment: String;
    lateSubmit: boolean;
    content: string;
}

@Injectable({
    providedIn: 'root',
})
export class SolutionService extends BaseService {
    constructor(
        private http: HttpClient,
        private downloadService: DownloadService,
        private signService: SignService
    ) {
        super();
    }

    getById(id: number): Observable<Solution> {
        return this.http
            .get<Solution>(`${environment.backendUrl}/solution/${id}`)
            .pipe(map((solution) => new Solution(solution)));
    }

    getLatestSolution(filter: SolutionFilter) {
        return this.getSolutions(filter, {
            sortBy: 'submittedAt',
            sortOrder: 'desc',
            pageSize: 1,
        }).pipe(
            map((solutions) => solutions.content[0]),
            switchMap((solution) =>
                solution ? this.getById(solution.id) : of<Solution>(null)
            )
        );
    }

    getSolutions(
        filter: SolutionFilter,
        projection?: DataQueryParams
    ): Observable<Page<Solution>> {
        var params = DataQueryParams.toHttpParams(projection);

        if (filter.userId != null) {
            params = params.set('userId', filter.userId);
        }

        if (filter.exerciseId != null) {
            params = params.set('exerciseId', filter.exerciseId);
        }

        if (filter.taskId != null) {
            params = params.set('taskId', filter.taskId);
        }

        if (filter.submissionId != null) {
            params = params.set('submissionId', filter.submissionId);
        }

        if (filter.valid != null) {
            params = params.set('valid', filter.valid);
        }

        return this.http
            .get<Page<Solution>>(`${environment.backendUrl}/solution`, {
                params: params,
            })
            .pipe(
                map(
                    (page) =>
                        new Page<Solution>(page, (item) => new Solution(item))
                )
            );
    }

    submitFileSolution(
        dto: SubmitFileSolution,
        file: File | null
    ): Observable<Solution> {
        var formData = new FormData();
        if (file) {
            formData.append('file', file);
        }

        formData.append(
            'solution',
            new Blob([JSON.stringify(dto)], { type: 'application/json' })
        );

        return this.http
            .post<Solution>(
                `${environment.backendUrl}/solution/submit`,
                formData
            )
            .pipe(map((response) => new Solution(response)));
    }

    getDownloadSolutionUrl(solutionId: number) {
        return `${environment.backendUrl}/solution/${solutionId}/download`;
    }

    submitInlineSolution(dto: SubmitInlineSolution): Observable<Solution> {
        return this.http
            .post<Solution>(`${environment.backendUrl}/solution/submit`, dto)
            .pipe(map((response) => new Solution(response)));
    }

    runAcceptance(solutionId: number): Observable<EvalState> {
        return this.http.post<EvalState>(
            `${environment.backendUrl}/solution/${solutionId}/acceptance`,
            {}
        );
        //.pipe(map((response) => new EvalState(response)));
    }

    getFileUrl(solutionId: number, uuid: string, download?: boolean): string {
        var url = `${environment.backendUrl}/solution/${solutionId}/file/${uuid}`;
        if (download) {
            url += '?download=true';
        }

        return url;
    }

    getFile(solutionId: number, item: FileTree): Observable<string> {
        return this.http
            .get(this.getFileUrl(solutionId, item.uuid), {
                responseType: 'text',
            })
            .pipe(map((data: any) => data));
    }

    downloadFile(solutionId: number, item: FileTree): Observable<void> {
        return this.signService
            .sign({ url: this.getFileUrl(solutionId, item.uuid) })
            .pipe(
                map((response) =>
                    this.downloadService.downloadFile(response.signedUrl)
                )
            );
    }
}
