import { Injectable } from '@angular/core';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { EMPTY, Observable } from 'rxjs';
import { BaseComponentStore } from 'app/shared/store/base-component.store';

import {
    catchError,
    map,
    switchMap,
    tap,
    withLatestFrom,
} from 'rxjs/operators';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { Solution } from 'app/solutions/models/solution';
import { SubatoError } from 'app/shared/error/error';
import { SolutionService } from 'app/solutions/services/solution.service';
import { EvalState } from 'app/solutions/models/eval-state';

interface SolutionAcceptanceState {
    solution: Solution;
    running: boolean;
    evalState: EvalState | null;
    error: SubatoError;
}

export const initialState: SolutionAcceptanceState = {
    solution: null,
    running: false,
    evalState: null,
    error: null,
};

@Injectable()
export class SolutionAcceptanceStore extends BaseComponentStore<SolutionAcceptanceState> {
    principal$ = this.store.select((s) => s.auth.principal);
    public readonly solution$ = this.select((s) => s.solution);
    public readonly running$ = this.select((s) => s.running);
    public readonly evalState$ = this.select((s) => s.evalState);
    public readonly error$ = this.select((s) => s.error);

    constructor(
        private store: Store<SubatoState>,
        private solutionService: SolutionService,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService
    ) {
        super(initialState);
    }

    readonly run = this.effect((trigger$: Observable<void>) =>
        trigger$.pipe(
            withLatestFrom(this.solution$),
            map(([, solution]) => solution),
            tap(() => this.patchState({ running: true })),
            switchMap((solution) =>
                this.solutionService.runAcceptance(solution.id).pipe(
                    tap((evalState) =>
                        this.patchState({
                            evalState: evalState,
                            running: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.patchState({ running: false, error: error });

                        this.notificationService.showError(
                            this.globalErrorHandler.format(
                                'Fehler bei Ausführung der Abnahmetests',
                                error
                            )
                        );

                        return EMPTY;
                    })
                )
            )
        )
    );

    readonly setResult$ = this.effect(() =>
        this.solution$.pipe(
            map((solution) => solution?.acceptanceEvalState),
            tap((evalState) => this.patchState({ evalState: evalState }))
        )
    );
}
