import { Injectable } from '@angular/core';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { EMPTY, NEVER, Observable } from 'rxjs';
import { BaseComponentStore } from 'app/shared/store/base-component.store';

import { catchError, map, switchMap, tap } from 'rxjs/operators';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { Router } from '@angular/router';
import { SolutionComment } from 'app/solutions/models/solution-comment';
import { Solution } from 'app/solutions/models/solution';
import { SolutionCommentService } from 'app/solutions/services/solution-comment.service';
import { CreateSolutionCommentDto } from 'app/solutions/dto/create-solution-comment.dto';

interface SolutionCommentsState {
    comments: SolutionComment[];
    saving: boolean;
    deletingComment: SolutionComment;
}

export const initialState: SolutionCommentsState = {
    comments: [],
    saving: false,
    deletingComment: null,
};

interface CreateCommentParams {
    dto: CreateSolutionCommentDto;
    onSuccess: (SolutionComment) => void;
}

@Injectable()
export class SolutionCommentsStore extends BaseComponentStore<SolutionCommentsState> {
    principal$ = this.store.select((s) => s.auth.principal);
    public readonly comments$ = this.select((s) => s.comments);
    public readonly saving$ = this.select((s) => s.saving);
    public readonly deletingComment$ = this.select((s) => s.deletingComment);

    constructor(
        private store: Store<SubatoState>,
        private commentService: SolutionCommentService,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService,
        private router: Router
    ) {
        super(initialState);
    }

    readonly loadComments = this.effect((solution$: Observable<Solution>) =>
        solution$.pipe(
            map((solution) => solution.comments),
            tap((comments) => this.patchState({ comments: comments }))
        )
    );

    readonly createComment = this.effect(
        (params$: Observable<CreateCommentParams>) =>
            params$.pipe(
                tap(() => this.patchState({ saving: true })),
                switchMap((params) =>
                    this.commentService.create(params.dto).pipe(
                        tap((comment) => {
                            this.patchState({ saving: false });
                            this.notificationService.showSuccess(
                                'Kommentar erfolgreich erstellt'
                            );

                            this.patchState((s) => ({
                                comments: [comment, ...s.comments],
                            }));

                            params.onSuccess(comment);
                        }),
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((error) => {
                            this.patchState({ saving: false });

                            this.notificationService.showError(
                                this.globalErrorHandler.format(
                                    'Fehler bei Erstellung des Kommentars',
                                    error
                                )
                            );

                            return EMPTY;
                        })
                    )
                )
            )
    );

    readonly deleteComment = this.effect(
        (comment$: Observable<SolutionComment>) =>
            comment$.pipe(
                tap((comment) => this.patchState({ deletingComment: comment })),
                switchMap((comment) =>
                    this.commentService.delete(comment.id).pipe(
                        tap(() => {
                            this.patchState((s) => ({
                                comments: s.comments.filter(
                                    (m) => m != comment
                                ),
                                deletingComment: null,
                            }));
                        }),
                        tap((membership) => {
                            this.notificationService.showSuccess(
                                'Kommentar erfolgreich gelöscht'
                            );
                        }),
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((error) => {
                            this.notificationService.showError(
                                this.globalErrorHandler.format(
                                    'Fehler bei Löschung des Kommentars',
                                    error
                                )
                            );

                            this.patchState({ deletingComment: null });
                            return NEVER;
                        })
                    )
                )
            )
    );
}
