import { Injectable } from '@angular/core';
import { combineLatest, EMPTY, of } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import {
    SolutionFilter,
    SolutionService,
} from 'app/solutions/services/solution.service';
import { Page } from 'app/shared/paging/page';
import { Solution } from 'app/solutions/models/solution';
import {
    createInitialState,
    PagingStore,
} from 'app/shared/paging/paging.store';
import { ComponentStore } from '@ngrx/component-store';
import { catchError, filter, map, switchMap, tap } from 'rxjs/operators';

interface SolutionHistoryState {
    filter: SolutionFilter;
}

export const initialState: SolutionHistoryState = {
    filter: null,
};

@Injectable()
export class SolutionHistoryStore extends ComponentStore<SolutionHistoryState> {
    public readonly filter$ = this.select((state) => state.filter);

    private _paging: PagingStore<Solution> = new PagingStore<Solution>({
        ...createInitialState(),
        projection: {
            sortBy: 'submittedAt',
            sortOrder: 'desc',
        },
    });

    get paging() {
        return this._paging;
    }

    constructor(
        private store: Store<SubatoState>,
        private solutionService: SolutionService,
        private globalErrorHandler: GlobalErrorHandler
    ) {
        super(initialState);
    }

    readonly updateFilter = this.updater(
        (state, filter: Partial<SolutionFilter>) => ({
            ...state,
            filter: { ...state.filter, ...filter },
        })
    );

    readonly resetItemsAfterFilterChange$ = this.effect(() =>
        this.filter$.pipe(
            tap(() => this.paging.patchState({ items: [], page: null }))
        )
    );

    readonly filterChange$ = this.effect(() =>
        combineLatest([this.filter$, this.paging.projection$]).pipe(
            filter(
                ([filter, projection]) => filter != null && projection != null
            ),
            tap(() => this.paging.patchState({ loading: true })),
            switchMap(([filter, projection]) =>
                filter.userId != null
                    ? this.solutionService
                          .getSolutions(filter, projection)
                          .pipe(
                              catchError((err) =>
                                  this.globalErrorHandler.handleError(err)
                              ),
                              catchError((error) => {
                                  this.paging.patchError(error);
                                  return EMPTY;
                              })
                          )
                    : of(new Page<Solution>())
            ),
            map((page) => this.paging.patchPage(page))
        )
    );
}
