import { EMPTY, Observable } from 'rxjs';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { inject, Injectable } from '@angular/core';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { SubatoError } from 'app/shared/error/error';
import { Solution } from 'app/solutions/models/solution';
import {
    SolutionFilter,
    SolutionService,
} from 'app/solutions/services/solution.service';

export interface SolutionState {
    solution: Solution | null;
    error: SubatoError;
    loading: boolean;
}

export function loadLatestSolution(
    updaterFn: (state: Partial<SolutionState>) => void
) {
    let op = inject(SolutionOperators);
    return (filter$: Observable<SolutionFilter>) =>
        op.loadLatestSolution(filter$, updaterFn);
}

export function loadSolution(
    updaterFn: (state: Partial<SolutionState>) => void
) {
    let op = inject(SolutionOperators);
    return (solutionId$: Observable<number>) =>
        op.loadSolution(solutionId$, updaterFn);
}

@Injectable({ providedIn: 'root' })
class SolutionOperators {
    constructor(
        private solutionService: SolutionService,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler
    ) {}

    public loadLatestSolution(
        filter$: Observable<SolutionFilter>,
        updaterFn: (state: Partial<SolutionState>) => void
    ) {
        return filter$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((filter) =>
                this.solutionService.getLatestSolution(filter).pipe(
                    tap((solution) =>
                        updaterFn({
                            solution: solution,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            solution: null,
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }

    public loadSolution(
        solutionId$: Observable<number>,
        updaterFn: (state: Partial<SolutionState>) => void
    ) {
        return solutionId$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((exerciseId) =>
                this.solutionService.getById(exerciseId).pipe(
                    tap((solution) =>
                        updaterFn({
                            solution: solution,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            solution: null,
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }
}
