import { Injectable, ViewContainerRef } from '@angular/core';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { combineLatest, EMPTY } from 'rxjs';
import { BaseComponentStore } from 'app/shared/store/base-component.store';

import {
    catchError,
    filter,
    map,
    switchMap,
    tap,
    withLatestFrom,
} from 'rxjs/operators';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { Router } from '@angular/router';
import { Solution } from 'app/solutions/models/solution';
import { SolutionCommentService } from 'app/solutions/services/solution-comment.service';
import { FileTree } from 'app/shared/fs/file-tree';
import { Viewer } from 'app/solutions/shared/solution-ide/viewer/viewer';
import { ViewerFactory } from 'app/solutions/shared/solution-ide/viewer/viewer.factory';

interface SolutionIdeState {
    selectedTree: FileTree;
    solution: Solution;
    compactMode: boolean;
    fileBrowserEnabled: boolean;
    viewer: Viewer;
    viewerLoading: boolean;
    viewerError: any;
    viewContainerRef: ViewContainerRef;
}

export const initialState: SolutionIdeState = {
    selectedTree: null,
    solution: null,
    compactMode: false,
    fileBrowserEnabled: false,
    viewer: null,
    viewerLoading: true,
    viewerError: null,
    viewContainerRef: null,
};

@Injectable()
export class SolutionIdeStore extends BaseComponentStore<SolutionIdeState> {
    public readonly solution$ = this.select((s) => s.solution);
    public readonly selectedTree$ = this.select((s) => s.selectedTree);
    public readonly compactMode$ = this.select((s) => s.compactMode);
    public readonly viewer$ = this.select((s) => s.viewer);
    public readonly viewerLoading$ = this.select((s) => s.viewerLoading);
    public readonly viewerError$ = this.select((s) => s.viewerError);
    public readonly viewContainerRef$ = this.select((s) => s.viewContainerRef);
    public readonly fileBrowserEnabled$ = this.select(
        (s) => s.fileBrowserEnabled
    );

    defaultBrowserWidth: number = 15;
    public readonly browserWidth$ = this.select(
        this.compactMode$,
        (compactMode) => (compactMode ? 0 : this.defaultBrowserWidth)
    );

    constructor(
        private store: Store<SubatoState>,
        private commentService: SolutionCommentService,
        private globalErrorHandler: GlobalErrorHandler,
        private viewerFactory: ViewerFactory,
        private notificationService: NotificationService,
        private router: Router
    ) {
        super(initialState);
    }

    readonly setup$ = this.effect(() =>
        this.solution$.pipe(
            filter(
                (solution) =>
                    solution != null && solution.tree.children.length != 0
            ),
            map((solution) => ({
                firstNode: solution.tree.children[0],
                firstNodeIsFile:
                    solution.tree.children.length == 1 &&
                    solution.tree.children[0].children.length == 0,
            })),
            tap(({ firstNode, firstNodeIsFile }) =>
                this.patchState({
                    selectedTree: firstNodeIsFile ? firstNode : null,
                    compactMode: firstNodeIsFile,
                    fileBrowserEnabled: !firstNodeIsFile,
                })
            )
        )
    );

    readonly initViewer$ = this.effect(() =>
        combineLatest([this.selectedTree$, this.viewContainerRef$]).pipe(
            withLatestFrom(this.solution$),
            filter(
                ([[, viewContainerRef], solution]) =>
                    solution != null && viewContainerRef != null
            ),
            tap(([[, viewContainerRef]]) => viewContainerRef.clear()),
            tap(() => this.patchState({ viewerLoading: true })),
            switchMap(([[tree, viewContainerRef], solution]) =>
                this.viewerFactory
                    .createViewer(solution, tree, viewContainerRef)
                    .pipe(
                        tap((viewer) =>
                            this.patchState({
                                viewer: viewer,
                                viewerError: null,
                                viewerLoading: false,
                            })
                        ),
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((err) => {
                            this.patchState({
                                viewerError: err,
                                viewerLoading: null,
                            });
                            return EMPTY;
                        })
                    )
            )
        )
    );
}
