import { Injectable } from '@angular/core';
import { filter, map, tap, withLatestFrom } from 'rxjs/operators';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { combineLatest, Observable } from 'rxjs';
import { Exercise } from 'app/exercises/models/exercise';
import { Task } from 'app/tasks/models/task';
import { FileType } from 'app/tasks/models/file-type';
import { LastSolutionService } from 'app/solutions/services/last-solution.service';
import {
    loadFile,
    loadFiles,
    TaskFileContentState,
    TaskFilesState,
} from 'app/tasks/shared/operators/task-file.operators';

interface EditorState {
    exercise: Exercise;
    task: Task;
    filesState: TaskFilesState;
    templateFileState: TaskFileContentState;
    text: string;
}

export const initialState: EditorState = {
    exercise: null,
    task: null,
    filesState: { files: [], error: null, loading: true },
    templateFileState: { content: '', error: null, loading: true },
    text: '',
};

@Injectable()
export class EditorStore extends BaseComponentStore<EditorState> {
    public readonly exercise$ = this.select((state) => state.exercise);
    public readonly task$ = this.select((state) => state.task);
    public readonly files$ = this.select((state) => state.filesState.files);
    public readonly templateRef$ = this.select(this.files$, (files) =>
        files.find((f) => f.type == FileType.Template && f.file.src)
    );
    public readonly templateContent$ = this.select(
        (state) => state.templateFileState.content
    );
    public readonly text$ = this.select((state) => state.text);

    public readonly loading$ = this.select(
        this.select((s) => s.filesState.loading),
        this.select((s) => s.templateFileState.loading),
        (l1, l2) => l1 || l2
    );

    public readonly error$ = this.select(
        this.select((s) => s.filesState.error),
        this.select((s) => s.templateFileState.error),
        (e1, e2) => e1 ?? e2
    );

    constructor(private lastSolutionService: LastSolutionService) {
        super(initialState);
    }

    readonly patchFiles = this.updaterFor('filesState');
    readonly patchTemplateFile = this.updaterFor('templateFileState');

    readonly setExercise = this.updater((state, exercise: Exercise) => ({
        ...state,
        exercise: exercise,
    }));

    readonly setTask = this.updater((state, task: Task) => ({
        ...state,
        task: task,
    }));

    readonly loadFiles$ = this.effect(() =>
        combineLatest([this.exercise$, this.task$]).pipe(
            filter(([, task]) => task != null),
            map(([exercise, task]) => ({ exercise: exercise, task: task })),
            loadFiles((s) => this.patchFiles(s))
        )
    );

    readonly loadTemplateContent$ = this.effect(() =>
        this.templateRef$.pipe(
            filter((ref) => ref != null),
            withLatestFrom(this.task$, this.exercise$),
            filter(
                ([, task]) => task != null && task.supportsTextSubmissionMode
            ),
            map(([ref, task, exercise]) => ({
                ref: ref,
                task: task,
                exercise: exercise,
            })),
            loadFile((s) => this.patchTemplateFile(s))
        )
    );

    readonly setInitialText$ = this.effect(() =>
        this.templateContent$.pipe(
            withLatestFrom(this.task$, this.exercise$),
            filter(
                ([, task]) => task != null && task.supportsTextSubmissionMode
            ),
            tap(([templateContent, task, exercise]) => {
                let lastSolution = this.lastSolutionService.getLastSolution(
                    task,
                    exercise
                );

                this.patchState({
                    text: lastSolution != null ? lastSolution : templateContent,
                });
            })
        )
    );

    readonly reset = this.effect((trigger$: Observable<void>) =>
        trigger$.pipe(
            withLatestFrom(this.templateContent$, this.task$, this.exercise$),
            tap(([, templateContent]) =>
                this.patchState({ text: templateContent })
            ),
            tap(([, , task, exercise]) =>
                this.lastSolutionService.clearLastSolution(task, exercise)
            )
        )
    );
}
