import {
    AfterViewInit,
    Component,
    forwardRef,
    Input,
    OnChanges,
    OnInit,
    QueryList,
    SimpleChanges,
    ViewChildren,
} from '@angular/core';
import { ControlValueAccessor, NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs';
import { take, takeUntil } from 'rxjs/operators';
import { Exercise } from 'app/exercises/models/exercise';
import { CodemirrorComponent } from '@ctrl/ngx-codemirror';
import { CodemirrorHelper } from 'app/shared/codemirror/helper/codemirror.helper';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { BaseComponent } from 'app/shared/misc/base.component';
import { Task } from 'app/tasks/models/task';
import { EditorStore } from 'app/solutions/shared/submit-solution-form/components/editor/editor-store.service';

@Component({
    selector: 'sb-submit-solution-form-editor',
    templateUrl: './editor.component.html',
    styleUrls: ['./editor.component.scss'],
    providers: [
        {
            provide: NG_VALUE_ACCESSOR,
            useExisting: forwardRef(() => EditorComponent),
            multi: true,
        },
        EditorStore,
    ],
})
export class EditorComponent
    extends BaseComponent
    implements OnInit, OnChanges, AfterViewInit, ControlValueAccessor
{
    @Input() task: Task;
    @Input() exercise: Exercise;

    @ViewChildren(CodemirrorComponent)
    codemirrorComponents: QueryList<CodemirrorComponent>;

    darkMode$: Observable<boolean> = this.store.select(
        (s) => s.layout.darkMode
    );

    error$ = this.solutionEditorStore.error$;
    loading$ = this.solutionEditorStore.loading$;
    templateRef$ = this.solutionEditorStore.templateRef$;
    text$ = this.solutionEditorStore.text$;

    constructor(
        private store: Store<SubatoState>,
        private solutionEditorStore: EditorStore
    ) {
        super();
    }

    ngAfterViewInit(): void {
        this.codemirrorComponents.changes.subscribe((c) => {
            this.refreshEditors();
        });
    }

    setValue(val) {
        this.solutionEditorStore.patchState({ text: val });
    }

    ngOnInit(): void {
        this.text$
            .pipe(takeUntil(this.destroy$))
            .subscribe((text) => this.onChange(text));
    }

    ngOnChanges(changes: SimpleChanges): void {
        if (changes.hasOwnProperty('exercise')) {
            this.solutionEditorStore.setExercise(this.exercise);
        }

        if (changes.hasOwnProperty('task')) {
            this.solutionEditorStore.setTask(this.task);
        }
    }

    public reset() {
        this.solutionEditorStore.reset();
    }

    refreshEditors() {
        this.codemirrorComponents.forEach((component) => {
            CodemirrorHelper.waitForEditor(component)
                .pipe(take(1))
                .subscribe(() => component.codeMirror.refresh());
        });
    }

    // ControlValueAccessor interface methods

    private onChange: (value: any) => void = () => {};
    private onTouched: () => void = () => {};

    writeValue(value: any): void {
        // Set the initial value when the form control is initialized
        this.solutionEditorStore.patchState({ text: value });
    }

    registerOnChange(fn: (value: any) => void): void {
        // Register the callback function to update the form control value
        this.onChange = fn;
    }

    registerOnTouched(fn: () => void): void {
        // Register the callback function for touch events
        this.onTouched = fn;
    }

    setDisabledState(isDisabled: boolean): void {
        // Optional: Implement this method if you want to disable the component
        // based on the form control's disabled state
    }
}
