import {
    Component,
    Input,
    OnChanges,
    OnInit,
    SimpleChanges,
} from '@angular/core';
import { UntypedFormBuilder, UntypedFormGroup } from '@angular/forms';
import { Exercise } from 'app/exercises/models/exercise';
import { SubmissionMode } from 'app/tasks/models/submission-mode';
import { BaseComponent } from 'app/shared/misc/base.component';
import { Task } from 'app/tasks/models/task';
import { Client } from 'app/solutions/models/client';
import {
    SubmitFileSolution,
    SubmitInlineSolution,
} from 'app/solutions/services/solution.service';
import { TaskInstance } from 'app/exercises/models/task-instance';
import { Submission } from 'app/assessment/models/submission';
import { SubmitSolutionFormStore } from 'app/solutions/shared/submit-solution-form/submit-solution-form.store';

@Component({
    selector: 'sb-submit-solution-form',
    templateUrl: './submit-solution-form.component.html',
    styleUrls: ['./submit-solution-form.component.scss'],
    providers: [SubmitSolutionFormStore],
})
export class SubmitSolutionFormComponent
    extends BaseComponent
    implements OnInit, OnChanges
{
    @Input() task: Task;
    @Input() exercise: Exercise;
    @Input() taskInstance: TaskInstance;
    @Input() submission: Submission;

    principal$ = this.submitSolutionStore.principal$;
    showLateSubmitHint$ = this.submitSolutionStore.showLateSubmitHint$;
    submitting$ = this.submitSolutionStore.submitting$;
    error$ = this.submitSolutionStore.error$;

    solutionForm: UntypedFormGroup;
    SubmissionMode = SubmissionMode;

    constructor(
        private submitSolutionStore: SubmitSolutionFormStore,
        private fb: UntypedFormBuilder
    ) {
        super();
    }

    ngOnInit(): void {
        let selectedMode = this.task.defaultSubmissionMode;
        if (selectedMode == SubmissionMode.TEXT && this.editorDisabled) {
            selectedMode = this.task.supportsFileSubmissionMode
                ? SubmissionMode.FILE
                : null;
        }

        this.solutionForm = this.fb.group({
            file: [''],
            solutionContent: [''],
            comment: [''],
            lateSubmit: [false],
            selectedMode: [selectedMode],
        });
    }

    ngOnChanges(changes: SimpleChanges): void {
        if (changes.hasOwnProperty('exercise')) {
            this.submitSolutionStore.patchState({ exercise: this.exercise });
        }

        if (changes.hasOwnProperty('task')) {
            this.submitSolutionStore.patchState({ task: this.task });
        }

        if (changes.hasOwnProperty('taskInstance')) {
            this.submitSolutionStore.patchState({
                taskInstance: this.taskInstance,
            });
        }

        if (changes.hasOwnProperty('submission')) {
            this.submitSolutionStore.patchState({
                submission: this.submission,
            });
        }
    }

    get editorDisabled() {
        return this.taskInstance != null
            ? this.taskInstance.exercise.course.disableEditor
            : false;
    }

    get selectedMode(): SubmissionMode {
        return this.solutionForm.get('selectedMode').value;
    }

    submitSolution() {
        if (this.taskInstance?.exercise?.course?.finished) {
            return;
        }

        if (this.selectedMode == SubmissionMode.TEXT) {
            let dto: SubmitInlineSolution = {
                exerciseId: this.exercise?.id,
                taskId: this.task.id,
                comment: this.solutionForm.get('comment').value,
                content: this.solutionForm.get('solutionContent').value,
                lateSubmit: this.solutionForm.get('lateSubmit').value,
            };

            this.submitSolutionStore.submitInline(dto);
        } else {
            let file = this.solutionForm.get('file').value;
            let dto: SubmitFileSolution = {
                exerciseId: this.exercise?.id,
                taskId: this.task.id,
                comment: this.solutionForm.get('comment').value,
                lateSubmit: this.solutionForm.get('lateSubmit').value,
                submittedWith: Client.Web,
            };

            this.submitSolutionStore.submitFile({
                dto: dto,
                file: file,
            });
        }
    }
}
