import { Injectable } from '@angular/core';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { EMPTY, Observable } from 'rxjs';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { ActivatedRoute, Router } from '@angular/router';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { Exercise } from 'app/exercises/models/exercise';
import { Task } from 'app/tasks/models/task';
import { TaskInstance } from 'app/exercises/models/task-instance';
import { Submission } from 'app/assessment/models/submission';
import { SubatoError } from 'app/shared/error/error';
import {
    SolutionService,
    SubmitFileSolution,
    SubmitInlineSolution,
} from 'app/solutions/services/solution.service';
import { LoadPrincipal } from 'app/auth/store/auth.actions';
import { LastSolutionService } from 'app/solutions/services/last-solution.service';
import { Solution } from 'app/solutions/models/solution';

interface SubmitSolutionFormState {
    exercise: Exercise;
    task: Task;
    taskInstance: TaskInstance;
    submission: Submission;

    submitting: boolean;
    error: SubatoError;
}

export const initialState: SubmitSolutionFormState = {
    exercise: null,
    task: null,
    taskInstance: null,
    submission: null,
    submitting: false,
    error: null,
};

interface SubmitFileParams {
    dto: SubmitFileSolution;
    file?: File;
}

@Injectable()
export class SubmitSolutionFormStore extends BaseComponentStore<SubmitSolutionFormState> {
    principal$ = this.store.select((s) => s.auth.principal);
    public readonly task$ = this.select((s) => s.task);
    public readonly exercise$ = this.select((s) => s.exercise);
    public readonly submission$ = this.select((s) => s.submission);
    public readonly submitting$ = this.select((s) => s.submitting);
    public readonly error$ = this.select((s) => s.error);

    showLateSubmitHint$ = this.select(
        this.principal$,
        this.exercise$,
        this.task$,
        this.submission$,
        (principal, exercise, task, submission) =>
            principal != null &&
            exercise != null &&
            (!exercise.isActive() ||
                (submission != null &&
                    submission.attemptsExceeded(task, exercise)))
    );

    constructor(
        private store: Store<SubatoState>,
        private solutionService: SolutionService,
        private lastSolutionService: LastSolutionService,
        private router: Router,
        private globalErrorHandler: GlobalErrorHandler,
        private route: ActivatedRoute,
        private notificationService: NotificationService
    ) {
        super(initialState);
    }

    readonly submitFile = this.effect((params$: Observable<SubmitFileParams>) =>
        params$.pipe(
            tap(() => this.patchState({ submitting: true })),
            switchMap((params) =>
                this.solutionService
                    .submitFileSolution(params.dto, params.file)
                    .pipe(
                        tap((solution) => this.handleSubmitSuccess(solution)),
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((error) => {
                            this.handleSubmitError(error);
                            return EMPTY;
                        })
                    )
            )
        )
    );

    readonly submitInline = this.effect(
        (dto$: Observable<SubmitInlineSolution>) =>
            dto$.pipe(
                tap(() => this.patchState({ submitting: true })),
                switchMap((dto) =>
                    this.solutionService.submitInlineSolution(dto).pipe(
                        tap((solution) =>
                            this.lastSolutionService.setLastSolution(
                                dto.content,
                                solution.task,
                                solution.exercise
                            )
                        ),
                        tap((solution) => this.handleSubmitSuccess(solution)),
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((error) => {
                            this.handleSubmitError(error);
                            return EMPTY;
                        })
                    )
                )
            )
    );

    private handleSubmitSuccess(solution: Solution) {
        this.patchState({ submitting: false });
        this.store.dispatch(new LoadPrincipal());

        this.router.navigate(['/', 'solutions', solution.id], {
            state: {
                justSubmitted: true,
            },
        });

        if (solution.evalFailed) {
            this.notificationService.showWarning(
                'Die Lösung wurde erfolgreich abgegeben, allerdings konnten die Tests nicht ausgeführt werden'
            );
        }
    }

    private handleSubmitError(error: SubatoError) {
        this.patchState({ submitting: false, error: error });

        this.notificationService.showError(
            this.globalErrorHandler.format(
                'Fehler beim Abgeben der Lösungen',
                error
            )
        );
    }
}
