import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { BaseComponent } from 'app/shared/misc/base.component';
import { EMPTY } from 'rxjs';
import {
    catchError,
    filter,
    map,
    switchMap,
    take,
    takeUntil,
} from 'rxjs/operators';
import { ErrorContainerConfig } from 'app/shared/error/error-container-config';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { EditLanguagePageStore } from 'app/tasks/pages/language-admin/edit-language/edit-language.page.store';
import { CreateLanguageDto } from 'app/tasks/dto/create-language.dto';
import { ExplicitConfirmationDialogComponent } from 'app/shared/dialogs/explicit-confirmation-dialog/explicit-confirmation-dialog.component';
import { NbDialogService } from '@nebular/theme';

@Component({
    templateUrl: './edit-language.page.component.html',
    styleUrls: ['./edit-language.page.component.scss'],
    providers: [EditLanguagePageStore],
})
export class EditLanguagePageComponent extends BaseComponent implements OnInit {
    languageState$ = this.editLanguageStore.languageState$;
    saving$ = this.editLanguageStore.saving$;
    deleting$ = this.editLanguageStore.deleting$;
    validationError$ = this.editLanguageStore.validationError$;
    isSuperAdmin$ = this.editLanguageStore.isSuperAdmin$;

    errorConfig: ErrorContainerConfig = {
        notFound: {
            message: 'Diese Sprache konnten wir leider nicht finden.',
        },
    };

    constructor(
        private editLanguageStore: EditLanguagePageStore,
        private globalErrorHandler: GlobalErrorHandler,
        private dialogService: NbDialogService,
        private router: Router,
        private route: ActivatedRoute
    ) {
        super();
    }

    ngOnInit(): void {
        this.route.paramMap
            .pipe(
                takeUntil(this.destroy$),
                map((params) => params.get('language_id')),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.editLanguageStore.patchLanguage({ error: err });
                    return EMPTY;
                })
            )
            .subscribe((languageId) =>
                this.editLanguageStore.loadLanguage(languageId)
            );
    }

    saveLanguage(dto: CreateLanguageDto) {
        this.editLanguageStore.updateLanguage(dto);
    }

    deleteLanguage() {
        this.languageState$
            .pipe(
                take(1),
                switchMap((languageState) =>
                    this.dialogService
                        .open(ExplicitConfirmationDialogComponent, {
                            context: {
                                title: `Löschung von Sprache: ${languageState.language.name}`,
                                text: `Bist du wirklich sicher, dass du die Sprache '${languageState.language.name}' löschen willst? Die Sprache wird bei allen verknüpften Aufgaben entfernt. Achte darauf, dass beim Aufgabenimport jetzt ein Fehler entsteht, wenn diese Sprache noch irgendwo referenziert wird.`,
                                actionText: 'Löschen',
                                expectedText: languageState.language.name,
                            },
                        })
                        .onClose.pipe(
                            takeUntil(this.destroy$),
                            filter((result) => result && result.confirmed),
                            map(() => languageState)
                        )
                )
            )
            .subscribe((languageState) =>
                this.editLanguageStore.deleteLanguage(languageState.language)
            );
    }
}
