import { Injectable } from '@angular/core';
import {
    catchError,
    map,
    switchMap,
    takeUntil,
    tap,
    withLatestFrom,
} from 'rxjs/operators';
import { EMPTY, NEVER, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { Router } from '@angular/router';
import { isValidationError, ValidationError } from 'app/shared/error/error';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { LanguageService } from 'app/tasks/services/language.service';
import { CreateLanguageDto } from 'app/tasks/dto/create-language.dto';
import {
    LanguageState,
    loadLanguage,
} from 'app/tasks/shared/operators/language.operators';
import { Language } from 'app/tasks/models/language';

interface EditLanguagePageState {
    languageState: LanguageState;
    validationError: ValidationError | null;
    saving: boolean;
    deleting: boolean;
}

export const initialState: EditLanguagePageState = {
    languageState: { language: null, error: null, loading: true },
    validationError: null,
    saving: false,
    deleting: false,
};

@Injectable()
export class EditLanguagePageStore extends BaseComponentStore<EditLanguagePageState> {
    public readonly saving$ = this.select((s) => s.saving);
    public readonly deleting$ = this.select((s) => s.deleting);
    public readonly languageState$ = this.select((s) => s.languageState);
    public readonly validationError$ = this.select(
        (state) => state.validationError
    );

    principal$ = this.store.select((s) => s.auth.principal);
    isSuperAdmin$ = this.principal$.pipe(
        takeUntil(this.destroy$),
        map((principal) => principal.hasSuperAdminRole())
    );

    constructor(
        private store: Store<SubatoState>,
        private languageService: LanguageService,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService,
        private router: Router
    ) {
        super(initialState);
    }

    readonly patchLanguage = this.updaterFor('languageState');

    readonly loadLanguage = this.effect((languageId$: Observable<string>) =>
        languageId$.pipe(loadLanguage((s) => this.patchLanguage(s)))
    );

    readonly updateLanguage = this.effect(
        (params: Observable<CreateLanguageDto>) =>
            params.pipe(
                tap((dto) => this.patchState({ saving: true })),
                withLatestFrom(this.languageState$),
                switchMap(([dto, languageState]) =>
                    this.languageService
                        .update(languageState.language.id, dto)
                        .pipe(
                            tap((pool) => {
                                this.patchState({ saving: false });
                                this.notificationService.showSuccess(
                                    'Sprache erfolgreich aktualisiert'
                                );
                                this.router.navigate(['/', 'language']);
                            }),
                            catchError((err) =>
                                this.globalErrorHandler.handleError(err)
                            ),
                            catchError((error) => {
                                this.patchState({ saving: false });

                                if (isValidationError(error)) {
                                    this.patchState({ validationError: error });
                                }

                                this.notificationService.showError(
                                    this.globalErrorHandler.format(
                                        'Fehler bei Aktualisierung der Sprache',
                                        error
                                    )
                                );

                                return EMPTY;
                            })
                        )
                )
            )
    );

    readonly deleteLanguage = this.effect((language: Observable<Language>) =>
        language.pipe(
            tap((language) => this.patchState({ deleting: true })),
            switchMap((language) =>
                this.languageService.delete(language.id).pipe(
                    tap(() => {
                        this.router.navigate(['/', 'language']);

                        this.patchState((s) => ({
                            deleting: false,
                        }));
                    }),
                    tap((language) => {
                        this.notificationService.showSuccess(
                            'Sprache erfolgreich gelöscht'
                        );
                    }),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.notificationService.showError(
                            this.globalErrorHandler.format(
                                'Fehler beim Löschen der Sprache',
                                error
                            )
                        );

                        this.patchState({ deleting: false });
                        return NEVER;
                    })
                )
            )
        )
    );
}
