import { Injectable } from '@angular/core';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { EMPTY, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { isValidationError, ValidationError } from 'app/shared/error/error';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { LanguageService } from 'app/tasks/services/language.service';
import { CreateLanguageDto } from 'app/tasks/dto/create-language.dto';
import { Router } from '@angular/router';

interface NewLanguagePageState {
    validationError: ValidationError | null;
    saving: boolean;
}

export const initialState: NewLanguagePageState = {
    validationError: null,
    saving: false,
};

@Injectable()
export class NewLanguagePageStore extends BaseComponentStore<NewLanguagePageState> {
    public readonly saving$ = this.select((s) => s.saving);
    public readonly validationError$ = this.select(
        (state) => state.validationError
    );

    constructor(
        private store: Store<SubatoState>,
        private languageService: LanguageService,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService,
        private router: Router
    ) {
        super(initialState);
    }

    readonly createLanguage = this.effect(
        (params: Observable<CreateLanguageDto>) =>
            params.pipe(
                tap((dto) => this.patchState({ saving: true })),
                switchMap((dto) =>
                    this.languageService.create(dto).pipe(
                        tap((language) => {
                            this.patchState({ saving: false });
                            this.notificationService.showSuccess(
                                'Sprache erfolgreich erstellt'
                            );
                            this.router.navigate(['/', 'language']);
                        }),
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((error) => {
                            this.patchState({ saving: false });

                            if (isValidationError(error)) {
                                this.patchState({ validationError: error });
                            }

                            this.notificationService.showError(
                                this.globalErrorHandler.format(
                                    'Fehler bei Erstellung der Sprache',
                                    error
                                )
                            );

                            return EMPTY;
                        })
                    )
                )
            )
    );
}
