import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { BaseComponent } from 'app/shared/misc/base.component';
import { ParamHelper } from 'app/shared/misc/helper/param.helper';
import { EMPTY } from 'rxjs';
import {
    catchError,
    filter,
    map,
    switchMap,
    take,
    takeUntil,
} from 'rxjs/operators';
import { ErrorContainerConfig } from 'app/shared/error/error-container-config';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { EditTaskPoolPageStore } from 'app/tasks/pages/task-pool-admin/edit-task-pool/edit-task-pool.page.store';
import { CreateTaskPoolDto } from 'app/tasks/dto/create-task-pool.dto';
import { ExplicitConfirmationDialogComponent } from 'app/shared/dialogs/explicit-confirmation-dialog/explicit-confirmation-dialog.component';
import { NbDialogService } from '@nebular/theme';

@Component({
    selector: 'ngx-edit-task-pool',
    templateUrl: './edit-task-pool.page.component.html',
    styleUrls: ['./edit-task-pool.page.component.scss'],
    providers: [EditTaskPoolPageStore],
})
export class EditTaskPoolPageComponent extends BaseComponent implements OnInit {
    pool$ = this.editTaskPoolStore.pool$;
    poolState$ = this.editTaskPoolStore.poolState$;
    saving$ = this.editTaskPoolStore.saving$;
    deleting$ = this.editTaskPoolStore.deleting$;
    isSuperAdmin$ = this.editTaskPoolStore.isSuperAdmin$;
    validationError$ = this.editTaskPoolStore.validationError$;

    errorConfig: ErrorContainerConfig = {
        notFound: {
            message: 'Diesen Aufgaben-Pool konnten wir leider nicht finden.',
        },
    };

    constructor(
        private editTaskPoolStore: EditTaskPoolPageStore,
        private dialogService: NbDialogService,
        private router: Router,
        private globalErrorHandler: GlobalErrorHandler,
        private route: ActivatedRoute
    ) {
        super();
    }

    ngOnInit(): void {
        this.route.paramMap
            .pipe(
                takeUntil(this.destroy$),
                map((params) => ParamHelper.parseIdOrThrow(params, 'pool_id')),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.editTaskPoolStore.patchPool({ error: err });
                    return EMPTY;
                })
            )
            .subscribe((exerciseId) =>
                this.editTaskPoolStore.loadPool(exerciseId)
            );
    }

    savePool(dto: CreateTaskPoolDto) {
        this.editTaskPoolStore.updatePool(dto);
    }

    deletePool() {
        this.pool$
            .pipe(
                take(1),
                switchMap((pool) =>
                    this.dialogService
                        .open(ExplicitConfirmationDialogComponent, {
                            context: {
                                title: `Löschung von Aufgaben-Pool: ${pool.name}`,
                                text: `Bist du wirklich sicher, dass du den Pool '${pool.name}' löschen willst? Alle verknüpften Aufgaben, Lösungen etc. werden damit permanent gelöscht.`,
                                actionText: 'Löschen',
                                expectedText: pool.name,
                            },
                        })
                        .onClose.pipe(
                            takeUntil(this.destroy$),
                            filter((result) => result && result.confirmed)
                        )
                )
            )
            .subscribe(() => this.editTaskPoolStore.deletePool());
    }
}
