import { Injectable } from '@angular/core';
import {
    catchError,
    map,
    switchMap,
    tap,
    withLatestFrom,
} from 'rxjs/operators';
import { EMPTY, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { Router } from '@angular/router';
import { isValidationError, ValidationError } from 'app/shared/error/error';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import {
    loadPool,
    TaskPoolState,
} from 'app/tasks/shared/operators/task-pool.operators';
import { TaskPoolService } from 'app/tasks/services/task-pool.service';
import { CreateTaskPoolDto } from 'app/tasks/dto/create-task-pool.dto';

interface EditTaskPoolPageState {
    poolState: TaskPoolState;
    validationError: ValidationError | null;
    saving: boolean;
    deleting: boolean;
}

export const initialState: EditTaskPoolPageState = {
    poolState: { pool: null, error: null, loading: true },
    validationError: null,
    saving: false,
    deleting: false,
};

@Injectable()
export class EditTaskPoolPageStore extends BaseComponentStore<EditTaskPoolPageState> {
    principal$ = this.store.select((s) => s.auth.principal);
    public readonly saving$ = this.select((s) => s.saving);
    public readonly pool$ = this.select((s) => s.poolState.pool);
    public readonly poolState$ = this.select((s) => s.poolState);
    public readonly validationError$ = this.select(
        (state) => state.validationError
    );
    public readonly deleting$ = this.select((s) => s.deleting);

    isSuperAdmin$ = this.select(this.principal$, (principal) =>
        principal.hasSuperAdminRole()
    );

    constructor(
        private store: Store<SubatoState>,
        private taskPoolService: TaskPoolService,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService,
        private router: Router
    ) {
        super(initialState);
    }

    readonly patchPool = this.updaterFor('poolState');

    readonly loadPool = this.effect((poolId$: Observable<number>) =>
        poolId$.pipe(loadPool((s) => this.patchPool(s)))
    );

    readonly updatePool = this.effect((params: Observable<CreateTaskPoolDto>) =>
        params.pipe(
            tap((dto) => this.patchState({ saving: true })),
            withLatestFrom(this.poolState$),
            switchMap(([dto, poolState]) =>
                this.taskPoolService.update(poolState.pool.id, dto).pipe(
                    tap((pool) => {
                        this.patchState({ saving: false });

                        this.notificationService.showSuccess(
                            'Pool erfolgreich aktualisiert'
                        );

                        this.router.navigate(['/', 'task_pools', pool.id]);
                    }),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.patchState({ saving: false });

                        if (isValidationError(error)) {
                            this.patchState({ validationError: error });
                        }

                        this.notificationService.showError(
                            this.globalErrorHandler.format(
                                'Fehler bei Aktualisierung des Pools',
                                error
                            )
                        );

                        return EMPTY;
                    })
                )
            )
        )
    );

    readonly deletePool = this.effect((trigger$) =>
        trigger$.pipe(
            withLatestFrom(this.pool$),
            map(([_, pool]) => pool),
            tap(() => this.patchState({ deleting: true })),
            switchMap((pool) =>
                this.taskPoolService.delete(pool.id).pipe(
                    tap((pool) => {
                        this.patchState({ deleting: false });

                        this.router.navigate(['task_pools']);
                        this.notificationService.showSuccess(
                            'Aufgaben-Pool erfolgreich gelöscht.'
                        );
                    }),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.patchState({ deleting: false });

                        this.notificationService.showError(
                            this.globalErrorHandler.format(
                                'Fehler beim Löschen des Aufgaben-Pools',
                                error
                            )
                        );

                        return EMPTY;
                    })
                )
            )
        )
    );
}
