import { Injectable } from '@angular/core';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { EMPTY, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { isValidationError, ValidationError } from 'app/shared/error/error';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { TaskPoolService } from 'app/tasks/services/task-pool.service';
import { CreateTaskPoolDto } from 'app/tasks/dto/create-task-pool.dto';
import { Router } from '@angular/router';

interface NewTaskPoolPageState {
    validationError: ValidationError | null;
    saving: boolean;
}

export const initialState: NewTaskPoolPageState = {
    validationError: null,
    saving: false,
};

@Injectable()
export class NewTaskPoolPageStore extends BaseComponentStore<NewTaskPoolPageState> {
    public readonly saving$ = this.select((s) => s.saving);
    public readonly validationError$ = this.select(
        (state) => state.validationError
    );

    constructor(
        private store: Store<SubatoState>,
        private taskPoolService: TaskPoolService,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService,
        private router: Router
    ) {
        super(initialState);
    }

    readonly createPool = this.effect((params: Observable<CreateTaskPoolDto>) =>
        params.pipe(
            tap((dto) => this.patchState({ saving: true })),
            switchMap((dto) =>
                this.taskPoolService.create(dto).pipe(
                    tap((result) => {
                        this.patchState({ saving: false });

                        if (
                            !result.syncFailed &&
                            (result.taskImportResult == null ||
                                result.taskImportResult.problems == 0)
                        ) {
                            this.notificationService.showSuccess(
                                'Pool erfolgreich erstellt'
                            );
                        } else {
                            this.notificationService.showWarning(
                                'Pool erstellt aber bei der Synchronisation sind Probleme aufgetreten.'
                            );
                        }

                        this.router.navigate([
                            '/',
                            'task_pools',
                            result.pool.id,
                        ]);
                    }),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.patchState({ saving: false });

                        if (isValidationError(error)) {
                            this.patchState({ validationError: error });
                        }

                        this.notificationService.showError(
                            this.globalErrorHandler.format(
                                'Fehler bei Erstellung des Pools',
                                error
                            )
                        );

                        return EMPTY;
                    })
                )
            )
        )
    );
}
