import {
    Component,
    EventEmitter,
    Input,
    OnChanges,
    OnInit,
    Output,
    SimpleChanges,
} from '@angular/core';
import {
    AbstractControl,
    UntypedFormBuilder,
    UntypedFormGroup,
    ValidationErrors,
    ValidatorFn,
    Validators,
} from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { FormComponent } from 'app/shared/validation/form.component';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { environment } from 'environments/environment';
import { CreateTaskPoolDto } from 'app/tasks/dto/create-task-pool.dto';
import { TaskPool } from 'app/tasks/models/task-pool';
import { TaskPoolService } from 'app/tasks/services/task-pool.service';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { ValidationError } from 'app/shared/error/error';
import { DocsService } from 'app/shared/misc/docs.service';

export const repositoryValidator: ValidatorFn = (
    control: AbstractControl
): ValidationErrors | null => {
    const useRepository = control.get('useRepository').value;
    const repositoryUrl = control.get('repositoryUrl').value;

    return useRepository && !repositoryUrl
        ? { repositoryUrlMissing: true }
        : null;
};

@Component({
    selector: 'sb-task-pool-form',
    templateUrl: './task-pool-form.component.html',
    styleUrls: ['./task-pool-form.component.scss'],
})
export class TaskPoolFormComponent
    extends FormComponent
    implements OnInit, OnChanges
{
    @Input() pool: TaskPool;
    @Input() saving: boolean = false;
    @Input() deleting: boolean = false;
    @Input() showDelete: boolean = false;
    @Input() validationError: ValidationError;
    @Output() onSave = new EventEmitter<CreateTaskPoolDto>();
    @Output() onDelete = new EventEmitter<void>();

    poolForm: UntypedFormGroup;
    docsUrls = this.docsService.urls;
    taskPoolRepoBaseUrl = environment.taskPoolRepoBaseUrl;

    constructor(
        private poolService: TaskPoolService,
        private router: Router,
        private globalErrorHandler: GlobalErrorHandler,
        private route: ActivatedRoute,
        private notificationService: NotificationService,
        private fb: UntypedFormBuilder,
        private store: Store<SubatoState>,
        private docsService: DocsService
    ) {
        super();
    }

    createForm(pool?: TaskPool) {
        this.poolForm = this.fb.group(
            {
                name: [pool?.name ?? '', Validators.required],
                description: [pool?.description ?? ''],
                useRepository: [pool?.linkedToRepository ?? false],
                repositoryUrl: [
                    pool?.repositoryUrl
                        ? pool.repositoryUrl
                        : `${this.taskPoolRepoBaseUrl}/<name>`,
                ],
                repositoryBranch: [
                    pool?.repositoryBranch ? pool.repositoryBranch : null,
                ],
            },
            { validators: repositoryValidator }
        );
    }

    ngOnInit(): void {
        this.createForm(this.pool);
    }

    ngOnChanges(changes: SimpleChanges): void {
        if (changes.hasOwnProperty('pool')) {
            this.createForm(this.pool);
        }

        if (changes.hasOwnProperty('validationError')) {
            this.errors = this.getValidationErrors(this.validationError);
        }
    }

    get useRepository(): boolean {
        return this.poolForm.get('useRepository').value;
    }

    save() {
        this.saving = true;

        let dto: CreateTaskPoolDto = {
            name: this.poolForm.get('name').value,
            description: this.poolForm.get('description').value,
            repositoryUrl: this.useRepository
                ? this.poolForm.get('repositoryUrl').value
                : null,
            repositoryBranch: this.useRepository
                ? this.poolForm.get('repositoryBranch').value
                : null,
        };

        this.onSave.emit(dto);
    }

    delete() {
        this.onDelete.emit();
    }
}
