import { Injectable } from '@angular/core';
import { catchError, filter, switchMap, tap } from 'rxjs/operators';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { EMPTY } from 'rxjs';
import { TaskPool } from 'app/tasks/models/task-pool';
import { SubatoError } from 'app/shared/error/error';
import { Course } from 'app/courses/models/course';
import { TaskPoolService } from 'app/tasks/services/task-pool.service';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';

interface CoursesState {
    courses: Course[];
    error: SubatoError;
    loading: boolean;
}

interface CourseListState {
    coursesState: CoursesState;
    pool: TaskPool;
}

export const initialState: CourseListState = {
    coursesState: { courses: [], error: null, loading: true },
    pool: null,
};

@Injectable()
export class CourseListStore extends BaseComponentStore<CourseListState> {
    public readonly pool$ = this.select((state) => state.pool);
    public readonly coursesState = this.select((state) => state.coursesState);

    constructor(
        private taskPoolService: TaskPoolService,
        private globalErrorHandler: GlobalErrorHandler
    ) {
        super(initialState);
    }

    readonly patchCourses = this.updaterFor('coursesState');

    readonly setPool = this.updater((state, pool: TaskPool) => ({
        ...state,
        pool: pool,
    }));

    readonly loadCourses$ = this.effect(() =>
        this.pool$.pipe(
            filter((pool) => pool != null),
            tap(() => this.patchCourses({ loading: true })),
            switchMap((pool) =>
                this.taskPoolService.getCourses(pool.id).pipe(
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.patchCourses({
                            error: error,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            ),
            tap((courses) =>
                this.patchCourses({
                    courses: courses,
                    loading: false,
                    error: null,
                })
            )
        )
    );
}
