import { Injectable } from '@angular/core';
import { ComponentStore } from '@ngrx/component-store';
import {
    catchError,
    filter,
    switchMap,
    tap,
    withLatestFrom,
} from 'rxjs/operators';
import { combineLatest, EMPTY, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';

import { Task } from 'app/tasks/models/task';
import { TaskPool } from 'app/tasks/models/task-pool';
import { TaskPoolService } from 'app/tasks/services/task-pool.service';
import { debounceWithInitialValue } from 'app/shared/misc/rxjs-operators';
import { environment } from 'environments/environment';
import {
    createInitialState,
    PagingStore,
} from 'app/shared/paging/paging.store';

interface TaskListState {
    query: string;
    pool: TaskPool;
    showArchived: boolean;
}

export const initialState: TaskListState = {
    query: '',
    pool: null,
    showArchived: false,
};

@Injectable()
export class TaskListStore extends ComponentStore<TaskListState> {
    public readonly query$ = this.select((state) => state.query);
    public readonly showArchived$ = this.select((state) => state.showArchived);
    public readonly debouncedQuery$ = this.query$.pipe(
        debounceWithInitialValue(environment.debounceTimeMs, initialState.query)
    );
    public readonly pool$ = this.select((state) => state.pool);

    private _paging: PagingStore<Task> = new PagingStore<Task>({
        ...createInitialState(),
    });

    get paging() {
        return this._paging;
    }

    constructor(
        private store: Store<SubatoState>,
        private taskPoolService: TaskPoolService,
        private globalErrorHandler: GlobalErrorHandler
    ) {
        super(initialState);
    }

    readonly resetTasksAfterFilterChange$ = this.effect(() =>
        combineLatest([this.debouncedQuery$, this.showArchived$]).pipe(
            tap(() =>
                this.paging.patchState({
                    items: [],
                    page: null,
                    projection: null,
                })
            )
        )
    );

    readonly refresh = this.effect<void>((params$: Observable<void>) =>
        params$.pipe(
            tap(() => this.paging.patchState({ items: [], page: null })),
            withLatestFrom(
                this.pool$,
                this.query$,
                this.paging.projection$,
                this.showArchived$
            ),
            filter(([_, pool]) => pool != null),
            tap(() => this.paging.patchState({ loading: true })),
            switchMap(([_, pool, query, projection, showArchived]) =>
                this.taskPoolService
                    .getTasks(pool.id, {
                        q: query,
                        projection: projection,
                        showArchived: showArchived,
                    })
                    .pipe(
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((error) => {
                            this.paging.patchError(error);
                            return EMPTY;
                        })
                    )
            ),
            tap((page) => this.paging.patchPage(page))
        )
    );

    readonly loadTasks$ = this.effect(() =>
        combineLatest([
            this.pool$,
            this.debouncedQuery$,
            this.paging.projection$,
            this.showArchived$,
        ]).pipe(
            filter(([pool]) => pool != null),
            tap(() => this.paging.patchState({ loading: true })),
            switchMap(([pool, query, projection, showArchived]) =>
                this.taskPoolService
                    .getTasks(pool.id, {
                        q: query,
                        showArchived: showArchived,
                        projection: projection,
                    })
                    .pipe(
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((error) => {
                            this.paging.patchError(error);
                            return EMPTY;
                        })
                    )
            ),
            tap((page) => this.paging.patchPage(page))
        )
    );
}
