import { Injectable } from '@angular/core';
import { catchError, switchMap, tap, withLatestFrom } from 'rxjs/operators';
import { EMPTY, Observable } from 'rxjs';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { Router } from '@angular/router';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { BaseComponentStore } from 'app/shared/store/base-component.store';
import { TaskPoolService } from 'app/tasks/services/task-pool.service';
import { TaskPool } from 'app/tasks/models/task-pool';

interface TaskUploadState {
    pool: TaskPool;
    importing: boolean;
}

export const initialState: TaskUploadState = {
    pool: null,
    importing: false,
};

interface ImportTaskParams {
    file: File;
    onSuccess: (Task) => void;
}

@Injectable()
export class TaskUploadStore extends BaseComponentStore<TaskUploadState> {
    public readonly importing$ = this.select((s) => s.importing);
    public readonly pool$ = this.select((s) => s.pool);

    constructor(
        private store: Store<SubatoState>,
        private taskPoolService: TaskPoolService,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService,
        private router: Router
    ) {
        super(initialState);
    }

    readonly setPool = this.updater((state, pool: TaskPool) => ({
        ...state,
        pool: pool,
    }));

    readonly importTask = this.effect((params$: Observable<ImportTaskParams>) =>
        params$.pipe(
            tap((params) => this.patchState({ importing: true })),
            withLatestFrom(this.pool$),
            switchMap(([params, pool]) =>
                this.taskPoolService.importTask(pool.id, params.file).pipe(
                    tap((task) => {
                        this.notificationService.showSuccess(
                            'Aufgabe erfolgreich importiert'
                        );

                        this.patchState({ importing: false });
                        params.onSuccess(task);
                    }),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.patchState({ importing: false });

                        this.notificationService.showError(
                            this.globalErrorHandler.format(
                                'Fehler beim Importieren der Aufgabe',
                                error
                            )
                        );

                        return EMPTY;
                    })
                )
            )
        )
    );
}
