import { Component, OnInit, ViewChild } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { BaseComponent } from 'app/shared/misc/base.component';
import { catchError, map, takeUntil } from 'rxjs/operators';
import { ImportLogViewerComponent } from 'app/tasks/pages/task-pool/components/import-log-viewer/import-log-viewer.component';
import { TaskListComponent } from 'app/tasks/pages/task-pool/components/task-list/task-list.component';
import { ParamHelper } from 'app/shared/misc/helper/param.helper';
import { ErrorContainerConfig } from 'app/shared/error/error-container-config';
import { SubatoState } from 'app/store/app.reducers';
import { Store } from '@ngrx/store';
import { combineLatest, EMPTY } from 'rxjs';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { TaskPoolPageStore } from 'app/tasks/pages/task-pool/task-pool.page.store';
import { ifAttached } from 'app/shared/route-reuse/route-reuse-life.cycle.directive';
import { PageHeaderAction } from 'app/shared/page-header/page-header/page-header.component';
import { TaskPool } from 'app/tasks/models/task-pool';

@Component({
    templateUrl: './task-pool.page.component.html',
    styleUrls: ['./task-pool.page.component.scss'],
    providers: [TaskPoolPageStore],
})
export class TaskPoolPageComponent extends BaseComponent implements OnInit {
    @ViewChild(ImportLogViewerComponent, { static: false })
    logsComponent!: ImportLogViewerComponent;

    @ViewChild(TaskListComponent, { static: false })
    tasksComponent!: TaskListComponent;

    poolState$ = this.taskPoolDetailStore.poolState$;
    syncing$ = this.taskPoolDetailStore.syncing$;

    errorConfig: ErrorContainerConfig = {
        notFound: {
            message: 'Diesen Aufgaben-Pool konnten wir leider nicht finden.',
        },
    };

    actions$ = combineLatest([this.poolState$, this.syncing$]).pipe(
        takeUntil(this.destroy$),
        ifAttached(this),
        map(([poolState, syncing]) =>
            this.buildActions(poolState.pool, syncing)
        )
    );

    constructor(
        private taskPoolDetailStore: TaskPoolPageStore,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler,
        private route: ActivatedRoute
    ) {
        super();
    }

    ngOnInit(): void {
        this.route.paramMap
            .pipe(
                takeUntil(this.destroy$),
                ifAttached(this),
                map((params) => ParamHelper.parseIdOrThrow(params, 'pool_id')),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.taskPoolDetailStore.patchPool({ error: err });
                    return EMPTY;
                })
            )
            .subscribe((poolId) => this.taskPoolDetailStore.loadPool(poolId));
    }

    onAttach() {
        super.onAttach();
        this.taskPoolDetailStore.onAttach();
    }

    onDetach() {
        super.onDetach();
        this.taskPoolDetailStore.onDetach();
    }

    onTaskUploaded() {
        this.tasksComponent.refresh();
    }

    sync() {
        this.taskPoolDetailStore.syncPool({
            done: () => {
                this.tasksComponent.refresh();
                this.logsComponent.refresh();
            },
        });
    }

    buildActions(pool: TaskPool, syncing: boolean): PageHeaderAction[] {
        let actions: PageHeaderAction[] = [];
        if (pool != null) {
            if (pool.linkedToRepository) {
                actions.push({
                    name: 'Synchronisieren',
                    icon: 'refresh-outline',
                    click: () => this.sync(),
                    loading: syncing,
                    disabled: syncing,
                    status: 'primary',
                });
            }

            actions.push({
                name: 'Bearbeiten',
                icon: 'edit-2-outline',
                link: {
                    commands: ['edit'],
                },
            });
        }

        return actions;
    }
}
