import { Injectable } from '@angular/core';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { EMPTY, Observable } from 'rxjs';
import { BaseComponentStore } from 'app/shared/store/base-component.store';

import {
    loadPool,
    TaskPoolState,
} from 'app/tasks/shared/operators/task-pool.operators';
import {
    catchError,
    map,
    switchMap,
    tap,
    withLatestFrom,
} from 'rxjs/operators';
import { TaskPoolService } from 'app/tasks/services/task-pool.service';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { NotificationService } from 'app/shared/notifications/notification.service';
import { Router } from '@angular/router';

interface TaskPoolPageState {
    poolState: TaskPoolState;
    syncing: boolean;
}

export const initialState: TaskPoolPageState = {
    poolState: { pool: null, error: null, loading: true },
    syncing: false,
};

interface SyncParams {
    done: () => void;
}

@Injectable()
export class TaskPoolPageStore extends BaseComponentStore<TaskPoolPageState> {
    public readonly pool$ = this.select((s) => s.poolState.pool);
    public readonly poolState$ = this.select((s) => s.poolState);
    public readonly syncing$ = this.select((s) => s.syncing);

    constructor(
        private store: Store<SubatoState>,
        private poolService: TaskPoolService,
        private globalErrorHandler: GlobalErrorHandler,
        private notificationService: NotificationService,
        private router: Router
    ) {
        super(initialState);
    }

    readonly patchPool = this.updaterFor('poolState');

    readonly loadPool = this.effect((poolId$: Observable<number>) =>
        poolId$.pipe(loadPool((s) => this.patchPool(s)))
    );

    readonly syncPool = this.effect((trigger$: Observable<SyncParams>) =>
        trigger$.pipe(
            withLatestFrom(this.pool$),
            tap(() => this.patchState({ syncing: true })),
            switchMap(([params, task]) =>
                this.poolService.sync(task.id).pipe(
                    map((result) => ({ params, result })),
                    tap(({ params, result }) => {
                        this.patchState({ syncing: false });

                        if (result.problems == 0) {
                            this.notificationService.showSuccess(
                                'Synchronisation erfolgreich'
                            );
                        } else {
                            this.notificationService.showWarning(
                                'Bei der Synchronisation sind Probleme aufgetreten'
                            );
                        }

                        params.done();
                    }),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((error) => {
                        this.patchState({ syncing: false });

                        this.notificationService.showError(
                            this.globalErrorHandler.format(
                                'Fehler bei Synchronisation',
                                error
                            )
                        );

                        return EMPTY;
                    })
                )
            )
        )
    );
}
