import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { BaseComponent } from 'app/shared/misc/base.component';
import { TaskService } from 'app/tasks/services/task.service';
import { combineLatest, EMPTY } from 'rxjs';
import { catchError, filter, map, takeUntil } from 'rxjs/operators';
import { ParamHelper } from 'app/shared/misc/helper/param.helper';
import { ErrorContainerConfig } from 'app/shared/error/error-container-config';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { TaskPageStore } from 'app/tasks/pages/task/task.page.store';
import { PageHeaderAction } from 'app/shared/page-header/page-header/page-header.component';
import { Task } from 'app/tasks/models/task';
import { ConfirmationDialogComponent } from 'app/shared/dialogs/confirmation-dialog/confirmation-dialog.component';
import { NbDialogService } from '@nebular/theme';

@Component({
    selector: 'ngx-task',
    templateUrl: './task.page.component.html',
    styleUrls: ['./task.page.component.scss'],
    providers: [TaskPageStore],
})
export class TaskPageComponent extends BaseComponent implements OnInit {
    taskState$ = this.taskDetailStore.taskState$;
    taskContentState$ = this.taskDetailStore.taskContentState$;
    archiving$ = this.taskDetailStore.archiving$;

    errorConfig: ErrorContainerConfig = {
        notFound: {
            message: 'Diese Aufgabe konnten wir leider nicht finden.',
        },
    };

    actions$ = combineLatest([this.taskState$, this.archiving$]).pipe(
        takeUntil(this.destroy$),
        map(([taskState, archiving]) =>
            this.buildActions(taskState.task, archiving)
        )
    );

    constructor(
        private taskService: TaskService,
        private taskDetailStore: TaskPageStore,
        private router: Router,
        private globalErrorHandler: GlobalErrorHandler,
        private dialogService: NbDialogService,
        private route: ActivatedRoute
    ) {
        super();
    }

    ngOnInit(): void {
        this.route.paramMap
            .pipe(
                takeUntil(this.destroy$),
                map((params) => ParamHelper.parseIdOrThrow(params, 'task_id')),
                catchError((err) => this.globalErrorHandler.handleError(err)),
                catchError((err) => {
                    this.taskDetailStore.patchTask({ error: err });
                    return EMPTY;
                })
            )
            .subscribe((exerciseId) =>
                this.taskDetailStore.loadTask(exerciseId)
            );
    }

    archiveTask(task: Task) {
        this.dialogService
            .open(ConfirmationDialogComponent, {
                context: {
                    title: `Archivierung von Aufgabe: ${task.name}`,
                    text: `Bist du wirklich sicher, dass du die Aufgabe '${task.name}' archivieren willst? Archivierte Aufgaben sind nicht mehr verfügbar und assoziierte Lösungen können nicht mehr direkt eingesehen werden. Die Aufgabe und die Verbindungen zu Übungsblättern und Lösungen bleiben jedoch erhalten und können von Admins wiederhergestellt werden.`,
                    actionText: 'Archivieren',
                },
            })
            .onClose.pipe(filter((result) => result && result.confirmed))
            .subscribe(() => this.taskDetailStore.archiveTask());
    }

    buildActions(task: Task, archiving: boolean): PageHeaderAction[] {
        let actions: PageHeaderAction[] = [];
        if (task != null) {
            actions.push({
                name: 'Herunterladen',
                icon: 'download-outline',
                externalLink: {
                    href: this.taskService.getSpecUrl(task.id),
                    download: true,
                },
            });

            if (!task.pool.linkedToRepository) {
                actions.push({
                    name: 'Archivieren',
                    icon: 'archive-outline',
                    click: () => this.archiveTask(task),
                    loading: archiving,
                    disabled: archiving,
                    status: 'danger',
                });
            }
        }

        return actions;
    }
}
