import { Injectable } from '@angular/core';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { EMPTY, Observable } from 'rxjs';
import { BaseComponentStore } from 'app/shared/store/base-component.store';

import {
    loadTaskContent,
    TaskContentState,
} from 'app/tasks/shared/operators/task-content.operators';
import { loadTask, TaskState } from 'app/tasks/shared/operators/task.operators';
import {
    catchError,
    filter,
    map,
    switchMap,
    tap,
    withLatestFrom,
} from 'rxjs/operators';
import { TaskService } from 'app/tasks/services/task.service';
import { ActivatedRoute, Router } from '@angular/router';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { NotificationService } from 'app/shared/notifications/notification.service';

interface TaskPageState {
    taskState: TaskState;
    taskContentState: TaskContentState;
    archiving: boolean;
}

export const initialState: TaskPageState = {
    taskState: { task: null, error: null, loading: true },
    taskContentState: { content: null, error: null, loading: true },
    archiving: false,
};

@Injectable()
export class TaskPageStore extends BaseComponentStore<TaskPageState> {
    principal$ = this.store.select((s) => s.auth.principal);
    public readonly task$ = this.select((s) => s.taskState.task);
    public readonly taskState$ = this.select((s) => s.taskState);

    public readonly taskContent$ = this.select(
        (state) => state.taskContentState.content
    );
    public readonly taskContentState$ = this.select((s) => s.taskContentState);
    public readonly archiving$ = this.select((s) => s.archiving);

    constructor(
        private store: Store<SubatoState>,
        private taskService: TaskService,
        private router: Router,
        private globalErrorHandler: GlobalErrorHandler,
        private route: ActivatedRoute,
        private notificationService: NotificationService
    ) {
        super(initialState);
    }

    readonly patchTaskContent = this.updaterFor('taskContentState');
    readonly patchTask = this.updaterFor('taskState');

    readonly loadTask = this.effect((taskId$: Observable<number>) =>
        taskId$.pipe(loadTask((s) => this.patchTask(s)))
    );

    readonly archiveTask = this.effect<void>((trigger$) =>
        trigger$.pipe(
            withLatestFrom(this.task$),
            map(([_, task]) => task),
            tap(() => this.patchState({ archiving: true })),
            switchMap((task) =>
                this.taskService
                    .archive(task.id)
                    .pipe(map(() => task))
                    .pipe(
                        tap((task) => {
                            this.patchState({ archiving: false });
                            this.router.navigate(['task_pools', task.pool.id]);
                            this.notificationService.showSuccess(
                                'Aufgabe erfolgreich archiviert.'
                            );
                        }),
                        catchError((err) =>
                            this.globalErrorHandler.handleError(err)
                        ),
                        catchError((error) => {
                            this.patchState({ archiving: false });

                            this.notificationService.showError(
                                this.globalErrorHandler.format(
                                    'Fehler beim Archivieren der Aufgabe',
                                    error
                                )
                            );

                            return EMPTY;
                        })
                    )
            )
        )
    );

    readonly loadTaskContent$ = this.effect(() =>
        this.task$.pipe(
            filter((task) => task != null && !task.archived),
            loadTaskContent((s) => this.patchTaskContent(s))
        )
    );
}
