import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { environment } from 'environments/environment';
import { Observable } from 'rxjs';
import { Task } from 'app/tasks/models/task';
import { map } from 'rxjs/operators';
import { TaskPool } from '../models/task-pool';
import { CreateTaskPoolDto } from '../dto/create-task-pool.dto';
import { Course } from 'app/courses/models/course';
import { DataQueryParams } from 'app/shared/paging/data-query-params';
import { Page } from 'app/shared/paging/page';
import { CreateTaskPoolResultDto } from '../dto/create-task-pool-result.dto';
import { TaskImportLog } from '../models/task-import-log';
import { BaseService } from 'app/shared/misc/base.service';
import { TaskImportResult } from '../models/task-import-result';

export interface TaskParams {
    q?: string;
    showArchived?: boolean;
    projection?: DataQueryParams;
}

@Injectable({
    providedIn: 'root',
})
export class TaskPoolService extends BaseService {
    constructor(private http: HttpClient) {
        super();
    }

    getById(id: number): Observable<TaskPool> {
        return this.http
            .get<TaskPool>(`${environment.backendUrl}/task_pool/${id}`)
            .pipe(map((pool) => new TaskPool(pool)));
    }

    getAll(): Observable<TaskPool[]> {
        return this.http
            .get<TaskPool[]>(`${environment.backendUrl}/task_pool`)
            .pipe(map((pools) => pools.map((pool) => new TaskPool(pool))));
    }

    getCourses(id: number): Observable<Course[]> {
        return this.http
            .get<Course[]>(`${environment.backendUrl}/task_pool/${id}/courses`)
            .pipe(
                map((courses) => courses.map((course) => new Course(course)))
            );
    }

    getLogs(id: number): Observable<TaskImportLog[]> {
        return this.http
            .get<TaskImportLog[]>(
                `${environment.backendUrl}/task_pool/${id}/logs`
            )
            .pipe(map((logs) => logs.map((log) => new TaskImportLog(log))));
    }

    getTasks(id: number, params?: TaskParams): Observable<Page<Task>> {
        let httpParams = DataQueryParams.toHttpParams(params?.projection);

        if (params?.q) {
            httpParams = httpParams.set('q', params.q);
        }

        if (params?.showArchived != null) {
            httpParams = httpParams.set('showArchived', params.showArchived);
        }

        return this.http
            .get<Page<Task>>(
                `${environment.backendUrl}/task_pool/${id}/tasks`,
                {
                    params: httpParams,
                }
            )
            .pipe(
                map((page) => new Page<Task>(page, (item) => new Task(item)))
            );
    }

    create(dto: CreateTaskPoolDto): Observable<CreateTaskPoolResultDto> {
        return this.http
            .post<CreateTaskPoolResultDto>(
                `${environment.backendUrl}/task_pool`,
                dto
            )
            .pipe(
                map((dto) => {
                    return new CreateTaskPoolResultDto(dto);
                })
            );
    }

    update(poolId: number, dto: CreateTaskPoolDto): Observable<TaskPool> {
        return this.http
            .put<TaskPool>(`${environment.backendUrl}/task_pool/${poolId}`, dto)
            .pipe(
                map((pool) => {
                    return new TaskPool(pool);
                })
            );
    }

    importTask(poolId: number, file: File): Observable<Task> {
        var formData = new FormData();
        formData.append('file', file);

        return this.http
            .put<Task>(
                `${environment.backendUrl}/task_pool/${poolId}/tasks`,
                formData
            )
            .pipe(
                map((task) => {
                    return new Task(task);
                })
            );
    }

    delete(id: number): Observable<void> {
        return this.http.delete<void>(
            `${environment.backendUrl}/task_pool/${id}`
        );
    }

    sync(poolId: number): Observable<TaskImportResult> {
        return this.http.put<TaskImportResult>(
            `${environment.backendUrl}/task_pool/${poolId}/sync`,
            {}
        );
    }
}
