import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { environment } from 'environments/environment';
import { Observable } from 'rxjs';
import { Task } from 'app/tasks/models/task';
import { map } from 'rxjs/operators';
import { BaseService } from 'app/shared/misc/base.service';
import { FileRef } from '../models/file-ref';
import { DataQueryParams } from 'app/shared/paging/data-query-params';
import { Page } from 'app/shared/paging/page';
import { DownloadService } from 'app/shared/download/download.service';

export interface TaskFilter {
    q?: string;
    poolId?: number;
    showArchived?: boolean;
}

@Injectable({
    providedIn: 'root',
})
export class TaskService extends BaseService {
    constructor(
        private http: HttpClient,
        private downloadService: DownloadService
    ) {
        super();
    }

    getAll(
        filter?: TaskFilter,
        projection?: DataQueryParams
    ): Observable<Page<Task>> {
        let httpParams = DataQueryParams.toHttpParams(projection);

        if (filter?.q) {
            httpParams = httpParams.set('q', filter.q);
        }

        if (filter?.poolId) {
            httpParams = httpParams.set('poolId', filter.poolId);
        }

        if (filter?.showArchived != null) {
            httpParams = httpParams.set('showArchived', filter.showArchived);
        }

        return this.http
            .get<Page<Task>>(`${environment.backendUrl}/task`, {
                params: httpParams,
            })
            .pipe(
                map((page) => new Page<Task>(page, (item) => new Task(item)))
            );
    }

    getById(id: number): Observable<Task> {
        return this.http
            .get<Task>(`${environment.backendUrl}/task/${id}`)
            .pipe(map((task) => new Task(task)));
    }

    getContent(id: number): Observable<string> {
        return this.http.get(`${environment.backendUrl}/task/${id}/content`, {
            responseType: 'text',
        });
    }

    getFiles(id: number): Observable<FileRef[]> {
        return this.http
            .get<FileRef[]>(`${environment.backendUrl}/task/${id}/files`)
            .pipe(map((files) => files.map((f) => new FileRef(f))));
    }

    getSpecUrl(taskId: number): string {
        return `${environment.backendUrl}/task/${taskId}/spec`;
    }

    getBundleUrl(taskId: number): string {
        return `${environment.backendUrl}/task/${taskId}/bundle`;
    }

    getFileUrl(taskId: number, ref: FileRef): string {
        return `${environment.backendUrl}/task/${taskId}/files/${ref.file.uuid}/download`;
    }

    getFile(taskId: number, ref: FileRef): Observable<string> {
        return this.http
            .get(this.getFileUrl(taskId, ref), {
                responseType: 'text',
            })
            .pipe(map((data: any) => data));
    }

    archive(id: number): Observable<void> {
        return this.http.delete<void>(`${environment.backendUrl}/task/${id}`);
    }
}
