import { EMPTY, Observable } from 'rxjs';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { inject, Injectable } from '@angular/core';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { SubatoError } from 'app/shared/error/error';
import { Language } from 'app/tasks/models/language';
import { LanguageService } from 'app/tasks/services/language.service';

export interface LanguagesState {
    languages: Language[];
    error: SubatoError;
    loading: boolean;
}

export interface LanguageState {
    language: Language | null;
    error: SubatoError;
    loading: boolean;
}

export function loadLanguages(
    updaterFn: (state: Partial<LanguagesState>) => void
) {
    let op = inject(LanguageOperators);
    return (trigger$: Observable<void>) =>
        op.loadLanguages(trigger$, updaterFn);
}

export function loadLanguage(
    updaterFn: (state: Partial<LanguageState>) => void
) {
    let op = inject(LanguageOperators);
    return (languageId$: Observable<string>) =>
        op.loadLanguage(languageId$, updaterFn);
}

@Injectable({ providedIn: 'root' })
class LanguageOperators {
    constructor(
        private languageService: LanguageService,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler
    ) {}

    public loadLanguages(
        trigger$: Observable<void>,
        updaterFn: (state: Partial<LanguagesState>) => void
    ) {
        return trigger$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((languageId) =>
                this.languageService.getAll().pipe(
                    tap((languages) =>
                        updaterFn({
                            languages: languages,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            languages: [],
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }

    public loadLanguage(
        languageId$: Observable<string>,
        updaterFn: (state: Partial<LanguageState>) => void
    ) {
        return languageId$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((languageId) =>
                this.languageService.getById(languageId).pipe(
                    tap((language) =>
                        updaterFn({
                            language: language,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            language: null,
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }
}
