import { EMPTY, Observable } from 'rxjs';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { inject, Injectable } from '@angular/core';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { SubatoError } from 'app/shared/error/error';
import { TaskService } from 'app/tasks/services/task.service';
import { Task } from 'app/tasks/models/task';

export interface TaskContentState {
    content: string | null;
    error: SubatoError;
    loading: boolean;
}

export function loadTaskContent(
    updaterFn: (state: Partial<TaskContentState>) => void
) {
    let op = inject(TaskInstanceOperators);
    return (task$: Observable<Task>) => op.loadTaskContent(task$, updaterFn);
}

@Injectable({ providedIn: 'root' })
class TaskInstanceOperators {
    constructor(
        private taskService: TaskService,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler
    ) {}

    public loadTaskContent(
        task$: Observable<Task>,
        updaterFn: (state: Partial<TaskContentState>) => void
    ) {
        return task$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((task) =>
                this.taskService.getContent(task.id).pipe(
                    tap((content) =>
                        updaterFn({
                            content: content,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            content: null,
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }
}
