import { EMPTY, Observable } from 'rxjs';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { inject, Injectable } from '@angular/core';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { SubatoError } from 'app/shared/error/error';
import { FileRef } from 'app/tasks/models/file-ref';
import { Exercise } from 'app/exercises/models/exercise';
import { Task } from 'app/tasks/models/task';
import { ExerciseService } from 'app/exercises/services/exercise.service';
import { TaskService } from 'app/tasks/services/task.service';

export interface TaskFileContentState {
    content: string;
    error: SubatoError;
    loading: boolean;
}

export interface TaskFilesState {
    files: FileRef[];
    error: SubatoError;
    loading: boolean;
}

interface LoadFileParams {
    exercise: Exercise;
    task: Task;
    ref: FileRef;
}

interface LoadFilesParams {
    exercise: Exercise;
    task: Task;
}

export function loadFile(
    updaterFn: (state: Partial<TaskFileContentState>) => void
) {
    let op = inject(TaskFileOperators);
    return (params$: Observable<LoadFileParams>) =>
        op.loadFile(params$, updaterFn);
}

export function loadFiles(updaterFn: (state: Partial<TaskFilesState>) => void) {
    let op = inject(TaskFileOperators);
    return (params$: Observable<LoadFilesParams>) =>
        op.loadFiles(params$, updaterFn);
}

@Injectable({ providedIn: 'root' })
class TaskFileOperators {
    constructor(
        private exerciseService: ExerciseService,
        private taskService: TaskService,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler
    ) {}

    public loadFiles(
        params$: Observable<LoadFilesParams>,
        updaterFn: (state: Partial<TaskFilesState>) => void
    ) {
        return params$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((params) =>
                this.getFiles(params.exercise, params.task).pipe(
                    tap((files) =>
                        updaterFn({
                            files: files,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            files: null,
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }

    private getFiles(exercise: Exercise, task: Task) {
        return exercise != null
            ? this.exerciseService.getTaskInstanceFiles(exercise.id, task.id)
            : this.taskService.getFiles(task.id);
    }

    public loadFile(
        params$: Observable<LoadFileParams>,
        updaterFn: (state: Partial<TaskFileContentState>) => void
    ) {
        return params$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((params) =>
                this.getFile(params.exercise, params.task, params.ref).pipe(
                    tap((content) =>
                        updaterFn({
                            content: content,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            content: null,
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }

    private getFile(exercise: Exercise, task: Task, ref: FileRef) {
        return exercise != null
            ? this.exerciseService.getFile(exercise.id, task.id, ref)
            : this.taskService.getFile(task.id, ref);
    }
}
