import { EMPTY, Observable } from 'rxjs';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { inject, Injectable } from '@angular/core';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { SubatoError } from 'app/shared/error/error';
import { TaskPoolService } from 'app/tasks/services/task-pool.service';
import { TaskImportLog } from 'app/tasks/models/task-import-log';

export interface TaskImportLogsState {
    logs: TaskImportLog[];
    error: SubatoError;
    loading: boolean;
}

export function loadLogs(
    updaterFn: (state: Partial<TaskImportLogsState>) => void
) {
    let op = inject(TaskImportLogOperators);
    return (poolId$: Observable<number>) => op.loadLogs(poolId$, updaterFn);
}

@Injectable({ providedIn: 'root' })
class TaskImportLogOperators {
    constructor(
        private taskPoolService: TaskPoolService,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler
    ) {}

    public loadLogs(
        poolId$: Observable<number>,
        updaterFn: (state: Partial<TaskImportLogsState>) => void
    ) {
        return poolId$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((poolId) =>
                this.taskPoolService.getLogs(poolId).pipe(
                    tap((logs) =>
                        updaterFn({
                            logs: logs,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            logs: null,
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }
}
