import { EMPTY, Observable } from 'rxjs';
import { catchError, switchMap, tap } from 'rxjs/operators';
import { inject, Injectable } from '@angular/core';
import { GlobalErrorHandler } from 'app/shared/error/global-error-handler';
import { Store } from '@ngrx/store';
import { SubatoState } from 'app/store/app.reducers';
import { SubatoError } from 'app/shared/error/error';
import { TaskPool } from 'app/tasks/models/task-pool';
import { TaskPoolService } from 'app/tasks/services/task-pool.service';

export interface TaskPoolsState {
    pools: TaskPool[];
    error: SubatoError;
    loading: boolean;
}

export interface TaskPoolState {
    pool: TaskPool | null;
    error: SubatoError;
    loading: boolean;
}

export function loadPools(updaterFn: (state: Partial<TaskPoolsState>) => void) {
    let op = inject(TaskPoolOperators);
    return (trigger$: Observable<void>) => op.loadPools(trigger$, updaterFn);
}

export function loadPool(updaterFn: (state: Partial<TaskPoolState>) => void) {
    let op = inject(TaskPoolOperators);
    return (poolId$: Observable<number>) => op.loadPool(poolId$, updaterFn);
}

@Injectable({ providedIn: 'root' })
class TaskPoolOperators {
    constructor(
        private taskPoolService: TaskPoolService,
        private store: Store<SubatoState>,
        private globalErrorHandler: GlobalErrorHandler
    ) {}

    public loadPools(
        trigger$: Observable<void>,
        updaterFn: (state: Partial<TaskPoolsState>) => void
    ) {
        return trigger$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((params) =>
                this.taskPoolService.getAll().pipe(
                    tap((pools) =>
                        updaterFn({
                            pools: pools,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            pools: [],
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }

    public loadPool(
        poolId$: Observable<number>,
        updaterFn: (state: Partial<TaskPoolState>) => void
    ) {
        return poolId$.pipe(
            tap(() =>
                updaterFn({
                    loading: true,
                })
            ),
            switchMap((taskId) =>
                this.taskPoolService.getById(taskId).pipe(
                    tap((pool) =>
                        updaterFn({
                            pool: pool,
                            loading: false,
                            error: null,
                        })
                    ),
                    catchError((err) =>
                        this.globalErrorHandler.handleError(err)
                    ),
                    catchError((err) => {
                        updaterFn({
                            pool: null,
                            error: err,
                            loading: false,
                        });
                        return EMPTY;
                    })
                )
            )
        );
    }
}
