package de.hsrm.sls.subato.aor.term.usecase;

import de.hsrm.sls.subato.aor.term.dto.ModulePartInstance;
import de.hsrm.sls.subato.aor.term.dto.ModulePartInstanceEvent;
import de.hsrm.sls.subato.courses.event.model.CourseUnitEvent;
import de.hsrm.sls.subato.courses.event.model.CourseUnitEventDefinition;
import de.hsrm.sls.subato.courses.unit.model.CourseUnit;
import de.hsrm.sls.subato.courses.unit.model.Weekday;
import de.hsrm.sls.subato.courses.unit.repository.CourseUnitRepository;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
@Transactional
class ImportEventsUseCase {

    @Autowired
    CourseUnitRepository courseUnitRepository;

    public CourseUnit importEvents(CourseUnit courseUnit, ModulePartInstance mpi) {
        if (mpi.getEvents().size() == 0) {
            return courseUnit;
        }

        courseUnit.getEventDefinitions().clear();

        var allUnitEvents = new ArrayList<CourseUnitEvent>();
        for (var mpiEvent : mpi.getEvents()) {
            var events = createEvents(courseUnit, mpiEvent);
            var eventDefinition = createEventDefinition(courseUnit, mpiEvent, events);
            events.forEach(e -> e.setEventDefinition(eventDefinition));

            allUnitEvents.addAll(events);
            courseUnit.getEventDefinitions().add(eventDefinition);
        }

        removeMissingEvents(courseUnit, allUnitEvents);
        return courseUnitRepository.save(courseUnit);
    }

    private CourseUnitEventDefinition createEventDefinition(CourseUnit courseUnit,
        ModulePartInstanceEvent mpiEvent, List<CourseUnitEvent> events) {
        var eventDefinition = new CourseUnitEventDefinition();
        eventDefinition.setUnit(courseUnit);
        eventDefinition.setRoom(mpiEvent.getRoom());
        eventDefinition.setTime(mpiEvent.getStart());

        var weekday = mpiEvent.getWeekday() != null ? getWeekday(mpiEvent) : getWeekdayOld(events);
        eventDefinition.setWeekday(weekday);

        return eventDefinition;
    }

    private Weekday getWeekday(ModulePartInstanceEvent mpiEvent) {
        var weekdayIdx = mpiEvent.getWeekday();
        var ourWeekdayIdx = Math.floorMod(weekdayIdx - 1,
            Weekday.values().length); // AoR: (Su-Sa = 0-6), Subato: (Mo-Su = 0-6)
        return Weekday.values()[ourWeekdayIdx];
    }

    // required because of old term data
    // todo: remove/cleanup
    private Weekday getWeekdayOld(List<CourseUnitEvent> evts) {
        if (evts.size() > 0) {
            var weekdayIdx =
                evts.stream().findFirst().get().getTakesPlaceAt().getDayOfWeek().getValue() - 1;
            return Weekday.values()[weekdayIdx];
        }

        return null;
    }

    private List<CourseUnitEvent> createEvents(CourseUnit courseUnit,
        ModulePartInstanceEvent mpiEvent) {
        List<CourseUnitEvent> events = new ArrayList<>();

        for (var termin : mpiEvent.getTermine()) {
            var date = LocalDate.parse(termin);
            var takesPlaceAt = LocalDateTime.of(date, mpiEvent.getStart());

            CourseUnitEvent event = getOrCreateEvent(courseUnit, takesPlaceAt);
            events.add(event);
        }

        return events;
    }

    private CourseUnitEvent getOrCreateEvent(CourseUnit courseUnit, LocalDateTime takesPlaceAt) {
        var event = courseUnit.getEvents().stream()
            .filter(e -> e.getTakesPlaceAt().equals(takesPlaceAt)).findFirst().orElse(null);
        if (event == null) {
            event = new CourseUnitEvent();
            event.setUnit(courseUnit);
            event.setTakesPlaceAt(takesPlaceAt);
            courseUnit.getEvents().add(event);
        }
        return event;
    }

    private void removeMissingEvents(CourseUnit courseUnit, ArrayList<CourseUnitEvent> events) {
        var removeEvents = courseUnit.getEvents().stream()
            .filter(e -> !events.contains(e))
            .collect(Collectors.toList());
        courseUnit.getEvents().removeAll(removeEvents);
    }
}
