package de.hsrm.sls.subato.aor.term.usecase;

import de.hsrm.sls.subato.aor.term.dto.ModulePartInstance;
import de.hsrm.sls.subato.aor.term.dto.TermData;
import de.hsrm.sls.subato.courses.course.model.Course;
import de.hsrm.sls.subato.courses.course.repository.CourseRepository;
import de.hsrm.sls.subato.courses.unit.CreateCourseUnitRequest;
import de.hsrm.sls.subato.courses.unit.model.CourseUnitType;
import de.hsrm.sls.subato.courses.unit.repository.CourseUnitRepository;
import de.hsrm.sls.subato.courses.unit.usecase.CreateUnitUseCase;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
@Transactional
public class ImportUnitsUseCase {

    @Autowired
    CourseUnitRepository courseUnitRepository;

    @Autowired
    ImportLecturersUseCase importLecturersUseCase;

    @Autowired
    ImportParticipantsUseCase importParticipantsUseCase;

    @Autowired
    ImportEventsUseCase importEventsUseCase;

    @Autowired
    CourseRepository courseRepository;

    @Autowired
    CreateUnitUseCase createUnitUseCase;

    public void importUnits(Course course, TermData termData) {
        course = clearUnits(course);

        var partInstances = getModulePartInstances(termData, course);
        var groupedPartInstances = groupPartInstances(partInstances);

        for (var entry : groupedPartInstances.entrySet()) {
            var modulePartInstances = entry.getValue();

            if (modulePartInstances.size() == 0) {
                continue;
            }

            var courseType = new CourseUnitTypeConverter().convert(entry.getKey().courseType());
            if (courseType == null) {
                continue;
            }

            var hasStudents = modulePartInstances.stream()
                .anyMatch(mpi -> mpi.getStudents().size() != 0);
            if (!hasStudents) {
                continue;
            }

            var leadingModulePartInstance = modulePartInstances.get(0);

            var lecturers = importLecturersUseCase.importLecturers(leadingModulePartInstance);
            var participants = importParticipantsUseCase.importParticipants(modulePartInstances);

            var group = entry.getKey().group().isEmpty() ? "A" : entry.getKey().group();

            var request = new CreateCourseUnitRequest.Builder()
                .setCourse(course)
                .setName(group)
                .setSupplementId(leadingModulePartInstance.getId())
                .setType(courseType)
                .setLecturers(lecturers)
                .setParticipants(participants)
                .build();

            var result = createUnitUseCase.create(request);
            importEventsUseCase.importEvents(result.unit(), leadingModulePartInstance);
        }
    }

    private Map<ModulePartInstance.Group, List<ModulePartInstance>> groupPartInstances(
        ArrayList<ModulePartInstance> partInstances) {
        Map<ModulePartInstance.Group, List<ModulePartInstance>> groupedPartInstances =
            partInstances.stream().collect(
                Collectors.groupingBy(p ->
                    new ModulePartInstance.Group(p.getGroup(), p.getLecturers(),
                        p.getModulePart().getCourseType())
                )
            );
        return groupedPartInstances;
    }

    private ArrayList<ModulePartInstance> getModulePartInstances(TermData termData, Course course) {
        var partInstances = new ArrayList<ModulePartInstance>();
        for (var programModule : course.getModules()) {
            var aorModuleInstance = termData.getModuleInstances().stream().filter(
                aorMi -> aorMi.getModuleNodeId().equals(programModule.getSupplementId())
            ).findFirst().orElse(null);

            if (aorModuleInstance == null) {
                // modul nicht im aktuellen semester vorhanden, sollte eigentlich nicht passieren
                continue;
            }

            partInstances.addAll(aorModuleInstance.getModulePartInstances());
        }
        return partInstances;
    }

    private Course clearUnits(Course course) {
        var units = course.getUnits().stream().filter(u -> u.getType() != CourseUnitType.Misc)
            .collect(Collectors.toList());
        course.getUnits().removeAll(units);
        course = courseRepository.save(course);
        return course;
    }
}
