package de.hsrm.sls.subato.assessment.progress;

import de.hsrm.sls.subato.assessment.progress.dto.UnitAssessmentProgressDto;
import de.hsrm.sls.subato.assessment.progress.dto.UnitAssessmentProgressMapper;
import de.hsrm.sls.subato.assessment.progress.model.UnitAssessmentProgress;
import de.hsrm.sls.subato.assessment.progress.usecase.GetAssessmentProgressUseCase;
import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.exercises.ExercisesAction;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.courses.course.CourseNotFoundException;
import de.hsrm.sls.subato.courses.unit.exception.CourseUnitNotFoundException;
import de.hsrm.sls.subato.courses.unit.repository.CourseUnitRepository;
import de.hsrm.sls.subato.exercises.exercise.ExerciseNotFoundException;
import de.hsrm.sls.subato.exercises.exercise.repository.ExerciseRepository;
import de.hsrm.sls.subato.shared.springext.ResponseStatusMapping;
import de.hsrm.sls.subato.shared.swagger.SubatoApiResponse;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import java.util.List;
import java.util.Optional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RestController
@Tag(name = "Bewertung")
public class AssessmentProgressController {

    @Autowired
    private ExerciseRepository exerciseRepository;

    @Autowired
    private PolicyEvaluator policy;

    @Autowired
    private UnitAssessmentProgressMapper unitAssessmentProgressMapper;

    @Autowired
    private GetAssessmentProgressUseCase getAssessmentProgressUseCase;

    @Autowired
    private CourseUnitRepository courseUnitRepository;

    @GetMapping("/exercise/{exerciseId}/assessment")
    @Operation(summary = "Zusammenfassung des Fortschritts der Abgabenbewertung (wie viele sind noch offen?) liefern")
    @SubatoApiResponse(ExerciseNotFoundException.class)
    @SubatoApiResponse(CourseUnitNotFoundException.class)
    @ResponseStatusMapping(exception = CourseNotFoundException.class, code = HttpStatus.BAD_REQUEST)
    @ResponseStatusMapping(exception = ExerciseNotFoundException.class, code = HttpStatus.BAD_REQUEST)
    List<UnitAssessmentProgressDto> getSubmissionsProgress(@PathVariable Long exerciseId,
        @RequestParam Optional<Long> unitId, @AuthenticationPrincipal SubatoPrincipal principal) {
        var exercise = exerciseRepository.findById(exerciseId)
            .orElseThrow(() -> new ExerciseNotFoundException(exerciseId));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(exercise)
            .setPermission(ExercisesAction.SUBMISSION_VIEW_PROGRESS).build();
        policy.checkAndRaise(context);

        List<UnitAssessmentProgress> unitProgresses;
        if (unitId.isPresent()) {
            var unit = courseUnitRepository.findById(unitId.get())
                .orElseThrow(() -> new CourseUnitNotFoundException(unitId.get()));
            var unitProgress = getAssessmentProgressUseCase.forExerciseAndUnit(exercise, unit);
            unitProgresses = List.of(unitProgress);
        } else {
            unitProgresses = getAssessmentProgressUseCase.forExercise(exercise);
        }

        return unitProgresses.stream()
            .map(unitProgress -> unitAssessmentProgressMapper.map(unitProgress)).toList();
    }
}
