package de.hsrm.sls.subato.assessment.progress.usecase;

import de.hsrm.sls.subato.assessment.progress.model.AssessmentProgress;
import de.hsrm.sls.subato.assessment.progress.model.BinaryAssessmentProgress;
import de.hsrm.sls.subato.assessment.progress.model.NullAssessmentProgress;
import de.hsrm.sls.subato.assessment.progress.model.PointsAssessmentProgress;
import de.hsrm.sls.subato.assessment.progress.model.UnitAssessmentProgress;
import de.hsrm.sls.subato.assessment.submission.model.Submission;
import de.hsrm.sls.subato.assessment.submission.repository.SubmissionFilter;
import de.hsrm.sls.subato.assessment.submission.repository.SubmissionQuery;
import de.hsrm.sls.subato.assessment.submission.repository.SubmissionRepository;
import de.hsrm.sls.subato.assessment.submission.repository.SubmissionSort;
import de.hsrm.sls.subato.courses.unit.model.CourseUnit;
import de.hsrm.sls.subato.courses.unit.model.CourseUnitType;
import de.hsrm.sls.subato.courses.unit.repository.CourseUnitRepository;
import de.hsrm.sls.subato.exercises.exercise.model.AssessmentMode;
import de.hsrm.sls.subato.exercises.exercise.model.Exercise;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class GetAssessmentProgressUseCase {

    @Autowired
    private SubmissionRepository submissionRepository;

    @Autowired
    private CourseUnitRepository courseUnitRepository;

    public UnitAssessmentProgress forExerciseAndUnit(Exercise exercise, CourseUnit unit) {
        var filter = new SubmissionFilter.Builder()
            .exerciseId(exercise.getId())
            .unitId(unit.getId())
            .build();
        var submissionQuery = new SubmissionQuery.Builder()
            .setFilter(filter)
            .setSort(SubmissionSort.bySubmittedBy())
            .build();
        var submissions = submissionRepository.findAll(submissionQuery);

        var unitProgress = new UnitAssessmentProgress();
        unitProgress.setUnit(unit);
        unitProgress.setProgress(createAssessmentProgress(exercise, submissions));

        var nextSubmission = submissions.stream().findFirst().orElse(null);
        if (nextSubmission != null) {
            unitProgress.setNextSubmissionId(nextSubmission.getId());
        }

        return unitProgress;
    }

    public List<UnitAssessmentProgress> forExercise(Exercise exercise) {
        var units = courseUnitRepository.findByCourse(exercise.getCourse()).stream()
            .filter(u -> u.getType() == CourseUnitType.Lab || u.getType() == CourseUnitType.Misc)
            .collect(Collectors.toList());

        var unitProgresses = new ArrayList<UnitAssessmentProgress>();

        for (var unit : units) {
            var unitProgress = forExerciseAndUnit(exercise, unit);
            unitProgresses.add(unitProgress);
        }

        return unitProgresses;
    }

    private static AssessmentProgress createAssessmentProgress(Exercise exercise,
        List<Submission> submissions) {
        if (exercise.getAssessmentMode() == AssessmentMode.Binary) {
            var progress = new BinaryAssessmentProgress();
            progress.setPassedTotal(
                submissions.stream().filter(s -> s.getPassed() != null && s.getPassed()).count());
            progress.setNotPassedTotal(
                submissions.stream().filter(s -> s.getPassed() != null && !s.getPassed()).count());
            progress.setUndecidedTotal(
                submissions.stream().filter(s -> s.getPassed() == null).count());
            progress.setTotal(submissions.size());
            return progress;
        }

        if (exercise.getAssessmentMode() == AssessmentMode.Points) {
            var progress = new PointsAssessmentProgress();
            progress.setFullyRatedTotal(submissions.stream()
                .filter(s -> s.getPointRatingCount() == s.getExercise().getTaskInstancesCount())
                .count());
            progress.setTotal(submissions.size());
            return progress;
        }

        if (exercise.getAssessmentMode() == null) {
            var progress = new NullAssessmentProgress();
            progress.setTotal(submissions.size());
            return progress;
        }

        return null;
    }
}
