package de.hsrm.sls.subato.assessment.submission;

import de.hsrm.sls.subato.assessment.submission.dto.SubmissionDetailDto;
import de.hsrm.sls.subato.assessment.submission.dto.SubmissionMapper;
import de.hsrm.sls.subato.assessment.submission.dto.SubmissionSummaryDto;
import de.hsrm.sls.subato.assessment.submission.dto.UpdatePassedDto;
import de.hsrm.sls.subato.assessment.submission.dto.UpdatePointsDto;
import de.hsrm.sls.subato.assessment.submission.repository.SubmissionFilter.Builder;
import de.hsrm.sls.subato.assessment.submission.repository.SubmissionQuery;
import de.hsrm.sls.subato.assessment.submission.repository.SubmissionRepository;
import de.hsrm.sls.subato.assessment.submission.repository.SubmissionSort;
import de.hsrm.sls.subato.assessment.submission.usecase.UpdatePassedUseCase;
import de.hsrm.sls.subato.assessment.submission.usecase.UpdatePointsUseCase;
import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.exercises.ExercisesAction;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.shared.swagger.SubatoApiResponse;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import java.util.List;
import java.util.Optional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RestController
@Tag(name = "Bewertung")
public class SubmissionController {

    @Autowired
    private SubmissionMapper submissionMapper;

    @Autowired
    private SubmissionRepository submissionRepository;

    @Autowired
    private UpdatePassedUseCase updatePassedUseCase;

    @Autowired
    private UpdatePointsUseCase updatePointsUseCase;

    @Autowired
    private PolicyEvaluator policy;

    @GetMapping("/submission/{id}")
    @SubatoApiResponse(SubmissionNotFoundException.class)
    SubmissionDetailDto getOne(@PathVariable Long id,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var submission = submissionRepository.findById(id)
            .orElseThrow(() -> new SubmissionNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(submission)
            .setPermission(ExercisesAction.SUBMISSION_VIEW).build();
        policy.checkAndRaise(context);

        return submissionMapper.mapDetail(submission);
    }

    @PostMapping("/submission/{id}/passed")
    @Operation(summary = "Über Abgabe entscheiden: Bestanden, nicht bestanden oder keine Entscheidung")
    @SubatoApiResponse(SubmissionNotFoundException.class)
    SubmissionDetailDto updatePassed(@PathVariable Long id, @RequestBody UpdatePassedDto dto,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var submission = submissionRepository.findById(id)
            .orElseThrow(() -> new SubmissionNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(submission)
            .setPermission(ExercisesAction.SUBMISSION_UPDATE).build();
        policy.checkAndRaise(context);

        submission = updatePassedUseCase.updatePassed(submission, dto, principal);
        return submissionMapper.mapDetail(submission);
    }

    @PostMapping("/submission/{id}/points")
    @Operation(summary = "Erreichte Punktzahl einer Aufgabeninstanz innerhalb der Abgabe (Aufgabe in Übungsblatt) setzen. Nur möglich falls Aufgabeninstanz eine konfigurierte Punktzahl hat und zugeordnete Übung bepunktet ist.")
    @SubatoApiResponse(SubmissionNotFoundException.class)
    SubmissionDetailDto updatePoints(@PathVariable Long id, @RequestBody UpdatePointsDto dto,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var submission = submissionRepository.findById(id)
            .orElseThrow(() -> new SubmissionNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(submission)
            .setPermission(ExercisesAction.SUBMISSION_UPDATE).build();
        policy.checkAndRaise(context);

        submission = updatePointsUseCase.updatePoints(submission, dto);
        return submissionMapper.mapDetail(submission);
    }

    @GetMapping("/submission")
    @PreAuthorize("isAuthenticated()")
    @Operation(summary = "Alle sichtbaren Abgaben im Kontext des angemeldeten Benutzers")
    List<SubmissionSummaryDto> getAll(@RequestParam Optional<Long> exerciseId,
        @RequestParam Optional<Long> userId, @RequestParam Optional<Long> unitId,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var context = new PolicyContext.Builder().setSubject(principal)
            .setPermission(ExercisesAction.SUBMISSION_VIEW).build();
        var filter = new Builder()
            .exerciseId(exerciseId.orElse(null))
            .unitId(unitId.orElse(null))
            .submittedById(userId.orElse(null))
            .build();

        var submissionQuery = new SubmissionQuery.Builder()
            .setFilter(filter)
            .setPolicyContext(context)
            .setSort(SubmissionSort.bySubmittedBy())
            .build();

        var submissions = submissionRepository.findAll(submissionQuery);
        return submissions.stream().map(submission -> submissionMapper.mapSummary(submission))
            .toList();
    }
}
