package de.hsrm.sls.subato.assessment.submission.repository;

import de.hsrm.sls.subato.assessment.submission.model.Submission;
import de.hsrm.sls.subato.assessment.submission.model.TaskInstanceAttempts;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.exercises.exercise.model.Exercise;
import de.hsrm.sls.subato.tasks.task.Task;
import de.hsrm.sls.subato.user.User;
import java.util.ArrayList;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.data.jpa.domain.Specification;

class SubmissionRepositoryImpl implements SubmissionRepositoryCustom {

    @Autowired
    @Lazy
    private SubmissionRepository submissionRepository;

    @Autowired
    private PolicyEvaluator policy;

    public List<Submission> findAll(SubmissionQuery query) {
        Specification<Submission> spec = createSpec(query);

        var submissions = submissionRepository.findAll(spec);

        // we don't use the repository sort parameter
        // because of postgres' distinct order by limitations
        if (query.getSort() != null) {
            submissions.sort(query.getSort());
        }

        return submissions;
    }

    public Submission findByExerciseAndUser(long exerciseId, long userId) {
        var filter = new SubmissionFilter.Builder()
            .submittedById(userId)
            .exerciseId(exerciseId)
            .build();
        var submissionQuery = new SubmissionQuery.Builder()
            .setFilter(filter)
            .build();

        var submissions = findAll(submissionQuery);
        return submissions.stream().findFirst().orElse(null);
    }

    public int getCurrentAttempts(Task task, Exercise exercise, User user) {
        var submission = findByExerciseAndUser(exercise.getId(), user.getId());
        var attempts =
            submission != null ? submission.getAttempts() : new ArrayList<TaskInstanceAttempts>();
        return attempts.stream()
            .filter(a -> a.getTaskInstance().getExercise().getId() == exercise.getId()
                && a.getTaskInstance().getTask().getId() == task.getId())
            .map(a -> a.getAttempts())
            .findFirst()
            .orElse(0);
    }

    private Specification<Submission> createSpec(SubmissionQuery query) {
        Specification<Submission> spec = query.getFilter().toSpecification();

        var policyContext = query.getPolicyContext();
        if (policyContext != null && policyContext.getPermission() != null) {
            var authSpec = policy.checkAsSpec(query.getPolicyContext());
            spec = spec.and(authSpec);
        }

        return spec;
    }
}
