package de.hsrm.sls.subato.attendances.attendance;

import de.hsrm.sls.subato.attendances.attendance.dto.AttendanceDto;
import de.hsrm.sls.subato.attendances.attendance.dto.AttendanceMapper;
import de.hsrm.sls.subato.attendances.attendance.dto.CreateAttendanceDto;
import de.hsrm.sls.subato.attendances.attendance.dto.ParticipantAttendanceDetailDto;
import de.hsrm.sls.subato.attendances.attendance.dto.ParticipantAttendanceMapper;
import de.hsrm.sls.subato.attendances.attendance.usecase.CreateAttendanceUseCase;
import de.hsrm.sls.subato.attendances.attendance.usecase.DeleteAttendanceUseCase;
import de.hsrm.sls.subato.attendances.attendance.usecase.GetAttendanceUseCase;
import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.attendance.AttendancePermission;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.courses.course.CourseNotFoundException;
import de.hsrm.sls.subato.courses.course.repository.CourseRepository;
import de.hsrm.sls.subato.courses.event.CourseUnitEventNotFoundException;
import de.hsrm.sls.subato.courses.event.repository.CourseUnitEventRepository;
import de.hsrm.sls.subato.shared.springext.ResponseStatusMapping;
import de.hsrm.sls.subato.shared.swagger.SubatoApiResponse;
import de.hsrm.sls.subato.user.UserNotFoundException;
import de.hsrm.sls.subato.user.repository.UserRepository;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import java.util.List;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

@RestController
@Tag(name = "Anwesenheitserfassung")
public class AttendanceController {

    @Autowired
    private AttendanceMapper attendanceMapper;

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private CourseUnitEventRepository courseUnitEventRepository;

    @Autowired
    private DeleteAttendanceUseCase deleteAttendanceUseCase;

    @Autowired
    private CreateAttendanceUseCase createAttendanceUseCase;

    @Autowired
    private CourseRepository courseRepository;

    @Autowired
    private GetAttendanceUseCase getAttendanceUseCase;

    @Autowired
    private ParticipantAttendanceMapper participantAttendanceMapper;

    @Autowired
    private PolicyEvaluator policy;

    @PutMapping("/attendance")
    @SubatoApiResponse(UserNotFoundException.class)
    @SubatoApiResponse(CourseUnitEventNotFoundException.class)
    @ResponseStatusMapping(exception = UserNotFoundException.class, code = HttpStatus.BAD_REQUEST)
    @ResponseStatusMapping(exception = CourseUnitEventNotFoundException.class, code = HttpStatus.BAD_REQUEST)
    AttendanceDto createAttendance(@RequestBody CreateAttendanceDto dto,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var event = courseUnitEventRepository.findById(dto.getEventId())
            .orElseThrow(() -> new CourseUnitEventNotFoundException(dto.getEventId()));
        var user = userRepository.findById(dto.getUserId())
            .orElseThrow(() -> new UserNotFoundException(dto.getUserId()));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(event)
            .setPermission(AttendancePermission.ATTENDANCE_UPDATE).build();
        policy.checkAndRaise(context);

        var attendance = createAttendanceUseCase.create(user, event);
        return attendanceMapper.map(attendance);
    }

    @DeleteMapping("/attendance/user/{userId}/event/{eventId}")
    @SubatoApiResponse(AttendanceNotFoundException.class)
    void deleteAttendance(@PathVariable Long userId, @PathVariable Long eventId,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var event = courseUnitEventRepository.findById(eventId)
            .orElseThrow(() -> new CourseUnitEventNotFoundException(eventId));
        var user = userRepository.findById(userId)
            .orElseThrow(() -> new UserNotFoundException(userId));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(event)
            .setPermission(AttendancePermission.ATTENDANCE_UPDATE).build();
        policy.checkAndRaise(context);

        deleteAttendanceUseCase.delete(user, event);
    }

    @GetMapping("/course/{courseId}/attendance/{userId}")
    @Operation(summary = "Zusammenfassung der Anwesenheitsinformationen des Benutzers in allen Kurseinheiten (z.B. Praktikumsgruppen)")
    @SubatoApiResponse(CourseNotFoundException.class)
    @SubatoApiResponse(UserNotFoundException.class)
    List<ParticipantAttendanceDetailDto> getCourseAttendance(@PathVariable Long courseId,
        @PathVariable Long userId, @AuthenticationPrincipal SubatoPrincipal principal) {
        var course = courseRepository.findById(courseId)
            .orElseThrow(() -> new CourseNotFoundException(courseId));
        var user = userRepository.findById(userId)
            .orElseThrow(() -> new UserNotFoundException(userId));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(Pair.of(course, user))
            .setPermission(AttendancePermission.ATTENDANCE_VIEW).build();
        policy.checkAndRaise(context);

        var courseUnitAttendance = getAttendanceUseCase.forCourseAndUser(course, user);
        return courseUnitAttendance.stream()
            .map(
                participantAttendance -> participantAttendanceMapper.map(participantAttendance))
            .toList();
    }
}
