package de.hsrm.sls.subato.auth;

import static org.springframework.security.config.Customizer.withDefaults;

import de.hsrm.sls.subato.auth.oauth.JwtAuthConverter;
import de.hsrm.sls.subato.auth.oauth.OAuth2AuthFilterPostProcessor;
import de.hsrm.sls.subato.auth.oauth.RolesGrantedAuthoritiesMapper;
import de.hsrm.sls.subato.auth.stream.StreamFilter;
import de.hsrm.sls.subato.shared.CustomHttpHeaderConstants;
import de.hsrm.sls.subato.shared.FilterChainExceptionHandler;
import de.hsrm.sls.subato.shared.config.SubatoConfig;
import java.net.URI;
import java.net.URL;
import java.util.List;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Profile;
import org.springframework.http.HttpHeaders;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configurers.AbstractHttpConfigurer;
import org.springframework.security.config.annotation.web.configurers.HeadersConfigurer.FrameOptionsConfig;
import org.springframework.security.core.authority.mapping.GrantedAuthoritiesMapper;
import org.springframework.security.core.session.SessionRegistryImpl;
import org.springframework.security.oauth2.client.registration.ClientRegistrationRepository;
import org.springframework.security.oauth2.client.web.DefaultOAuth2AuthorizationRequestResolver;
import org.springframework.security.oauth2.client.web.OAuth2AuthorizationRequestRedirectFilter;
import org.springframework.security.oauth2.client.web.OAuth2AuthorizationRequestResolver;
import org.springframework.security.oauth2.server.resource.web.authentication.BearerTokenAuthenticationFilter;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.authentication.logout.LogoutFilter;
import org.springframework.security.web.authentication.session.RegisterSessionAuthenticationStrategy;
import org.springframework.security.web.authentication.session.SessionAuthenticationStrategy;
import org.springframework.web.cors.CorsConfiguration;
import org.springframework.web.cors.CorsConfigurationSource;
import org.springframework.web.cors.UrlBasedCorsConfigurationSource;

@Configuration
@EnableWebSecurity
public class SecurityConfig {

    /**
     * See <a
     * href="https://github.com/spring-projects/spring-security/blob/58a08050c1bc0586f5cfc8f8ca4f42d5f665b676/docs/modules/ROOT/pages/reactive/oauth2/login/advanced.adoc#oauth-20-login-page">Spring
     * docs</a>. the last segment has to match with the configuration in application.properties
     * (spring.security.oauth2.client.registration.--->keycloak<----.)
     */
    public static final String OAUTH_LOGIN_URL = "/oauth2/authorization/keycloak";

    @Autowired
    JwtAuthConverter jwtAuthConverter;

    @Autowired
    private StreamFilter streamFilter;

    @Autowired
    private OAuth2AuthFilterPostProcessor oAuth2AuthFilterPostProcessor;

    @Autowired
    private FilterChainExceptionHandler filterChainExceptionHandler;

    @Autowired
    private SubatoConfig subatoConfig;


    @Bean
    @Profile("runner")
    public ClientRegistrationRepository clientRegistrationRepository() {
        return registrationId -> null;
    }

    @Bean
    protected SessionAuthenticationStrategy sessionAuthenticationStrategy() {
        return new RegisterSessionAuthenticationStrategy(new SessionRegistryImpl());
    }

    @Bean
    public CorsConfigurationSource corsConfigurationSource() {
        final CorsConfiguration configuration = new CorsConfiguration();
        configuration.setAllowedOrigins(List.of("*"));
        configuration.setAllowedMethods(List.of("GET", "POST", "PUT", "DELETE"));
        configuration.setAllowedHeaders(
            List.of(HttpHeaders.AUTHORIZATION, HttpHeaders.CACHE_CONTROL,
                HttpHeaders.CONTENT_TYPE,
                CustomHttpHeaderConstants.CONFIRM_DANGEROUS_ACTION,
                CustomHttpHeaderConstants.API_VERSION));

        final UrlBasedCorsConfigurationSource source = new UrlBasedCorsConfigurationSource();
        source.registerCorsConfiguration("/**", configuration);

        return source;
    }

    @Bean
    public SecurityFilterChain filterChain(HttpSecurity http,
        ClientRegistrationRepository clientRegistrationRepository)
        throws Exception {
        http.authorizeHttpRequests(
                authorizeHttpRequests -> authorizeHttpRequests.anyRequest().permitAll())
            .cors(withDefaults()).csrf(AbstractHttpConfigurer::disable);

        http.headers(headers -> headers.frameOptions(FrameOptionsConfig::disable));

        // we use the oauth2 client for accessing protected resources, so we need a session cookie.
        // thus, the API is not completely stateless because it supports both session and JWT authentication.
        // http.sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS);

        http.oauth2Login(
            oauth2Login -> oauth2Login
                .withObjectPostProcessor(oAuth2AuthFilterPostProcessor)
                .authorizationEndpoint(
                    a -> {
                        if (clientRegistrationRepository != null) {
                            a.authorizationRequestResolver(
                                customAuthorizationRequestResolver(clientRegistrationRepository));
                        }
                    }));

        http.oauth2ResourceServer(oauth2ResourceServer -> oauth2ResourceServer
            .jwt(jwt -> jwt.jwtAuthenticationConverter(jwtAuthConverter)));

        http.addFilterBefore(filterChainExceptionHandler, LogoutFilter.class);
        http.addFilterAfter(streamFilter, BearerTokenAuthenticationFilter.class);
        return http.build();
    }


    /**
     * configures a custom resolver for changing the redirectUri if behind a proxy. we should
     * configure X-Forwarded-* headers in the future.
     * <p>
     * see: <a
     * href="https://docs.spring.io/spring-security/reference/features/exploits/http.html#http-proxy-server">Proxy
     * Server</a> <a
     * href="https://docs.spring.io/spring-security/reference/servlet/oauth2/client/authorization-grants.html#oauth2Client-auth-code-redirect-uri">Redirect
     * Uri config</a>
     */
    private OAuth2AuthorizationRequestResolver customAuthorizationRequestResolver(
        ClientRegistrationRepository clientRegistrationRepository) {
        var r = new DefaultOAuth2AuthorizationRequestResolver(
            clientRegistrationRepository,
            OAuth2AuthorizationRequestRedirectFilter.DEFAULT_AUTHORIZATION_REQUEST_BASE_URI
        );

        r.setAuthorizationRequestCustomizer(builder -> {
            var request = builder.build();

            if (StringUtils.isNotBlank(subatoConfig.getPublicHostname())) {
                var redirectUri = rewriteHost(request.getRedirectUri(),
                    subatoConfig.getPublicHostname());
                builder.redirectUri(redirectUri);
            }
        });
        return r;
    }

    private String rewriteHost(String uri, String replacement) {
        try {
            var originalUrl = new URL(uri);

            var updatedUri = new URI(
                originalUrl.getProtocol(),
                replacement,
                originalUrl.getPath(),
                originalUrl.getQuery(),
                originalUrl.getRef()
            );

            var updatedUrl = updatedUri.toURL();
            return updatedUrl.toString();
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }

    @Bean
    public GrantedAuthoritiesMapper rolesGrantedAuthoritiesMapper() {
        return new RolesGrantedAuthoritiesMapper();
    }
}