package de.hsrm.sls.subato.auth.masking;

import com.fasterxml.jackson.databind.BeanDescription;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializationConfig;
import com.fasterxml.jackson.databind.ser.BeanSerializerModifier;
import de.hsrm.sls.subato.auth.principal.PrincipalService;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * Masking requires changing the dto and then serializing that changed dto.
 * Masking is implemented by hooking into the serialization process from Jackson.
 * For serialization, we want to reuse the default serializer from jackson for that dto to prevent code duplication.
 * Because the default serializer is overwritten with our masking serializer,
 * we need to extract the default serializer using this {@link BeanSerializerModifier} as soon as it has been created.
 * </p>
 *
 * <p>
 * <b>Note</b>: We cannot use converters, because both the input and output types are identical. Using converters has been tried
 * but results in stackoverflow exceptions or problems with DI of spring. Maskers need access to external components/services.
 * </p>
 *
 * <p>
 * This implementation is a generic version of
 * <a href="https://www.baeldung.com/jackson-call-default-serializer-from-custom-serializer">this</a>
 * code.
 * </p>
 */
class MaskerSerializerModifier extends BeanSerializerModifier {

    private PrincipalService principalService;

    private Map<Class<?>, Masker<?>> maskers = new HashMap<>();


    public MaskerSerializerModifier(PrincipalService principalService) {
        this.principalService = principalService;
    }

    public <T> void addMasker(Masker<T> masker, Class<T> clazz) {
        maskers.put(clazz, masker);
    }

    @Override
    public JsonSerializer<?> modifySerializer(
        SerializationConfig config, BeanDescription beanDesc, JsonSerializer<?> serializer) {

        for (var e : maskers.entrySet()) {
            if (beanDesc.getBeanClass().equals(e.getKey())) {
                return new MaskerSerializer(e.getValue(), serializer, principalService);
            }
        }

        return serializer;
    }
}
