package de.hsrm.sls.subato.auth.oauth;

import de.hsrm.sls.subato.auth.principal.PrincipalService;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.modules.program.DegreeProgramRepository;
import de.hsrm.sls.subato.shared.helper.KeycloakRoleMapper;
import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.user.repository.UserRepository;
import java.util.List;
import java.util.stream.Collectors;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.cache.interceptor.KeyGenerator;
import org.springframework.context.annotation.Bean;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * Responsible for importing users authenticated by keycloak into the system, using the information
 * from the supplied token.
 * <p>
 * Be careful: This implementation is not thread-safe, the method calling
 * {@link #handleToken(KeycloakAuthenticationToken)} should be declared as synchronized.
 * Synchronization is not done in this class because @Transactional does not work properly with
 * synchronized
 */
@Service
@Transactional
public class KeycloakLoginHandler {

    @Autowired
    private PrincipalService principalService;

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private DegreeProgramRepository degreeProgramRepository;

    private static Logger logger = LoggerFactory.getLogger(KeycloakLoginHandler.class);

    @Bean
    public KeyGenerator keycloakAuthKeyGenerator() {
        return (target, method, params) -> {
            var userInfo = (KeycloakUserInfo) params[0];
            var alias = userInfo.getAlias();
            if (alias != null) {
                return alias;
            }

            return userInfo.getUsername();
        };
    }

    /**
     * if not cached, the user should be imported or updated (if exists) from the token otherwise
     * the cached principal for that user is returned
     * <p>
     * Caching may be seen as redundant with the one in
     * {@link PrincipalService#loadUserByUsername(String)}. However, user details (the principal)
     * can be loaded without a token, so it is necessary to do that. In integration tests (for
     * example) the user is loaded directly (without a keycloak token).
     * <p>
     * caching here will prevent the user from getting updated from the token on every request
     */
    @Cacheable(value = "currentUser", keyGenerator = "keycloakAuthKeyGenerator")
    public SubatoPrincipal handleToken(KeycloakUserInfo userInfo) {
        String alias = userInfo.getAlias();
        String username = alias;

        if (alias != null) {
            logger.debug(String.format("Logged in with alias user, skipping import/update..."));
        } else {
            username = userInfo.getUsername();
            var mail = userInfo.getMail();

            var user = userRepository.findByLoginNameIgnoreCase(username).orElse(null);
            if (user == null && mail != null) {
                logger.debug(
                    String.format("User with loginName %s not found, trying email %s", username,
                        mail));
                user = userRepository.findByMailIgnoreCase(userInfo.getMail()).orElse(null);
            }

            if (user == null) {
                logger.debug("User not found, importing now...");
                user = new User();
            }

            updateUserFromToken(user, userInfo);
        }

        return (SubatoPrincipal) principalService.loadUserByUsername(username);
    }

    private User updateUserFromToken(User user, KeycloakUserInfo userInfo) {

        user.setLoginName(userInfo.getUsername());
        user.setFirstName(userInfo.getFirstName());
        user.setLastName(userInfo.getLastName());
        user.setMail(userInfo.getMail());
        user.setActive(true);
        user.setProgram(null);

        if (StringUtils.isNotBlank(userInfo.getStudentInfo())) {
            var splittedInfo = userInfo.getStudentInfo().split(";");
            var posCode = splittedInfo.length > 3 ? splittedInfo[3] : null;
            Integer poYear = null;
            try {
                poYear = splittedInfo.length > 5 ? Integer.parseInt(splittedInfo[5]) : null;
            } catch (NumberFormatException ignored) {
            }

            if (StringUtils.isNotBlank(posCode) && poYear != null) {
                var program = degreeProgramRepository.findByPosCodeAndPoYear(posCode, poYear);
                user.setProgram(program);
            }
        }

        user.getRoles().clear();
        user = userRepository.save(user);

        List<String> plainRoles = userInfo.getRoles();

        var roles = plainRoles.stream().map(r -> KeycloakRoleMapper.getRole(r))
            .filter(r -> r != null)
            .collect(Collectors.toList());
        user.getRoles().addAll(roles);
        user = userRepository.save(user);
        logger.debug("Imported/updated user %s".formatted(user.getLoginName()));

        return user;
    }
}
