package de.hsrm.sls.subato.auth.policy.exercises;

import de.hsrm.sls.subato.auth.Role;
import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.PolicyRule;
import de.hsrm.sls.subato.auth.policy.SubatoPrincipalSpecifications;
import de.hsrm.sls.subato.auth.policy.courses.CourseViewPolicy;
import de.hsrm.sls.subato.auth.policy.courses.CoursesAction;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.courses.course.model.CourseVisibility;
import de.hsrm.sls.subato.courses.course.model.Course_;
import de.hsrm.sls.subato.exercises.exercise.model.Exercise;
import de.hsrm.sls.subato.exercises.exercise.model.Exercise_;
import java.time.LocalDateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Component;

/**
 * The current user is allowed to view an exercise within a certain course
 */
@Component
public class ExerciseViewPolicy implements PolicyRule {

    @Autowired
    @Lazy
    private PolicyEvaluator policy;

    @Override
    public boolean check(PolicyContext request) {
        if (!request.matches(ExercisesAction.EXERCISE_VIEW)) {
            return false;
        }

        var exercise = (Exercise) request.getObj();

        var context = new PolicyContext.Builder()
            .setSubject(request.getSubject())
            .setObj(exercise.getCourse())
            .setPermission(CoursesAction.COURSE_VIEW).build();

        var canViewCourse = policy.check(context);
        return canViewCourse && (exercise.isVisible()
            || request.getSubject() != null && request.getSubject()
            .hasAnyGroup(exercise.getCourse().getId()));
    }

    @Override
    public Specification checkAsSpec(PolicyContext context) {
        if (!context.matches(ExercisesAction.EXERCISE_VIEW)) {
            return Specification.where(null);
        }
        return isCourseVisible(context.getSubject()).and(
            isVisible().or(isCourseStaff(context.getSubject())));
    }

    /**
     * copied from {@link CourseViewPolicy#isVisible(SubatoPrincipal)} and adapted to Exercise
     * entity sorry, I don't know how to reuse it yet.
     * todo
     */
    public static Specification<Exercise> isCourseVisible(SubatoPrincipal principal) {
        return (ex, cq, cb) -> {
            var exerciseCourseJoin = ex.join(Exercise_.course);

            var participantInBuilder = SubatoPrincipalSpecifications.participantIn(principal, cb,
                exerciseCourseJoin.get(Course_.id));
            var courseStaffInBuilder = SubatoPrincipalSpecifications.courseStaffIn(principal, cb,
                exerciseCourseJoin.get(Course_.id));

            return cb.or(
                cb.equal(ex.get(Exercise_.course).get(Course_.visibility), CourseVisibility.PUBLIC),
                cb.and(
                    cb.equal(ex.get(Exercise_.course).get(Course_.visibility),
                        CourseVisibility.INTERNAL),
                    cb.equal(cb.literal(true), principal != null)
                ),
                cb.and(
                    cb.equal(ex.get(Exercise_.course).get(Course_.visibility),
                        CourseVisibility.PARTICIPANTS),
                    cb.equal(cb.literal(true), principal != null),
                    participantInBuilder
                ),
                cb.or(
                    cb.equal(cb.literal(true), principal != null && principal.hasRole(Role.Staff)),
                    courseStaffInBuilder
                )
            );
        };
    }

    public static Specification<Exercise> isVisible() {
        var now = LocalDateTime.now();

        return (ex, cq, cb) -> cb.or(
            cb.isNull(ex.get(Exercise_.visibleFrom)),
            cb.lessThan(ex.get(Exercise_.visibleFrom), now)
        );
    }

    public static Specification<Exercise> isCourseStaff(SubatoPrincipal principal) {
        return (ex, cq, cb) -> {
            var exerciseCourseJoin = ex.join(Exercise_.course);
            var courseStaffInBuilder = SubatoPrincipalSpecifications.courseStaffIn(principal, cb,
                exerciseCourseJoin.get(Course_.id));
            return courseStaffInBuilder;
        };
    }
}
