package de.hsrm.sls.subato.auth.principal;

import de.hsrm.sls.subato.auth.Role;
import de.hsrm.sls.subato.courses.course.model.Course;
import de.hsrm.sls.subato.courses.staff.model.StaffGroup;
import java.io.Serializable;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.oauth2.core.user.OAuth2User;

/**
 * Refers to the currently logged in user. Replicates and enhances the keycloak user information and
 * contains convenience methods for authorization.
 * <p>
 * This class (and all types of the contained fields) need to be marked with Serializable for
 * session persistence.
 */
public class SubatoPrincipal implements UserDetails, OAuth2User, Serializable {

    private long id;
    private String loginName;
    private String firstName;
    private String lastName;
    private String name;
    private String mail;
    private DegreeProgram program;
    private List<CourseStaffMembership> staffMemberships;
    private List<CourseUnit> participatingUnits;
    private List<CourseParticipantProfile> participantProfiles;
    private List<Role> roles;
    private boolean consentedToCollection;

    public boolean hasGroup(Course course, StaffGroup group) {
        return hasGroup(course.getId(), group);
    }

    public boolean hasGroup(Long courseId, StaffGroup group) {
        return staffMemberships.stream().anyMatch(
            m -> m.getCourseId() == courseId && m.getGroup().equals(group)
        );
    }

    public boolean hasAnyGroup(Long courseId) {
        return staffMemberships.stream().anyMatch(
            m -> m.getCourseId() == courseId
        );
    }

    public boolean hasAnyGroup() {
        return staffMemberships.size() != 0;
    }

    public boolean isParticipant(long courseId) {
        return participatingUnits.stream().anyMatch(u -> u.getCourseId() == courseId);
    }

    public boolean hasRole(Role role) {
        return roles.stream().anyMatch(r -> r.equals(role));
    }

    public boolean hasAnyRole(Role... roles) {
        return Arrays.stream(roles).anyMatch(r -> hasRole(r));
    }

    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getLoginName() {
        return loginName;
    }

    public void setLoginName(String loginName) {
        this.loginName = loginName;
    }

    public String getFirstName() {
        return firstName;
    }

    public void setFirstName(String firstName) {
        this.firstName = firstName;
    }

    public String getLastName() {
        return lastName;
    }

    public void setLastName(String lastName) {
        this.lastName = lastName;
    }

    public List<CourseStaffMembership> getStaffMemberships() {
        return staffMemberships;
    }

    public void setStaffMemberships(List<CourseStaffMembership> memberships) {
        this.staffMemberships = memberships;
    }

    public List<Role> getRoles() {
        return roles;
    }

    public void setRoles(List<Role> roles) {
        this.roles = roles;
    }

    public List<CourseUnit> getParticipatingUnits() {
        return participatingUnits;
    }

    public void setParticipatingUnits(List<CourseUnit> participatingUnits) {
        this.participatingUnits = participatingUnits;
    }

    public List<CourseParticipantProfile> getParticipantProfiles() {
        return participantProfiles;
    }

    public void setParticipantProfiles(
        List<CourseParticipantProfile> participantProfiles) {
        this.participantProfiles = participantProfiles;
    }

    public String getMail() {
        return mail;
    }

    public void setMail(String mail) {
        this.mail = mail;
    }

    public DegreeProgram getProgram() {
        return program;
    }

    public void setProgram(DegreeProgram program) {
        this.program = program;
    }

    public boolean isConsentedToCollection() {
        return consentedToCollection;
    }

    public SubatoPrincipal setConsentedToCollection(boolean consentedToCollection) {
        this.consentedToCollection = consentedToCollection;
        return this;
    }

    @Override
    public <A> A getAttribute(String name) {
        return OAuth2User.super.getAttribute(name);
    }

    @Override
    public Map<String, Object> getAttributes() {
        return new HashMap<>();
    }

    @Override
    public Collection<? extends GrantedAuthority> getAuthorities() {
        return roles.stream().map(r ->
            new SimpleGrantedAuthority(r.toString())).collect(Collectors.toList()
        );
    }

    @Override
    public String getPassword() {
        return null;
    }

    @Override
    public String getUsername() {
        return loginName;
    }

    @Override
    public boolean isAccountNonExpired() {
        return true;
    }

    @Override
    public boolean isAccountNonLocked() {
        return true;
    }

    @Override
    public boolean isCredentialsNonExpired() {
        return true;
    }

    @Override
    public boolean isEnabled() {
        return true;
    }
}
