package de.hsrm.sls.subato.auth.stream.token;

import static de.hsrm.sls.subato.shared.helper.UriHelper.parseQueryParameters;

import de.hsrm.sls.subato.auth.principal.PrincipalService;
import de.hsrm.sls.subato.shared.config.SubatoConfig;
import de.hsrm.sls.subato.shared.helper.UriHelper;
import jakarta.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;

/**
 * Coordinates requests for signing urls, manages the state of signed tokens and activates tokens on
 * request.
 */
@Service
public class StreamTokenManager {

    private static Logger logger = LoggerFactory.getLogger(StreamTokenManager.class);

    @Autowired
    private StreamTokenGenerator tokenGenerator;

    @Autowired
    private PrincipalService principalService;

    @Autowired
    private SubatoConfig subatoConfig;

    @Autowired
    StreamTokenConfig streamTokenConfig;

    /**
     * Lookup map required for token activation. If the token was extracted from the query param, we
     * need to get the context to check expiration, principal, ...
     */
    private Map<String, StreamTokenContext> tokenContextMap = new HashMap<>();

    /**
     * Reverse lookup map required for signing. If we already signed a url for the same user, we
     * don't need to sign again.
     */
    private Map<ReverseStreamTokenKey, String> reverseTokenMap = new HashMap<>();


    /**
     * Signs the given url by generating a stream token (with respect to the current user), stores
     * the token in-memory and returns the signed url for redirection to a client.
     *
     * @param url     may be ABSOLUTE or RELATIVE (with respect to the backend root)
     * @param request for generating the absolute url
     * @return the ABSOLUTE url with the appended stream token
     */
    public synchronized String sign(String url, HttpServletRequest request) {
        logger.trace("Attempting to sign URL %s".formatted(url));

        var principal = principalService.getPrincipal();
        if (principal == null) {
            logger.trace("No principal, no signing required");
            return url;
        }

        // parse the url to get the relative url and the base url (if it's not already absolute)
        var parseResult = UriHelper.parseUrl(url, request, subatoConfig);
        if (parseResult == null) {
            logger.trace("URL does not require signing");
            return url;
        }

        // reuse signed urls if possible
        var key = new ReverseStreamTokenKey(parseResult.relativeUrl(), principal.getUsername());
        StreamTokenContext context;
        if (reverseTokenMap.containsKey(key)) {
            var token = reverseTokenMap.get(key);
            context = tokenContextMap.get(token);
            context = context.refresh();
            tokenContextMap.put(context.getToken(), context);
            logger.trace(
                "Refreshed signed URL for principal %s".formatted(principal.getUsername()));
        } else {
            context = tokenGenerator.generate(parseResult.relativeUrl(), principal.getUsername());
            tokenContextMap.put(context.getToken(), context);
            reverseTokenMap.put(key, context.getToken());
            logger.trace("URL signed: %s".formatted(context));
        }

        var signedAbsoluteUrl = String.format("%s%s", parseResult.baseUrl(),
            context.getSignedUrl());
        logger.trace("Signed absolute URL: %s".formatted(signedAbsoluteUrl));
        return signedAbsoluteUrl;
    }

    private String parseTokenFromUrl(String url) {
        var params = parseQueryParameters(url);
        return params.getOrDefault(StreamTokenConfig.QUERY_PARAM_NAME, null);
    }


    /**
     * Extract the token from the request and activate it by setting the current user to the
     * principal associated with the token.
     */
    public synchronized void activate(HttpServletRequest request) {
        String relativeUrl = request.getRequestURI();
        String queryString = request.getQueryString();

        if (queryString != null) {
            relativeUrl += "?" + queryString;
        }
        logger.trace("URL: %s".formatted(relativeUrl));

        var token = parseTokenFromUrl(relativeUrl);
        if (token == null) {
            logger.trace("No token provided");
            return;
        }

        if (!tokenContextMap.containsKey(token)) {
            logger.trace("Token %s does not exist (anymore)".formatted(token));
            throw new InvalidStreamTokenException();
        }

        var context = tokenContextMap.get(token);
        logger.trace("Retrieved context for token %s: %s".formatted(token, context.toString()));

        if (context.isExpired(streamTokenConfig.getExpiration())) {
            logger.trace("Token expired");
            throw new InvalidStreamTokenException();
        }

        var principal = principalService.loadUserByUsername(context.getUsername());
        var authToken = new UsernamePasswordAuthenticationToken(principal, null,
            principal.getAuthorities());
        SecurityContextHolder.getContext().setAuthentication(authToken);
        logger.trace("Activated principal %s".formatted(context.getUsername()));
    }

    /**
     * Removes expired tokens from memory
     */
    public synchronized void cleanupExpiredTokens() {
        var expiredEntries = tokenContextMap.entrySet().stream()
            .filter(e -> e.getValue().isExpired(streamTokenConfig.getExpiration()))
            .toList();
        logger.debug("%s tokens expired since last check".formatted(expiredEntries.size()));

        for (var e : expiredEntries) {
            var context = tokenContextMap.remove(e.getKey());

            var key = new ReverseStreamTokenKey(context.getOriginalUrl(), context.getUsername());
            reverseTokenMap.remove(key);

            logger.trace("Removed expired token %s".formatted(e.getKey()));
        }
    }
}
