package de.hsrm.sls.subato.auth.websocket;

import static de.hsrm.sls.subato.auth.websocket.AuthenticationHelper.getPrincipalFrom;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.lang.NonNull;
import org.springframework.messaging.Message;
import org.springframework.messaging.MessageChannel;
import org.springframework.messaging.simp.stomp.StompCommand;
import org.springframework.messaging.simp.stomp.StompHeaderAccessor;
import org.springframework.messaging.support.ChannelInterceptor;
import org.springframework.messaging.support.MessageBuilder;
import org.springframework.messaging.support.MessageHeaderAccessor;
import org.springframework.security.core.Authentication;
import org.springframework.security.oauth2.server.resource.authentication.BearerTokenAuthenticationToken;
import org.springframework.security.oauth2.server.resource.authentication.JwtAuthenticationProvider;
import org.springframework.stereotype.Component;

/**
 * Unfortunately, spring does not support Token authentication through STOMP. This interceptor will
 * run before the interceptors from Spring and intercept CONNECT-requests to authenticate the
 * client. If a client connects, it needs to provide the JWT for authentication.
 * <p>
 * After authentication, the principal is stored in the STOMP session. <strong>We currently check if
 * the token has expired at the time of the CONNECT. Token expiration is not checked on "subsequent
 * requests".</strong>
 */
@Component
public class AuthChannelInterceptor implements ChannelInterceptor {

    private static Logger logger = LoggerFactory.getLogger(AuthChannelInterceptor.class);


    @Autowired
    JwtAuthenticationProvider webSocketAuthenticationProvider;

    /*
    private SecurityContextHolderStrategy securityContextHolderStrategy = SecurityContextHolder
        .getContextHolderStrategy();
    */

    @Override
    public Message<?> preSend(@NonNull Message<?> message, @NonNull MessageChannel channel) {
        StompHeaderAccessor accessor =
            MessageHeaderAccessor.getAccessor(message, StompHeaderAccessor.class);
        if (accessor != null && StompCommand.CONNECT.equals(accessor.getCommand())) {
            var token = accessor.getNativeHeader("token").get(0);
            var auth = authenticate(token);
            var principal = getPrincipalFrom(() -> auth);
            logger.debug("Authenticated through STOMP: %s".formatted(principal.getUsername()));

            accessor.setUser(auth);
            accessor.setLeaveMutable(true);
            return MessageBuilder.createMessage(message.getPayload(), accessor.getMessageHeaders());
        }

        return message;
    }


    private Authentication authenticate(String token) {
        var authenticationRequest = new BearerTokenAuthenticationToken(
            token);
        var authenticationResult = webSocketAuthenticationProvider.authenticate(
            authenticationRequest);
        /*
        SecurityContext context = this.securityContextHolderStrategy.createEmptyContext();
        context.setAuthentication(authenticationResult);
        this.securityContextHolderStrategy.setContext(context);
        */
        return authenticationResult;
    }

}
