package de.hsrm.sls.subato.auth.websocket;

import static de.hsrm.sls.subato.auth.websocket.AuthenticationHelper.getPrincipalFrom;

import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.solutions.SolutionsAction;
import de.hsrm.sls.subato.exercises.exercise.ExerciseNotFoundException;
import de.hsrm.sls.subato.exercises.exercise.repository.ExerciseRepository;
import java.util.function.Supplier;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.messaging.simp.stomp.StompHeaderAccessor;
import org.springframework.messaging.support.MessageHeaderAccessor;
import org.springframework.security.authorization.AuthorizationDecision;
import org.springframework.security.authorization.AuthorizationManager;
import org.springframework.security.core.Authentication;
import org.springframework.security.messaging.access.intercept.MessageAuthorizationContext;
import org.springframework.stereotype.Component;

/**
 * Handles authorization logic for bulk evaluation of solutions. Updates of the state are send to a
 * specific topic, which can be subscribed by clients. The subscription should only be possible for
 * authorized users.
 */
@Component
public class ExerciseAcceptanceAuthorizationManager implements
    AuthorizationManager<MessageAuthorizationContext<?>> {

    @Autowired
    PolicyEvaluator policyEvaluator;

    @Autowired
    ExerciseRepository exerciseRepository;

    @Override
    public AuthorizationDecision check(Supplier<Authentication> authentication,
        MessageAuthorizationContext<?> ctx) {
        var exerciseId = parseExerciseId(ctx);
        if (exerciseId != null) {
            var exercise = exerciseRepository.findById(exerciseId)
                .orElseThrow(() -> new ExerciseNotFoundException(exerciseId));

            var context = new PolicyContext.Builder()
                .setSubject(getPrincipalFrom(authentication))
                .setObj(exercise)
                .setPermission(SolutionsAction.SOLUTION_RUN_BULK_ACCEPTANCE_TESTS).build();
            var granted = policyEvaluator.check(context);
            return new AuthorizationDecision(granted);
        }

        return new AuthorizationDecision(false);
    }

    private Long parseExerciseId(MessageAuthorizationContext<?> message) {
        StompHeaderAccessor accessor =
            MessageHeaderAccessor.getAccessor(message.getMessage(), StompHeaderAccessor.class);

        String destination = accessor != null ? accessor.getDestination() : null;
        if (destination != null) {
            String[] destinationParts = destination.split("/");

            if (destinationParts.length >= 4) {
                try {
                    String topicId = destinationParts[3];
                    return Long.parseLong(topicId);
                } catch (NumberFormatException ex) {
                    return null;
                }
            }
        }

        return null;
    }
}
