package de.hsrm.sls.subato.courses.course;

import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.courses.CoursesAction;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.auth.stream.HtmlUrlSigner;
import de.hsrm.sls.subato.auth.stream.SignedUrlSanitizer;
import de.hsrm.sls.subato.courses.course.dto.CourseDto;
import de.hsrm.sls.subato.courses.course.dto.CourseMapper;
import de.hsrm.sls.subato.courses.course.dto.CreateCourseDto;
import de.hsrm.sls.subato.courses.course.dto.CreateCourseRequest;
import de.hsrm.sls.subato.courses.course.dto.UpdateCourseDto;
import de.hsrm.sls.subato.courses.course.dto.UpdateFinishedDto;
import de.hsrm.sls.subato.courses.course.repository.CourseFilter;
import de.hsrm.sls.subato.courses.course.repository.CourseQuery;
import de.hsrm.sls.subato.courses.course.repository.CourseRepository;
import de.hsrm.sls.subato.courses.course.usecase.CreateCourseUseCase;
import de.hsrm.sls.subato.courses.course.usecase.DeleteCourseUseCase;
import de.hsrm.sls.subato.courses.course.usecase.FinishCourseUseCase;
import de.hsrm.sls.subato.courses.course.usecase.GetCourseDescriptionUseCase;
import de.hsrm.sls.subato.courses.course.usecase.GetCourseUseCase;
import de.hsrm.sls.subato.courses.course.usecase.ReimportCourseUseCase;
import de.hsrm.sls.subato.courses.course.usecase.UpdateCourseUseCase;
import de.hsrm.sls.subato.courses.file.dto.CourseFileDto;
import de.hsrm.sls.subato.courses.file.dto.CourseFileMapper;
import de.hsrm.sls.subato.courses.unit.dto.CourseUnitDto;
import de.hsrm.sls.subato.courses.unit.dto.CourseUnitMapper;
import de.hsrm.sls.subato.courses.unit.repository.CourseUnitRepository;
import de.hsrm.sls.subato.modules.term.TermNotFoundException;
import de.hsrm.sls.subato.shared.springext.ResponseStatusMapping;
import de.hsrm.sls.subato.shared.swagger.SubatoApiResponse;
import de.hsrm.sls.subato.shared.validation.ValidationException;
import de.hsrm.sls.subato.user.dto.UserDetailDto;
import de.hsrm.sls.subato.user.dto.UserMapper;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Pageable;
import org.springframework.data.web.PageableDefault;
import org.springframework.http.HttpStatus;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RestController
@Tag(name = "Kursverwaltung")
public class CourseController {

    @Autowired
    private CourseMapper courseMapper;

    @Autowired
    private CourseUnitMapper courseUnitMapper;

    @Autowired
    private CourseUnitRepository courseUnitRepository;

    @Autowired
    private CourseFileMapper courseFileMapper;

    @Autowired
    private PolicyEvaluator policy;

    @Autowired
    UserMapper userMapper;

    @Autowired
    private CourseRepository courseRepository;

    @Autowired
    private CreateCourseUseCase createCourseUseCase;

    @Autowired
    private UpdateCourseUseCase updateCourseUseCase;

    @Autowired
    private ReimportCourseUseCase reimportCourseUseCase;

    @Autowired
    private FinishCourseUseCase finishCourseUseCase;

    @Autowired
    private DeleteCourseUseCase deleteCourseUseCase;

    @Autowired
    private GetCourseUseCase getCourseUseCase;

    @Autowired
    private GetCourseDescriptionUseCase getCourseDescriptionUseCase;

    @Autowired
    private HtmlUrlSigner htmlUrlSigner;

    @Autowired
    private SignedUrlSanitizer signedUrlSanitizer;

    @GetMapping("/course/{id}")
    @Operation(summary = "Liefert Metadaten zu einem Kurs")
    @SubatoApiResponse(CourseNotFoundException.class)
    CourseDto getOne(@PathVariable Long id, @AuthenticationPrincipal SubatoPrincipal principal) {
        var course = getCourseUseCase.findById(id);

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(course)
            .setPermission(CoursesAction.COURSE_VIEW).build();
        policy.checkAndRaise(context);
        return courseMapper.mapDetail(course);
    }

    @GetMapping("/course/{id}/description")
    @Operation(summary = "Liefert die Kursbeschreibung (entweder in Rohform als Markdown oder gerendert als HTML)")
    @SubatoApiResponse(CourseNotFoundException.class)
    String getDescription(@PathVariable Long id,
        @Parameter(description = "false wenn nicht angegeben") @RequestParam Optional<Boolean> raw,
        HttpServletRequest req, @AuthenticationPrincipal SubatoPrincipal principal) {
        var course = courseRepository.findById(id)
            .orElseThrow(() -> new CourseNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(course)
            .setPermission(CoursesAction.COURSE_VIEW).build();
        policy.checkAndRaise(context);

        var isRaw = raw.orElse(false);
        var description = getCourseDescriptionUseCase.getDescription(course, isRaw);
        if (!isRaw) {
            description = htmlUrlSigner.sign(description, req);
        }

        return description;
    }

    @GetMapping("/course")
    @Operation(summary = "Liefert Metadaten zu allen Kursen")
    List<CourseDto> getAll(@RequestParam Optional<Long> termId,
        @RequestParam("q") Optional<String> query, @PageableDefault Pageable page,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var context = new PolicyContext.Builder().setSubject(principal)
            .setPermission(CoursesAction.COURSE_VIEW).build();
        var filter = new CourseFilter.Builder()
            .termId(termId.orElse(null))
            .query(query.orElse(null))
            .build();
        var courseQuery = new CourseQuery.Builder()
            .setFilter(filter)
            .setPolicyContext(context)
            .setSort(page.getSort())
            .build();

        var courses = courseRepository.findAll(courseQuery);
        return courses.stream()
            .map(c -> courseMapper.mapDetail(c))
            .collect(Collectors.toList());
    }

    @PostMapping("/course")
    @SubatoApiResponse(TermNotFoundException.class)
    @SubatoApiResponse(ValidationException.class)
    @ResponseStatusMapping(exception = TermNotFoundException.class, code = HttpStatus.BAD_REQUEST)
    public CourseDto create(@RequestBody CreateCourseDto dto,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setPermission(CoursesAction.COURSE_CREATE).build();
        policy.checkAndRaise(context);

        var request = new CreateCourseRequest.Builder()
            .setName(dto.getName())
            .setShortName(dto.getShortName())
            .setCourseId(dto.getCourseId())
            .setTermId(dto.getTermId())
            .setShowEmails(true)
            .setModuleIds(dto.getModuleIds())
            .build();

        var course = createCourseUseCase.create(request, principal);
        return courseMapper.mapDetail(course);
    }

    @PutMapping("/course/{id}")
    @SubatoApiResponse(CourseNotFoundException.class)
    @SubatoApiResponse(ValidationException.class)
    CourseDto update(@PathVariable Long id, @RequestBody UpdateCourseDto dto,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var course = courseRepository.findById(id)
            .orElseThrow(() -> new CourseNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(course)
            .setPermission(CoursesAction.COURSE_UPDATE).build();
        policy.checkAndRaise(context);

        dto.setDescription(signedUrlSanitizer.sanitize(dto.getDescription()));
        course = updateCourseUseCase.update(course, dto);
        return courseMapper.mapDetail(course);
    }

    @DeleteMapping("/course/{id}")
    @SubatoApiResponse(CourseNotFoundException.class)
    public void deleteCourse(@PathVariable Long id,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var course = courseRepository.findById(id)
            .orElseThrow(() -> new CourseNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(course)
            .setPermission(CoursesAction.COURSE_DELETE).build();
        policy.checkAndRaise(context);

        deleteCourseUseCase.delete(course);
    }

    @GetMapping("/course/{id}/units")
    @SubatoApiResponse(CourseNotFoundException.class)
    List<CourseUnitDto> getUnits(@PathVariable Long id,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var course = courseRepository.findById(id)
            .orElseThrow(() -> new CourseNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(course)
            .setPermission(CoursesAction.COURSE_VIEW).build();
        policy.checkAndRaise(context);

        var units = courseUnitRepository.findByCourse(course);
        return units.stream().map(unit -> courseUnitMapper.map(unit)).toList();
    }

    @GetMapping("/course/{id}/files")
    @SubatoApiResponse(CourseNotFoundException.class)
    List<CourseFileDto> getFiles(@PathVariable Long id,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var course = courseRepository.findById(id)
            .orElseThrow(() -> new CourseNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(course)
            .setPermission(CoursesAction.COURSE_VIEW).build();
        policy.checkAndRaise(context);

        var files = course.getFiles();
        return files.stream().map(file -> courseFileMapper.map(file)).toList();
    }

    @GetMapping("/course/{id}/participants")
    @SubatoApiResponse(CourseNotFoundException.class)
    List<UserDetailDto> getParticipants(@PathVariable Long id,
        @RequestParam("q") Optional<String> query,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var course = courseRepository.findById(id)
            .orElseThrow(() -> new CourseNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(course)
            .setPermission(CoursesAction.COURSE_VIEW_PARTICIPANTS).build();
        policy.checkAndRaise(context);

        var participants = courseRepository.getParticipants(course, query.orElse(null));
        return participants.stream().map(u -> userMapper.mapDetail(u))
            .collect(Collectors.toList());
    }

    @PostMapping("/course/{id}/reimport")
    @SubatoApiResponse(CourseNotFoundException.class)
    public CourseDto reimport(@PathVariable Long id,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var course = courseRepository.findById(id)
            .orElseThrow(() -> new CourseNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(course)
            .setPermission(CoursesAction.COURSE_REIMPORT).build();
        policy.checkAndRaise(context);

        course = reimportCourseUseCase.reimport(course);
        return courseMapper.mapDetail(course);
    }

    @PutMapping("/course/{id}/finished")
    @SubatoApiResponse(CourseNotFoundException.class)
    public CourseDto finish(@PathVariable Long id,
        @RequestBody UpdateFinishedDto dto,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var course = courseRepository.findById(id)
            .orElseThrow(() -> new CourseNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(course)
            .setPermission(CoursesAction.COURSE_UPDATE).build();
        policy.checkAndRaise(context);

        course = finishCourseUseCase.updateFinished(course, dto.isFinished());
        return courseMapper.mapDetail(course);
    }
}
