package de.hsrm.sls.subato.courses.course.repository;

import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.courses.course.model.Course;
import de.hsrm.sls.subato.courses.staff.model.CourseStaffMembership;
import de.hsrm.sls.subato.courses.staff.model.StaffGroup;
import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.user.repository.UserFilter;
import de.hsrm.sls.subato.user.repository.UserQuery;
import de.hsrm.sls.subato.user.repository.UserRepository;
import de.hsrm.sls.subato.user.repository.UserSort;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.data.jpa.domain.Specification;

class CourseRepositoryImpl implements CourseRepositoryCustom {

    @Autowired
    @Lazy
    private CourseRepository courseRepository;

    @Autowired
    private PolicyEvaluator policy;

    @Autowired
    private UserRepository userRepository;


    public List<Course> findParticipatingCourses(User user, Long termId) {
        var participatingCourseIds = user.getParticipatingUnits().stream()
            .map(unit -> unit.getCourse())
            .filter(c -> c.getTerm() == null || (termId != null && c.getTerm().getId() == termId))
            .map(c -> c.getId())
            .distinct()
            .toList();

        return courseRepository.findAllById(participatingCourseIds);
    }

    public List<Course> findCoursesWhereIsMember(User user, Long termId) {
        var courseIds = user.getStaffMemberships().stream()
            .map(m -> m.getCourse())
            .distinct()
            .filter(c -> c.getTerm() == null || (termId != null && c.getTerm().getId() == termId))
            .map(c -> c.getId())
            .toList();

        return courseRepository.findAllById(courseIds);
    }

    public List<Course> findAll(CourseQuery query) {
        Specification<Course> spec = createSpec(query);
        return courseRepository.findAll(spec, query.getSort());
    }

    public List<User> getParticipants(Course course, String query) {
        var filter = new UserFilter.Builder().courseId(course.getId()).query(query).build();
        var userQuery = new UserQuery.Builder().setFilter(filter).setSort(UserSort.byName())
            .build();

        return userRepository.findAll(userQuery);
    }

    public boolean hasMembership(Course course, User user, StaffGroup group) {
        return course.getMemberships().stream()
            .filter(m -> m.getUser().getId() == user.getId() && m.getGroup() == group)
            .count() != 0;
    }

    public AddStaffResult addStaff(Course course, User user, StaffGroup group) {
        var membership = new CourseStaffMembership(user, course, group);
        if (!course.getMemberships().contains(membership)) {
            course.getMemberships().add(membership);
        }
        return new AddStaffResult(courseRepository.save(course), membership);
    }

    @Override
    public CourseStaffMembership getStaffMembership(Course course, User user, StaffGroup group) {
        var membership = course.getMemberships().stream()
            .filter(m -> m.getUser().getId() == user.getId() && m.getGroup() == group)
            .findFirst().orElse(null);
        return membership;
    }

    public Course removeStaffMembership(Course course, CourseStaffMembership membership) {
        course.getMemberships().remove(membership);
        return courseRepository.save(course);
    }

    private Specification<Course> createSpec(CourseQuery query) {
        Specification<Course> spec = query.getFilter().toSpecification();

        var policyContext = query.getPolicyContext();
        if (policyContext != null && policyContext.getPermission() != null) {
            var authSpec = policy.checkAsSpec(query.getPolicyContext());
            spec = spec.and(authSpec);
        }

        return spec;
    }
}
