package de.hsrm.sls.subato.courses.course.usecase;

import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.courses.course.CourseNotFoundException;
import de.hsrm.sls.subato.courses.course.dto.CreateCourseRequest;
import de.hsrm.sls.subato.courses.course.model.Course;
import de.hsrm.sls.subato.courses.course.model.CourseVisibility;
import de.hsrm.sls.subato.courses.course.model.SubmitPolicy;
import de.hsrm.sls.subato.courses.course.repository.CourseRepository;
import de.hsrm.sls.subato.courses.course.repository.DefaultDescriptionRepository;
import de.hsrm.sls.subato.courses.course.supplement.CourseSupplementFactory;
import de.hsrm.sls.subato.courses.staff.model.CourseStaffMembership;
import de.hsrm.sls.subato.courses.staff.model.StaffGroup;
import de.hsrm.sls.subato.courses.unit.CreateCourseUnitRequest;
import de.hsrm.sls.subato.courses.unit.model.CourseUnitType;
import de.hsrm.sls.subato.courses.unit.usecase.CreateUnitUseCase;
import de.hsrm.sls.subato.exercises.exercise.model.Exercise;
import de.hsrm.sls.subato.exercises.exercise.repository.ExerciseFilter;
import de.hsrm.sls.subato.exercises.exercise.repository.ExerciseQuery;
import de.hsrm.sls.subato.exercises.exercise.repository.ExerciseRepository;
import de.hsrm.sls.subato.exercises.exercise.repository.ExerciseSort;
import de.hsrm.sls.subato.exercises.taskinstance.TaskInstance;
import de.hsrm.sls.subato.exercises.taskinstance.repository.TaskInstanceFilter;
import de.hsrm.sls.subato.exercises.taskinstance.repository.TaskInstanceQuery;
import de.hsrm.sls.subato.exercises.taskinstance.repository.TaskInstanceRepository;
import de.hsrm.sls.subato.exercises.taskinstance.repository.TaskInstanceSort;
import de.hsrm.sls.subato.modules.progmodule.ProgramModuleRepository;
import de.hsrm.sls.subato.modules.term.TermNotFoundException;
import de.hsrm.sls.subato.modules.term.TermRepository;
import de.hsrm.sls.subato.shared.helper.StreamHelper;
import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.user.repository.UserRepository;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class CreateCourseUseCase {

    @Autowired
    private CourseRepository courseRepository;

    @Autowired
    private DefaultDescriptionRepository defaultDescriptionRepository;

    @Autowired
    private TermRepository termRepository;

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private ProgramModuleRepository programModuleRepository;

    @Autowired
    TaskInstanceRepository taskInstanceRepository;

    @Autowired
    ExerciseRepository exerciseRepository;

    @Autowired
    private CreateUnitUseCase createUnitUseCase;

    /**
     * @throws de.hsrm.sls.subato.modules.term.TermNotFoundException
     * @throws de.hsrm.sls.subato.user.UserNotFoundException
     * @throws CourseNotFoundException                               if the template course was not
     *                                                               found
     */
    public Course create(CreateCourseRequest dto, SubatoPrincipal principal) {
        var templateCourse = dto.getCourseId() != null
            ? courseRepository.findById(dto.getCourseId())
            .orElseThrow(() -> new CourseNotFoundException(dto.getCourseId()))
            : null;

        var user = principal != null ? userRepository.findByPrincipal(principal) : null;
        var term = dto.getTermId() != null
            ? termRepository.findById(dto.getTermId())
            .orElseThrow(() -> new TermNotFoundException(dto.getTermId()))
            : null;
        var submitPolicy = dto.getSubmitPolicy() == null ?
            dto.getTermId() == null ? SubmitPolicy.EVERYONE : SubmitPolicy.PARTICIPANTS
            : dto.getSubmitPolicy();
        var visibility =
            dto.getVisibility() == null ? CourseVisibility.PRIVATE : dto.getVisibility();

        var course = new Course();
        course.setName(dto.getName());
        course.setShortName(dto.getShortName());
        course.setShowEmails(dto.isShowEmails());
        course.setDescription(defaultDescriptionRepository.getDefaultDescription());
        course.setVisibility(visibility);
        course.setSubmitPolicy(submitPolicy);
        course.setTerm(term);
        course.setSourceId(dto.getSourceId());

        var modules = programModuleRepository.findAllById(dto.getModuleIds());
        course.setModules(modules);
        course = courseRepository.save(course);

        if (user != null) {
            var result = courseRepository.addStaff(course, user, StaffGroup.ADMIN);
            course = result.course();
        }

        if (templateCourse != null) {
            course = copyCourse(templateCourse, course);
        }

        createDefaultUnits(course);

        var supplement = CourseSupplementFactory.create();
        if (supplement != null) {
            course = supplement.supplement(course);
        }

        return course;
    }

    private void createDefaultUnits(Course course) {
        if (course.getSandbox() == null) {
            var staffMembers = course.getMemberships().stream()
                .map(CourseStaffMembership::getUser)
                .filter(StreamHelper.distinctByKey(User::getId))
                .collect(Collectors.toList());

            var request = new CreateCourseUnitRequest.Builder()
                .setName("Sandkasten")
                .setCourse(course)
                .setParticipants(staffMembers)
                .setType(CourseUnitType.Misc)
                .build();
            createUnitUseCase.create(request);
        }

        if (course.getDefault() == null) {
            var request = new CreateCourseUnitRequest.Builder()
                .setName("Standard")
                .setCourse(course)
                .setType(CourseUnitType.Misc)
                .build();
            createUnitUseCase.create(request);
        }
    }

    private Course copyCourse(Course source, Course target) {
        target.setDescription(source.getDescription());
        target.setSubmitPolicy(source.getSubmitPolicy());
        target.setShowEmails(source.isShowEmails());

        for (var m : source.getMemberships()) {
            var result = courseRepository.addStaff(target, m.getUser(), m.getGroup());
            target = result.course();
        }

        copyExercises(source, target);
        return target;
    }

    private void copyExercises(Course source, Course target) {
        var exerciseFilter = new ExerciseFilter.Builder().courseId(source.getId()).build();
        var exerciseQuery = new ExerciseQuery.Builder()
            .setFilter(exerciseFilter)
            .setSort(ExerciseSort.byId())
            .build();
        var exercises = exerciseRepository.findAll(exerciseQuery);

        for (var ex : exercises) {
            var exercise = new Exercise();
            exercise.setSummary(ex.getSummary());
            exercise.setName(ex.getName());
            exercise.setCourse(target);
            exercise.setAssessmentMode(ex.getAssessmentMode());
            exercise.setDescription(ex.getDescription());

            var taskInstanceFilter = new TaskInstanceFilter.Builder().exerciseId(ex.getId())
                .build();
            var taskInstanceQuery = new TaskInstanceQuery.Builder()
                .setFilter(taskInstanceFilter)
                .setSort(TaskInstanceSort.byOrderNumber()).build();
            var taskInstances = taskInstanceRepository.findAll(taskInstanceQuery);

            for (var ti : taskInstances) {
                var taskInstance = new TaskInstance();
                taskInstance.setExercise(exercise);
                taskInstance.setTask(ti.getTask());
                taskInstance.setOrderNumber(ti.getOrderNumber());
                taskInstance.setReachablePoints(ti.getReachablePoints());
                exercise.getTaskInstances().add(taskInstance);
            }

            exercise = exerciseRepository.save(exercise);
            target.getExercises().add(exercise);
        }
    }
}
