package de.hsrm.sls.subato.courses.file;

import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.courses.CoursesAction;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.courses.course.CourseNotFoundException;
import de.hsrm.sls.subato.courses.course.repository.CourseRepository;
import de.hsrm.sls.subato.courses.file.dto.CourseFileDto;
import de.hsrm.sls.subato.courses.file.dto.CourseFileMapper;
import de.hsrm.sls.subato.courses.file.dto.CreateCourseFileDto;
import de.hsrm.sls.subato.courses.file.repository.CourseFileRepository;
import de.hsrm.sls.subato.courses.file.usecase.CreateFileUseCase;
import de.hsrm.sls.subato.courses.file.usecase.DeleteFileUseCase;
import de.hsrm.sls.subato.courses.file.usecase.GetFileUseCase;
import de.hsrm.sls.subato.shared.fs.FileContent;
import de.hsrm.sls.subato.shared.response.FileResponseBuilderFactory;
import de.hsrm.sls.subato.shared.springext.ResponseStatusMapping;
import de.hsrm.sls.subato.shared.swagger.SubatoApiResponse;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestPart;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

@RestController
@Tag(name = "Kursverwaltung")
public class CourseFileController {

    @Autowired
    private CourseRepository courseRepository;

    @Autowired
    private CourseFileRepository courseFileRepository;

    @Autowired
    private CourseFileMapper mapper;

    @Autowired
    private PolicyEvaluator policy;

    @Autowired
    private FileResponseBuilderFactory fileResponseBuilderFactory;

    @Autowired
    private CreateFileUseCase createFileUseCase;

    @Autowired
    private DeleteFileUseCase deleteFileUseCase;

    @Autowired
    private GetFileUseCase getFileUseCase;


    @GetMapping("/course_file/{id}/download")
    @SubatoApiResponse(CourseFileNotFoundException.class)
    public ResponseEntity<Resource> download(@PathVariable Long id,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var courseFile = courseFileRepository.findById(id)
            .orElseThrow(() -> new CourseFileNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(courseFile.getCourse())
            .setPermission(CoursesAction.COURSE_VIEW).build();
        policy.checkAndRaise(context);

        var file = getFileUseCase.getFile(courseFile);
        return fileResponseBuilderFactory.forFile(file)
            .setAsAttachment(true)
            .build();
    }

    @PostMapping("/course_file")
    @ResponseStatusMapping(exception = CourseNotFoundException.class, code = HttpStatus.BAD_REQUEST)
    CourseFileDto create(@RequestPart("file") MultipartFile file,
        @RequestPart("metadata") CreateCourseFileDto dto,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var course = courseRepository.findById(dto.getCourseId())
            .orElseThrow(() -> new CourseNotFoundException(dto.getCourseId()));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(course)
            .setPermission(CoursesAction.COURSE_UPDATE).build();
        policy.checkAndRaise(context);

        var courseFile = createFileUseCase.create(FileContent.fromMultipartFile(file), course,
            dto);
        return mapper.map(courseFile);
    }

    @DeleteMapping("/course_file/{id}")
    @SubatoApiResponse(CourseFileNotFoundException.class)
    void delete(@PathVariable Long id, @AuthenticationPrincipal SubatoPrincipal principal) {
        var courseFile = courseFileRepository.findById(id)
            .orElseThrow(() -> new CourseFileNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(courseFile.getCourse())
            .setPermission(CoursesAction.COURSE_UPDATE).build();
        policy.checkAndRaise(context);

        deleteFileUseCase.delete(courseFile);
    }
}
