package de.hsrm.sls.subato.courses.result;

import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.courses.CoursesAction;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.courses.course.CourseNotFoundException;
import de.hsrm.sls.subato.courses.course.repository.CourseRepository;
import de.hsrm.sls.subato.courses.result.dto.CourseResultDto;
import de.hsrm.sls.subato.courses.result.dto.CourseResultMapper;
import de.hsrm.sls.subato.courses.result.usecase.BuildCourseResultUseCase;
import de.hsrm.sls.subato.courses.result.usecase.CourseResultFilter;
import de.hsrm.sls.subato.shared.swagger.SubatoApiResponse;
import de.hsrm.sls.subato.user.UserNotFoundException;
import de.hsrm.sls.subato.user.repository.UserRepository;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import java.util.Optional;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RestController
@Tag(name = "Kursverwaltung")
public class CourseResultController {

    @Autowired
    private BuildCourseResultUseCase buildCourseResultUseCase;

    @Autowired
    private CourseResultMapper courseResultMapper;

    @Autowired
    private CourseRepository courseRepository;

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private PolicyEvaluator policy;

    @GetMapping("/course/{courseId}/result/{userId}")
    @Operation(summary = "Liefert Informationen zum Fortschritt (Zusammenfassung) eines Benutzers über alle Übungen eines Kurses hineweg")
    @SubatoApiResponse(CourseNotFoundException.class)
    @SubatoApiResponse(UserNotFoundException.class)
    CourseResultDto getProgressForUser(@PathVariable Long courseId, @PathVariable Long userId,
        @RequestParam Optional<Boolean> showInvisibleExercises,
        @RequestParam("q") Optional<String> query,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(Pair.of(courseId, userId))
            .setPermission(CoursesAction.COURSE_PROGRESS_VIEW).build();
        policy.checkAndRaise(context);

        var course = courseRepository.findById(courseId)
            .orElseThrow(() -> new CourseNotFoundException(courseId));
        var user = userRepository.findById(userId)
            .orElseThrow(() -> new UserNotFoundException(userId));

        var filter = new CourseResultFilter.Builder()
            .setShowInvisibleExercises(showInvisibleExercises.orElse(null))
            .setQuery(query.orElse(null))
            .build();
        var courseResult = buildCourseResultUseCase.build(course, user, filter);
        return courseResultMapper.map(courseResult);
    }
}
