package de.hsrm.sls.subato.courses.result.usecase;

import de.hsrm.sls.subato.assessment.submission.repository.SubmissionRepository;
import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.exercises.ExercisesAction;
import de.hsrm.sls.subato.auth.principal.PrincipalService;
import de.hsrm.sls.subato.courses.course.model.Course;
import de.hsrm.sls.subato.courses.result.model.CourseResult;
import de.hsrm.sls.subato.courses.result.model.ExerciseResult;
import de.hsrm.sls.subato.exercises.exercise.model.Exercise;
import de.hsrm.sls.subato.exercises.exercise.repository.ExerciseFilter;
import de.hsrm.sls.subato.exercises.exercise.repository.ExerciseQuery;
import de.hsrm.sls.subato.exercises.exercise.repository.ExerciseRepository;
import de.hsrm.sls.subato.exercises.exercise.repository.ExerciseSort;
import de.hsrm.sls.subato.exercises.taskinstance.repository.TaskInstanceFilter;
import de.hsrm.sls.subato.exercises.taskinstance.repository.TaskInstanceQuery;
import de.hsrm.sls.subato.exercises.taskinstance.repository.TaskInstanceRepository;
import de.hsrm.sls.subato.exercises.taskinstance.repository.TaskInstanceSort;
import de.hsrm.sls.subato.solutions.solution.model.Solution;
import de.hsrm.sls.subato.solutions.solution.repository.SolutionSort;
import de.hsrm.sls.subato.user.User;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class BuildCourseResultUseCase {

    @Autowired
    private SubmissionRepository submissionRepository;

    @Autowired
    private ExerciseRepository exerciseRepository;

    @Autowired
    private TaskInstanceRepository taskInstanceService;

    @Autowired
    private PrincipalService principalService;


    public CourseResult build(Course course, User user, CourseResultFilter filter) {
        List<ExerciseResult> exerciseResults = new ArrayList<>();

        var exercises = getExercises(course, filter);
        for (var exercise : exercises) {
            var exerciseResult = new ExerciseResult();
            exerciseResult.setExercise(exercise);

            var taskInstanceFilter = new TaskInstanceFilter.Builder().exerciseId(exercise.getId())
                .build();
            var taskInstanceQuery = new TaskInstanceQuery.Builder()
                .setFilter(taskInstanceFilter)
                .setSort(TaskInstanceSort.byOrderNumber()).build();

            var taskInstances = taskInstanceService.findAll(taskInstanceQuery);
            exerciseResult.setTaskInstances(taskInstances);

            var submission = submissionRepository.findByExerciseAndUser(exercise.getId(),
                user.getId());
            if (submission != null) {
                exerciseResult.setSubmission(submission);

                var solutionsToConsider = submission.getSolution().stream().toList();
                var taskInstanceSolutions = solutionsToConsider.stream()
                    .filter(s -> s.getTaskInstance() != null)
                    .collect(Collectors.groupingBy(s -> s.getTaskInstance()));
                var latestSolutions = taskInstanceSolutions.entrySet().stream()
                    .map(entry -> getLatestSolution(entry.getValue())).collect(Collectors.toList());
                exerciseResult.setSolutions(latestSolutions);

                var testsTotal = latestSolutions.stream().filter(
                        s -> s.getEvalState() != null && s.getEvalState().getResult() != null
                            && s.getEvalState().getResult().getTest() != null)
                    .map(s -> s.getEvalState().getResult().getTest().getTestcases())
                    .reduce(0, (a, b) -> a + b);
                var testsPassed = latestSolutions.stream().filter(
                        s -> s.getEvalState() != null && s.getEvalState().getResult() != null
                            && s.getEvalState().getResult().getTest() != null)
                    .map(s -> s.getEvalState().getResult().getTest().getPassed())
                    .reduce(0, (a, b) -> a + b);
                var leaks = latestSolutions.stream().filter(
                        s -> s.getEvalState() != null && s.getEvalState().getResult() != null
                            && s.getEvalState().getResult().getTest() != null
                            && s.getEvalState().getResult().getTest().getLeaks() != null)
                    .map(s -> s.getEvalState().getResult().getTest().getLeaks())
                    .reduce(0, (a, b) -> a + b);
                var styleErrors = latestSolutions.stream().filter(
                        s -> s.getEvalState() != null && s.getEvalState().getResult() != null
                            && s.getEvalState().getResult().getStyle() != null)
                    .map(s -> s.getEvalState().getResult().getStyle().getErrors())
                    .reduce(0, (a, b) -> a + b);

                exerciseResult.setTestsTotal(testsTotal);
                exerciseResult.setTestsPassed(testsPassed);
                exerciseResult.setLeaksTotal(leaks);
                exerciseResult.setStyleErrorsTotal(styleErrors);
            }

            exerciseResults.add(exerciseResult);
        }

        exerciseResults = filterByQuery(exerciseResults, filter.getQuery());
        exerciseResults.sort(
            Comparator.comparing(ExerciseResult::getExercise, ExerciseSort.byId().reversed()));
        return new CourseResult(exerciseResults);
    }

    private List<Exercise> getExercises(Course course, CourseResultFilter filter) {
        var exerciseFilter = new ExerciseFilter.Builder().courseId(course.getId()).build();
        var context = new PolicyContext.Builder()
            .setSubject(principalService.getPrincipal())
            .setPermission(ExercisesAction.EXERCISE_VIEW).build();
        var exerciseQuery = new ExerciseQuery.Builder()
            .setPolicyContext(context)
            .setFilter(exerciseFilter)
            .setSort(ExerciseSort.byId()).build();
        var exercises = exerciseRepository.findAll(exerciseQuery);

        var showInvisibleExercises =
            filter.isShowInvisibleExercises() != null && filter.isShowInvisibleExercises();
        return exercises.stream()
            .filter(e -> e.isVisible() || showInvisibleExercises)
            .toList();
    }

    private List<ExerciseResult> filterByQuery(List<ExerciseResult> exerciseResults, String query) {
        if (query == null) {
            return exerciseResults;
        }

        return exerciseResults.stream().filter(
                r -> r.getExercise().getName().toLowerCase().contains(query.toLowerCase()) ||
                    r.getExercise().getTaskInstances().stream().anyMatch(
                        ti -> ti.getTask().getName().toLowerCase().contains(query.toLowerCase())
                    )
            )
            .collect(Collectors.toList());
    }

    // todo
    private Solution getLatestSolution(List<Solution> solutions) {
        solutions.sort(SolutionSort.bySubmittedAtDesc());
        return solutions.stream().findFirst().orElse(null);
    }
}
