package de.hsrm.sls.subato.courses.staff;

import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.courses.CoursesAction;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.courses.course.CourseNotFoundException;
import de.hsrm.sls.subato.courses.course.repository.CourseRepository;
import de.hsrm.sls.subato.courses.staff.dto.CourseStaffMembershipDetailDto;
import de.hsrm.sls.subato.courses.staff.dto.CourseStaffMembershipMapper;
import de.hsrm.sls.subato.courses.staff.model.StaffGroup;
import de.hsrm.sls.subato.courses.staff.usecase.AddStaffUseCase;
import de.hsrm.sls.subato.courses.staff.usecase.RemoveStaffUseCase;
import de.hsrm.sls.subato.shared.swagger.SubatoApiResponse;
import de.hsrm.sls.subato.user.UserNotFoundException;
import de.hsrm.sls.subato.user.repository.UserRepository;
import io.swagger.v3.oas.annotations.tags.Tag;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
@Tag(name = "Kursverwaltung")
public class CourseStaffController {

    @Autowired
    private CourseRepository courseRepository;

    @Autowired
    private AddStaffUseCase addStaffUseCase;

    @Autowired
    private RemoveStaffUseCase removeStaffUseCase;

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private PolicyEvaluator policy;

    @Autowired
    private CourseStaffMembershipMapper courseStaffMembershipMapper;


    @GetMapping("/course/{id}/staff")
    @SubatoApiResponse(CourseNotFoundException.class)
    List<CourseStaffMembershipDetailDto> getStaff(@PathVariable Long id,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var course = courseRepository.findById(id)
            .orElseThrow(() -> new CourseNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(course)
            .setPermission(CoursesAction.COURSE_VIEW).build();
        policy.checkAndRaise(context);

        var memberships = course.getMemberships();
        return memberships.stream()
            .map(membership -> courseStaffMembershipMapper.mapDetail(membership)).toList();
    }

    @DeleteMapping("/course/{id}/staff/{userId}/group/{group}")
    @SubatoApiResponse(CourseNotFoundException.class)
    @SubatoApiResponse(UserNotFoundException.class)
    @SubatoApiResponse(CourseStaffMembershipNotFoundException.class)
    public void removeStaff(@PathVariable Long id, @PathVariable Long userId,
        @PathVariable StaffGroup group,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var course = courseRepository.findById(id)
            .orElseThrow(() -> new CourseNotFoundException(id));
        var user = userRepository.findById(userId)
            .orElseThrow(() -> new UserNotFoundException(userId));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(course)
            .setPermission(CoursesAction.COURSE_UPDATE).build();
        policy.checkAndRaise(context);

        removeStaffUseCase.removeStaff(course, user, group);
    }

    @PutMapping("/course/{id}/staff/{userId}/group/{group}")
    @SubatoApiResponse(CourseNotFoundException.class)
    @SubatoApiResponse(UserNotFoundException.class)
    @SubatoApiResponse(CourseStaffMembershipAlreadyExistsException.class)
    public CourseStaffMembershipDetailDto addStaff(@PathVariable Long id, @PathVariable Long userId,
        @PathVariable StaffGroup group, @AuthenticationPrincipal SubatoPrincipal principal) {
        var course = courseRepository.findById(id)
            .orElseThrow(() -> new CourseNotFoundException(id));
        var user = userRepository.findById(userId)
            .orElseThrow(() -> new UserNotFoundException(userId));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(course)
            .setPermission(CoursesAction.COURSE_UPDATE).build();
        policy.checkAndRaise(context);

        var result = addStaffUseCase.addStaff(course, user, group);
        return courseStaffMembershipMapper.mapDetail(result.membership());
    }
}
