package de.hsrm.sls.subato.courses.unit;

import de.hsrm.sls.subato.attendances.attendance.dto.ParticipantAttendanceDetailDto;
import de.hsrm.sls.subato.attendances.attendance.dto.ParticipantAttendanceMapper;
import de.hsrm.sls.subato.attendances.attendance.usecase.GetAttendanceUseCase;
import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.attendance.AttendancePermission;
import de.hsrm.sls.subato.auth.policy.courses.CoursesAction;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.courses.event.dto.CourseUnitEventMapper;
import de.hsrm.sls.subato.courses.event.dto.CourseUnitEventSummaryDto;
import de.hsrm.sls.subato.courses.unit.dto.CourseUnitDto;
import de.hsrm.sls.subato.courses.unit.dto.CourseUnitMapper;
import de.hsrm.sls.subato.courses.unit.exception.AlreadyJoinedException;
import de.hsrm.sls.subato.courses.unit.exception.CourseUnitNotFoundException;
import de.hsrm.sls.subato.courses.unit.exception.JoinNotPossibleException;
import de.hsrm.sls.subato.courses.unit.exception.JoinsExceededException;
import de.hsrm.sls.subato.courses.unit.exception.UserIsAlreadyParticipantException;
import de.hsrm.sls.subato.courses.unit.exception.UserIsNotAParticipantException;
import de.hsrm.sls.subato.courses.unit.repository.CourseUnitRepository;
import de.hsrm.sls.subato.courses.unit.usecase.AddParticipantUseCase;
import de.hsrm.sls.subato.courses.unit.usecase.JoinUnitUseCase;
import de.hsrm.sls.subato.courses.unit.usecase.RemoveParticipantUseCase;
import de.hsrm.sls.subato.shared.swagger.SubatoApiResponse;
import de.hsrm.sls.subato.user.UserNotFoundException;
import de.hsrm.sls.subato.user.dto.UserDetailDto;
import de.hsrm.sls.subato.user.dto.UserMapper;
import de.hsrm.sls.subato.user.repository.UserRepository;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RestController
@Tag(name = "Kursverwaltung")
public class CourseUnitController {

    @Autowired
    CourseUnitRepository courseUnitRepository;

    @Autowired
    private UserRepository userRepository;

    @Autowired
    CourseUnitMapper courseUnitMapper;

    @Autowired
    CourseUnitEventMapper courseUnitEventMapper;

    @Autowired
    private ParticipantAttendanceMapper participantAttendanceMapper;

    @Autowired
    private GetAttendanceUseCase getAttendanceUseCase;

    @Autowired
    UserMapper userMapper;

    @Autowired
    private PolicyEvaluator policy;

    @Autowired
    private AddParticipantUseCase addParticipantUseCase;

    @Autowired
    private RemoveParticipantUseCase removeParticipantUseCase;

    @Autowired
    private JoinUnitUseCase joinUnitUseCase;

    @GetMapping("/course_unit/{id}")
    @SubatoApiResponse(CourseUnitNotFoundException.class)
    CourseUnitDto getOne(@PathVariable Long id,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var courseUnit = courseUnitRepository.findById(id)
            .orElseThrow(() -> new CourseUnitNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(courseUnit.getCourse())
            .setPermission(CoursesAction.COURSE_VIEW).build();
        policy.checkAndRaise(context);

        return courseUnitMapper.map(courseUnit);
    }

    @GetMapping("/course_unit/{id}/events")
    @SubatoApiResponse(CourseUnitNotFoundException.class)
    List<CourseUnitEventSummaryDto> getEvents(@PathVariable Long id,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var courseUnit = courseUnitRepository.findById(id)
            .orElseThrow(() -> new CourseUnitNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(courseUnit.getCourse())
            .setPermission(CoursesAction.COURSE_VIEW).build();
        policy.checkAndRaise(context);

        var events = courseUnitRepository.getEvents(courseUnit);
        return events.stream().map(e -> courseUnitEventMapper.mapSummary(e)).toList();
    }

    @GetMapping("/course_unit/{id}/participants")
    @SubatoApiResponse(CourseUnitNotFoundException.class)
    List<UserDetailDto> getParticipants(@PathVariable Long id,
        @RequestParam("q") Optional<String> query,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var courseUnit = courseUnitRepository.findById(id)
            .orElseThrow(() -> new CourseUnitNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(courseUnit.getCourse())
            .setPermission(CoursesAction.COURSE_VIEW_PARTICIPANTS).build();
        policy.checkAndRaise(context);

        var participants = courseUnitRepository.getParticipants(courseUnit, query.orElse(null));
        return participants.stream().map(u -> userMapper.mapDetail(u))
            .collect(Collectors.toList());
    }

    @PostMapping("/course_unit/{id}/join")
    @SubatoApiResponse(CourseUnitNotFoundException.class)
    @SubatoApiResponse(AlreadyJoinedException.class)
    @SubatoApiResponse(JoinNotPossibleException.class)
    @SubatoApiResponse(JoinsExceededException.class)
    public void join(@PathVariable Long id,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var courseUnit = courseUnitRepository.findById(id)
            .orElseThrow(() -> new CourseUnitNotFoundException(id));
        var user = userRepository.findByPrincipal(principal);

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(courseUnit)
            .setPermission(CoursesAction.COURSE_UNIT_JOIN).build();
        policy.checkAndRaise(context);

        joinUnitUseCase.join(courseUnit, user);
    }

    @PutMapping("/course_unit/{id}/participants/{userId}")
    @SubatoApiResponse(CourseUnitNotFoundException.class)
    @SubatoApiResponse(UserNotFoundException.class)
    @SubatoApiResponse(UserIsAlreadyParticipantException.class)
    public void addParticipant(@PathVariable Long id, @PathVariable Long userId,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var courseUnit = courseUnitRepository.findById(id)
            .orElseThrow(() -> new CourseUnitNotFoundException(id));
        var user = userRepository.findById(userId)
            .orElseThrow(() -> new UserNotFoundException(userId));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(courseUnit)
            .setPermission(CoursesAction.COURSE_UNIT_UPDATE).build();
        policy.checkAndRaise(context);

        addParticipantUseCase.add(courseUnit, user);
    }

    @DeleteMapping("/course_unit/{id}/participants/{userId}")
    @SubatoApiResponse(CourseUnitNotFoundException.class)
    @SubatoApiResponse(UserIsNotAParticipantException.class)
    public void removeParticipant(@PathVariable Long id, @PathVariable Long userId,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var courseUnit = courseUnitRepository.findById(id)
            .orElseThrow(() -> new CourseUnitNotFoundException(id));
        var user = userRepository.findById(userId)
            .orElseThrow(() -> new UserNotFoundException(userId));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(courseUnit)
            .setPermission(CoursesAction.COURSE_UNIT_UPDATE).build();
        policy.checkAndRaise(context);

        removeParticipantUseCase.remove(courseUnit, user);
    }

    @GetMapping("/course_unit/{unitId}/attendance/{userId}")
    @Operation(summary = "Zusammenfassung der Anwesenheit des Benutzers innerhalb einer Kurseinheit (z.B. Praktikumsgruppe)")
    @SubatoApiResponse(CourseUnitNotFoundException.class)
    ParticipantAttendanceDetailDto getCourseUnitAttendance(@PathVariable Long unitId,
        @PathVariable Long userId, @AuthenticationPrincipal SubatoPrincipal principal) {
        var courseUnit = courseUnitRepository.findById(unitId)
            .orElseThrow(() -> new CourseUnitNotFoundException(unitId));
        var user = userRepository.findById(userId)
            .orElseThrow(() -> new UserNotFoundException(userId));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(Pair.of(courseUnit.getCourse(), user))
            .setPermission(AttendancePermission.ATTENDANCE_VIEW).build();
        policy.checkAndRaise(context);

        var courseUnitAttendance = getAttendanceUseCase.forUnitAndUser(courseUnit, user, true);
        return participantAttendanceMapper.map(courseUnitAttendance);
    }
}
