package de.hsrm.sls.subato.courses.unit.usecase;

import de.hsrm.sls.subato.auth.principal.PrincipalService;
import de.hsrm.sls.subato.courses.course.CourseFinishedException;
import de.hsrm.sls.subato.courses.course.model.Course;
import de.hsrm.sls.subato.courses.participantprofile.model.CourseParticipantProfile;
import de.hsrm.sls.subato.courses.participantprofile.model.CourseParticipantProfileRepository;
import de.hsrm.sls.subato.courses.unit.exception.AlreadyJoinedException;
import de.hsrm.sls.subato.courses.unit.exception.JoinNotPossibleException;
import de.hsrm.sls.subato.courses.unit.exception.JoinsExceededException;
import de.hsrm.sls.subato.courses.unit.model.CourseUnit;
import de.hsrm.sls.subato.courses.unit.model.CourseUnitType;
import de.hsrm.sls.subato.courses.unit.repository.CourseUnitRepository;
import de.hsrm.sls.subato.shared.config.SubatoConfig;
import de.hsrm.sls.subato.user.User;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class JoinUnitUseCase {

    @Autowired
    private CourseUnitRepository courseUnitRepository;

    @Autowired
    private CourseParticipantProfileRepository courseParticipantRepository;

    @Autowired
    private PrincipalService principalService;

    @Autowired
    private RemoveFromOtherLabsUseCase removeFromOtherLabsUseCase;

    @Autowired
    private SubatoConfig subatoConfig;

    public synchronized void join(CourseUnit unit, User user) {
        var course = unit.getCourse();
        if (course.isFinished()) {
            throw new CourseFinishedException();
        }

        var isParticipant = courseUnitRepository.isParticipant(unit, user);
        if (isParticipant) {
            throw new AlreadyJoinedException();
        }

        if (unit.getType() != CourseUnitType.Lab) {
            throw new JoinNotPossibleException();
        }

        if (!isJoinAllowed(unit, user)) {
            throw new JoinsExceededException(subatoConfig.getMaxLabChanges());
        }

        unit.getParticipants().add(user);
        unit = courseUnitRepository.save(unit);

        removeFromOtherLabsUseCase.execute(unit, user);
        updateLabChangedCounter(user, course);

        principalService.evictPrincipal(user);
    }

    private boolean isJoinAllowed(CourseUnit unit, User user) {
        var profile = courseParticipantRepository.findFirstByCourseIdAndUserId(
            unit.getCourse().getId(), user.getId());
        var labChangedCounter = profile != null ? profile.getLabChangedCounter() : 0;
        return labChangedCounter < subatoConfig.getMaxLabChanges();
    }

    private void updateLabChangedCounter(User user, Course course) {
        var profile = courseParticipantRepository.findFirstByCourseIdAndUserId(
            course.getId(), user.getId());
        if (profile == null) {
            profile = new CourseParticipantProfile(user, course);
        }

        profile.setLabChangedCounter(profile.getLabChangedCounter() + 1);
        courseParticipantRepository.save(profile);
    }
}
