package de.hsrm.sls.subato.exercises.exercise;

import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.exercises.ExercisesAction;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.auth.stream.HtmlUrlSigner;
import de.hsrm.sls.subato.auth.stream.SignedUrlSanitizer;
import de.hsrm.sls.subato.courses.course.CourseNotFoundException;
import de.hsrm.sls.subato.courses.course.repository.CourseRepository;
import de.hsrm.sls.subato.exercises.exercise.dto.CreateExerciseDto;
import de.hsrm.sls.subato.exercises.exercise.dto.ExerciseDto;
import de.hsrm.sls.subato.exercises.exercise.dto.ExerciseMapper;
import de.hsrm.sls.subato.exercises.exercise.repository.ExerciseFilter;
import de.hsrm.sls.subato.exercises.exercise.repository.ExerciseQuery;
import de.hsrm.sls.subato.exercises.exercise.repository.ExerciseRepository;
import de.hsrm.sls.subato.exercises.exercise.repository.ExerciseSort;
import de.hsrm.sls.subato.exercises.exercise.usecase.DeleteExerciseUseCase;
import de.hsrm.sls.subato.exercises.exercise.usecase.UpdateExerciseUseCase;
import de.hsrm.sls.subato.exercises.taskinstance.dto.TaskInstanceDetailDto;
import de.hsrm.sls.subato.exercises.taskinstance.dto.TaskInstanceMapper;
import de.hsrm.sls.subato.exercises.taskinstance.repository.TaskInstanceFilter;
import de.hsrm.sls.subato.exercises.taskinstance.repository.TaskInstanceQuery;
import de.hsrm.sls.subato.exercises.taskinstance.repository.TaskInstanceRepository;
import de.hsrm.sls.subato.exercises.taskinstance.repository.TaskInstanceSort;
import de.hsrm.sls.subato.shared.CustomHttpHeaderConstants;
import de.hsrm.sls.subato.shared.exceptions.DangerousActionException;
import de.hsrm.sls.subato.shared.markdown.MarkdownParser;
import de.hsrm.sls.subato.shared.springext.ResponseStatusMapping;
import de.hsrm.sls.subato.shared.swagger.SubatoApiResponse;
import de.hsrm.sls.subato.shared.validation.ValidationException;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import org.apache.tika.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RestController
@Tag(name = "Übungsverwaltung")
public class ExerciseController {

    @Autowired
    private ExerciseMapper exerciseMapper;

    @Autowired
    private ExerciseRepository exerciseRepository;

    @Autowired
    private TaskInstanceMapper taskInstanceMapper;

    @Autowired
    private TaskInstanceRepository taskInstanceRepository;

    @Autowired
    private CourseRepository courseRepository;

    @Autowired
    private PolicyEvaluator policy;

    @Autowired
    private MarkdownParser markdownParser;

    @Autowired
    private DeleteExerciseUseCase deleteExerciseUseCase;

    @Autowired
    private UpdateExerciseUseCase updateExerciseUseCase;

    @Autowired
    private HtmlUrlSigner htmlUrlSigner;

    @Autowired
    private SignedUrlSanitizer signedUrlSanitizer;

    @GetMapping("/exercise/{id}")
    @Operation(summary = "Liefert Metadaten zu einer Übung")
    @SubatoApiResponse(ExerciseNotFoundException.class)
    ExerciseDto getOne(@PathVariable Long id, @AuthenticationPrincipal SubatoPrincipal principal) {
        var exercise = exerciseRepository.findById(id)
            .orElseThrow(() -> new ExerciseNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(exercise)
            .setPermission(ExercisesAction.EXERCISE_VIEW).build();
        policy.checkAndRaise(context);

        return exerciseMapper.map(exercise);
    }

    @PostMapping("/exercise")
    @Operation(summary = "Erstellt eine Übung")
    @SubatoApiResponse(CourseNotFoundException.class)
    @SubatoApiResponse(ValidationException.class)
    @ResponseStatusMapping(exception = CourseNotFoundException.class, code = HttpStatus.BAD_REQUEST)
    ExerciseDto create(@RequestBody CreateExerciseDto dto,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var course = courseRepository.findById(dto.getCourseId())
            .orElseThrow(
                () -> new CourseNotFoundException(dto.getCourseId()));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(course)
            .setPermission(ExercisesAction.EXERCISE_CREATE).build();
        policy.checkAndRaise(context);

        dto.setDescription(signedUrlSanitizer.sanitize(dto.getDescription()));
        var exercise = updateExerciseUseCase.create(course, dto);
        return exerciseMapper.map(exercise);
    }

    @PutMapping("/exercise/{id}")
    @Operation(
        summary = "Aktualisiert eine Übung",
        description = "Wenn dabei eine Aufgabeninstanz gelöscht wird, zu der Lösungen existieren, wird ein spezielles Fehlerobjekt (ToDo) mit dem Status 400 zurückgegeben, welches angibt, welche Aufgabeninstanz betroffen ist und wie viele Lösung gelöscht würden. Wenn das Update dennoch erfolgen soll, muss die Anfrage mit dem ConfirmDangerousAction Header wiederholt werden."
    )
    @SubatoApiResponse(DangerousActionException.class)
    @SubatoApiResponse(ExerciseNotFoundException.class)
    @SubatoApiResponse(ValidationException.class)
    ExerciseDto update(
        @PathVariable Long id,
        @RequestBody CreateExerciseDto dto,
        @Parameter(description = "Bestätigung der gefährlichen Aktion mit dem Wert true. Bei initialer Anfrage leer lassen und erst nach expliziter Bestätigung durch den Benutzer (über Dialog o.ä.) bei der wiederholten Anfrage diesen Header setzen.")
        @RequestHeader(value = CustomHttpHeaderConstants.CONFIRM_DANGEROUS_ACTION, required = false) boolean confirmDangerousAction,
        @AuthenticationPrincipal SubatoPrincipal principal
    ) {
        var exercise = exerciseRepository.findById(id)
            .orElseThrow(() -> new ExerciseNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(exercise)
            .setPermission(ExercisesAction.EXERCISE_UPDATE).build();
        policy.checkAndRaise(context);

        dto.setDescription(signedUrlSanitizer.sanitize(dto.getDescription()));
        exercise = updateExerciseUseCase.update(exercise, dto, confirmDangerousAction);
        return exerciseMapper.map(exercise);
    }

    @GetMapping("/exercise/{id}/description")
    @Operation(summary = "Liefert die Beschreibung der Übung (entweder in Rohform als Markdown oder gerendert als HTML)")
    @SubatoApiResponse(ExerciseNotFoundException.class)
    String getDescription(@PathVariable Long id,
        @Parameter(description = "false wenn nicht angegeben") @RequestParam Optional<Boolean> raw,
        HttpServletRequest req,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var exercise = exerciseRepository.findById(id)
            .orElseThrow(() -> new ExerciseNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(exercise)
            .setPermission(ExercisesAction.EXERCISE_VIEW).build();
        policy.checkAndRaise(context);

        if (StringUtils.isBlank(exercise.getDescription())) {
            return "";
        }

        var isRaw = raw.orElse(false);
        if (isRaw) {
            return exercise.getDescription();
        } else {
            var html = markdownParser.convertToHtml(exercise.getDescription());
            return htmlUrlSigner.sign(html, req);
        }
    }

    @GetMapping("/exercise")
    @Operation(summary = "Liefert Metadaten zu allen Übungen eines Kurses")
    List<ExerciseDto> getExercises(@RequestParam Long courseId,
        @RequestParam Optional<String> query, @AuthenticationPrincipal SubatoPrincipal principal) {
        var context = new PolicyContext.Builder().setSubject(principal)
            .setPermission(ExercisesAction.EXERCISE_VIEW).build();
        var filter = new ExerciseFilter.Builder().courseId(courseId).query(query.orElse(null))
            .build();

        var exerciseQuery = new ExerciseQuery.Builder()
            .setFilter(filter)
            .setPolicyContext(context)
            .setSort(ExerciseSort.byId())
            .build();

        var exercises = exerciseRepository.findAll(exerciseQuery);
        return exercises.stream()
            .map(e -> exerciseMapper.map(e))
            .collect(Collectors.toList());
    }

    @DeleteMapping("/exercise/{id}")
    @Operation(summary = "Löscht eine Übung mit allen verknüpften Lösungen, Abgaben etc.")
    @SubatoApiResponse(ExerciseNotFoundException.class)
    void delete(@PathVariable Long id, @AuthenticationPrincipal SubatoPrincipal principal) {
        var exercise = exerciseRepository.findById(id)
            .orElseThrow(() -> new ExerciseNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(exercise)
            .setPermission(ExercisesAction.EXERCISE_DELETE).build();
        policy.checkAndRaise(context);

        deleteExerciseUseCase.delete(exercise);
    }

    @GetMapping("/exercise/{id}/task")
    @Operation(summary = "Liefert die unarchivierten Aufgabeninstanzen, die zur Übung zugeordnet sind")
    @SubatoApiResponse(ExerciseNotFoundException.class)
    List<TaskInstanceDetailDto> getTaskInstances(@PathVariable Long id,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var exercise = exerciseRepository.findById(id)
            .orElseThrow(() -> new ExerciseNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(exercise)
            .setPermission(ExercisesAction.EXERCISE_VIEW).build();
        policy.checkAndRaise(context);

        var filter = new TaskInstanceFilter.Builder().exerciseId(exercise.getId()).build();
        var query = new TaskInstanceQuery.Builder()
            .setFilter(filter)
            .setSort(TaskInstanceSort.byOrderNumber())
            .build();

        var taskInstances = taskInstanceRepository.findAll(query);
        return taskInstances.stream()
            .map(taskInstance -> taskInstanceMapper.mapDetail(taskInstance)).toList();
    }
}
