package de.hsrm.sls.subato.exercises.exercise.usecase;

import de.hsrm.sls.subato.courses.course.CourseFinishedException;
import de.hsrm.sls.subato.courses.course.model.Course;
import de.hsrm.sls.subato.exercises.exercise.dto.CreateExerciseDto;
import de.hsrm.sls.subato.exercises.exercise.model.Exercise;
import de.hsrm.sls.subato.exercises.exercise.repository.ExerciseRepository;
import de.hsrm.sls.subato.exercises.taskinstance.TaskInstance;
import de.hsrm.sls.subato.shared.exceptions.DangerousActionException;
import de.hsrm.sls.subato.shared.validation.ConstraintViolationMapper;
import de.hsrm.sls.subato.shared.validation.ValidationException;
import de.hsrm.sls.subato.solutions.solution.repository.SolutionRepository;
import de.hsrm.sls.subato.solutions.solution.usecase.DeleteSolutionUseCase;
import de.hsrm.sls.subato.tasks.task.repository.TaskRepository;
import jakarta.validation.Validation;
import java.util.HashMap;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

@Component
public class UpdateExerciseUseCase {

    @Autowired
    private ExerciseRepository exerciseRepository;

    @Autowired
    private TaskRepository taskRepository;

    @Autowired
    private SolutionRepository solutionRepository;

    @Autowired
    private DeleteSolutionUseCase deleteSolutionUseCase;


    public Exercise create(Course course, CreateExerciseDto dto) {
        if (course.isFinished()) {
            throw new CourseFinishedException();
        }

        var exercise = new Exercise();
        exercise.setCourse(course);
        setExerciseFields(exercise, dto, false);

        validateExercise(exercise);
        return exerciseRepository.save(exercise);
    }

    /**
     * updates the exercise with the given field values
     *
     * @param exercise                to update
     * @param dto                     field values to update
     * @param confirmDangerousActions if this update action was explicitly confirmed by the user.
     *                                required if the action is dangerous and needs to be confirmed
     *                                again. the update is dangerous if task instances are removed
     *                                that have linked solutions because these solutions would be
     *                                deleted
     * @return updated exercise
     * @throws DangerousActionException if solutions will be removed while updating and
     *                                  {@code confirmDangerousActions} is false
     */
    @Transactional
    public Exercise update(Exercise exercise, CreateExerciseDto dto,
        boolean confirmDangerousActions) {
        if (exercise.getCourse().isFinished()) {
            throw new CourseFinishedException();
        }

        setExerciseFields(exercise, dto, confirmDangerousActions);
        validateExercise(exercise);
        return exerciseRepository.save(exercise);
    }

    private void updateTaskInstances(Exercise exercise, CreateExerciseDto dto,
        boolean confirmDangerousActions) {
        var toRemove = exercise.getTaskInstances().stream()
            .filter(ti -> dto.getTaskInstances().stream()
                .noneMatch(tiDto -> ti.getTask().getId() == tiDto.getTaskId()))
            .collect(Collectors.toList());

        var taskInstancesWithSolutions = new HashMap<TaskInstance, Long>();
        for (var taskInstance : toRemove) {
            var solutionCount = solutionRepository.getSolutionCountForTaskInstance(taskInstance);
            if (solutionCount != 0) {
                taskInstancesWithSolutions.put(taskInstance, solutionCount);
            }
        }

        if (taskInstancesWithSolutions.size() != 0 && !confirmDangerousActions) {
            var infoString = taskInstancesWithSolutions.entrySet().stream().map(
                    e -> "%s (%s Lösungen)".formatted(e.getKey().getTask().getName(), e.getValue()))
                .collect(Collectors.joining(", "));
            throw new DangerousActionException(
                "Einige Aufgaben haben Lösungen, die beim Speichern gelöscht werden würden: %s".formatted(
                    infoString),
                "Eventuell wäre es sinnvoller die Aufgaben nicht zu löschen, sondern stattdessen als optional zu markieren"
            );
        }

        exercise.getTaskInstances().removeAll(toRemove);
        for (var taskInstance : toRemove) {
            deleteSolutionUseCase.deleteByTaskInstance(taskInstance);
        }

        for (var taskInstanceDto : dto.getTaskInstances()) {
            var taskInstance = exercise.getTaskInstances().stream()
                .filter(ti -> ti.getTask().getId() == taskInstanceDto.getTaskId())
                .findFirst()
                .orElse(null);

            if (taskInstance == null) {
                taskInstance = new TaskInstance();
                exercise.getTaskInstances().add(taskInstance);

                taskInstance.setExercise(exercise);
                var task = taskRepository.findById(taskInstanceDto.getTaskId()).orElse(null);
                if (task == null) {
                    throw new RuntimeException(
                        "Task not found with id %s".formatted(taskInstanceDto.getTaskId()));
                }
                taskInstance.setTask(task);
            }

            taskInstance.setReachablePoints(taskInstanceDto.getReachablePoints());
            taskInstance.setOrderNumber(taskInstanceDto.getOrderNumber());
        }
    }

    private void setExerciseFields(Exercise exercise, CreateExerciseDto dto,
        boolean confirmDangerousActions) {
        exercise.setSummary(dto.getSummary());
        exercise.setName(dto.getName());
        exercise.setDescription(dto.getDescription());
        exercise.setAssessmentMode(dto.getAssessmentMode());
        exercise.setDueTo(dto.getDueTo());
        exercise.setVisibleFrom(dto.getVisibleFrom());
        exercise.setOrderNumber(dto.getOrderNumber());
        updateTaskInstances(exercise, dto, confirmDangerousActions);
    }

    private void validateExercise(Exercise exercise) {
        var validator = Validation.buildDefaultValidatorFactory().getValidator();
        var violations = validator.validate(exercise);
        if (!violations.isEmpty()) {
            throw new ValidationException(ConstraintViolationMapper.map(violations));
        }
    }
}
