package de.hsrm.sls.subato.exercises.taskinstance;

import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.exercises.ExercisesAction;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.auth.stream.HtmlUrlSigner;
import de.hsrm.sls.subato.exercises.exercise.dto.UpdatePublicDto;
import de.hsrm.sls.subato.exercises.taskinstance.dto.TaskInstanceDetailDto;
import de.hsrm.sls.subato.exercises.taskinstance.dto.TaskInstanceMapper;
import de.hsrm.sls.subato.exercises.taskinstance.repository.TaskInstanceRepository;
import de.hsrm.sls.subato.exercises.taskinstance.usecase.PublishSolutionUseCase;
import de.hsrm.sls.subato.shared.response.FileResponseBuilderFactory;
import de.hsrm.sls.subato.shared.swagger.SubatoApiResponse;
import de.hsrm.sls.subato.tasks.stef.dto.FileRefDto;
import de.hsrm.sls.subato.tasks.stef.dto.FileRefMapper;
import de.hsrm.sls.subato.tasks.task.TaskContentAssetUrlRewriter;
import de.hsrm.sls.subato.tasks.task.exception.AssetNotFoundException;
import de.hsrm.sls.subato.tasks.task.exception.FileNotFoundException;
import de.hsrm.sls.subato.tasks.task.exception.TaskArchivedException;
import de.hsrm.sls.subato.tasks.task.exception.TaskNotFoundException;
import de.hsrm.sls.subato.tasks.task.repository.BundleRepository;
import de.hsrm.sls.subato.tasks.task.usecase.DownloadAssetUseCase;
import de.hsrm.sls.subato.tasks.task.usecase.DownloadFileUseCase;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.Resource;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RestController
@Tag(name = "Übungsverwaltung")
public class TaskInstanceController {

    @Autowired
    private FileRefMapper fileRefMapper;

    @Autowired
    private TaskInstanceMapper taskInstanceMapper;

    @Autowired
    private TaskInstanceRepository taskInstanceRepository;

    @Autowired
    private PolicyEvaluator policy;

    @Autowired
    private FileResponseBuilderFactory fileResponseBuilderFactory;

    @Autowired
    private TaskContentAssetUrlRewriter taskContentAssetUrlRewriter;

    @Autowired
    private BundleRepository bundleRepository;

    @Autowired
    private PublishSolutionUseCase publishSolutionUseCase;

    @Autowired
    private DownloadFileUseCase downloadFileUseCase;

    @Autowired
    private DownloadAssetUseCase downloadAssetUseCase;

    @Autowired
    private HtmlUrlSigner htmlUrlSigner;


    @PutMapping("/exercise/{exerciseId}/task/{taskId}/public")
    @Operation(summary = "Veröffentlicht (oder versteckt) die Musterlösung, die mit der Aufgabe verknüpft ist, im Rahmen der angegebenen Aufgabeninstanz")
    @SubatoApiResponse(TaskInstanceNotFoundException.class)
    void publishSolution(@PathVariable Long exerciseId, @PathVariable Long taskId,
        @RequestBody UpdatePublicDto dto, @AuthenticationPrincipal SubatoPrincipal principal) {
        var taskInstance = taskInstanceRepository.findByExerciseAndTask(exerciseId, taskId)
            .orElseThrow(() -> new TaskInstanceNotFoundException(exerciseId, taskId));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(taskInstance.getExercise())
            .setPermission(ExercisesAction.EXERCISE_UPDATE).build();
        policy.checkAndRaise(context);

        publishSolutionUseCase.publish(taskInstance, dto.isPub());
    }

    @GetMapping("exercise/{exerciseId}/task/{taskId}/content")
    @SubatoApiResponse(TaskInstanceNotFoundException.class)
    @SubatoApiResponse(TaskArchivedException.class)
        // todo: not thrown?
    String getContent(@PathVariable Long exerciseId, @PathVariable Long taskId,
        @RequestParam Optional<Boolean> raw, HttpServletRequest req,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        // todo: why is raw not used?
        var taskInstance = taskInstanceRepository.findByExerciseAndTask(exerciseId, taskId)
            .orElseThrow(() -> new TaskInstanceNotFoundException(exerciseId, taskId));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(taskInstance.getExercise())
            .setPermission(ExercisesAction.EXERCISE_VIEW).build();
        policy.checkAndRaise(context);

        if (taskInstance.getTask().isArchived()) {
            throw new TaskArchivedException();
        }

        var content = taskContentAssetUrlRewriter.rewiteContent(taskInstance, req);
        return htmlUrlSigner.sign(content, req);
    }

    @GetMapping("/exercise/{exerciseId}/task/{taskId}")
    @Operation(summary = "Liefert eine Aufgabeninstanz, die eine konfigurierte Aufgabe in einem Übungsblatt mit zusätzlichen Informationen (Punktzahl etc.) enthält")
    @SubatoApiResponse(TaskInstanceNotFoundException.class)
    TaskInstanceDetailDto getTaskInstance(@PathVariable Long exerciseId,
        @PathVariable Long taskId, @AuthenticationPrincipal SubatoPrincipal principal) {
        var taskInstance = taskInstanceRepository.findByExerciseAndTask(exerciseId, taskId)
            .orElseThrow(() -> new TaskInstanceNotFoundException(exerciseId, taskId));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(taskInstance.getExercise())
            .setPermission(ExercisesAction.EXERCISE_VIEW).build();
        policy.checkAndRaise(context);

        return taskInstanceMapper.mapDetail(taskInstance);
    }

    @GetMapping("/exercise/{exerciseId}/task/{taskId}/asset")
    @Operation(
        summary = "Endpunkt /task/{id}/asset nur nochmal für Aufgabeninstanzen zwecks Berechtigungsprüfung"
    )
    @SubatoApiResponse(TaskNotFoundException.class)
    @SubatoApiResponse(AssetNotFoundException.class)
    ResponseEntity<Resource> downloadAsset(@PathVariable Long exerciseId, @PathVariable Long taskId,
        @RequestParam("path") String path, @AuthenticationPrincipal SubatoPrincipal principal) {

        var taskInstance = taskInstanceRepository.findByExerciseAndTask(exerciseId, taskId)
            .orElseThrow(() -> new TaskInstanceNotFoundException(exerciseId, taskId));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(taskInstance.getExercise())
            .setPermission(ExercisesAction.EXERCISE_VIEW).build();
        policy.checkAndRaise(context);

        var file = downloadAssetUseCase.getAsset(taskInstance.getTask(), path);
        return fileResponseBuilderFactory.forFile(file)
            .setAsAttachment(false)
            .build();
    }

    @GetMapping("/exercise/{id}/task/{taskId}/files")
    @Operation(summary = "Für Bearbeitung der Aufgabe relevante Dateien, gefiltert nach Sichtbarkeit im Kontext des angemeldeten Nutzers. Lösungsdateien werden z.B. nicht Studenten angezeigt.")
    @SubatoApiResponse(TaskInstanceNotFoundException.class)
    @SubatoApiResponse(TaskArchivedException.class)
        // todo: this is not thrown here?
    List<FileRefDto> getTaskInstanceFiles(@PathVariable Long id, @PathVariable Long taskId,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var taskInstance = taskInstanceRepository.findByExerciseAndTask(id, taskId)
            .orElseThrow(() -> new TaskInstanceNotFoundException(id, taskId));
        var files = taskInstance.getTask().getFileRefs();

        files = files.stream()
            .filter(file -> {
                var context = new PolicyContext.Builder()
                    .setSubject(principal)
                    .setObj(Pair.of(file, taskInstance))
                    .setPermission(ExercisesAction.TASK_FILE_VIEW).build();
                return policy.check(context);
            })
            .toList();

        return files.stream().map(task -> fileRefMapper.map(task))
            .collect(Collectors.toList());
    }

    @GetMapping("/exercise/{exerciseId}/task/{taskId}/bundle")
    @SubatoApiResponse(TaskInstanceNotFoundException.class)
    @SubatoApiResponse(TaskArchivedException.class)
    ResponseEntity<Resource> downloadBundle(@PathVariable Long exerciseId,
        @PathVariable Long taskId, @AuthenticationPrincipal SubatoPrincipal principal) {
        var taskInstance = taskInstanceRepository.findByExerciseAndTask(exerciseId, taskId)
            .orElseThrow(() -> new TaskInstanceNotFoundException(exerciseId, taskId));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(taskInstance.getExercise())
            .setPermission(ExercisesAction.EXERCISE_VIEW).build();
        policy.checkAndRaise(context);

        var bundle = bundleRepository.getBundle(taskInstance.getTask());
        return fileResponseBuilderFactory.forFile(bundle)
            .setAsAttachment(true)
            .build();
    }

    @GetMapping("/exercise/{exerciseId}/task/{taskId}/files/{uuid}/download")
    @Operation(summary = "Zugriff auf den Inhalt der Datei zu einer Aufgabeninstanz")
    @SubatoApiResponse(TaskInstanceNotFoundException.class)
    @SubatoApiResponse(TaskArchivedException.class)
    @SubatoApiResponse(FileNotFoundException.class)
    ResponseEntity<Resource> downloadFile(@PathVariable Long exerciseId, @PathVariable Long taskId,
        @PathVariable String uuid, @AuthenticationPrincipal SubatoPrincipal principal) {
        var taskInstance = taskInstanceRepository.findByExerciseAndTask(exerciseId, taskId)
            .orElseThrow(() -> new TaskInstanceNotFoundException(exerciseId, taskId));
        var fileRef = taskInstance.getTask().getFileRefById(uuid);

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(Pair.of(fileRef, taskInstance))
            .setPermission(ExercisesAction.TASK_FILE_VIEW).build();
        policy.checkAndRaise(context);

        var file = downloadFileUseCase.getFile(taskInstance.getTask(), fileRef);
        return fileResponseBuilderFactory.forFile(file)
            .setAsAttachment(true)
            .build();
    }
}
