package de.hsrm.sls.subato.modules.term;

import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.modules.ModulesAction;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.shared.swagger.SubatoApiResponse;
import de.hsrm.sls.subato.shared.validation.ValidationException;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import java.util.List;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

@RestController
@Tag(name = "Modulverwaltung")
public class TermController {

    @Autowired
    private TermRepository termRepository;

    @Autowired
    private UpdateTermUseCase updateTermUseCase;

    @Autowired
    private FinishTermUseCase finishTermUseCase;

    @Autowired
    private TermMapper termMapper;

    @Autowired
    private PolicyEvaluator policy;

    @GetMapping("/term/{id}")
    @SubatoApiResponse(TermNotFoundException.class)
    TermDto getOne(@PathVariable Long id, @AuthenticationPrincipal SubatoPrincipal principal) {
        var term = termRepository.findById(id).orElseThrow(() -> new TermNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(term)
            .setPermission(ModulesAction.TERM_VIEW).build();
        policy.checkAndRaise(context);

        return termMapper.map(term);
    }

    @GetMapping("/term")
    @Operation(summary = "Alle sichtbaren Semester im Kontext des angemeldeten Benutzers")
    List<TermDto> getAll(@AuthenticationPrincipal SubatoPrincipal principal) {
        var terms = termRepository.findAll(TermSort.byLecturesStartAt());

        terms = terms.stream()
            .filter(term -> {
                var context = new PolicyContext.Builder()
                    .setSubject(principal)
                    .setObj(term)
                    .setPermission(ModulesAction.TERM_VIEW).build();
                return policy.check(context);
            })
            .collect(Collectors.toList());

        return terms.stream().map(t -> termMapper.map(t)).collect(Collectors.toList());
    }

    @PostMapping("/term")
    @PreAuthorize("hasPermission(null, @ModulesAction.TERM_CREATE)")
    @SubatoApiResponse(ValidationException.class)
    TermDto create(@RequestBody CreateTermDto dto) {
        var term = updateTermUseCase.create(dto);
        return termMapper.map(term);
    }

    @PutMapping("/term/{id}")
    @PreAuthorize("hasPermission(null, @ModulesAction.TERM_UPDATE)")
    @SubatoApiResponse(TermNotFoundException.class)
    @SubatoApiResponse(ValidationException.class)
    TermDto update(@PathVariable Long id, @RequestBody CreateTermDto dto) {
        var term = updateTermUseCase.update(id, dto);
        return termMapper.map(term);
    }

    @PostMapping("/term/{id}/finish")
    @SubatoApiResponse(TermNotFoundException.class)
    public void finish(@PathVariable Long id,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var term = termRepository.findById(id).orElseThrow(() -> new TermNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(term)
            .setPermission(ModulesAction.TERM_UPDATE).build();
        policy.checkAndRaise(context);

        finishTermUseCase.finish(term);
    }
}
