package de.hsrm.sls.subato.search;

import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.courses.CoursesAction;
import de.hsrm.sls.subato.auth.policy.exercises.ExercisesAction;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.courses.course.CourseNotFoundException;
import de.hsrm.sls.subato.courses.course.dto.CourseMapper;
import de.hsrm.sls.subato.courses.course.model.Course;
import de.hsrm.sls.subato.courses.course.repository.CourseRepository;
import de.hsrm.sls.subato.exercises.exercise.dto.ExerciseMapper;
import de.hsrm.sls.subato.exercises.exercise.model.Exercise;
import de.hsrm.sls.subato.exercises.taskinstance.TaskInstance;
import de.hsrm.sls.subato.exercises.taskinstance.dto.TaskInstanceMapper;
import de.hsrm.sls.subato.search.item.CourseParticipantItem;
import de.hsrm.sls.subato.search.item.ExerciseItem;
import de.hsrm.sls.subato.search.item.Item;
import de.hsrm.sls.subato.search.item.TaskInstanceItem;
import de.hsrm.sls.subato.shared.springext.ResponseStatusMapping;
import de.hsrm.sls.subato.shared.swagger.SubatoApiResponse;
import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.user.dto.UserMapper;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;


@RestController
@Tag(name = "Querschnitt")
public class SearchController {

    @Autowired
    SearchUseCase searchUseCase;

    @Autowired
    CourseMapper courseMapper;

    @Autowired
    TaskInstanceMapper taskInstanceMapper;

    @Autowired
    ExerciseMapper exerciseMapper;

    @Autowired
    UserMapper userMapper;

    @Autowired
    CourseRepository courseRepository;

    @Autowired
    PolicyEvaluator policy;

    static String exerciseGroup(Exercise exercise) {
        return exercise.getName();
    }

    List<TaskInstanceItem> getTaskItems(List<TaskInstance> tasks) {
        return tasks.stream()
            .map(t -> {
                var dto = taskInstanceMapper.mapDetail(t);
                return new TaskInstanceItem(dto, exerciseGroup(t.getExercise()));
            }).collect(Collectors.toList());
    }

    List<ExerciseItem> getExerciseItems(List<Exercise> exercises) {
        return exercises.stream().map(m -> {
            var dto = exerciseMapper.map(m);
            return new ExerciseItem(dto, "Übungen");
        }).collect(Collectors.toList());
    }

    List<CourseParticipantItem> getCourseParticipantItems(List<User> users, Course course) {
        return users.stream().map(m -> {
            var dto = userMapper.mapDetail(m);
            return new CourseParticipantItem(dto, course.getId(), "Teilnehmer");
        }).collect(Collectors.toList());
    }

    @GetMapping("/search")
    @Operation(summary = "Querschnittliche Suche innerhalb eines Kurses über alle Aspekte hinweg (Aufgaben, Kursteilnehmer, ...)")
    @SubatoApiResponse(CourseNotFoundException.class)
    @ResponseStatusMapping(exception = CourseNotFoundException.class, code = HttpStatus.BAD_REQUEST)
    SearchResult search(@RequestParam("courseId") Long courseId, @RequestParam("q") String query,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var items = new ArrayList<Item>();

        var course = courseRepository.findById(courseId)
            .orElseThrow(() -> new CourseNotFoundException(courseId));

        var exercises = searchUseCase.searchExercises(course, query);
        items.addAll(getExerciseItems(exercises));

        var taskInstances = searchUseCase.searchTaskInstances(course, query)
            .stream()
            .filter(taskInstance -> {
                var context = new PolicyContext.Builder()
                    .setSubject(principal)
                    .setObj(taskInstance.getExercise())
                    .setPermission(ExercisesAction.EXERCISE_VIEW).build();

                return policy.check(context);
            })
            .collect(Collectors.toList());
        items.addAll(getTaskItems(taskInstances));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(course)
            .setPermission(CoursesAction.COURSE_VIEW_PARTICIPANTS).build();
        if (policy.check(context)) {
            var participants = searchUseCase.searchCourseParticipants(query, course);
            items.addAll(getCourseParticipantItems(participants, course));
        }

        return new SearchResult(query, items);
    }
}
