package de.hsrm.sls.subato.seed.logic.course;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import de.hsrm.sls.subato.courses.course.model.Course;
import de.hsrm.sls.subato.courses.course.model.CourseVisibility;
import de.hsrm.sls.subato.courses.course.model.SubmitPolicy;
import de.hsrm.sls.subato.courses.course.repository.CourseRepository;
import de.hsrm.sls.subato.courses.course.repository.DefaultDescriptionRepository;
import de.hsrm.sls.subato.courses.event.model.CourseUnitEvent;
import de.hsrm.sls.subato.courses.event.model.CourseUnitEventDefinition;
import de.hsrm.sls.subato.courses.staff.model.CourseStaffMembership;
import de.hsrm.sls.subato.courses.unit.model.CourseUnit;
import de.hsrm.sls.subato.courses.unit.model.CourseUnitType;
import de.hsrm.sls.subato.modules.progmodule.ProgramModuleRepository;
import de.hsrm.sls.subato.modules.term.TermRepository;
import de.hsrm.sls.subato.seed.logic.SeedingLogic;
import de.hsrm.sls.subato.shared.helper.FileHelper;
import de.hsrm.sls.subato.shared.helper.StreamHelper;
import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.user.repository.UserRepository;
import java.io.File;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
@Transactional
public class CourseSeedingLogic implements SeedingLogic {

    private static Logger logger = LoggerFactory.getLogger(CourseSeedingLogic.class);

    @Autowired
    CourseRepository courseRepository;

    @Autowired
    TermRepository termRepository;

    @Autowired
    UserRepository userRepository;

    @Autowired
    ProgramModuleRepository programModuleRepository;

    @Autowired
    DefaultDescriptionRepository defaultDescriptionRepository;


    public void run(File seedDir, ObjectMapper objectMapper) {
        var coursesFile = seedDir.toPath().resolve("courses.json").toFile();
        if (!coursesFile.exists()) {
            return;
        }

        try {
            var json = FileHelper.readString(coursesFile);
            var courseSeeds = objectMapper.readValue(json, new TypeReference<List<CourseSeed>>() {
            });

            logger.debug("Seeding %s courses...".formatted(courseSeeds.size()));

            for (var courseSeed : courseSeeds) {
                if (courseSeed.getId() != null && courseRepository.existsBySourceId(
                    courseSeed.getId())) {
                    continue;
                }

                createCourse(courseSeed);
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private void createCourse(CourseSeed courseSeed) {
        var course = new Course();
        course.setName(courseSeed.getName());
        course.setSourceId(courseSeed.getId());
        course.setDescription(defaultDescriptionRepository.getDefaultDescription());
        course.setShortName(courseSeed.getShortName());
        course.setMotd(courseSeed.getMotd());
        course.setShowEmails(courseSeed.isShowEmails());
        course.setVisibility(courseSeed.getVisibility() != null ? courseSeed.getVisibility()
            : CourseVisibility.PRIVATE);
        course.setSubmitPolicy(
            courseSeed.getSubmitPolicy() != null ? courseSeed.getSubmitPolicy()
                : SubmitPolicy.EVERYONE);
        course.setTerm(
            courseSeed.getTermId() != null ? termRepository.findBySourceId(courseSeed.getTermId())
                .orElse(null) : null);

        createDefaultUnits(course);
        for (var courseUnitSeed : courseSeed.getUnits()) {
            createUnit(course, courseUnitSeed);
        }

        var modules = programModuleRepository.findBySupplementIdIn(
            courseSeed.getModuleIds());
        course.setModules(modules);

        for (var staffMembership : courseSeed.getStaffMemberships()) {
            var user = userRepository.findByLoginNameIgnoreCase(
                staffMembership.getLoginName()).get();
            course.getMemberships()
                .add(new CourseStaffMembership(user, course, staffMembership.getGroup()));
        }

        courseRepository.save(course);
    }

    private void createUnit(Course course, CourseUnitSeed courseUnitSeed) {
        var lecturers = courseUnitSeed.getLecturers().stream()
            .filter(StreamHelper.distinctByKey(s -> s))
            .map(loginName -> userRepository.findByLoginNameIgnoreCase(loginName).get())
            .toList();
        var participants = courseUnitSeed.getParticipants().stream()
            .filter(StreamHelper.distinctByKey(s -> s))
            .map(loginName -> userRepository.findByLoginNameIgnoreCase(loginName).get())
            .toList();

        var unit = new CourseUnit();
        unit.setName(courseUnitSeed.getName());
        unit.setType(courseUnitSeed.getType());
        unit.setLecturers(lecturers);
        unit.setParticipants(participants);

        for (var eventDefinitionSeed : courseUnitSeed.getEventDefinitions()) {
            var eventDefinition = new CourseUnitEventDefinition();
            eventDefinition.setUnit(unit);
            eventDefinition.setRoom(eventDefinitionSeed.getRoom());
            eventDefinition.setTime(eventDefinitionSeed.getTime());
            eventDefinition.setWeekday(eventDefinitionSeed.getWeekday());

            for (var event : eventDefinitionSeed.getEvents()) {
                var ev = new CourseUnitEvent();
                ev.setUnit(unit);
                ev.setEventDefinition(eventDefinition);
                ev.setTakesPlaceAt(LocalDateTime.of(event, eventDefinition.getTime()));
                unit.getEvents().add(ev);
            }

            unit.getEventDefinitions().add(eventDefinition);
        }

        unit.setCourse(course);
        course.getUnits().add(unit);
    }

    private void createDefaultUnits(Course course) {
        var staffMembers = course.getMemberships().stream()
            .map(CourseStaffMembership::getUser)
            .filter(StreamHelper.distinctByKey(User::getId))
            .collect(Collectors.toList());

        var unit = new CourseUnit();
        unit.setName("Sandkasten");
        unit.setType(CourseUnitType.Misc);
        unit.setParticipants(staffMembers);
        unit.setCourse(course);
        course.getUnits().add(unit);

        unit = new CourseUnit();
        unit.setName("Standard");
        unit.setType(CourseUnitType.Misc);
        unit.setCourse(course);
        course.getUnits().add(unit);
    }
}
