package de.hsrm.sls.subato.shared.error;

import de.hsrm.sls.subato.shared.error.mapping.ExceptionMapper;
import de.hsrm.sls.subato.shared.error.rendering.ExceptionRenderer;
import de.hsrm.sls.subato.shared.error.rendering.ExceptionRendererArgs;
import de.hsrm.sls.subato.shared.error.rendering.HtmlExceptionRenderer;
import de.hsrm.sls.subato.shared.error.response.ErrorResponse;
import de.hsrm.sls.subato.shared.error.response.ErrorResponseBuilder;
import jakarta.annotation.Nullable;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;
import org.apache.catalina.connector.ClientAbortException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.util.MimeType;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.method.HandlerMethod;

/**
 * Responsible for processing exceptions that were not catched in controllers, in the filter chain
 * or in websocket handlers.
 */
@ControllerAdvice
public class GlobalExceptionHandler {

    private static Logger logger = LoggerFactory.getLogger(GlobalExceptionHandler.class);

    /**
     * see {@link ExceptionMapper} for more information.
     */
    @Autowired
    List<ExceptionMapper> exceptionMappers;

    @Autowired
    ErrorResponseBuilder errorResponseBuilder;

    @Autowired
    private List<ExceptionRenderer> exceptionRenderers;

    @Autowired
    private HtmlExceptionRenderer defaultExceptionRenderer;

    @Autowired
    GlobalExceptionLogger globalExceptionLogger;

    // https://stackoverflow.com/questions/35451244/how-to-handle-clientabortexception-in-spring-mvc
    @ExceptionHandler(ClientAbortException.class)
    public void handleClientAbortException(ClientAbortException exception,
        HttpServletRequest request) {
        final String message = "ClientAbortException generated by request {} {} from remote address {} with X-FORWARDED-FOR {}";
        final String headerXFF = request.getHeader("X-FORWARDED-FOR");
        logger.debug(message, request.getMethod(), request.getRequestURL(), request.getRemoteAddr(),
            headerXFF);
    }

    @ExceptionHandler(Exception.class)
    public Object handleException(Exception ex,
        HttpServletRequest request,
        HttpServletResponse response, @Nullable HandlerMethod handlerMethod) {
        var matchingMapper = exceptionMappers.stream().filter(m -> m.canMap(ex)).findFirst()
            .orElse(null);

        try {
            var errorResponse = matchingMapper != null
                ? matchingMapper.map(ex)
                : errorResponseBuilder.buildForException(ex, handlerMethod);
            globalExceptionLogger.log(errorResponse, ex);

            var args = new ExceptionRendererArgs.Builder().setErrorResponse(errorResponse)
                .setRequest(request).setResponse(response).build();
            var exceptionRenderer = selectRenderer(request);

            return exceptionRenderer.render(args);
        } catch (Exception e) {
            logger.error("Failed to process exception", e);

            // if any exception is thrown while processing the exception,
            // we return a generic internal server error as default
            return ResponseEntity.status(ErrorConstants.DEFAULT_STATUS).body(
                new ErrorResponse(HttpStatus.INTERNAL_SERVER_ERROR.value(),
                    ErrorConstants.DEFAULT_REASON,
                    null));
        }
    }

    private ExceptionRenderer selectRenderer(HttpServletRequest httpResponse) {
        ExceptionRenderer renderer = defaultExceptionRenderer;

        String acceptHeader = httpResponse.getHeader("Accept");
        if (acceptHeader != null) {
            var mimeTypes = acceptHeader.split(",");
            for (var exceptionRenderer : exceptionRenderers) {
                var supports = Arrays.stream(mimeTypes)
                    .anyMatch(m -> exceptionRenderer.supportedContentTypes().contains(
                        MimeType.valueOf(m)));

                if (supports) {
                    renderer = exceptionRenderer;
                    break;
                }
            }
        }

        return renderer;
    }
}
