package de.hsrm.sls.subato.shared.error.response;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.databind.introspect.AnnotatedMember;
import com.fasterxml.jackson.databind.introspect.JacksonAnnotationIntrospector;
import java.util.Map;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * Serializes properties/methods in exceptions, that were explicitly annotated with {@link Expose}.
 * It can be used to provide additional error information for clients. <p>
 * <b>Important</b>: {@link Expose}
 * needs to be provided on the field AND the getter. This is currently a limitation.
 */
@Component
class ErrorDataSerializer {

    private static Logger logger = LoggerFactory.getLogger(ErrorDataSerializer.class);

    ObjectMapper objectMapper;

    public ErrorDataSerializer(@Autowired ObjectMapper originalObjectMapper) {
        objectMapper = createObjectMapper(originalObjectMapper);
    }

    /**
     * uses the original mapper but disables every form of auto detection to prevent exposing
     * information that should be kept internal.
     */
    private ObjectMapper createObjectMapper(ObjectMapper originalObjectMapper) {
        var errorObjectMapper = originalObjectMapper.copy();

        /*
        the first idea was about using @JsonProperty if something should be exposed while
        disabling auto detection at the same time:

        errorObjectMapper.disable(MapperFeature.AUTO_DETECT_CREATORS,
            MapperFeature.AUTO_DETECT_FIELDS,
            MapperFeature.AUTO_DETECT_GETTERS,
            MapperFeature.AUTO_DETECT_IS_GETTERS);

        for some reason "stackTrace", "message" and other properties are still added for RuntimeExceptions
        even if we disable all auto detection features. So we use the current
        approach by marking everything with a custom @Expose annotation.
        */
        errorObjectMapper.disable(SerializationFeature.FAIL_ON_EMPTY_BEANS);

        errorObjectMapper.setAnnotationIntrospector(new JacksonAnnotationIntrospector() {
            @Override
            public boolean hasIgnoreMarker(final AnnotatedMember m) {
                return !m.hasAnnotation(Expose.class) || super.hasIgnoreMarker(m);
            }
        });

        return errorObjectMapper;
    }

    public Map<String, Object> serialize(Throwable exception) {
        var serializedData = objectMapper.convertValue(exception, Map.class);
        return serializedData;
    }
}
