package de.hsrm.sls.subato.shared.markdown.latex;


import org.commonmark.internal.util.Parsing;
import org.commonmark.node.Block;
import org.commonmark.parser.SourceLine;
import org.commonmark.parser.block.AbstractBlockParser;
import org.commonmark.parser.block.AbstractBlockParserFactory;
import org.commonmark.parser.block.BlockContinue;
import org.commonmark.parser.block.BlockStart;
import org.commonmark.parser.block.MatchedBlockParser;
import org.commonmark.parser.block.ParserState;

/**
 * Parses both inline and multi-line Math expressions.
 */
class LatexMathBlockParser extends AbstractBlockParser {

    private static final char DOLLAR = '$';
    private static final char SPACE = ' ';

    private final LatexMathBlock block = new LatexMathBlock();

    private final StringBuilder builder = new StringBuilder();

    private final int signs;

    private boolean alreadyFinished;

    LatexMathBlockParser(int signs) {
        this.signs = signs;
    }

    @Override
    public Block getBlock() {
        return block;
    }

    @Override
    public BlockContinue tryContinue(ParserState parserState) {
        if (alreadyFinished) {
            return BlockContinue.none();
        }

        final int nextNonSpaceIndex = parserState.getNextNonSpaceIndex();
        final CharSequence line = parserState.getLine().getContent();
        final int length = line.length();

        // check for closing in new line
        if (parserState.getIndent() < Parsing.CODE_BLOCK_INDENT) {
            if (consume(DOLLAR, line, nextNonSpaceIndex, length) == signs) {
                // okay, we have our number of signs
                // let's consume spaces until the end
                if (Parsing.skip(SPACE, line, nextNonSpaceIndex + signs, length) == length) {
                    return BlockContinue.finished();
                }
            }
        }

        return BlockContinue.atIndex(parserState.getIndex());
    }

    @Override
    public void addLine(SourceLine line) {
        // the parser should terminate early if the line is finished with the number
        // of $ signs that were used to start the expression
        var l = line.getContent().toString();
        if (l.endsWith("$".repeat(signs))) {
            alreadyFinished = true;
            l = l.substring(0, l.length() - signs);
        }

        builder.append(l);
        builder.append('\n');
    }

    @Override
    public void closeBlock() {
        block.latex(builder.toString());
        block.signs(signs);
    }

    public static class Factory extends AbstractBlockParserFactory {

        @Override
        public BlockStart tryStart(ParserState state, MatchedBlockParser matchedBlockParser) {

            // let's define the spec:
            //  * 0-3 spaces before are allowed (Parsing.CODE_BLOCK_INDENT = 4)
            //  * at least one `$` sign
            //  * any optional amount of spaces
            //  * new line
            //  * block is closed when the same amount of opening signs is met

            final int indent = state.getIndent();

            // check if it's an indented code block
            if (indent >= Parsing.CODE_BLOCK_INDENT) {
                return BlockStart.none();
            }

            final int nextNonSpaceIndex = state.getNextNonSpaceIndex();
            final CharSequence line = state.getLine().getContent();
            final int length = line.length();

            final int signs = consume(DOLLAR, line, nextNonSpaceIndex, length);

            if (signs < 1) {
                return BlockStart.none();
            }

            // consume spaces
            var startIndex = Parsing.skip(SPACE, line, nextNonSpaceIndex + signs, length);

            return BlockStart.of(new LatexMathBlockParser(signs)).atIndex(startIndex);
        }
    }

    @SuppressWarnings("SameParameterValue")
    private static int consume(char c, CharSequence line, int start, int end) {
        for (int i = start; i < end; i++) {
            if (c != line.charAt(i)) {
                return i - start;
            }
        }
        // all consumed
        return end - start;
    }
}