package de.hsrm.sls.subato.shared.mime;

import de.hsrm.sls.subato.shared.config.SubatoConfig;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.apache.tika.Tika;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * Responsible for mapping file names to mime types.
 * <p>
 * Uses the Apache Tika library for reliable detection. However, some files are detected as
 * application/octet-stream by tika, so we need to map the exceptions here
 */
@Component
public class MimeDetector {

    @Autowired
    private SubatoConfig subatoConfig;

    /**
     * @return actual content type or application/octet-stream if it can't be guessed
     */
    public String getContentType(String fileName, List<MimeMapping> mimeMappingOverrides) {
        var mergedMappings = mergeMappings(mimeMappingOverrides);

        for (var entry : mergedMappings.entrySet()) {
            var map = entry.getKey();
            var as = entry.getValue();

            if (fileName.endsWith(map)) {
                return as;
            }
        }

        return new Tika().detect(fileName);
    }

    private Map<String, String> mergeMappings(List<MimeMapping> overrides) {
        var mimeMappings = new ArrayList<MimeMapping>();
        mimeMappings.addAll(subatoConfig.getMime());
        mimeMappings.addAll(overrides);

        var d = new HashMap<String, String>();
        for (var mapping : mimeMappings) {
            d.put(mapping.getMap(), mapping.getAs());
        }

        return d;
    }

    public String getContentType(String fileName) {
        return getContentType(fileName, new ArrayList<>());
    }

    public String getContentType(File file, List<MimeMapping> mimeMappingOverrides) {
        return getContentType(file.getName(), mimeMappingOverrides);
    }
}
