package de.hsrm.sls.subato.shared.springext;

import java.lang.annotation.ElementType;
import java.lang.annotation.Repeatable;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import org.springframework.http.HttpStatus;

/**
 * Provides the ability to remap the status code in exceptions annotated with
 * {@link org.springframework.web.bind.annotation.ResponseStatus} for individual controller methods.
 * This annotation can be applied multiple times with different remapping configurations on the same
 * method.
 * <p>
 * Example
 * <p>
 * <p>
 * The following exception was annoted with NOT_FOUND: {@code
 *
 * @ResponseStatus(code = HttpStatus.NOT_FOUND, reason = "Der Kurs existiert nicht.") public class
 * CourseNotFoundException extends RuntimeException { } }
 * <p>
 * Depending on the context, we want to return BAD_REQUEST instead, so we can remap the status in
 * the controller method like this:
 * <p>
 * {@code
 * @ResponseStatusMapping(exception = CourseNotFoundException.class, code = HttpStatus.BAD_REQUEST)
 * ExerciseDto create(@RequestBody CreateExerciseDto dto) { } }
 * <p>
 * Motivation: Handling exceptions with AOP is best practice in Spring. It reduces boilerplate code
 * and eliminates redundancy. However, it comes at the cost that the status code/reason is not
 * context-dependent. This can a problem if the same exception should be mapped to different status
 * codes/reasons in different scenarios. Creating additional Exception classes (such as:
 * CourseNotFoundInExceptionCreationException) leads to ridiculously long exception names and
 * cumbersome code. This annotation will solve this problem. Note, that this is not a complete
 * replacement for additional exception classes. If it makes sense to dinstinguish the exceptions,
 * do so. Otherwise use this annotation.
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.METHOD)
@Repeatable(ResponseStatusMappings.class)
public @interface ResponseStatusMapping {

    HttpStatus code();

    Class<? extends Exception> exception();
}
