package de.hsrm.sls.subato.shared.springext;

import de.hsrm.sls.subato.shared.error.ErrorConstants;
import java.lang.annotation.Annotation;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.method.HandlerMethod;

/**
 * Applies the logic for {@link ResponseStatusMapping}.
 */
public class ResponseStatusResolver {

    public static ResponseStatus resolve(Class clazz, HandlerMethod handlerMethod) {
        var responseStatus = AnnotationUtils.findAnnotation(clazz, ResponseStatus.class);
        if (responseStatus == null) {
            return createResponseStatus(ErrorConstants.DEFAULT_STATUS,
                ErrorConstants.DEFAULT_REASON);
        }

        var annotations = getMappingAnnotations(handlerMethod);
        var annotation = annotations.stream().filter(a -> a.exception().equals(clazz)).findFirst()
            .orElse(null);

        return merge(responseStatus, annotation);
    }

    private static ResponseStatus createResponseStatus(HttpStatus status, String reason) {
        return new ResponseStatus() {
            @Override
            public Class<? extends Annotation> annotationType() {
                return ResponseStatus.class;
            }

            @Override
            public HttpStatus code() {
                return status;
            }

            @Override
            public HttpStatus value() {
                return status;
            }

            @Override
            public String reason() {
                return reason;
            }
        };
    }

    private static List<ResponseStatusMapping> getMappingAnnotations(HandlerMethod handlerMethod) {
        var annotations = new ArrayList<ResponseStatusMapping>();
        if (handlerMethod == null) {
            return annotations;
        }

        // ResponseStatusMappings is a wrapper annotation, that is present if the method was annotated
        // multiple times with the ResponseStatusMapping annotation
        var responsesAnnotation = handlerMethod.getMethodAnnotation(ResponseStatusMappings.class);
        if (responsesAnnotation != null) {
            annotations.addAll(Arrays.stream(responsesAnnotation.value()).toList());
        }

        // otherwise the ResponseStatusMapping can be looked up directly
        var responseAnnotation = handlerMethod.getMethodAnnotation(ResponseStatusMapping.class);
        if (responseAnnotation != null) {
            annotations.add(responseAnnotation);
        }

        return annotations;
    }

    private static ResponseStatus merge(ResponseStatus status, ResponseStatusMapping mapping) {
        return createResponseStatus(mapping != null ? mapping.code() : status.code(),
            status.reason());
    }
}
