package de.hsrm.sls.subato.shared.swagger;

import java.lang.annotation.ElementType;
import java.lang.annotation.Repeatable;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Used for annotating methods in a REST controller to signal that the method may throw an
 * exception. This is used to document different API responses in swagger. Using this annotation
 * requires, that the exception is annotated with the
 * {@link org.springframework.web.bind.annotation.ResponseStatus} annotation.
 * <p>
 * example:
 * <p>
 * {@code
 *
 * @SubatoApiResponse(CourseNotFoundForExerciseException.class) ExerciseDto create(@RequestBody
 * CreateExerciseDto dto) { ... } }
 * <p>
 * <p>
 * A method can be annotated multiple times: {@code
 * @SubatoApiResponse(CourseNotFoundForExerciseException.class)
 * @SubatoApiResponse(SomeOtherException.class) ExerciseDto create(@RequestBody CreateExerciseDto
 * dto) { ... } }
 * <p>
 * <p>
 * Motivation for this annotation: The current springdoc implementation supports
 * {@link org.springframework.web.bind.annotation.ResponseStatus} annotations but does not show the
 * reason of the annotation. Additionally it does not support cases where multiple reasons map to
 * the same response code.
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.METHOD)
@Repeatable(SubatoApiResponses.class)
public @interface SubatoApiResponse {

    Class<? extends Exception> value();
}

