package de.hsrm.sls.subato.shared.swagger;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;

import de.hsrm.sls.subato.shared.error.response.ErrorResponse;
import de.hsrm.sls.subato.shared.error.response.ErrorResponseBuilder;
import de.hsrm.sls.subato.shared.springext.ResponseStatusResolver;
import io.swagger.v3.oas.models.OpenAPI;
import io.swagger.v3.oas.models.Operation;
import io.swagger.v3.oas.models.examples.Example;
import io.swagger.v3.oas.models.media.Content;
import io.swagger.v3.oas.models.media.MediaType;
import io.swagger.v3.oas.models.responses.ApiResponse;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;
import org.springdoc.core.customizers.OperationCustomizer;
import org.springdoc.core.utils.SpringDocAnnotationsUtils;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.method.HandlerMethod;


/**
 * Generates the descriptions, schemas and examples in the API documentation for every operation.
 * Based on the {@link SubatoApiResponse}/{@link SubatoApiResponses} annotations, that are applied
 * to the endpoint methods in the REST controllers.
 */
public class SubatoApiResponseCustomizer implements OperationCustomizer {

    private OpenAPI api;
    private ExampleDefinitions exampleDefinitions;
    private ErrorResponseBuilder errorResponseBuilder;

    public SubatoApiResponseCustomizer(OpenAPI api, ExampleDefinitions exampleDefinitions,
        ErrorResponseBuilder errorResponseBuilder) {
        this.api = api;
        this.exampleDefinitions = exampleDefinitions;
        this.errorResponseBuilder = errorResponseBuilder;
    }

    @Override
    public Operation customize(Operation operation, HandlerMethod handlerMethod) {
        var annotations = getAnnotations(handlerMethod);

        // because we cannot create multiple ApiResponse objects with the same response code
        // (these will be "lost", idk why), we need to group the annotations by their annotated
        // status code
        var responses = operation.getResponses();
        var responseGroups = annotations.stream()
            .map(a -> new ResolvedResponse(a, getResponseStatusAnnotation(a, handlerMethod)))
            .collect(Collectors.groupingBy(g -> g.responseStatus().code()));

        for (var group : responseGroups.entrySet()) {
            var apiResponse = new ApiResponse();

            var content = configureContent(group.getValue(), handlerMethod);
            apiResponse.setContent(content);

            // if a status code is unique (i.e. there is only one annotated exception that uses
            // that status code), we can create the ApiResponse as-is.
            if (group.getValue().size() == 1) {
                apiResponse.setDescription(group.getValue().get(0).getDescription());
            } else {
                // otherwise we use markdown to add a list of possible reasons to that status code
                var sb = new StringBuilder();
                sb.append("Gründe: \n");
                for (var resolvedResponse : group.getValue()) {
                    sb.append("* %s\n".formatted(resolvedResponse.getDescription()));
                }

                apiResponse.setDescription(sb.toString());
            }

            var statusCode = group.getKey().value();
            responses.addApiResponse(String.valueOf(statusCode), apiResponse);
        }

        return operation;
    }

    /**
     * Generates the schema and examples section for a response group
     */
    private Content configureContent(List<ResolvedResponse> responseGroup,
        HandlerMethod handlerMethod) {
        var c = new Content();
        var mt = new MediaType();

        var schema = SpringDocAnnotationsUtils.extractSchema(api.getComponents(),
            ErrorResponse.class, null, null);

        for (var resolvedResponse : responseGroup) {
            var exceptionObject = exampleDefinitions.getExample(
                resolvedResponse.response().value());
            if (exceptionObject == null) {
                continue;
            }

            var errorResponse = errorResponseBuilder.buildForException(exceptionObject,
                handlerMethod);

            var example = new Example();
            example.setValue(errorResponse);
            mt.addExamples(resolvedResponse.generateErrorIdentifier(), example);
        }

        mt.setSchema(schema);
        c.addMediaType(APPLICATION_JSON_VALUE, mt);
        return c;
    }

    /**
     * Resolves all {@link SubatoApiResponse} annotations on a controller method.
     *
     * @param handlerMethod
     * @return
     */
    private List<SubatoApiResponse> getAnnotations(HandlerMethod handlerMethod) {
        var annotations = new ArrayList<SubatoApiResponse>();

        // SubatoApiResponses is a wrapper annotation, that is present if the method was annotated
        // multiple times with the SubatoApiResponse annotation
        var responsesAnnotation = handlerMethod.getMethodAnnotation(SubatoApiResponses.class);
        if (responsesAnnotation != null) {
            annotations.addAll(Arrays.stream(responsesAnnotation.value()).toList());
        }

        // otherwise the SubatoApiResponse can be looked up directly
        var responseAnnotation = handlerMethod.getMethodAnnotation(SubatoApiResponse.class);
        if (responseAnnotation != null) {
            annotations.add(responseAnnotation);
        }

        return annotations;
    }

    private ResponseStatus getResponseStatusAnnotation(SubatoApiResponse apiResponseAnnotation,
        HandlerMethod handlerMethod) {
        var clazz = apiResponseAnnotation.value();
        return ResponseStatusResolver.resolve(clazz, handlerMethod);
    }
}
