package de.hsrm.sls.subato.shared.swagger;

import de.hsrm.sls.subato.shared.ApiConstants;
import de.hsrm.sls.subato.shared.config.KeycloakConfig;
import de.hsrm.sls.subato.shared.error.response.ErrorResponseBuilder;
import io.swagger.v3.oas.models.Components;
import io.swagger.v3.oas.models.ExternalDocumentation;
import io.swagger.v3.oas.models.OpenAPI;
import io.swagger.v3.oas.models.info.Contact;
import io.swagger.v3.oas.models.info.Info;
import io.swagger.v3.oas.models.security.SecurityRequirement;
import io.swagger.v3.oas.models.security.SecurityScheme;
import io.swagger.v3.oas.models.tags.Tag;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.stream.Collectors;
import org.springdoc.core.customizers.OperationCustomizer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.ResourceLoader;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.util.PropertyPlaceholderHelper;
import org.springframework.web.servlet.config.annotation.ViewControllerRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

@Configuration
public class SwaggerConfiguration implements WebMvcConfigurer {

    @Autowired
    private KeycloakConfig keycloakConfig;

    @Autowired
    @Value("${subato.swagger.docs-url}")
    private String docsUrl;

    @Value("${subato.swagger.mail}")
    private String mail;

    @Autowired
    ResourceLoader resourceLoader;

    // https://github.com/swagger-api/swagger-ui/issues/6462
    public SwaggerConfiguration(MappingJackson2HttpMessageConverter converter) {
        var supportedMediaTypes = new ArrayList<>(converter.getSupportedMediaTypes());
        supportedMediaTypes.add(new MediaType("application", "octet-stream"));
        converter.setSupportedMediaTypes(supportedMediaTypes);
    }

    private String getApiDescription() {
        try {
            var inputStream = resourceLoader.getResource("classpath:static/api-description.md")
                .getInputStream();
            var description = new BufferedReader(new InputStreamReader(inputStream))
                .lines()
                .collect(Collectors.joining(System.lineSeparator()));

            var props = new Properties();
            props.setProperty("KEYCLOAK_URL", keycloakConfig.getAuthServerUrl());

            var templateEngine = new PropertyPlaceholderHelper("${", "}");
            return templateEngine.replacePlaceholders(description, props);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Bean
    OperationCustomizer sampleCustomizer(OpenAPI api,
        @Autowired ExampleDefinitions exampleDefinitions,
        @Autowired ErrorResponseBuilder errorResponseBuilder) {
        return new SubatoApiResponseCustomizer(api, exampleDefinitions, errorResponseBuilder);
    }

    @Bean
    public OpenAPI subatoOpenApi() {
        return new OpenAPI()
            .info(new Info().title("Subato API Dokumentation")
                .description(getApiDescription())
                .version(ApiConstants.API_VERSION)
                .contact(new Contact().name("Support").email(mail)))
            .externalDocs(new ExternalDocumentation()
                .description("Subato Dokumentation")
                .url(docsUrl))
            .tags(List.of(
                new Tag().name("Kursverwaltung").description("Abfrage und Verwaltung von Kursen."),
                new Tag().name("Anwesenheitserfassung")
                    .description("Abfrage und Verwaltung der Anwesenheit in Kursen."),
                new Tag().name("Übungsverwaltung").description(
                    "Abfrage und Verwaltung von Übungen mit ihren konfigurierten Aufgaben (Aufgabeninstanzen)"),
                new Tag().name("Bewertung").description(
                    "Verwaltung von Abgaben/Bewertung"),
                new Tag().name("Lösungsverwaltung")
                    .description("Abgabe, Verwaltung und automatische Auswertung von Lösungen"),
                new Tag().name("Modulverwaltung").description(
                    "Abfrage von Studiengängen, Modulen und anderen Stammdaten wie dem Semester."),
                new Tag().name("Aufgabenverwaltung")
                    .description("Abfrage und Verwaltung von Aufgaben bzw. Aufgaben-Pools"),
                new Tag().name("Querschnitt")
                    .description("Dashboard, Suche, Authentifizierung usw."),
                new Tag().name("Fehlvorstellungen").description("Intern (Forschung)")
            ))
            .addSecurityItem(new SecurityRequirement().addList("OIDC"))
            .components(
                new Components()
                    .addSecuritySchemes("OIDC",
                        new SecurityScheme()
                            .name("OIDC")
                            .type(SecurityScheme.Type.OPENIDCONNECT)
                            .openIdConnectUrl(
                                "%s/realms/%s/.well-known/openid-configuration".formatted(
                                    keycloakConfig.getAuthServerUrl(), keycloakConfig.getRealm()))
                    )
            );
    }

    @Override
    public void addViewControllers(ViewControllerRegistry registry) {
        registry.addRedirectViewController("/swagger-ui", "/swagger-ui.html")
            .setKeepQueryParams(true)
            .setStatusCode(HttpStatus.PERMANENT_REDIRECT);
        registry.addRedirectViewController("/swagger-ui/", "/swagger-ui.html")
            .setKeepQueryParams(true)
            .setStatusCode(HttpStatus.PERMANENT_REDIRECT);
    }

}
