package de.hsrm.sls.subato.shared.swagger;


import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;
import org.reflections.Reflections;
import org.reflections.scanners.MethodAnnotationsScanner;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.context.event.EventListener;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.ResponseStatus;

/**
 * This job runs on start of the application and verifies, that the API documentation for error
 * responses can be generated from exceptions. API documentation requires the presence of the
 * {@link ResponseStatus} annotation on the exception class. If it is not present, then the
 * application will terminate with an error message.
 */
@Component
public class VerifyApiAnnotationsStartupJob {

    @EventListener(ContextRefreshedEvent.class)
    public void run() {
        verifySubclassesHaveAttribute();
    }

    public void verifySubclassesHaveAttribute() {
        var clazzez = findAllSubatoApiResponseExceptionClasses("de.hsrm.sls.subato");
        for (var clazz : clazzez) {
            if (!clazz.isAnnotationPresent(ResponseStatus.class)) {
                throw new RuntimeException(
                    "Class %s was used in SubatoApiResponse annotation but has not been annotated with ResponseStatus. This is required for generating the API documentation.".formatted(
                        clazz.getName()));
            }
        }
    }

    private List<Class<?>> findAllSubatoApiResponseExceptionClasses(
        String packageName) {

        Reflections reflections = new Reflections(packageName, new MethodAnnotationsScanner());

        var multiAnnotations = reflections.getMethodsAnnotatedWith(SubatoApiResponses.class)
            .stream().map(m -> AnnotationUtils.findAnnotation(m, SubatoApiResponses.class).value())
            .flatMap(Arrays::stream)
            .collect(Collectors.toList());

        var singleAnnotations = reflections.getMethodsAnnotatedWith(SubatoApiResponse.class)
            .stream().map(m -> AnnotationUtils.findAnnotation(m, SubatoApiResponse.class))
            .collect(Collectors.toList());

        var annotations = new ArrayList<SubatoApiResponse>();
        annotations.addAll(multiAnnotations);
        annotations.addAll(singleAnnotations);

        return annotations.stream().map(SubatoApiResponse::value).distinct()
            .collect(Collectors.toList());
    }
}