package de.hsrm.sls.subato.shared.websocket;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import de.hsrm.sls.subato.shared.error.ErrorConstants;
import de.hsrm.sls.subato.shared.error.GlobalExceptionLogger;
import de.hsrm.sls.subato.shared.error.mapping.ExceptionMapper;
import de.hsrm.sls.subato.shared.error.response.ErrorResponse;
import de.hsrm.sls.subato.shared.error.response.ErrorResponseBuilder;
import java.nio.charset.StandardCharsets;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.lang.NonNull;
import org.springframework.messaging.Message;
import org.springframework.messaging.MessageDeliveryException;
import org.springframework.messaging.simp.stomp.StompCommand;
import org.springframework.messaging.simp.stomp.StompHeaderAccessor;
import org.springframework.messaging.support.MessageBuilder;
import org.springframework.messaging.support.MessageHeaderAccessor;
import org.springframework.stereotype.Component;
import org.springframework.util.MimeTypeUtils;
import org.springframework.web.socket.messaging.StompSubProtocolErrorHandler;

/**
 * Custom error handler for translating exceptions to proper error messages. The default handler
 * just returns a generic "The message could not be send" if an exception occurs.
 */
@Component
public class StompErrorHandler extends StompSubProtocolErrorHandler {

    private static Logger logger = LoggerFactory.getLogger(StompErrorHandler.class);

    @Autowired
    ErrorResponseBuilder errorResponseBuilder;

    /**
     * see {@link ExceptionMapper} for more information.
     */
    @Autowired
    List<ExceptionMapper> exceptionMappers;

    @Autowired
    ObjectMapper objectMapper;

    @Autowired
    GlobalExceptionLogger globalExceptionLogger;

    @Override
    public Message<byte[]> handleClientMessageProcessingError(Message<byte[]> clientMessage,
        @NonNull Throwable ex) {
        var exception = unwrapException(ex);

        var matchingMapper = exceptionMappers.stream().filter(m -> m.canMap(exception)).findFirst()
            .orElse(null);

        try {
            var errorResponse = matchingMapper != null
                ? matchingMapper.map(exception)
                : errorResponseBuilder.buildForException(exception, null);
            globalExceptionLogger.log(errorResponse, exception);

            return buildErrorMessage(clientMessage, errorResponse);
        } catch (Exception e) {
            logger.error("Failed to process exception", e);
            try {
                return buildErrorMessage(clientMessage,
                    new ErrorResponse(ErrorConstants.DEFAULT_STATUS.value(),
                        ErrorConstants.DEFAULT_REASON,
                        null));
            } catch (JsonProcessingException exc) {
                throw new RuntimeException("Failed to send default error", exc);
            }
        }
    }

    /**
     * exceptions thrown in custom authorization handler and interceptrs get wrapped into
     * {@link MessageDeliveryException}, so we need to unwrap.
     */
    private Throwable unwrapException(Throwable ex) {
        var exception = ex;
        if (exception instanceof MessageDeliveryException) {
            exception = exception.getCause();
        }

        return exception;
    }

    private Message<byte[]> buildErrorMessage(Message<byte[]> clientMessage,
        ErrorResponse errorResponse)
        throws JsonProcessingException {
        var message = objectMapper.writeValueAsString(errorResponse);
        StompHeaderAccessor accessor = StompHeaderAccessor.create(StompCommand.ERROR);
        accessor.setContentType(MimeTypeUtils.APPLICATION_JSON);
        accessor.setLeaveMutable(true);

        StompHeaderAccessor clientHeaderAccessor = null;
        if (clientMessage != null) {
            clientHeaderAccessor = MessageHeaderAccessor.getAccessor(clientMessage,
                StompHeaderAccessor.class);
            if (clientHeaderAccessor != null) {
                String receiptId = clientHeaderAccessor.getReceipt();
                if (receiptId != null) {
                    accessor.setReceiptId(receiptId);
                }

                // this may be needed later for correlating errors with the requests
                //accessor.setSubscriptionId(clientHeaderAccessor.getSubscriptionId());
            }
        }

        return MessageBuilder.createMessage(
            message.getBytes(StandardCharsets.UTF_8),
            accessor.getMessageHeaders());
    }
}