package de.hsrm.sls.subato.solutions.evaluation.acceptance;

import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.solutions.SolutionsAction;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.exercises.exercise.ExerciseNotFoundException;
import de.hsrm.sls.subato.exercises.exercise.repository.ExerciseRepository;
import de.hsrm.sls.subato.shared.swagger.SubatoApiResponse;
import de.hsrm.sls.subato.solutions.evaluation.TestMode;
import de.hsrm.sls.subato.solutions.evaluation.bulk.BulkEvalManager;
import de.hsrm.sls.subato.solutions.evaluation.bulk.BulkEvalOptions;
import de.hsrm.sls.subato.solutions.evaluation.bulk.state.dto.BulkEvalStateDto;
import de.hsrm.sls.subato.solutions.evaluation.bulk.state.dto.BulkEvalStateMapper;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

@RestController("AcceptanceExerciseController")
@Tag(name = "Lösungsverwaltung")
public class ExerciseController {

    @Autowired
    private ExerciseRepository exerciseRepository;

    @Autowired
    private BulkEvalManager bulkEvalManager;

    @Autowired
    private PolicyEvaluator policy;

    @Autowired
    private BulkEvalStateMapper bulkEvalStateMapper;

    @GetMapping("/exercise/{id}/acceptance")
    @Operation(summary = "Liefert den aktuellen Zustand der Abnahmeauswertung für eine Übung")
    @SubatoApiResponse(ExerciseNotFoundException.class)
    BulkEvalStateDto getState(@PathVariable Long id,
        @AuthenticationPrincipal SubatoPrincipal principal
    ) {
        var exercise = exerciseRepository.findById(id)
            .orElseThrow(() -> new ExerciseNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(exercise)
            .setPermission(SolutionsAction.SOLUTION_RUN_BULK_ACCEPTANCE_TESTS).build();
        policy.checkAndRaise(context);

        var state = bulkEvalManager.getState(exercise, TestMode.ACCEPTANCE);
        return bulkEvalStateMapper.map(state);
    }

    @PostMapping("/exercise/{id}/acceptance")
    @Operation(summary = "Startet die Abnahmeauswertung für alle Lösungen einer Übung")
    @SubatoApiResponse(ExerciseNotFoundException.class)
    BulkEvalStateDto run(@PathVariable Long id,
        @RequestBody RunExerciseAcceptanceDto dto,
        @AuthenticationPrincipal SubatoPrincipal principal
    ) {
        var exercise = exerciseRepository.findById(id)
            .orElseThrow(() -> new ExerciseNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(exercise)
            .setPermission(SolutionsAction.SOLUTION_RUN_BULK_ACCEPTANCE_TESTS).build();
        policy.checkAndRaise(context);

        var options = new BulkEvalOptions.Builder()
            .setExercise(exercise)
            .setMode(TestMode.ACCEPTANCE)
            .setReset(dto.isReset()).build();
        var state = bulkEvalManager.runEvaluation(options);
        return bulkEvalStateMapper.map(state);
    }

    @PostMapping("/exercise/{id}/acceptance/cancel")
    @Operation(summary = "Bricht die aktuelle Abnahmeauswertung ab")
    @SubatoApiResponse(ExerciseNotFoundException.class)
    void cancel(@PathVariable Long id,
        @AuthenticationPrincipal SubatoPrincipal principal
    ) {
        var exercise = exerciseRepository.findById(id)
            .orElseThrow(() -> new ExerciseNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(exercise)
            .setPermission(SolutionsAction.SOLUTION_RUN_BULK_ACCEPTANCE_TESTS).build();
        policy.checkAndRaise(context);

        bulkEvalManager.cancel(exercise.getId());
    }

    @PostMapping("/exercise/{id}/acceptance/reset")
    @Operation(summary = "Setzt die aktuelle Abnahmeauswertung zurück (und bricht diese ab, falls sie gerade läuft)")
    @SubatoApiResponse(ExerciseNotFoundException.class)
    void reset(@PathVariable Long id,
        @AuthenticationPrincipal SubatoPrincipal principal
    ) {
        var exercise = exerciseRepository.findById(id)
            .orElseThrow(() -> new ExerciseNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(exercise)
            .setPermission(SolutionsAction.SOLUTION_RUN_BULK_ACCEPTANCE_TESTS).build();
        policy.checkAndRaise(context);

        bulkEvalManager.reset(exercise, TestMode.ACCEPTANCE);
    }
}
