package de.hsrm.sls.subato.solutions.evaluation.bulk;

import de.hsrm.sls.subato.assessment.submission.model.Submission;
import de.hsrm.sls.subato.courses.course.CourseFinishedException;
import de.hsrm.sls.subato.exercises.exercise.model.Exercise;
import de.hsrm.sls.subato.exercises.taskinstance.repository.TaskInstanceSort;
import de.hsrm.sls.subato.solutions.evaluation.TestMode;
import de.hsrm.sls.subato.solutions.evaluation.bulk.state.BulkEvalState;
import de.hsrm.sls.subato.solutions.evaluation.bulk.state.LogType;
import de.hsrm.sls.subato.solutions.evaluation.bulk.state.SolutionState;
import de.hsrm.sls.subato.solutions.evaluation.bulk.state.SolutionStateResult;
import de.hsrm.sls.subato.solutions.evaluation.bulk.state.TaskInstanceState;
import de.hsrm.sls.subato.solutions.evaluation.bulk.task.BulkEvalTask;
import de.hsrm.sls.subato.solutions.evaluation.bulk.task.BulkEvalTaskFactory;
import de.hsrm.sls.subato.solutions.solution.model.Solution;
import de.hsrm.sls.subato.solutions.solution.repository.SolutionRepository;
import de.hsrm.sls.subato.solutions.solution.repository.SolutionSort;
import jakarta.transaction.Transactional;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Future;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.task.AsyncTaskExecutor;
import org.springframework.stereotype.Service;

/**
 * Responsible for managing the bulk evaluation of solutions.
 * <p>
 * Some notes on the implementation: The manager prepares the {@link BulkEvalState}, which holds all
 * information required for the bulk execution. Then, the manager passes the state to the
 * {@link BulkEvalTask}, which is responsible for the executing and updating the state. This task
 * will run in another thread. Because the manager and the task have to access the database, both
 * threads have their own hibernate session. This is important because hibernate sessions are not
 * threadsafe. Therefore, the state should not contain any ORM objects. Instead, IDs should be
 * passed, which can be used to load objects in the other session.
 * <p>
 * Currently, bulk execution is using a single thread for execution because Eva has no support for
 * multithreading (yet). This can be changed later by splitting the work into multiple tasks and
 * configuring the thread pool to have more than 1 thread
 */
@Service
public class BulkEvalManager {

    @Autowired
    private BulkEvalTaskFactory bulkEvalTaskFactory;

    @Autowired
    private AsyncTaskExecutor bulkEvalExecutor;

    @Autowired
    private SolutionRepository solutionRepository;

    @Autowired
    private BulkEvalHandler handler;

    private final Object lock = new Object();

    private final Map<Long, Future<?>> runningTasks = new HashMap<>();
    private final Map<Long, BulkEvalState> stateMap = new HashMap<>();

    public void cancel(long exerciseId) {
        synchronized (lock) {
            if (runningTasks.containsKey(exerciseId)) {
                var future = runningTasks.remove(exerciseId);
                future.cancel(true);
            }
        }
    }

    public void reset(Exercise exercise, TestMode mode) {
        cancel(exercise.getId());

        synchronized (lock) {
            stateMap.remove(exercise.getId());
        }

        var state = getState(exercise, mode);
        handler.onStateChange(state);
    }

    public BulkEvalState getState(Exercise exercise, TestMode mode) {
        synchronized (lock) {
            if (stateMap.containsKey(exercise.getId())) {
                return stateMap.get(exercise.getId());
            }
        }

        var solutions = collectSolutionsToEvaluate(exercise, mode);
        return createState(exercise, solutions, mode);
    }

    public void notifyChanged(Solution solution, TestMode affectedMode) {
        if (solution.getExercise() == null) {
            return;
        }

        synchronized (lock) {
            if (stateMap.containsKey(solution.getExercise().getId())) {
                var state = stateMap.get(solution.getExercise().getId());

                var possiblyOutdated = (affectedMode == null && !solution.isDelayed()
                    && !solution.isAttemptsExceeded()) || affectedMode == state.getMode();
                if (possiblyOutdated) {
                    state.setPossiblyOutdated(true);
                    state.log(LogType.WARNING,
                        "Eine Lösung wurde geändert bzw. kam hinzu (ID %s). Der angezeigte Zustand der Auswertung ist jetzt eventuell veraltet.".formatted(
                            solution.getId()));
                    state.markChanged();
                    handler.onStateChange(state);
                }
            }
        }
    }

    @Transactional
    public BulkEvalState runEvaluation(BulkEvalOptions options) {
        if (options.getExercise().getCourse().isFinished()) {
            throw new CourseFinishedException();
        }

        var solutions = collectSolutionsToEvaluate(options.getExercise(), options.getMode());
        if (solutions.size() == 0) {
            throw new NoSolutionsException();
        }

        solutions = resetSolutions(solutions, options.getMode(), options.isReset());

        var state = createState(options.getExercise(), solutions, options.getMode());
        var task = bulkEvalTaskFactory.createTask(state, options);
        var future = bulkEvalExecutor.submit(task);

        synchronized (lock) {
            runningTasks.put(state.getExerciseId(), future);
            stateMap.put(state.getExerciseId(), state);
            state.setDetached(true);
        }

        return state;
    }

    private List<Solution> resetSolutions(List<Solution> solutions, TestMode mode,
        boolean resetAll) {
        var tmpSolutions = new ArrayList<Solution>();

        for (var solution : solutions) {
            var shouldReset =
                getStateResult(solution, mode) == SolutionStateResult.ERROR || resetAll;

            if (shouldReset) {
                if (mode == TestMode.ACCEPTANCE) {
                    solution.setAcceptanceEvalState(null);
                } else {
                    solution.setEvalState(null);
                }

                var changedSolution = solutionRepository.save(solution);
                tmpSolutions.add(changedSolution);
            } else {
                tmpSolutions.add(solution);
            }
        }

        // because the solutions will be loaded again in another thread,
        // we need to ensure that the changes are written to the database before the execution starts
        // to prevent race conditions. (hibernate does not write changes directly to the database on save)
        solutionRepository.flush();
        return tmpSolutions;
    }

    private List<Solution> collectSolutionsToEvaluate(Exercise exercise, TestMode mode) {
        var submissions = exercise.getSubmissions();
        return submissions.stream()
            .map(Submission::getSolution)
            .flatMap(solutions -> solutions.stream()
                .filter(s -> mode != TestMode.ACCEPTANCE || s.getTask().isAcceptanceTestAvailable())
                .filter(s -> !s.isDelayed() && !s.isAttemptsExceeded())
                .collect(Collectors.groupingBy(s -> s.getTaskInstance().getId()))
                .values().stream()
                .flatMap(taskInstanceSolutions -> taskInstanceSolutions.stream()
                    .min(SolutionSort.bySubmittedAtDesc()).stream())
            )
            .toList();
    }

    private BulkEvalState createState(Exercise exercise, List<Solution> solutions, TestMode mode) {
        var taskInstanceStates = solutions.stream()
            .collect(Collectors.groupingBy(Solution::getTaskInstance))
            .entrySet().stream().sorted(Map.Entry.comparingByKey(TaskInstanceSort.byOrderNumber()))
            .map(e ->
                new TaskInstanceState(
                    e.getKey(),
                    e.getValue().stream()
                        .map(s -> new SolutionState(s.getId(), getStateResult(s, mode)))
                        .toList()
                )
            )
            .collect(Collectors.toList());

        return new BulkEvalState(exercise.getId(), mode, taskInstanceStates);
    }


    private SolutionStateResult getStateResult(Solution solution, TestMode mode) {
        var evalResult = mode == TestMode.ACCEPTANCE
            ? solution.getAcceptanceEvalState()
            : solution.getEvalState();
        return evalResult == null
            ? SolutionStateResult.PENDING
            : evalResult.isFailed()
                ? SolutionStateResult.ERROR
                : SolutionStateResult.SUCCESS;
    }
}
