package de.hsrm.sls.subato.solutions.evaluation.bulk.state;

import de.hsrm.sls.subato.solutions.evaluation.TestMode;
import de.hsrm.sls.subato.solutions.evaluation.bulk.BulkEvalManager;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * Represents the state of the bulk execution for a given exercise. See {@link BulkEvalManager} for
 * more info.
 */
public class BulkEvalState {

    private final long exerciseId;
    private final TestMode mode;
    private final List<TaskInstanceState> taskInstanceStates;
    private boolean running;
    private List<BulkEvalLog> logs = new ArrayList<>();
    private boolean possiblyOutdated;
    private boolean detached;
    private LocalDateTime initializedAt;
    private LocalDateTime lastUpdatedAt;

    public BulkEvalState(long exerciseId, TestMode mode,
        List<TaskInstanceState> taskInstanceStates) {
        this.exerciseId = exerciseId;
        this.mode = mode;
        this.taskInstanceStates = taskInstanceStates;
        this.initializedAt = LocalDateTime.now();
        this.lastUpdatedAt = this.initializedAt;
    }

    public long getProcessed() {
        return taskInstanceStates.stream().map(TaskInstanceState::getProcessed)
            .reduce(0L, Long::sum);
    }

    public long getErrors() {
        return taskInstanceStates.stream().map(TaskInstanceState::getErrors)
            .reduce(0L, Long::sum);
    }

    public long getSuccessful() {
        return getProcessed() - getErrors();
    }

    public long getTotal() {
        return taskInstanceStates.stream().map(TaskInstanceState::getTotal)
            .reduce(0L, Long::sum);
    }

    public long getExerciseId() {
        return exerciseId;
    }

    public TestMode getMode() {
        return mode;
    }

    public List<TaskInstanceState> getTaskInstanceStates() {
        return taskInstanceStates;
    }

    public boolean isRunning() {
        return running;
    }

    public void setRunning(boolean running) {
        this.running = running;
    }

    public boolean isPossiblyOutdated() {
        return possiblyOutdated;
    }

    public void setPossiblyOutdated(boolean possiblyOutdated) {
        this.possiblyOutdated = possiblyOutdated;
    }

    public boolean isDetached() {
        return detached;
    }

    public void setDetached(boolean detached) {
        this.detached = detached;
    }

    public LocalDateTime getInitializedAt() {
        return initializedAt;
    }

    public void setInitializedAt(LocalDateTime initializedAt) {
        this.initializedAt = initializedAt;
    }

    public LocalDateTime getLastUpdatedAt() {
        return lastUpdatedAt;
    }

    public void setLastUpdatedAt(LocalDateTime lastUpdatedAt) {
        this.lastUpdatedAt = lastUpdatedAt;
    }

    public List<BulkEvalLog> getLogs() {
        return logs;
    }

    public void log(LogType type, String message) {
        var entry = new BulkEvalLog();
        entry.setType(type);
        entry.setAt(LocalDateTime.now());
        entry.setMessage(message);

        logs.add(entry);
    }

    public void markChanged() {
        lastUpdatedAt = LocalDateTime.now();
    }
}
