package de.hsrm.sls.subato.solutions.evaluation.bulk.task;

import de.hsrm.sls.subato.solutions.evaluation.EvalOptions.Builder;
import de.hsrm.sls.subato.solutions.evaluation.bulk.BulkEvalHandler;
import de.hsrm.sls.subato.solutions.evaluation.bulk.BulkEvalManager;
import de.hsrm.sls.subato.solutions.evaluation.bulk.BulkEvalOptions;
import de.hsrm.sls.subato.solutions.evaluation.bulk.state.BulkEvalState;
import de.hsrm.sls.subato.solutions.evaluation.bulk.state.LogType;
import de.hsrm.sls.subato.solutions.evaluation.bulk.state.SolutionState;
import de.hsrm.sls.subato.solutions.evaluation.bulk.state.SolutionStateResult;
import de.hsrm.sls.subato.solutions.evaluation.exception.EvaluationInterruptedException;
import de.hsrm.sls.subato.solutions.evaluation.usecase.EvalSolutionUseCase;
import de.hsrm.sls.subato.solutions.solution.repository.SolutionRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Responsible for the actual bulk execution. See {@link BulkEvalManager} for more info.
 */
public class BulkEvalTask implements Runnable {

    private static Logger logger = LoggerFactory.getLogger(BulkEvalTask.class);

    private final SolutionRepository solutionRepository;

    private final EvalSolutionUseCase evalSolutionUseCase;

    private final BulkEvalHandler handler;

    private final BulkEvalState state;

    private final BulkEvalOptions options;

    BulkEvalTask(BulkEvalState state, BulkEvalOptions options,
        EvalSolutionUseCase evalSolutionUseCase, BulkEvalHandler handler,
        SolutionRepository solutionRepository) {
        this.state = state;
        this.options = options;
        this.evalSolutionUseCase = evalSolutionUseCase;
        this.handler = handler;
        this.solutionRepository = solutionRepository;
    }

    @Override
    public void run() {
        state.setRunning(true);
        state.log(LogType.INFO, "Ausführung gestartet: %s Lösungen".formatted(state.getTotal()));
        handler.onStateChange(state);

        var cancelled = false;
        for (var taskInstanceState : state.getTaskInstanceStates()) {
            if (cancelled) {
                break;
            }

            for (var solutionState : taskInstanceState.getSolutionStates()) {
                var solution = solutionRepository.findById(solutionState.getSolutionId())
                    .orElse(null);

                if (Thread.currentThread().isInterrupted()) {
                    cancelled = true;
                    break;
                }

                if (solutionState.getResult() == SolutionStateResult.SUCCESS) {
                    continue;
                }

                try {
                    var evalOpts = new Builder()
                        .setSolution(solution)
                        .setMode(options.getMode())
                        .setSendNotificationEvent(false)
                        .build();

                    var evalResponse = evalSolutionUseCase.evaluateAndStore(evalOpts);
                    solution = evalResponse.getSolution();

                    if (evalResponse.getState().isFailed()) {
                        processError(state, solutionState);
                    } else {
                        solutionState.setResult(SolutionStateResult.SUCCESS);
                    }
                } catch (EvaluationInterruptedException ex) {
                    cancelled = true;
                    break;
                } catch (Exception ex) {
                    processError(state, solutionState);
                }

                logger.debug("Processed solution: %s".formatted(solution.getId()));
                state.markChanged();

                logProgress(state);
                handler.onStateChange(state);
            }
        }

        state.setRunning(false);
        state.log(LogType.INFO,
            cancelled ? "Ausführung abgebrochen" : "Ausführung abgeschlossen");

        handler.onStateChange(state);
    }

    private void logProgress(BulkEvalState state) {
        if (state.getProcessed() % 10 == 0) {
            logger.info("Progress: %s / %s (%s failed)".formatted(state.getProcessed(),
                state.getTotal(), state.getTotal()));
        }
    }

    private void processError(BulkEvalState state, SolutionState solutionState) {
        solutionState.setResult(SolutionStateResult.ERROR);
        state.log(LogType.PROBLEM,
            "Lösung %s konnte nicht getestet werden".formatted(
                solutionState.getSolutionId()));
    }
}
