package de.hsrm.sls.subato.solutions.evaluation.request;

import de.hsrm.sls.subato.shared.fs.FileTree;
import de.hsrm.sls.subato.solutions.evaluation.EvalOptions;
import de.hsrm.sls.subato.solutions.evaluation.TestMode;
import de.hsrm.sls.subato.solutions.evaluation.exception.MissingFileException;
import de.hsrm.sls.subato.solutions.solution.model.Solution;
import de.hsrm.sls.subato.solutions.solution.repository.SolutionContentRepository;
import de.hsrm.sls.subato.tasks.stef.spec.FileRef;
import de.hsrm.sls.subato.tasks.stef.spec.FileType;
import de.hsrm.sls.subato.tasks.task.Task;
import java.util.ArrayList;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class EvalRequestGenerator {

    @Autowired
    private SolutionContentRepository solutionContentRepository;

    /**
     * selects the files to send to the evaluator and configures other parameters
     *
     * @throws MissingFileException
     */
    public EvalRequest generateRequest(EvalOptions options) {
        Task task = options.getSolution().getTask();
        var canEvaluate = options.getMode() == TestMode.REGULAR && task.evaluationPossible()
            || options.getMode() == TestMode.ACCEPTANCE && task.isAcceptanceTestAvailable();
        if (!canEvaluate) {
            throw new UnsupportedOperationException(
                "Cannot evaluate solution %s because evaluator or test files are missing".formatted(
                    options.getSolution().getId()));
        }

        var templateFile = task.getSpec().getMeta().getTemplateFiles().stream().findFirst().get();
        FileTree codeFile = getMatchingSolutionFile(options.getSolution(), templateFile);

        var filesToTransmit = new ArrayList<FileRef>();
        for (var taskFile : task.getSpec().getMeta().getFiles()) {
            var shouldInclude = taskFile.getType() == FileType.INCLUDE ||
                (options.getMode() == TestMode.REGULAR && taskFile.getType() == FileType.TEST) ||
                (options.getMode() == TestMode.ACCEPTANCE
                    && taskFile.getType() == FileType.ACCEPTANCETEST);

            if (shouldInclude) {
                filesToTransmit.add(taskFile);
            }
        }

        var request = new EvalRequest();
        request.setOptions(options);
        request.setCodeFile(codeFile);
        request.setFilesToTransmit(filesToTransmit);
        request.setTemplateFile(templateFile);
        request.setParams(task.getSpec().getMeta().getEvaluatorParams());

        return request;
    }

    /**
     * resolves the file in the solution, which best matches the template
     *
     * @throws MissingFileException if no file was matched
     */
    private FileTree getMatchingSolutionFile(Solution solution, FileRef templateFile) {
        var solutionTree = solutionContentRepository.getTree(solution);

        var matchers = new FileMatcher[]{
            new FirstFileMatcher(),
            new ExactMatcher(),
            new PrefixMatcher("src"),
            new PrefixMatcher("*"),
            new PrefixMatcher("*/src"),
        };

        FileTree codeFile = null;
        for (var matcher : matchers) {
            var item = matcher.match(solutionTree, templateFile);
            if (item != null) {
                codeFile = item;
                break;
            }
        }

        if (codeFile == null) {
            throw new MissingFileException(templateFile);
        }

        return codeFile;
    }
}
