package de.hsrm.sls.subato.solutions.evaluation.usecase;

import de.hsrm.sls.eva.EvaConstants;
import de.hsrm.sls.eva.request.EvaFile;
import de.hsrm.sls.eva.request.RequestedCapability;
import de.hsrm.sls.subato.courses.course.CourseFinishedException;
import de.hsrm.sls.subato.shared.helper.FileHelper;
import de.hsrm.sls.subato.solutions.evaluation.EvalMonitor;
import de.hsrm.sls.subato.solutions.evaluation.EvalOptions;
import de.hsrm.sls.subato.solutions.evaluation.EvalResultChangedEvent;
import de.hsrm.sls.subato.solutions.evaluation.TestMode;
import de.hsrm.sls.subato.solutions.evaluation.exception.EvaluationInterruptedException;
import de.hsrm.sls.subato.solutions.evaluation.exception.EvaluationTimeoutException;
import de.hsrm.sls.subato.solutions.evaluation.request.EvalRequest;
import de.hsrm.sls.subato.solutions.evaluation.request.EvalRequestGenerator;
import de.hsrm.sls.subato.solutions.evaluation.request.EvalResponse;
import de.hsrm.sls.subato.solutions.evaluation.result.EvalState;
import de.hsrm.sls.subato.solutions.solution.model.Solution;
import de.hsrm.sls.subato.solutions.solution.repository.SolutionContentRepository;
import de.hsrm.sls.subato.solutions.solution.repository.SolutionRepository;
import de.hsrm.sls.subato.tasks.stef.spec.FileType;
import de.hsrm.sls.subato.tasks.task.repository.TaskContentRepository;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.UncategorizedAmqpException;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.stereotype.Service;

@Service
public class EvalSolutionUseCase {

    private static Logger logger = LoggerFactory.getLogger(EvalSolutionUseCase.class);

    @Qualifier("eva2Template")
    @Autowired
    private RabbitTemplate rabbitTemplate;

    @Autowired
    private RefreshEvaluatorUseCase refreshEvaluatorUseCase;

    @Autowired
    private EvalRequestGenerator requestGenerator;

    @Autowired
    private SolutionContentRepository solutionContentRepository;

    @Autowired
    private SolutionRepository solutionRepository;

    @Autowired
    private TaskContentRepository taskContentRepository;

    @Autowired
    private EvalMonitor monitor;

    @Autowired
    private ApplicationEventPublisher applicationEventPublisher;

    /**
     * @return result of the evaluation, returned by the evalautor
     */
    public EvalResponse evaluateAndStore(EvalOptions options) {
        var exercise = options.getSolution().getExercise();
        if (exercise != null && exercise.getCourse().isFinished()) {
            throw new CourseFinishedException();
        }

        monitor.start();

        try {
            var request = requestGenerator.generateRequest(options);
            var evalState = tryEvaluate(request);
            var solution = storeResult(evalState, options);

            if (options.isSendNotificationEvent()) {
                applicationEventPublisher.publishEvent(
                    new EvalResultChangedEvent(this, solution, options.getMode()));
            }

            return new EvalResponse(evalState, solution);
        } finally {
            monitor.done();
        }
    }

    private EvalState tryEvaluate(EvalRequest request) {
        try {
            return submitRequest(request);
        } catch (EvaluationTimeoutException ex) {
            logger.error(
                "Evaluation of solution %s failed: Timeout".formatted(
                    request.getOptions().getSolution().getId()), ex);
            monitor.failure(true);
        } catch (EvaluationInterruptedException ex) {
            throw ex;
        } catch (Exception ex) {
            logger.error(
                "Evaluation of solution %s failed".formatted(
                    request.getOptions().getSolution().getId()), ex);
            monitor.failure(false);
        }

        var evalState = new EvalState();
        evalState.setFailed(true);
        return evalState;
    }

    private Solution storeResult(EvalState evalState, EvalOptions options) {
        var solution = options.getSolution();
        if (options.getMode() == TestMode.ACCEPTANCE) {
            solution.setAcceptanceEvalState(evalState);
        } else {
            solution.setEvalState(evalState);
        }
        return solutionRepository.save(solution);
    }

    private EvalState submitRequest(EvalRequest evalRequest) {
        var stack = refreshEvaluatorUseCase.refresh(
            evalRequest.getOptions().getSolution().getTask());

        var codeFile = solutionContentRepository.getFile(
            evalRequest.getOptions().getSolution(),
            evalRequest.getCodeFile().getUuid()
        );

        var includeFiles = evalRequest.getFilesToTransmit().stream()
            .filter(f -> f.getType() == FileType.INCLUDE)
            .map(f -> {
                var file = taskContentRepository.getFileByRef(
                    evalRequest.getOptions().getSolution().getTask(), f);
                return new EvaFile.Builder().setPath(f.getPath())
                    .setContent(FileHelper.readString(file)).build();
            }).toList();

        var testFiles = evalRequest.getFilesToTransmit().stream()
            .filter(f -> (f.getType() == FileType.TEST
                && evalRequest.getOptions().getMode() == TestMode.REGULAR) || (
                f.getType() == FileType.ACCEPTANCETEST
                    && evalRequest.getOptions().getMode() == TestMode.ACCEPTANCE))
            .map(f -> {
                var file = taskContentRepository.getFileByRef(
                    evalRequest.getOptions().getSolution().getTask(), f);
                return new EvaFile.Builder().setPath(f.getPath())
                    .setContent(FileHelper.readString(file)).build();
            }).toList();

        var reqBuilder = new de.hsrm.sls.eva.request.EvalRequest.Builder()
            .setStackRef(stack.getRef())
            .setSolutionFiles(List.of(new EvaFile.Builder()
                .setContent(FileHelper.readString(codeFile))
                .setPath(evalRequest.getTemplateFile().getPath())
                .build()))
            .setTestFiles(testFiles)
            .setIncludeFiles(includeFiles)
            .setCapability(new RequestedCapability.Builder()
                .setCompile(stack.getEvaluator().getCapability().isCompile())
                .setTest(stack.getEvaluator().getCapability().isTest())
                .setStyle(stack.getEvaluator().getCapability().isStyle())
                .build())
            .setParams(evalRequest.getParams());
        var req = reqBuilder.build();

        try {
            var response = (de.hsrm.sls.eva.EvalResponse) rabbitTemplate.convertSendAndReceiveAsType(
                "amq.topic",
                "eva.request", req,
                message -> {
                    message.getMessageProperties().setHeader("VERSION", EvaConstants.VERSION);
                    return message;
                },
                ParameterizedTypeReference.forType(de.hsrm.sls.eva.EvalResponse.class));

            if (response == null) {
                throw new EvaluationTimeoutException();
            }

            if (!response.isSuccess()) {
                throw new RuntimeException(
                    "Eva returned an error: %s".formatted(
                        response.getMessage()));
            }

            var evalState = new EvalState();
            evalState.setResult(response.getResult());
            evalState.setStackRef(stack.getRef());
            return evalState;
        } catch (UncategorizedAmqpException ex) {
            if (ex.getCause() instanceof InterruptedException) {
                throw new EvaluationInterruptedException(ex.getCause());
            }

            throw ex;
        }
    }
}
