package de.hsrm.sls.subato.solutions.solution;


import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.solutions.SolutionsAction;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.shared.fs.FileTreeMapper;
import de.hsrm.sls.subato.shared.response.FileResponseBuilderFactory;
import de.hsrm.sls.subato.shared.swagger.SubatoApiResponse;
import de.hsrm.sls.subato.solutions.solution.dto.SolutionDetailDto;
import de.hsrm.sls.subato.solutions.solution.dto.SolutionMapper;
import de.hsrm.sls.subato.solutions.solution.dto.SolutionSummaryDto;
import de.hsrm.sls.subato.solutions.solution.repository.SolutionContentRepository;
import de.hsrm.sls.subato.solutions.solution.repository.SolutionFilter;
import de.hsrm.sls.subato.solutions.solution.repository.SolutionQuery;
import de.hsrm.sls.subato.solutions.solution.repository.SolutionRepository;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import java.util.Optional;
import java.util.UUID;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.Resource;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.web.PageableDefault;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RestController
@Tag(name = "Lösungsverwaltung")
public class SolutionController {

    @Autowired
    private SolutionRepository solutionRepository;

    @Autowired
    private SolutionContentRepository solutionContentRepository;

    @Autowired
    private SolutionMapper solutionMapper;

    @Autowired
    private FileTreeMapper fileTreeMapper;

    @Autowired
    private PolicyEvaluator policy;

    @Autowired
    private FileResponseBuilderFactory fileResponseBuilderFactory;

    @GetMapping("/solution/{id}")
    @SubatoApiResponse(SolutionNotFoundException.class)
    SolutionDetailDto getOne(@PathVariable Long id,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var solution = solutionRepository.findById(id)
            .orElseThrow(() -> new SolutionNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(solution)
            .setPermission(SolutionsAction.SOLUTION_VIEW).build();
        policy.checkAndRaise(context);

        var tree = solutionContentRepository.getTree(solution);
        var dto = solutionMapper.mapDetail(solution);
        dto.setTree(fileTreeMapper.map(tree));
        return dto;
    }

    @GetMapping("/solution")
    @PreAuthorize("isAuthenticated()")
    @Operation(summary = "Alle sichtbaren Lösungen im Kontext des angemeldeten Benutzers")
    Page<SolutionSummaryDto> getAll(
        @PageableDefault Pageable page,
        @RequestParam Optional<Long> exerciseId,
        @RequestParam Optional<Long> taskId,
        @RequestParam Optional<Long> userId,
        @RequestParam Optional<Long> submissionId,
        @RequestParam Optional<Boolean> valid,
        @AuthenticationPrincipal SubatoPrincipal principal) {

        var context = new PolicyContext.Builder().setSubject(principal)
            .setPermission(SolutionsAction.SOLUTION_VIEW).build();
        var filter = new SolutionFilter.Builder()
            .exerciseId(exerciseId.orElse(null))
            .taskId(taskId.orElse(null))
            .submittedById(userId.orElse(null))
            .submissionId(submissionId.orElse(null))
            .valid(valid.orElse(null))
            .build();

        var solutionQuery = new SolutionQuery.Builder()
            .setFilter(filter)
            .setPolicyContext(context)
            .build();

        var solutions = solutionRepository.findAll(solutionQuery, page);
        return solutions.map(solution -> solutionMapper.mapSummary(solution));
    }

    @GetMapping("/solution/{id}/file/{uuid}")
    @Operation(summary = "Zugriff auf Dateien in der Lösungen, ermöglicht Browsing von komplexeren Lösungen die einen Verzeichnisbaum beinhalten")
    @SubatoApiResponse(SolutionNotFoundException.class)
    ResponseEntity<Resource> getFile(@PathVariable Long id, @PathVariable UUID uuid,
        @RequestParam Optional<Boolean> download,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var solution = solutionRepository.findById(id)
            .orElseThrow(() -> new SolutionNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(solution)
            .setPermission(SolutionsAction.SOLUTION_VIEW).build();
        policy.checkAndRaise(context);

        var file = solutionContentRepository.getFile(solution, uuid);
        return fileResponseBuilderFactory.forFile(file)
            .setAsAttachment(download.isPresent() && download.get())
            .build();
    }

    @GetMapping("/solution/{id}/download")
    @SubatoApiResponse(SolutionNotFoundException.class)
    ResponseEntity<Resource> downloadSolution(@PathVariable Long id,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var solution = solutionRepository.findById(id)
            .orElseThrow(() -> new SolutionNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(solution)
            .setPermission(SolutionsAction.SOLUTION_VIEW).build();
        policy.checkAndRaise(context);

        var file = solutionContentRepository.getSolutionAsZip(solution);
        return fileResponseBuilderFactory.forFile(file)
            .setFileName("solution%s.zip".formatted(solution.getId()))
            .setAsAttachment(true)
            .build();
    }
}
