package de.hsrm.sls.subato.solutions.solution.model;

import de.hsrm.sls.subato.assessment.submission.model.Submission;
import de.hsrm.sls.subato.exercises.exercise.model.Exercise;
import de.hsrm.sls.subato.exercises.taskinstance.TaskInstance;
import de.hsrm.sls.subato.solutions.comment.SolutionComment;
import de.hsrm.sls.subato.solutions.evaluation.result.EvalState;
import de.hsrm.sls.subato.tasks.task.Task;
import de.hsrm.sls.subato.user.User;
import jakarta.annotation.Nullable;
import jakarta.persistence.CascadeType;
import jakarta.persistence.Column;
import jakarta.persistence.Entity;
import jakarta.persistence.GeneratedValue;
import jakarta.persistence.GenerationType;
import jakarta.persistence.Id;
import jakarta.persistence.ManyToOne;
import jakarta.persistence.OneToMany;
import jakarta.persistence.Table;
import jakarta.validation.constraints.NotNull;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import org.hibernate.annotations.JdbcTypeCode;
import org.hibernate.type.SqlTypes;

/**
 * Represents a complex solution for a {@link de.hsrm.sls.subato.tasks.task.Task} in an
 * {@link Exercise} (represented by {@link TaskInstance}), that may be evaluated automatically
 */
@Entity
@Table(name = "solution")
public class Solution {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private long id;

    @ManyToOne
    private Task task;

    @ManyToOne
    private Exercise exercise;

    @ManyToOne
    private User submittedBy;

    @Column(columnDefinition = "TIMESTAMP")
    @NotNull
    private LocalDateTime submittedAt;

    /**
     * internal reference, indicating if the solution is from the legacy system. if it is, then the
     * solution might be inconsistent. not all fields may be present
     */
    @Column(unique = true)
    Long sourceId;

    /**
     * results of the evaluation. null if the evaluation was never attempted
     */
    @JdbcTypeCode(SqlTypes.JSON)
    @Nullable
    private EvalState evalState;

    /**
     * same as {@link #evalState} but for acceptance tests
     */
    @JdbcTypeCode(SqlTypes.JSON)
    @Nullable
    private EvalState acceptanceEvalState;

    /**
     * submission, this solution belongs to.
     * <p>
     * null if 1.) the solution was submitted after deadline or 2.) the solution belongs to an
     * exercise for which checking/grading is disabled
     */
    @ManyToOne
    @Nullable
    private Submission submission;

    /**
     * if the solution was submitted after deadline and the user explicitly stated to show the
     * solution to the lecturers
     */
    private boolean delayed;

    /**
     * if this solution exceeded the value at {@link Task#attempts}
     */
    private boolean attemptsExceeded;

    /**
     * how was this solution submitted?
     */
    @NotNull
    private Client submittedWith;

    @OneToMany(mappedBy = "solution", cascade = {CascadeType.ALL})
    private List<SolutionComment> comments = new ArrayList<>();

    public Solution() {
    }

    public Long getSourceId() {
        return sourceId;
    }

    public void setSourceId(Long sourceId) {
        this.sourceId = sourceId;
    }

    public Long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public Task getTask() {
        return task;
    }

    public void setTask(Task task) {
        this.task = task;
    }

    public Exercise getExercise() {
        return exercise;
    }

    public void setExercise(Exercise exercise) {
        this.exercise = exercise;
    }


    public TaskInstance getTaskInstance() {
        return exercise.getTaskInstances().stream()
            .filter(ti -> ti.getTask().getId() == task.getId()).findFirst().orElse(null);
    }

    public User getSubmittedBy() {
        return submittedBy;
    }

    @Nullable
    public EvalState getEvalState() {
        return evalState;
    }

    public void setEvalState(@Nullable EvalState evalState) {
        this.evalState = evalState;
    }

    public void setSubmittedBy(User creator) {
        this.submittedBy = creator;
    }

    public LocalDateTime getSubmittedAt() {
        return submittedAt;
    }

    public void setSubmittedAt(LocalDateTime submittedAt) {
        this.submittedAt = submittedAt;
    }

    public Submission getSubmission() {
        return submission;
    }

    public void setSubmission(Submission submission) {
        this.submission = submission;
    }

    public boolean isDelayed() {
        return delayed;
    }

    public void setDelayed(boolean delayed) {
        this.delayed = delayed;
    }

    public boolean isAttemptsExceeded() {
        return attemptsExceeded;
    }

    public void setAttemptsExceeded(boolean attemptsExceeded) {
        this.attemptsExceeded = attemptsExceeded;
    }

    public List<SolutionComment> getComments() {
        return comments;
    }

    public void setComments(List<SolutionComment> comments) {
        this.comments = comments;
    }

    public Client getSubmittedWith() {
        return submittedWith;
    }

    public void setSubmittedWith(Client submittedWith) {
        this.submittedWith = submittedWith;
    }

    @Nullable
    public EvalState getAcceptanceEvalState() {
        return acceptanceEvalState;
    }

    public void setAcceptanceEvalState(
        @Nullable EvalState acceptanceEvalState) {
        this.acceptanceEvalState = acceptanceEvalState;
    }
}
