package de.hsrm.sls.subato.solutions.submit;

import de.hsrm.sls.subato.assessment.submission.model.Submission;
import de.hsrm.sls.subato.assessment.submission.repository.SubmissionRepository;
import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.solutions.SolutionsAction;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.courses.course.model.Course;
import de.hsrm.sls.subato.courses.unit.usecase.AddParticipantUseCase;
import de.hsrm.sls.subato.shared.validation.ConstraintViolationMapper;
import de.hsrm.sls.subato.shared.validation.ValidationException;
import de.hsrm.sls.subato.solutions.comment.SolutionComment;
import de.hsrm.sls.subato.solutions.evaluation.EvalOptions;
import de.hsrm.sls.subato.solutions.evaluation.exception.MissingFileException;
import de.hsrm.sls.subato.solutions.evaluation.usecase.EvalSolutionUseCase;
import de.hsrm.sls.subato.solutions.solution.model.Solution;
import de.hsrm.sls.subato.solutions.solution.repository.SolutionContentRepository;
import de.hsrm.sls.subato.solutions.solution.repository.SolutionRepository;
import de.hsrm.sls.subato.user.UserNotFoundException;
import de.hsrm.sls.subato.user.repository.UserRepository;
import jakarta.validation.Validation;
import java.time.LocalDateTime;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Service;

/**
 * handles the submission of a solution to an exercise by storing it in the database, eventually
 * adding the solution to the submission from the submitting user for the affected exercise, adding
 * the submitting user as a participant to the course and evaluating the solution if an evaluator
 * has been configured for the task
 * <p>
 * The solution will only be added to the submission if it is not delayed or the attempts are not
 * exceeded. Except if the submitting user has explicitly stated so. If a submission does not exist
 * yet, it will be created.
 */
@Service
public class ExerciseSubmitHandler implements SubmitHandler {

    @Autowired
    private SolutionRepository solutionRepository;

    @Autowired
    private SolutionContentRepository solutionContentRepository;

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private EvalSolutionUseCase evalSolutionUseCase;

    @Autowired
    private AddParticipantUseCase addParticipantUseCase;

    @Autowired
    private PolicyEvaluator policy;

    @Autowired
    private SubmissionRepository submissionRepository;

    @Autowired
    private ApplicationEventPublisher applicationEventPublisher;

    /**
     * @throws MissingFileException if the solution cannot be evaluated because it was incorrectly
     *                              submitted
     */
    @Override
    public Solution handle(SubmitSolutionRequest request, SubatoPrincipal principal,
        LocalDateTime submittedAt) {
        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(Pair.of(request.exercise(), request.task()))
            .setPermission(SolutionsAction.SOLUTION_SUBMIT_EXERCISE).build();
        policy.checkAndRaise(context);

        var user = principal != null ? userRepository.findById(principal.getId())
            .orElseThrow(() -> new UserNotFoundException(principal.getId())) : null;

        Solution solution = new Solution();
        solution.setSubmittedBy(user);
        solution.setTask(request.task());
        solution.setSubmittedAt(submittedAt);
        solution.setSubmittedWith(request.client());
        solution.setExercise(request.exercise());

        if (user != null && StringUtils.isNotBlank(request.comment())) {
            var comment = new SolutionComment();
            comment.setText(request.comment());
            comment.setFrom(user);
            comment.setSolution(solution);
            comment.setCreatedAt(LocalDateTime.now());
            solution.getComments().add(comment);
        }

        var validator = Validation.buildDefaultValidatorFactory().getValidator();
        var violations = validator.validate(solution);
        if (!violations.isEmpty()) {
            throw new ValidationException(ConstraintViolationMapper.map(violations));
        }

        solution = solutionRepository.save(solution);

        if (user != null) {
            addAsParticipantIfNoParticipant(principal, request.exercise().getCourse());

            var spec = request.task().getSpec();
            var currentAttempts = submissionRepository.getCurrentAttempts(request.task(),
                request.exercise(), user);
            var attemptsExceeded =
                spec.getMeta().getAttempts() != null && currentAttempts > spec.getMeta()
                    .getAttempts();
            solution.setAttemptsExceeded(attemptsExceeded);

            solution.setDelayed(request.exercise().getDueTo() != null && submittedAt.isAfter(
                request.exercise().getDueTo()));
            // we eventually add the solution to the submission
            if (!solution.isDelayed() && !attemptsExceeded || request.lateSubmit()) {
                addSolutionToSubmission(solution);
            }

            solution = solutionRepository.save(solution);
        }

        solutionContentRepository.store(request.content(), solution);

        // eventually evaluate the solution if the task is configured for automated evaluation
        var task = solution.getTask();
        if (task.evaluationPossible()) {
            var evalResponse = evalSolutionUseCase.evaluateAndStore(
                new EvalOptions.Builder().setSolution(solution).build());
            solution = evalResponse.getSolution();
        }

        applicationEventPublisher.publishEvent(new SolutionSubmittedEvent(this, solution));
        return solution;
    }

    private void addAsParticipantIfNoParticipant(SubatoPrincipal principal, Course course) {
        if (!principal.isParticipant(course.getId())) {
            var defaultUnit = course.getDefault();
            var user = userRepository.findById(principal.getId())
                .orElseThrow(() -> new UserNotFoundException(principal.getId()));
            addParticipantUseCase.add(defaultUnit, user);
        }
    }

    private void addSolutionToSubmission(Solution solution) {
        var submission = submissionRepository.findByExerciseAndUser(solution.getExercise().getId(),
            solution.getSubmittedBy().getId());
        if (submission == null) {
            submission = new Submission();
            submission.setExercise(solution.getExercise());
            submission.setSubmittedBy(solution.getSubmittedBy());
        }

        submission.getSolution().add(solution);
        submission = submissionRepository.save(submission);
        solution.setSubmission(submission);
    }
}
