package de.hsrm.sls.subato.solutions.submit;

import de.hsrm.sls.subato.exercises.exercise.ExerciseNotFoundException;
import de.hsrm.sls.subato.shared.springext.ResponseStatusMapping;
import de.hsrm.sls.subato.shared.swagger.SubatoApiResponse;
import de.hsrm.sls.subato.shared.validation.ValidationException;
import de.hsrm.sls.subato.solutions.evaluation.exception.MissingFileException;
import de.hsrm.sls.subato.solutions.solution.dto.SolutionDetailDto;
import de.hsrm.sls.subato.solutions.solution.dto.SolutionMapper;
import de.hsrm.sls.subato.tasks.task.exception.TaskArchivedException;
import de.hsrm.sls.subato.tasks.task.exception.TaskNotFoundException;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestPart;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

@RestController
@Tag(name = "Lösungsverwaltung")
public class SubmitController {

    @Autowired
    private SubmitGate submitGate;

    @Autowired
    private SolutionMapper solutionMapper;

    private static final String solutionSubmitDescription = """
        Die Abgabe von Lösungen kann in zwei Modi erfolgen, weshalb dieser Endpunkt zwei verschiedene Eingaben entgegen nehmen kann:  \n
        1.) Abgabe einer Datei bzw. eines zip-Archives (multipart/form-data)\n
        2.) Abgabe von Inhalt einer Datei als String (application/json)\n
        \n
        Welcher Modus verwendet werden kann, ergibt sich aus der Aufgabe. Siehe `SubmissionMode` in der STEF Spezifikation für weitere Informationen.
        \n
        Die Angabe von `lateSubmit` hat erst eine Bedeutung, wenn die Abgabefrist erreicht ist. Ist dann lateSubmit = `false`, wird die Lösung nicht mehr einer Abgabe zugeordnet. Wenn lateSubmit = `true`, wird die Lösung als verspätet gekennzeichnet und der Abgabe hinzugefügt.
        `comment` ist Optional und kann angegeben werden um einen Kommentar zur Lösung hinzuzfügen.
            
        Bei der Abgabe muss zu Forschungszwecken auch angegeben werden, mit welchem Client (z.B. SPA, IntelliJ) abgegeben wurde.
        """;

    private static final String solutionSubmitSummary = "Speicherung und automatische Auswertung von Lösungen (wenn für die Aufgabe konfiguriert).";


    @PostMapping(path = "/solution/submit", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    @Operation(summary = solutionSubmitSummary, description = solutionSubmitDescription)
    @SubatoApiResponse(ExerciseNotFoundException.class)
    @SubatoApiResponse(TaskNotFoundException.class)
    @SubatoApiResponse(TaskArchivedException.class)
    @SubatoApiResponse(InvalidModeException.class)
    @SubatoApiResponse(ExtensionNotAllowedException.class)
    @SubatoApiResponse(MissingFileException.class)
    @SubatoApiResponse(MissingFileException.class)
    @SubatoApiResponse(ValidationException.class)
    @ResponseStatusMapping(exception = TaskNotFoundException.class, code = HttpStatus.BAD_REQUEST)
    @ResponseStatusMapping(exception = ExerciseNotFoundException.class, code = HttpStatus.BAD_REQUEST)
    SolutionDetailDto submit(
        @Parameter(description = "Lösungsdatei (Datei oder zip-Archiv)", required = false)
        @RequestPart("file")
        MultipartFile file,
        @Parameter(description = "Metadaten wie Verweise auf Aufgabe und Übung, die für die Zuordnung der Lösung erforderlich sind. Das DTO heißt `SubmitFileSolutionDto`, hier wird leider das Schema nicht angezeigt aber man kann es unten in der Auflistung finden.")
        @RequestPart("solution")
        SubmitFileSolutionDto dto) {
        var solution = submitGate.submit(file, dto);
        return solutionMapper.mapDetail(solution);
    }

    @PostMapping(path = "/solution/submit", consumes = MediaType.APPLICATION_JSON_VALUE)
    @Operation(summary = solutionSubmitSummary, description = solutionSubmitDescription)
    @SubatoApiResponse(ExerciseNotFoundException.class)
    @SubatoApiResponse(TaskNotFoundException.class)
    @SubatoApiResponse(TaskArchivedException.class)
    @SubatoApiResponse(InvalidModeException.class)
    @SubatoApiResponse(ExtensionNotAllowedException.class)
    @SubatoApiResponse(MissingFileException.class)
    @SubatoApiResponse(ValidationException.class)
    @ResponseStatusMapping(exception = TaskNotFoundException.class, code = HttpStatus.BAD_REQUEST)
    @ResponseStatusMapping(exception = ExerciseNotFoundException.class, code = HttpStatus.BAD_REQUEST)
    SolutionDetailDto submit(
        @Parameter(description = "Enthält die Lösung selbst und Metadaten, wie Verweise auf Aufgabe und Übung, die für die Zuordnung der Lösung erforderlich sind.")
        @RequestBody SubmitInlineSolution dto) {
        var solution = submitGate.submit(dto);
        return solutionMapper.mapDetail(solution);
    }
}
