package de.hsrm.sls.subato.solutions.submit;

import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.solutions.SolutionsAction;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.shared.validation.ConstraintViolationMapper;
import de.hsrm.sls.subato.shared.validation.ValidationException;
import de.hsrm.sls.subato.solutions.evaluation.EvalOptions;
import de.hsrm.sls.subato.solutions.evaluation.exception.MissingFileException;
import de.hsrm.sls.subato.solutions.evaluation.usecase.EvalSolutionUseCase;
import de.hsrm.sls.subato.solutions.solution.model.Solution;
import de.hsrm.sls.subato.solutions.solution.repository.SolutionContentRepository;
import de.hsrm.sls.subato.solutions.solution.repository.SolutionRepository;
import de.hsrm.sls.subato.user.UserNotFoundException;
import de.hsrm.sls.subato.user.repository.UserRepository;
import jakarta.validation.Validation;
import java.time.LocalDateTime;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * handles the submission of a solution to a task, which will occur if lecturers are testing their
 * tasks with sample solutions. The solutions have to be persisted in order to be able to
 * automatically evaluate them.
 * <p>
 * Solutions will not be cleaned up automatically, they remain in the system to ensure that
 * solutions can be accessed later for debugging purposes
 */
@Service
public class TaskSubmitHandler implements SubmitHandler {

    @Autowired
    private SolutionRepository solutionRepository;

    @Autowired
    private SolutionContentRepository solutionContentRepository;

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private EvalSolutionUseCase evalSolutionUseCase;

    @Autowired
    private PolicyEvaluator policy;

    /**
     * @throws MissingFileException if the solution cannot be evaluated because it was incorrectly
     *                              submitted
     */
    @Override
    public Solution handle(SubmitSolutionRequest request, SubatoPrincipal principal,
        LocalDateTime submittedAt) {
        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(Pair.of(request.exercise(), request.task()))
            .setPermission(SolutionsAction.SOLUTION_SUBMIT_TASK).build();
        policy.checkAndRaise(context);

        var user = principal != null ? userRepository.findById(principal.getId())
            .orElseThrow(() -> new UserNotFoundException(principal.getId())) : null;

        Solution solution = new Solution();
        solution.setSubmittedBy(user);
        solution.setTask(request.task());
        solution.setSubmittedAt(submittedAt);
        solution.setSubmittedWith(request.client());

        var validator = Validation.buildDefaultValidatorFactory().getValidator();
        var violations = validator.validate(solution);
        if (!violations.isEmpty()) {
            throw new ValidationException(ConstraintViolationMapper.map(violations));
        }

        solution = solutionRepository.save(solution);

        solutionContentRepository.store(request.content(), solution);

        var task = solution.getTask();
        if (task.evaluationPossible()) {
            var evalResponse = evalSolutionUseCase.evaluateAndStore(
                new EvalOptions.Builder().setSolution(solution).build());
            solution = evalResponse.getSolution();
        }

        return solution;
    }
}
