package de.hsrm.sls.subato.tasks.imp;

import de.hsrm.sls.subato.shared.fs.FileContent;
import de.hsrm.sls.subato.shared.fs.TmpDirProvider;
import de.hsrm.sls.subato.solutions.evaluation.registry.EvaRegistry;
import de.hsrm.sls.subato.tasks.imp.log.LogImportUseCase;
import de.hsrm.sls.subato.tasks.imp.log.LogType;
import de.hsrm.sls.subato.tasks.lang.LanguageRepository;
import de.hsrm.sls.subato.tasks.pool.TaskPool;
import de.hsrm.sls.subato.tasks.pool.exception.TaskPoolNotFoundException;
import de.hsrm.sls.subato.tasks.pool.repository.TaskPoolRepository;
import de.hsrm.sls.subato.tasks.stef.cache.StefSpecCache;
import de.hsrm.sls.subato.tasks.stef.parser.InvalidSpecException;
import de.hsrm.sls.subato.tasks.stef.parser.SpecParser;
import de.hsrm.sls.subato.tasks.stef.spec.StefSpec;
import de.hsrm.sls.subato.tasks.task.Task;
import de.hsrm.sls.subato.tasks.task.repository.BundleRepository;
import de.hsrm.sls.subato.tasks.task.repository.TaskContentRepository;
import de.hsrm.sls.subato.tasks.task.repository.TaskRepository;
import de.hsrm.sls.subato.tasks.task.usecase.ArchiveTaskUseCase;
import de.hsrm.sls.subato.tasks.task.usecase.BuildBundleUseCase;
import java.io.File;
import java.util.ArrayList;
import java.util.stream.Collectors;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.FileSystemUtils;
import org.springframework.web.multipart.MultipartFile;

/***
 * Responsible for importing or updating a {@link Task}, represented by a {@link StefSpec}, into the database
 */
@Service
public class ImportTaskUseCase {

    private static Logger logger = LoggerFactory.getLogger(ImportTaskUseCase.class);

    @Autowired
    private TaskRepository taskRepository;

    @Autowired
    private SpecParser specService;

    @Autowired
    private TaskPoolRepository taskPoolRepository;

    @Autowired
    private LogImportUseCase logImportUseCase;

    @Autowired
    private TaskContentRepository taskContentRepository;

    @Autowired
    private TmpDirProvider tmpDirProvider;

    @Autowired
    private LanguageRepository languageRepository;

    @Autowired
    private ApplicationEventPublisher applicationEventPublisher;

    @Autowired
    private BuildBundleUseCase buildBundleUseCase;

    @Autowired
    private BundleRepository bundleRepository;

    @Autowired
    private ArchiveTaskUseCase archiveTaskUseCase;

    @Autowired
    private EvaRegistry evaRegistry;


    /**
     * checks whether there already is a task in the pool with the id given in the spec or not. if
     * not, a new task will be created and linked to the pool. in both cases the task will be
     * updated to reflect the metadata of the spec
     *
     * @param request target task pool, spec and directory of the task
     * @return created or updated task
     * @throws TaskImportException
     */
    private Task importTask(ImportTaskRequest request) throws TaskImportException {
        try {
            // update task metadata in db
            var task = taskRepository.findByTrackingIdAndPoolId(
                request.spec().getMeta().getTrackingId(),
                request.pool().getId());
            if (task == null) {
                task = new Task();
                task.setPool(request.pool());
                task.setTrackingId(request.spec().getMeta().getTrackingId());
                request.pool().getTasks().add(task);
            }

            task.setSoftId(request.spec().getId());
            task.setName(request.spec().getMeta().getName());
            task.setArchived(false);
            task.setSpecCache(
                new StefSpecCache(StefSpecCache.SERIALIZATION_VERSION, request.spec()));

            var stack = request.spec().getMeta().getEvaluator() != null ? evaRegistry.get(
                request.spec().getMeta().getEvaluator()) : null;
            task.setEvaluationStack(stack);

            var language =
                request.spec().getMeta().getLanguage() != null ? languageRepository.findById(
                    request.spec().getMeta().getLanguage()).get() : null;
            task.setLanguage(language);

            if (request.spec().getMeta().getLegacyId() != null) {
                var existingLegacyTask = taskRepository.findBySourceId(
                        request.spec().getMeta().getLegacyId())
                    .orElse(null);
                if (existingLegacyTask != null && task.getId() != existingLegacyTask.getId()) {
                    // prevent exception because of unique constraint on sourceId
                    logger.warn(
                        "Ignoring sourceId because there already is a task with sourceId=%s".formatted(
                            request.spec().getMeta().getLegacyId()));
                } else {
                    task.setSourceId(request.spec().getMeta().getLegacyId());
                }
            }

            task = taskRepository.save(task);

            // copy the tmp dir
            taskContentRepository.store(task, request.dir());

            var tmpBundleFile = buildBundleUseCase.buildBundle(task);
            if (tmpBundleFile != null) {
                var bundleFile = bundleRepository.storeBundle(task, tmpBundleFile);
                task.setBundle(bundleFile.getName());
            } else {
                task.setBundle(null);
            }

            task = taskRepository.save(task);

            logger.debug("Imported task %s".formatted(task.getSoftId()));
            return task;
        } catch (Exception ex) {
            logger.error("Failed to import task in directory %s".formatted(request.dir().getName()),
                ex);
            throw new TaskImportException(ex);
        }
    }

    /**
     * @see #importTask(ImportTaskRequest)
     */
    @Transactional
    public Task importTask(TaskPool pool, MultipartFile file) {
        var tmpPath = tmpDirProvider.createTmpPath();

        applicationEventPublisher.publishEvent(new PreTaskImportEvent(this, pool));

        try {
            var content = FileContent.fromMultipartFile(file);
            var dir = tmpPath.resolve(content.getFileName().split("\\.")[0]).toFile();
            content.saveTo(dir, true);

            var spec = specService.parseSpec(dir.toPath().toString());
            return importTask(new ImportTaskRequest(pool, spec, dir));
        } finally {
            // delete tmp dir
            FileSystemUtils.deleteRecursively(tmpPath.toFile());
        }
    }

    /**
     * Batch-imports multiple tasks at once from the given directory and deletes tasks which are in
     * the pool but not present in the root directory. Each task needs to be in its own folder
     * within the root directory.
     *
     * @see #importTask(ImportTaskRequest)
     */
    @Transactional
    public TaskImportResult importTasks(TaskPool pool, File rootDir) {
        if (!rootDir.exists()) {
            throw new TaskImportException(
                "Directory %s does not exist".formatted(rootDir.getAbsolutePath()));
        }

        var currentTasks = pool.getTasks().stream().filter(p -> !p.isArchived())
            .collect(Collectors.toList());
        var problems = 0L;

        applicationEventPublisher.publishEvent(new PreTaskImportEvent(this, pool));

        logger.debug("Importing tasks in %s for pool %s...".formatted(rootDir.getAbsolutePath(),
            pool.getName()));

        var collectedTasks = new ArrayList<ImportTaskRequest>();
        for (var dir : rootDir.listFiles()) {
            if (!dir.isDirectory() || dir.isHidden()) {
                continue;
            }

            try {
                var spec = specService.parseSpec(dir.toPath().toString());
                collectedTasks.add(new ImportTaskRequest(pool, spec, dir));
            } catch (InvalidSpecException ex) {
                logImportUseCase.log(pool, LogType.PROBLEM,
                    "%s: %s".formatted(dir.getName(), ex.getReason()));
                problems += 1;
            }
        }

        var groups = collectedTasks.stream().collect(Collectors.groupingBy(
            p -> p.spec().getMeta().getTrackingId()));
        for (var group : groups.entrySet()) {
            if (group.getValue().size() == 1) {
                continue;
            }

            var taskDirs = group.getValue().stream().map(r -> r.dir().getName())
                .collect(Collectors.joining(", "));
            logImportUseCase.log(pool, LogType.PROBLEM,
                "Die ID %s wird von mehr als einer Aufgabe verwendet: %s.".formatted(
                    group.getKey(), taskDirs));
            problems += 1;
            collectedTasks.removeAll(group.getValue());
        }

        var importedTasks = new ArrayList<Task>();
        for (var request : collectedTasks) {
            try {
                var task = importTask(request);

                var relativePath = rootDir.toPath().relativize(request.dir().toPath());
                task.setRepositoryPath(relativePath.toString());
                task = taskRepository.save(task);

                importedTasks.add(task);
            } catch (TaskImportException ex) {
                logImportUseCase.log(pool, LogType.PROBLEM,
                    "%s: Konnte nicht importiert werden".formatted(request.dir().getName()));
                problems += 1;
            }
        }

        var missingTasks = currentTasks.stream()
            .filter(currentTask -> !importedTasks.stream()
                .anyMatch(importedTask -> importedTask.getId() == currentTask.getId()))
            .collect(Collectors.toList());

        for (var missingTask : missingTasks) {
            archiveTaskUseCase.archive(missingTask);
            logger.debug("Archived task %s".formatted(missingTask.getSoftId()));
            logImportUseCase.log(pool, LogType.INFO,
                "Aufgabe %s archiviert".formatted(missingTask.getSoftId()));
        }

        logImportUseCase.log(pool, LogType.INFO,
            "Import durchgeführt. %s Aufgaben importiert/aktualisiert, %s Fehler, %s Aufgaben archiviert".formatted(
                importedTasks.size(), problems, missingTasks.size()));
        logger.info(
            "Imported %s tasks for pool %s, %s problems occured and %s tasks have been archived.".formatted(
                importedTasks.size(), pool.getName(), problems, missingTasks.size()));
        return new TaskImportResult(collectedTasks.size(), importedTasks.size(),
            missingTasks.size(),
            problems);
    }

    /**
     * @see #importTasks(TaskPool, File)
     */
    @Transactional
    public TaskImportResult importTasks(long poolId, File rootDir) {
        var pool = taskPoolRepository.findById(poolId)
            .orElseThrow(() -> new TaskPoolNotFoundException(poolId));
        return importTasks(pool, rootDir);
    }
}
