package de.hsrm.sls.subato.tasks.lang;

import de.hsrm.sls.subato.shared.swagger.SubatoApiResponse;
import de.hsrm.sls.subato.shared.validation.ValidationException;
import de.hsrm.sls.subato.tasks.lang.dto.CreateLanguageDto;
import de.hsrm.sls.subato.tasks.lang.dto.LanguageDto;
import de.hsrm.sls.subato.tasks.lang.dto.LanguageMapper;
import de.hsrm.sls.subato.tasks.lang.usecase.DeleteLanguageUseCase;
import de.hsrm.sls.subato.tasks.lang.usecase.UpdateLanguageUseCase;
import io.swagger.v3.oas.annotations.tags.Tag;
import java.util.List;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

@RestController
@Tag(name = "Aufgabenverwaltung")
public class LanguageController {

    @Autowired
    private UpdateLanguageUseCase updateLanguageUseCase;

    @Autowired
    private DeleteLanguageUseCase deleteLanguageUseCase;

    @Autowired
    private LanguageRepository languageRepository;

    @Autowired
    private LanguageMapper languageMapper;

    @GetMapping("/language/{id}")
    @SubatoApiResponse(LanguageNotFoundException.class)
    LanguageDto getOne(@PathVariable String id) {
        var language = languageRepository.findById(id)
            .orElseThrow(() -> new LanguageNotFoundException(id));
        return languageMapper.map(language);
    }

    @GetMapping("/language")
    List<LanguageDto> getAll() {
        var languages = languageRepository.findAll();
        return languages.stream().map(lang -> languageMapper.mapWithCount(lang))
            .collect(Collectors.toList());
    }

    @PostMapping("/language")
    @PreAuthorize("hasPermission(null, @TasksAction.LANGUAGE_CREATE)")
    @SubatoApiResponse(ValidationException.class)
    LanguageDto create(@RequestBody CreateLanguageDto dto) {
        var language = updateLanguageUseCase.create(dto);
        return languageMapper.map(language);
    }

    @PutMapping("/language/{id}")
    @PreAuthorize("hasPermission(null, @TasksAction.LANGUAGE_UPDATE)")
    @SubatoApiResponse(ValidationException.class)
    @SubatoApiResponse(LanguageNotFoundException.class)
    LanguageDto update(@PathVariable String id, @RequestBody CreateLanguageDto dto) {
        var language = updateLanguageUseCase.update(id, dto);
        return languageMapper.map(language);
    }

    @DeleteMapping("/language/{id}")
    @PreAuthorize("hasPermission(null, @TasksAction.LANGUAGE_DELETE)")
    @SubatoApiResponse(LanguageNotFoundException.class)
    void delete(@PathVariable String id) {
        deleteLanguageUseCase.delete(id);
    }
}
