package de.hsrm.sls.subato.tasks.pool;

import de.hsrm.sls.subato.courses.course.dto.CourseDto;
import de.hsrm.sls.subato.courses.course.dto.CourseMapper;
import de.hsrm.sls.subato.shared.swagger.SubatoApiResponse;
import de.hsrm.sls.subato.shared.validation.ValidationException;
import de.hsrm.sls.subato.tasks.imp.ImportTaskUseCase;
import de.hsrm.sls.subato.tasks.imp.TaskImportResult;
import de.hsrm.sls.subato.tasks.imp.log.TaskImportLogDto;
import de.hsrm.sls.subato.tasks.imp.log.TaskImportLogMapper;
import de.hsrm.sls.subato.tasks.pool.dto.CreateTaskPoolDto;
import de.hsrm.sls.subato.tasks.pool.dto.CreateTaskPoolResultDto;
import de.hsrm.sls.subato.tasks.pool.dto.TaskPoolDto;
import de.hsrm.sls.subato.tasks.pool.dto.TaskPoolMapper;
import de.hsrm.sls.subato.tasks.pool.exception.TaskPoolNotFoundException;
import de.hsrm.sls.subato.tasks.pool.exception.TaskPoolReadOnlyException;
import de.hsrm.sls.subato.tasks.pool.repository.TaskPoolRepository;
import de.hsrm.sls.subato.tasks.pool.sync.SyncTaskPoolUseCase;
import de.hsrm.sls.subato.tasks.pool.sync.TaskPoolSyncException;
import de.hsrm.sls.subato.tasks.pool.sync.repository.NotLinkedToRepositoryException;
import de.hsrm.sls.subato.tasks.pool.sync.repository.RepositoryDoesNotExistException;
import de.hsrm.sls.subato.tasks.pool.usecase.DeleteTaskPoolUseCase;
import de.hsrm.sls.subato.tasks.pool.usecase.UpdateTaskPoolUseCase;
import de.hsrm.sls.subato.tasks.stef.parser.InvalidSpecException;
import de.hsrm.sls.subato.tasks.task.dto.TaskDto;
import de.hsrm.sls.subato.tasks.task.dto.TaskMapper;
import de.hsrm.sls.subato.tasks.task.repository.TaskFilter;
import de.hsrm.sls.subato.tasks.task.repository.TaskSort;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.web.PageableDefault;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RequestPart;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

@RestController
@Tag(name = "Aufgabenverwaltung")
public class TaskPoolController {

    @Autowired
    private TaskPoolMapper taskPoolMapper;

    @Autowired
    private TaskMapper taskMapper;

    @Autowired
    private CourseMapper courseMapper;

    @Autowired
    private ImportTaskUseCase importTaskUseCase;

    @Autowired
    private TaskPoolRepository taskPoolRepository;

    @Autowired
    private SyncTaskPoolUseCase syncTaskPoolUseCase;

    @Autowired
    private TaskImportLogMapper taskImportLogMapper;

    @Autowired
    private DeleteTaskPoolUseCase deleteTaskPoolUseCase;

    @Autowired
    private UpdateTaskPoolUseCase updateTaskPoolUseCase;

    @GetMapping("/task_pool/{id}")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_POOL_VIEW)")
    @SubatoApiResponse(TaskPoolNotFoundException.class)
    TaskPoolDto getOne(@PathVariable Long id) {
        var pool = taskPoolRepository.findById(id)
            .orElseThrow(() -> new TaskPoolNotFoundException(id));
        return taskPoolMapper.map(pool);
    }

    @GetMapping("/task_pool/{id}/tasks")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_POOL_VIEW)")
    @Operation(summary = "Liefert eine Übersicht aller unarchivierten Aufgaben aus dem Aufgaben-Pool")
    @SubatoApiResponse(TaskPoolNotFoundException.class)
    Page<TaskDto> getTasks(@PageableDefault Pageable page, @PathVariable Long id,
        @RequestParam("q") Optional<String> query, @RequestParam Optional<Boolean> showArchived) {
        var updatedPage = PageRequest.of(page.getPageNumber(), page.getPageSize(),
            page.getSortOr(TaskSort.Native.byId()));

        var filter = new TaskFilter.Builder()
            .poolId(id)
            .query(query.orElse(null))
            .setShowArchived(showArchived.orElse(null))
            .build();
        var tasks = taskPoolRepository.getTasks(updatedPage, filter);
        return tasks.map(c -> taskMapper.map(c));
    }

    @GetMapping("/task_pool/{id}/courses")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_POOL_VIEW)")
    @Operation(summary = "Liefert eine Übersicht aller Kurse, die mindestens eine Aufgabe aus dem angegebenen Aufgaben-Pool in einem Übungsblatt verwendet haben")
    @SubatoApiResponse(TaskPoolNotFoundException.class)
    List<CourseDto> getCourses(@PathVariable Long id) {
        var pool = taskPoolRepository.findById(id)
            .orElseThrow(() -> new TaskPoolNotFoundException(id));
        var courses = taskPoolRepository.getCourses(pool);
        return courses.stream().map(c -> courseMapper.mapDetail(c)).toList();
    }

    @GetMapping("/task_pool/{id}/logs")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_POOL_VIEW)")
    @Operation(summary = "Liefert Logs, die beim Import entstanden sind.")
    @SubatoApiResponse(TaskPoolNotFoundException.class)
    List<TaskImportLogDto> getLogs(@PathVariable Long id) {
        var pool = taskPoolRepository.findById(id)
            .orElseThrow(() -> new TaskPoolNotFoundException(id));
        var logs = taskPoolRepository.getLogs(pool);
        return logs.stream().map(c -> taskImportLogMapper.map(c)).toList();
    }

    @GetMapping("/task_pool")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_POOL_VIEW)")
    List<TaskPoolDto> getAll() {
        var pools = taskPoolRepository.findAll();
        return pools.stream().map(task -> taskPoolMapper.map(task)).collect(Collectors.toList());
    }

    @PutMapping("/task_pool/{id}/tasks")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_POOL_UPDATE)")
    @Operation(
        summary = "Importiert eine Aufgabe aus der übermittelten STEF-Spezifikation (als zip) in den Aufgaben-Pool bzw. aktualisiert sie",
        description = "Ob eine Aufgabe bereits existiert, wird anhand des id-Attributs in meta.xml festgestellt"
    )
    @SubatoApiResponse(InvalidSpecException.class)
    @SubatoApiResponse(TaskPoolReadOnlyException.class)
    @SubatoApiResponse(TaskPoolNotFoundException.class)
    TaskDto importTask(@PathVariable Long id, @RequestPart("file") MultipartFile file) {
        var pool = taskPoolRepository.findById(id)
            .orElseThrow(() -> new TaskPoolNotFoundException(id));
        if (pool.isLinkedToRepository()) {
            throw new TaskPoolReadOnlyException();
        }

        var task = importTaskUseCase.importTask(pool, file);
        return taskMapper.map(task);
    }

    @PostMapping("/task_pool")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_POOL_CREATE)")
    @SubatoApiResponse(RepositoryDoesNotExistException.class)
    @SubatoApiResponse(ValidationException.class)
    CreateTaskPoolResultDto create(@RequestBody CreateTaskPoolDto dto) {
        TaskPool pool = updateTaskPoolUseCase.create(dto);

        if (pool.isLinkedToRepository()) {
            try {
                var result = syncTaskPoolUseCase.sync(pool, false);
                return new CreateTaskPoolResultDto(taskPoolMapper.map(pool), false, result);
            } catch (TaskPoolSyncException e) {
                return new CreateTaskPoolResultDto(taskPoolMapper.map(pool), true);
            }
        }

        return new CreateTaskPoolResultDto(taskPoolMapper.map(pool), false);
    }

    @PutMapping("/task_pool/{id}")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_POOL_UPDATE)")
    @SubatoApiResponse(TaskPoolNotFoundException.class)
    @SubatoApiResponse(RepositoryDoesNotExistException.class)
    @SubatoApiResponse(ValidationException.class)
    TaskPoolDto update(@PathVariable Long id, @RequestBody CreateTaskPoolDto dto) {
        var taskPool = updateTaskPoolUseCase.update(id, dto);
        return taskPoolMapper.map(taskPool);
    }

    @DeleteMapping("/task_pool/{id}")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_POOL_DELETE)")
    @Operation(summary = "Löscht einen Aufgaben-Pool mit allen Aufgaben, verknüpfte Lösungen etc.")
    @SubatoApiResponse(TaskPoolNotFoundException.class)
    void delete(@PathVariable Long id) {
        deleteTaskPoolUseCase.delete(id);
    }

    @PreAuthorize("hasPermission(null, @TasksAction.TASK_POOL_UPDATE)")
    @PutMapping("/task_pool/{id}/sync")
    @SubatoApiResponse(TaskPoolNotFoundException.class)
    @SubatoApiResponse(NotLinkedToRepositoryException.class)
    @SubatoApiResponse(RepositoryDoesNotExistException.class)
    TaskImportResult syncPool(@PathVariable Long id) {
        var pool = taskPoolRepository.findById(id)
            .orElseThrow(() -> new TaskPoolNotFoundException(id));
        return syncTaskPoolUseCase.sync(pool, true);
    }
}
