package de.hsrm.sls.subato.tasks.pool.sync;


import de.hsrm.sls.subato.tasks.imp.ImportTaskUseCase;
import de.hsrm.sls.subato.tasks.imp.TaskImportResult;
import de.hsrm.sls.subato.tasks.pool.TaskPool;
import de.hsrm.sls.subato.tasks.pool.repository.TaskPoolRepository;
import de.hsrm.sls.subato.tasks.pool.sync.repository.NotLinkedToRepositoryException;
import de.hsrm.sls.subato.tasks.pool.sync.repository.TaskPoolRepositoryManager;
import java.time.LocalDateTime;
import org.eclipse.jgit.api.Git;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
public class SyncTaskPoolUseCase {

    private static Logger logger = LoggerFactory.getLogger(SyncTaskPoolUseCase.class);

    @Autowired
    ImportTaskUseCase importTaskUseCase;

    @Autowired
    TaskPoolRepository taskPoolRepository;

    @Autowired
    TaskPoolRepositoryManager repositoryManager;

    /**
     * checks if the repository for the given task pool has changed and if it has,
     * reimports/archives the tasks in the repository
     *
     * @param pool        pool to sync
     * @param forceUpdate if true, the sync will be run - even if the repository did not change
     * @return null if no sync was performed (because there were no changes), otherwise the result
     * of the sync
     */
    @Transactional
    public TaskImportResult sync(TaskPool pool, boolean forceUpdate) {
        if (!pool.isLinkedToRepository()) {
            throw new NotLinkedToRepositoryException();
        }

        try {
            Git repo;
            var repositoryChanged = false;
            if (!repositoryManager.repositoryExists(pool)) {
                repo = repositoryManager.cloneRepository(pool);
                repositoryChanged = true;
            } else {
                repo = repositoryManager.getRepository(pool);
                repositoryChanged = repositoryManager.refreshRepository(repo);
            }

            if (pool.getLastChecked() == null) {
                repositoryChanged = true;
            }

            var now = LocalDateTime.now();
            pool.setLastChecked(now);
            pool.setRepositoryBranch(repo.getRepository().getBranch());
            pool = taskPoolRepository.save(pool);

            if (repositoryChanged || forceUpdate) {
                var result = importTaskUseCase.importTasks(pool,
                    repo.getRepository().getDirectory().getParentFile());

                pool.setLastUpdated(now);
                pool = taskPoolRepository.save(pool);
                return result;
            }

            return null;
        } catch (Exception ex) {
            throw new TaskPoolSyncException(ex);
        }
    }

    public void syncAll(boolean forceUpdate) {
        logger.info("Syncing all task pools...");

        try {
            var pools = taskPoolRepository.getAllWithRepository();
            for (var pool : pools) {
                sync(pool, forceUpdate);
            }

            logger.info("Finished sync.");
        } catch (Exception e) {
            logger.error("Sync failed", e);
        }
    }
}
