package de.hsrm.sls.subato.tasks.pool.sync.repository;

import de.hsrm.sls.subato.shared.fs.TmpDirProvider;
import de.hsrm.sls.subato.tasks.pool.TaskPool;
import java.io.File;
import java.io.IOException;
import org.apache.commons.io.FileUtils;
import org.eclipse.jgit.api.Git;
import org.eclipse.jgit.api.errors.GitAPIException;
import org.eclipse.jgit.api.errors.InvalidRemoteException;
import org.eclipse.jgit.storage.file.FileRepositoryBuilder;
import org.eclipse.jgit.transport.CredentialsProvider;
import org.eclipse.jgit.transport.UsernamePasswordCredentialsProvider;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * Responsible for managing the state of git repositories for task pools
 */
@Service
public class TaskPoolRepositoryManager {

    @Autowired
    private GitConfig gitConfig;

    @Autowired
    TmpDirProvider tmpDirProvider;

    /**
     * pulls the given repository and returns if there were changes since the last pull
     */
    public boolean refreshRepository(Git git) {
        try {
            var credentials = getCredentials();

            var oldHead = git.getRepository().resolve("HEAD^{tree}");
            git.pull().setCredentialsProvider(credentials).call();
            var newHead = git.getRepository().resolve("HEAD^{tree}");

            var hasChanged = !oldHead.equals(newHead);
            return hasChanged;
        } catch (GitAPIException ex) {
            throw new RepositoryException(ex);
        } catch (IOException e) {
            throw new RepositoryException(e);
        }
    }

    public boolean repositoryExists(TaskPool pool) {
        if (!pool.isLinkedToRepository()) {
            throw new NotLinkedToRepositoryException();
        }

        var directory = getCloneDirectory(pool);

        FileRepositoryBuilder repositoryBuilder = new FileRepositoryBuilder();
        repositoryBuilder.findGitDir(directory);
        return repositoryBuilder.getGitDir() != null;
    }

    public Git cloneRepository(TaskPool pool, File cloneDir) {
        if (!pool.isLinkedToRepository()) {
            throw new NotLinkedToRepositoryException();
        }

        var credentials = getCredentials();

        try {
            var git = Git.cloneRepository()
                .setURI(pool.getRepositoryUrl())
                .setDirectory(cloneDir)
                .setCredentialsProvider(credentials)
                .setBranch(pool.getRepositoryBranch())
                .call();
            return git;
        } catch (InvalidRemoteException e) {
            throw new RepositoryDoesNotExistException(pool.getRepositoryUrl());
        } catch (GitAPIException e) {
            throw new RepositoryException(e);
        }
    }

    public Git cloneRepository(TaskPool pool) {
        return cloneRepository(pool, getCloneDirectory(pool));
    }

    /**
     * checks if the repository for the given pool can be cloned by cloning into a temp directory
     */
    public void testRepository(TaskPool pool) {
        var cloneDir = tmpDirProvider.createTmpPath().toFile();
        cloneRepository(pool, cloneDir);

        try {
            FileUtils.deleteDirectory(cloneDir);
        } catch (IOException e) {
            throw new RepositoryException(e);
        }
    }

    public void removeRepository(TaskPool pool) {
        var cloneDir = getCloneDirectory(pool);
        if (cloneDir.exists()) {
            try {
                FileUtils.deleteDirectory(cloneDir);
            } catch (IOException ex) {
                throw new RuntimeException(ex);
            }
        }
    }


    private File getCloneDirectory(TaskPool pool) {
        return tmpDirProvider.createTmpPath(String.valueOf(pool.getId())).toFile();
    }

    public Git getRepository(TaskPool pool) {
        try {
            return Git.open(getCloneDirectory(pool));
        } catch (IOException e) {
            throw new RepositoryException(e);
        }
    }

    private CredentialsProvider getCredentials() {
        return new UsernamePasswordCredentialsProvider(
            this.gitConfig.getUser(),
            this.gitConfig.getToken() != null ? this.gitConfig.getToken() : ""
        );
    }
}
