package de.hsrm.sls.subato.tasks.pool.usecase;

import de.hsrm.sls.subato.shared.validation.ConstraintViolationMapper;
import de.hsrm.sls.subato.shared.validation.ValidationException;
import de.hsrm.sls.subato.tasks.pool.TaskPool;
import de.hsrm.sls.subato.tasks.pool.dto.CreateTaskPoolDto;
import de.hsrm.sls.subato.tasks.pool.exception.TaskPoolNotFoundException;
import de.hsrm.sls.subato.tasks.pool.repository.TaskPoolRepository;
import de.hsrm.sls.subato.tasks.pool.sync.repository.TaskPoolRepositoryManager;
import de.hsrm.sls.subato.tasks.task.repository.TaskRepository;
import jakarta.validation.ConstraintViolationException;
import jakarta.validation.Validation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

@Component
public class UpdateTaskPoolUseCase {

    @Autowired
    private TaskPoolRepository taskPoolRepository;

    @Autowired
    private TaskRepository taskRepository;

    @Autowired
    private TaskPoolRepositoryManager taskPoolRepositoryManager;

    /**
     * @param dto attributes for the new pool
     * @return created pool
     * @throws ConstraintViolationException
     */
    @Transactional
    public TaskPool create(CreateTaskPoolDto dto) {
        var taskPool = new TaskPool();
        updateTaskPool(taskPool, dto);
        validateTaskPool(taskPool);

        var pool = taskPoolRepository.save(taskPool);

        if (taskPool.isLinkedToRepository()) {
            taskPoolRepositoryManager.testRepository(pool);
        }

        return pool;
    }

    /**
     * @param id  pool to be changed
     * @param dto changes
     * @return changed pool
     * @throws ConstraintViolationException
     * @throws TaskPoolNotFoundException
     */
    public TaskPool update(long id, CreateTaskPoolDto dto) {
        var taskPool = taskPoolRepository.findById(id)
            .orElseThrow(() -> new TaskPoolNotFoundException(id));
        var repositoryChanged = hasRepositoryChanged(taskPool, dto);

        updateTaskPool(taskPool, dto);
        validateTaskPool(taskPool);

        if (taskPool.isLinkedToRepository()) {
            taskPoolRepositoryManager.testRepository(taskPool);
        }

        if (repositoryChanged) {
            taskPoolRepositoryManager.removeRepository(taskPool);
        }

        return taskPoolRepository.save(taskPool);
    }

    private boolean hasRepositoryChanged(TaskPool pool, CreateTaskPoolDto dto) {
        if (!pool.isLinkedToRepository()) {
            return false;
        }

        var oldRepositoryUrl = pool.getRepositoryUrl();
        var oldRepositoryBranch = pool.getRepositoryBranch();

        var urlChanged = !oldRepositoryUrl.equals(pool.getRepositoryUrl());
        var branchChanged = !StringUtils.equals(oldRepositoryBranch, dto.getRepositoryBranch());
        return urlChanged || branchChanged;
    }


    private void updateTaskPool(TaskPool pool, CreateTaskPoolDto dto) {
        pool.setDescription(dto.getDescription());
        pool.setName(dto.getName());
        pool.setRepositoryUrl(dto.getRepositoryUrl());
        pool.setRepositoryBranch(dto.getRepositoryBranch());
    }

    /**
     * @throws ConstraintViolationException if invalid
     */
    private void validateTaskPool(TaskPool pool) {
        var validator = Validation.buildDefaultValidatorFactory().getValidator();
        var violations = validator.validate(pool);
        if (!violations.isEmpty()) {
            throw new ValidationException(ConstraintViolationMapper.map(violations));
        }
    }
}
