package de.hsrm.sls.subato.tasks.stef.parser.meta;

import de.hsrm.sls.subato.shared.helper.UUIDHelper;
import de.hsrm.sls.subato.tasks.stef.parser.InvalidSpecException;
import de.hsrm.sls.subato.tasks.stef.parser.meta.xml.TaskXml;
import de.hsrm.sls.subato.tasks.stef.spec.Meta;
import jakarta.xml.bind.JAXBContext;
import jakarta.xml.bind.JAXBException;
import jakarta.xml.bind.Unmarshaller;
import java.nio.file.Path;
import javax.xml.XMLConstants;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.SchemaFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ResourceLoader;
import org.springframework.stereotype.Service;

/**
 * Responsible for parsing and resolving the meta.xml file
 */
@Service
public class MetaParser {

    @Autowired
    private LanguageResolver languageResolver;

    @Autowired
    private SubmissionModeResolver submissionModeResolver;

    @Autowired
    private EvaluatorResolver evaluatorResolver;

    @Autowired
    private EvaluatorParamResolver evaluatorParamResolver;

    @Autowired
    private FileRefResolver fileRefResolver;

    @Autowired
    private MimeMappingResolver mimeMappingResolver;

    @Autowired
    private CompetenceResolver competenceResolver;

    @Autowired
    private ResourceLoader resourceLoader;

    public Meta parse(String fileRoot) {
        var meta = new Meta();

        var doc = parseXmlDocument(fileRoot);

        if (!UUIDHelper.isValidUUID(doc.getTrackingId())) {
            throw new InvalidSpecException(
                "Das id-Attribut in meta.xml ist keine gültige UUID. Ab sofort müssen IDs über den ID-Generator (siehe oben) erzeugt werden. Die Ordner dürfen nach wie vor beliebig benannt werden");
        }

        meta.setTrackingId(doc.getTrackingId());
        meta.setName(doc.getName());
        meta.setAttempts(doc.getAttempts());
        meta.setDescription(doc.getDescription());
        meta.setLegacyId(doc.getSubatoId());

        var evaluator = evaluatorResolver.resolve(doc);
        meta.setEvaluator(evaluator != null ? evaluator.getRef() : null);

        var evaluatorParams = evaluatorParamResolver.resolve(doc);
        meta.setEvaluatorParams(evaluatorParams);

        var language = languageResolver.resolve(doc);
        meta.setLanguage(language != null ? language.getId() : null);

        var mimeMappings = mimeMappingResolver.resolve(doc);
        meta.setMimeMappings(mimeMappings);

        var fileRefs = fileRefResolver.resolve(doc, mimeMappings, fileRoot);
        meta.setFiles(fileRefs);

        var submissionModeConfig = submissionModeResolver.resolve(doc, meta.getTemplateFiles());
        meta.setSubmissionModeConfig(submissionModeConfig);

        var competences = competenceResolver.resolve(doc);
        meta.setCompetences(competences);

        return meta;
    }

    private TaskXml parseXmlDocument(String fileRoot) {
        var metaFile = Path.of(fileRoot).resolve("meta.xml").toFile();
        if (!metaFile.exists()) {
            throw new InvalidSpecException(
                "Die Datei meta.xml existiert nicht im Wurzelverzeichnis, ist jedoch erforderlich");
        }

        var parser = setupParser();

        try {
            var task = (TaskXml) parser.unmarshaller.unmarshal(metaFile);

            if (parser.handler.getValidationMessages().size() != 0) {
                var first = parser.handler.getValidationMessages().stream().findFirst().get();
                throw new InvalidSpecException(
                    "Die Datei meta.xml ist ungültig: %s".formatted(first));
            }

            return task;
        } catch (JAXBException e) {
            var msg = e.getMessage() != null ? e.getMessage()
                : e.getLinkedException() != null ? e.getLinkedException().getMessage() : "?";
            throw new InvalidSpecException(
                "Die Datei meta.xml konnte nicht geparsed werden: %s".formatted(msg));
        }
    }

    private ParserSetup setupParser() {
        try {
            JAXBContext jaxbContext = JAXBContext.newInstance(TaskXml.class);
            Unmarshaller jaxbUnmarshaller = jaxbContext.createUnmarshaller();

            try (var xsdStream = resourceLoader.getResource("classpath:stef.xsd")
                .getInputStream()) {
                // link XSD to the input document
                var schemaFactory = SchemaFactory.newInstance(
                    XMLConstants.W3C_XML_SCHEMA_NS_URI);
                var schema = schemaFactory.newSchema(new StreamSource(xsdStream));
                jaxbUnmarshaller.setSchema(schema);

                // catch xsd validation failures
                var handler = new MetaValidationEventHandler();
                jaxbUnmarshaller.setEventHandler(handler);

                return new ParserSetup(jaxbUnmarshaller, handler);
            }
        } catch (Exception ex) {
            throw new RuntimeException("Failed to setup jaxb parser", ex);
        }
    }

    private record ParserSetup(Unmarshaller unmarshaller, MetaValidationEventHandler handler) {

    }
}
