package de.hsrm.sls.subato.tasks.stef.parser.meta;

import de.hsrm.sls.subato.tasks.stef.parser.InvalidSpecException;
import de.hsrm.sls.subato.tasks.stef.parser.meta.xml.SubmissionModeXml;
import de.hsrm.sls.subato.tasks.stef.parser.meta.xml.SubmissionModesXml;
import de.hsrm.sls.subato.tasks.stef.parser.meta.xml.TaskXml;
import de.hsrm.sls.subato.tasks.stef.spec.FileRef;
import de.hsrm.sls.subato.tasks.stef.spec.FileSubmissionMode;
import de.hsrm.sls.subato.tasks.stef.spec.SubmissionMode;
import de.hsrm.sls.subato.tasks.stef.spec.SubmissionModeConfig;
import de.hsrm.sls.subato.tasks.stef.spec.SubmissionModeType;
import de.hsrm.sls.subato.tasks.stef.spec.TextSubmissionMode;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

@Service
class SubmissionModeResolver {

    public SubmissionModeConfig resolve(TaskXml doc, List<FileRef> templateFiles) {
        SubmissionModeConfig config;

        if (doc.getSubmissionModes() != null) {
            var modes = new ArrayList<SubmissionMode>();
            for (var xmlSubmissionMode : doc.getSubmissionModes().getSubmissionModes()) {
                var submissionMode = extractSubmissionMode(doc, templateFiles, xmlSubmissionMode);
                modes.add(submissionMode);
            }

            SubmissionModeType defaultMode = extractDefaultMode(doc, modes);
            if (defaultMode == null) {
                defaultMode = getDefaultMode(modes);
            }

            config = new SubmissionModeConfig(modes, defaultMode);
        } else {
            config = configureDefaultSubmissionModes(doc, templateFiles);
        }

        addDefaultFileExtensionIfRequired(config, templateFiles);
        return config;
    }

    private SubmissionModeConfig configureDefaultSubmissionModes(TaskXml doc,
        List<FileRef> templateFiles) {
        var modes = new ArrayList<SubmissionMode>();

        var fileSubmissionMode = new FileSubmissionMode();
        fileSubmissionMode.setAllowArchives(getAllowArchivesDefaultValue());
        fileSubmissionMode.setExtractArchives(getExtractArchivesValue(doc));
        fileSubmissionMode.setExtensions(getDefaultExtensions(templateFiles));
        modes.add(fileSubmissionMode);

        var templateFilesCount = templateFiles.size();
        if (templateFilesCount == 1) {
            modes.add(new TextSubmissionMode());
        }

        var config = new SubmissionModeConfig(modes, getDefaultMode(modes));
        return config;
    }

    private SubmissionMode extractSubmissionMode(TaskXml doc, List<FileRef> templateFiles,
        SubmissionModeXml xmlSubmissionMode) {

        SubmissionModeType type;
        try {
            type = SubmissionModeType.valueOf(xmlSubmissionMode.getName());
        } catch (IllegalArgumentException ex) {
            throw new InvalidSpecException("SubmissionMode %s existiert nicht im System".formatted(
                xmlSubmissionMode.getName()));
        }

        switch (type) {
            case FILE: {
                var submissionMode = new FileSubmissionMode();

                var allowArchives =
                    xmlSubmissionMode.getArchives() != null ? xmlSubmissionMode.getArchives()
                        : getAllowArchivesDefaultValue();
                submissionMode.setAllowArchives(allowArchives);

                var extractArchives =
                    xmlSubmissionMode.getExtract() != null ? xmlSubmissionMode.getExtract()
                        : getExtractArchivesValue(doc);
                if (extractArchives && !allowArchives) {
                    throw new InvalidSpecException(
                        "%s kann nicht true sein wenn %s false ist".formatted(
                            SubmissionModeXml.EXTRACT, SubmissionModeXml.ARCHIVES));
                }
                submissionMode.setExtractArchives(extractArchives);

                var extensionsString = xmlSubmissionMode.getExtensions();
                if (StringUtils.isNotBlank(extensionsString)) {
                    var extensions = Arrays.stream(extensionsString.split(";")).toList();
                    submissionMode.setExtensions(extensions);
                } else {
                    submissionMode.setExtensions(getDefaultExtensions(templateFiles));
                }

                return submissionMode;
            }
            case TEXT: {
                var extensionsString = xmlSubmissionMode.getExtensions();
                if (StringUtils.isNotBlank(extensionsString)) {
                    throw new InvalidSpecException(
                        "Angabe von %s für %s %s nicht möglich".formatted(
                            SubmissionModeXml.EXTENSIONS, SubmissionModesXml.SUBMISSION_MODE,
                            SubmissionModeType.TEXT));
                }

                if (xmlSubmissionMode.getArchives() != null) {
                    throw new InvalidSpecException(
                        "Angabe von %s für %s %s nicht möglich".formatted(
                            SubmissionModeXml.ARCHIVES, SubmissionModesXml.SUBMISSION_MODE,
                            SubmissionModeType.TEXT));
                }

                if (xmlSubmissionMode.getExtract() != null) {
                    throw new InvalidSpecException(
                        "Angabe von %s für %s %s nicht möglich".formatted(SubmissionModeXml.EXTRACT,
                            SubmissionModesXml.SUBMISSION_MODE, SubmissionModeType.TEXT));
                }

                return new TextSubmissionMode();
            }
            default: {
                throw new RuntimeException("submissionMode %s not handled".formatted(type));
            }
        }
    }

    private SubmissionModeType extractDefaultMode(TaskXml doc,
        List<SubmissionMode> submissionModes) {
        SubmissionModeType defaultMode = null;
        if (doc.getSubmissionModes().getDefaultMode() != null) {
            try {
                defaultMode = SubmissionModeType.valueOf(doc.getSubmissionModes().getDefaultMode());

                var isDefaultModeConfigured = submissionModes.stream().map(m -> m.getType())
                    .toList().contains(defaultMode);
                if (!isDefaultModeConfigured) {
                    throw new InvalidSpecException(
                        "SubmissionMode %s kann nicht als default verwendet werden, da er nicht konfiguriert wurde".formatted(
                            doc.getSubmissionModes().getDefaultMode()));
                }
            } catch (IllegalArgumentException ex) {
                throw new InvalidSpecException(
                    "SubmissionMode %s existiert nicht im System".formatted(
                        doc.getSubmissionModes().getDefaultMode()));
            }
        }
        return defaultMode;
    }

    private SubmissionModeType getDefaultMode(List<SubmissionMode> modes) {
        return modes.stream().map(m -> m.getType()).toList().contains(SubmissionModeType.TEXT)
            ? SubmissionModeType.TEXT
            : SubmissionModeType.FILE;
    }

    private boolean getAllowArchivesDefaultValue() {
        return true;
    }

    private boolean getExtractArchivesValue(TaskXml doc) {
        return StringUtils.isNotBlank(doc.getEvaluator());
    }

    private List<String> getDefaultExtensions(List<FileRef> templateFiles) {
        var extensions = new ArrayList<String>();

        var templateExtensions = templateFiles
            .stream()
            .map(f -> f.getFile().getExtension())
            .collect(Collectors.toList());

        extensions.addAll(templateExtensions);
        return extensions;
    }

    private void addDefaultFileExtensionIfRequired(SubmissionModeConfig submissionModeConfig,
        List<FileRef> templateFiles) {
        var fileSubmissionMode = submissionModeConfig.getFileSubmissionMode();
        var textSubmissionMode = submissionModeConfig.getTextSubmissionMode();
        var templateFilesCount = templateFiles.size();

        if (submissionModeConfig != null && textSubmissionMode != null && templateFilesCount == 0) {
            if (!fileSubmissionMode.getExtensions().contains(FileRef.DEFAULT_FILE_EXT)) {
                fileSubmissionMode.getExtensions().add(FileRef.DEFAULT_FILE_EXT);
            }
        }
    }
}
