package de.hsrm.sls.subato.tasks.task;

import static de.hsrm.sls.subato.shared.helper.UriHelper.concat;

import de.hsrm.sls.subato.exercises.taskinstance.TaskInstance;
import de.hsrm.sls.subato.solutions.evaluation.registry.EvaluationStack;
import de.hsrm.sls.subato.solutions.solution.model.Solution;
import de.hsrm.sls.subato.tasks.lang.Language;
import de.hsrm.sls.subato.tasks.pool.TaskPool;
import de.hsrm.sls.subato.tasks.stef.cache.StefSpecCache;
import de.hsrm.sls.subato.tasks.stef.spec.FileRef;
import de.hsrm.sls.subato.tasks.stef.spec.FileSubmissionMode;
import de.hsrm.sls.subato.tasks.stef.spec.FileType;
import de.hsrm.sls.subato.tasks.stef.spec.StefSpec;
import de.hsrm.sls.subato.tasks.stef.spec.SubmissionModeType;
import de.hsrm.sls.subato.tasks.stef.spec.SupportedArchiveExtensions;
import de.hsrm.sls.subato.tasks.stef.spec.TextSubmissionMode;
import de.hsrm.sls.subato.tasks.task.exception.FileNotFoundException;
import jakarta.persistence.CascadeType;
import jakarta.persistence.Column;
import jakarta.persistence.Entity;
import jakarta.persistence.GeneratedValue;
import jakarta.persistence.GenerationType;
import jakarta.persistence.Id;
import jakarta.persistence.ManyToOne;
import jakarta.persistence.OneToMany;
import jakarta.persistence.Table;
import jakarta.persistence.UniqueConstraint;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import java.util.UUID;
import org.apache.commons.lang3.StringUtils;
import org.hibernate.annotations.JdbcTypeCode;
import org.hibernate.type.SqlTypes;

/**
 * represents a task (STEF) in the database (used for linking to other entities)
 */
@Entity
@Table(name = "task", uniqueConstraints = {
    // task ids are unique within a task pool
    @UniqueConstraint(columnNames = {"trackingId", "pool_id"})
})
public class Task {

    /**
     * unique across all tasks, just for internal use
     */
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private long id;

    /**
     * STEF
     */
    private String trackingId;

    /**
     * STEF
     */
    private String softId;

    /**
     * STEF
     */
    private String name;

    /**
     * actual occurences of this task in exercises
     */
    @OneToMany(mappedBy = "task", cascade = CascadeType.REMOVE)
    private List<TaskInstance> instances;

    @ManyToOne
    private TaskPool pool;

    /**
     * STEF maps to subatoId
     */
    @Column(unique = true)
    private Long sourceId;

    /**
     * STEF just for display purposes
     */
    @ManyToOne
    private Language language;

    /**
     * STEF null if solutions are not evaluated automatically
     */
    //@ManyToOne(cascade = {CascadeType.PERSIST, CascadeType.MERGE, CascadeType.REFRESH})
    //private Evaluator2 evaluator;

    /**
     * flag for soft deletion.
     * <p>
     * soft deleted tasks are preserved in order to keep references from solutions. useful if
     * someone deletes a task accidentally.
     * <p>
     * archived tasks may have an invalid or missing spec.
     */
    private boolean archived;

    @OneToMany(mappedBy = "task", cascade = CascadeType.REMOVE)
    private List<Solution> solutions;

    /**
     * relative url to the task in the repository only applicable if the linked task pool is linked
     * to a git repository
     */
    private String repositoryPath;

    @JdbcTypeCode(SqlTypes.JSON)
    private StefSpecCache specCache;

    private String bundle;

    @JdbcTypeCode(SqlTypes.JSON)
    private EvaluationStack evaluationStack;

    public Task() {
    }

    public Long getSourceId() {
        return sourceId;
    }

    public void setSourceId(Long sourceId) {
        this.sourceId = sourceId;
    }

    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public List<TaskInstance> getInstances() {
        return instances;
    }

    public void setInstances(List<TaskInstance> instances) {
        this.instances = instances;
    }

    public TaskPool getPool() {
        return pool;
    }

    public void setPool(TaskPool pool) {
        this.pool = pool;
    }

    public Language getLanguage() {
        return language;
    }

    public void setLanguage(Language language) {
        this.language = language;
    }

    @Deprecated
    public String getIdName() {
        return getTrackingId();
    }

    @Deprecated
    public void setIdName(String idName) {
        setTrackingId(idName);
    }

    public String getTrackingId() {
        return trackingId;
    }

    public void setTrackingId(String trackingId) {
        this.trackingId = trackingId;
    }


    public SubmissionModeType getDefaultSubmissionMode() {
        var spec = getSpec();
        return spec != null ? spec.getMeta().getSubmissionModeConfig().getDefaultMode() : null;
    }

    public boolean isArchived() {
        return archived;
    }

    public void setArchived(boolean archived) {
        this.archived = archived;
    }

    public List<Solution> getSolutions() {
        return solutions;
    }

    public void setSolutions(List<Solution> solutions) {
        this.solutions = solutions;
    }

    /**
     * determines if lecturers can run additional acceptance tests while checking/grading the
     * solution
     */
    public boolean isAcceptanceTestAvailable() {
        var spec = getSpec();
        return spec != null && spec.getMeta().getEvaluator() != null && spec.getMeta().getFiles()
            .stream().anyMatch(f -> f.getType() == FileType.ACCEPTANCETEST);
    }

    public boolean isSupportsFileSubmissionMode() {
        return getFileSubmissionMode() != null;
    }

    public boolean isSupportsTextSubmissionMode() {
        return getTextSubmissionMode() != null;
    }

    /**
     * determines if solutions for this task can be evaluated automatically
     */
    public boolean evaluationPossible() {
        var spec = getSpec();
        return spec != null && spec.getMeta().getEvaluator() != null && spec.getMeta().getFiles()
            .stream().anyMatch(f -> f.getType() == FileType.TEST);
    }

    public List<FileRef> getFileRefs() {
        var spec = getSpec();
        var files = spec.getMeta().getFiles();
        files.sort(Comparator.comparingInt(a -> a.getType().getOrder()));
        return files;
    }

    public FileRef getFileRefById(String uuid) {
        return getFileRefs().stream()
            .filter(f -> f.getFile().getUuid().equals(UUID.fromString(uuid))).findFirst()
            .orElseThrow(() -> new FileNotFoundException(uuid, id));
    }

    public String getRepositoryPath() {
        return repositoryPath;
    }

    public void setRepositoryPath(String repositoryPath) {
        this.repositoryPath = repositoryPath;
    }

    public String getSoftId() {
        return softId;
    }

    public void setSoftId(String softId) {
        this.softId = softId;
    }

    public boolean isSupportedByEclipsePlugin() {
        var supportedLanguages = new String[]{Language.JAVA};
        return !archived &&
            language != null &&
            Arrays.stream(supportedLanguages).anyMatch(langId -> language.getId().equals(langId)) &&
            isAllowArchives();
    }

    public StefSpecCache getSpecCache() {
        return specCache;
    }

    public StefSpec getSpec() {
        return hasValidSpec() ? specCache.getSpec() : null;
    }

    public void setSpecCache(StefSpecCache specCache) {
        this.specCache = specCache;
    }

    public String getRepositoryUrl() {
        if (pool.isLinkedToRepository() && repositoryPath != null) {
            var branch =
                StringUtils.isNotBlank(pool.getRepositoryBranch()) ? pool.getRepositoryBranch()
                    : "main";
            return concat(pool.getRepositoryUrl(),
                Path.of("-").resolve("tree").resolve(branch).resolve(repositoryPath));
        }

        return null;
    }

    public boolean isAllowArchives() {
        var fileSubmissionMode = getFileSubmissionMode();
        return fileSubmissionMode != null && fileSubmissionMode.isAllowArchives();
    }

    public List<String> getAllowedExtensions() {
        var fileSubmissionMode = getFileSubmissionMode();
        if (fileSubmissionMode == null) {
            return new ArrayList<>();
        }

        var allowedExtensions = fileSubmissionMode.getExtensions();
        if (isAllowArchives()) {
            allowedExtensions.addAll(SupportedArchiveExtensions.getSupportedExtensions());
        }

        return allowedExtensions;
    }

    public FileSubmissionMode getFileSubmissionMode() {
        var spec = getSpec();
        if (spec == null) {
            return null;
        }

        return spec.getMeta().getSubmissionModeConfig().getFileSubmissionMode();
    }

    public TextSubmissionMode getTextSubmissionMode() {
        var spec = getSpec();
        if (spec == null) {
            return null;
        }

        return spec.getMeta().getSubmissionModeConfig().getTextSubmissionMode();
    }

    public boolean isExtractArchives() {
        var fileSubmissionMode = getFileSubmissionMode();
        return fileSubmissionMode != null && fileSubmissionMode.isExtractArchives();
    }

    public boolean hasValidSpec() {
        return specCache != null;
    }

    public String getBundle() {
        return bundle;
    }

    public void setBundle(String bundle) {
        this.bundle = bundle;
    }

    public EvaluationStack getEvaluationStack() {
        return evaluationStack;
    }

    public void setEvaluationStack(EvaluationStack evaluationStack) {
        this.evaluationStack = evaluationStack;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        Task task = (Task) o;
        return id == task.id;
    }

    @Override
    public int hashCode() {
        return Objects.hash(id);
    }
}
