package de.hsrm.sls.subato.tasks.task;

import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.stream.HtmlUrlSigner;
import de.hsrm.sls.subato.shared.response.FileResponseBuilderFactory;
import de.hsrm.sls.subato.shared.swagger.SubatoApiResponse;
import de.hsrm.sls.subato.tasks.pool.exception.TaskPoolReadOnlyException;
import de.hsrm.sls.subato.tasks.stef.dto.FileRefDto;
import de.hsrm.sls.subato.tasks.stef.dto.FileRefMapper;
import de.hsrm.sls.subato.tasks.task.dto.TaskDto;
import de.hsrm.sls.subato.tasks.task.dto.TaskMapper;
import de.hsrm.sls.subato.tasks.task.exception.AssetNotFoundException;
import de.hsrm.sls.subato.tasks.task.exception.TaskArchivedException;
import de.hsrm.sls.subato.tasks.task.exception.TaskNotFoundException;
import de.hsrm.sls.subato.tasks.task.repository.BundleRepository;
import de.hsrm.sls.subato.tasks.task.repository.TaskContentRepository;
import de.hsrm.sls.subato.tasks.task.repository.TaskFilter;
import de.hsrm.sls.subato.tasks.task.repository.TaskRepository;
import de.hsrm.sls.subato.tasks.task.repository.TaskSort;
import de.hsrm.sls.subato.tasks.task.usecase.DeleteTaskUseCase;
import de.hsrm.sls.subato.tasks.task.usecase.DownloadAssetUseCase;
import de.hsrm.sls.subato.tasks.task.usecase.DownloadFileUseCase;
import de.hsrm.sls.subato.tasks.task.usecase.ListFilesUseCase;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.Resource;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.web.PageableDefault;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RestController
@Tag(name = "Aufgabenverwaltung")
public class TaskController {

    @Autowired
    private TaskMapper taskMapper;

    @Autowired
    private FileRefMapper fileRefMapper;

    @Autowired
    private PolicyEvaluator policy;

    @Autowired
    private FileResponseBuilderFactory fileResponseBuilderFactory;

    @Autowired
    private TaskContentAssetUrlRewriter taskContentAssetUrlRewriter;

    @Autowired
    private BundleRepository bundleRepository;

    @Autowired
    private DownloadFileUseCase downloadFileUseCase;

    @Autowired
    private DeleteTaskUseCase deleteTaskUseCase;

    @Autowired
    private DownloadAssetUseCase downloadAssetUseCase;

    @Autowired
    private ListFilesUseCase listFilesUseCase;

    @Autowired
    private TaskContentRepository taskContentRepository;

    @Autowired
    private TaskRepository taskRepository;

    @Autowired
    private HtmlUrlSigner htmlUrlSigner;

    @GetMapping("/task")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_VIEW)")
    @Operation(summary = "Liefert alle Aufgaben über alle Aufgaben-Pools hinweg")
    Page<TaskDto> getTasks(
        @PageableDefault Pageable page,
        @Parameter(description = "Case-insensitive Filter über Name und Beschreibung") @RequestParam("q") Optional<String> query,
        @Parameter(description = "Aufgaben-Pool Filter") @RequestParam("poolId") Optional<Long> poolId,
        @RequestParam Optional<Boolean> showArchived
    ) {
        var updatedPage = PageRequest.of(page.getPageNumber(), page.getPageSize(),
            page.getSortOr(TaskSort.Native.byId()));

        var filter = new TaskFilter.Builder()
            .query(query.orElse(null))
            .poolId(poolId.orElse(null))
            .setShowArchived(showArchived.orElse(null))
            .build();
        var tasks = taskRepository.getAll(filter, updatedPage);
        return tasks.map(task -> taskMapper.map(task));
    }

    @GetMapping("/task/{id}")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_VIEW)")
    @Operation(
        summary = "Liefert eine Aufgabe, wobei die relativen Verweise auf Bilder und andere Medien im Inhalt vorher aufgelöst werden",
        description = "Die Verweise werden so aufgelöst, sodass mit dem Endpunkt /task/{id}/assets/** die Inhalte direkt per HTTP eingebunden werden können"
    )
    @SubatoApiResponse(TaskNotFoundException.class)
    TaskDto getOne(@PathVariable Long id, HttpServletRequest req) {
        var task = taskRepository.findById(id).orElseThrow(() -> new TaskNotFoundException(id));
        var dto = taskMapper.map(task);
        return dto;
    }

    @GetMapping("/task/{id}/content")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_VIEW)")
    @SubatoApiResponse(TaskNotFoundException.class)
    @SubatoApiResponse(TaskArchivedException.class)
    String getContent(@PathVariable Long id, HttpServletRequest req) {
        var task = taskRepository.findById(id).orElseThrow(() -> new TaskNotFoundException(id));
        if (task.isArchived()) {
            throw new TaskArchivedException();
        }

        var content = taskContentAssetUrlRewriter.rewiteContent(task, req);
        return htmlUrlSigner.sign(content, req);
    }

    @GetMapping("/task/{id}/files")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_VIEW)")
    @Operation(summary = "Alle Dateien zur Aufgabe")
    @SubatoApiResponse(TaskNotFoundException.class)
    @SubatoApiResponse(TaskArchivedException.class)
    List<FileRefDto> getFiles(@PathVariable Long id, HttpServletRequest req) {
        var task = taskRepository.findById(id).orElseThrow(() -> new TaskNotFoundException(id));
        var files = listFilesUseCase.getFiles(task);
        return files.stream().map(file -> fileRefMapper.map(file)).collect(Collectors.toList());
    }

    @GetMapping("/task/{id}/files/{uuid}/download")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_VIEW)")
    @Operation(summary = "Zugriff auf den Inhalt der Datei zu einer Aufgabe")
    @SubatoApiResponse(TaskNotFoundException.class)
    @SubatoApiResponse(TaskArchivedException.class)
    ResponseEntity<Resource> downloadFile(@PathVariable Long id, @PathVariable String uuid) {
        var task = taskRepository.findById(id).orElseThrow(() -> new TaskNotFoundException(id));
        var file = downloadFileUseCase.getFile(task, uuid);

        return fileResponseBuilderFactory.forFile(file)
            .setAsAttachment(true)
            .build();
    }

    @GetMapping("/task/{id}/bundle")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_VIEW)")
    @SubatoApiResponse(TaskNotFoundException.class)
    @SubatoApiResponse(TaskArchivedException.class)
    ResponseEntity<Resource> downloadBundle(@PathVariable Long id) {
        var task = taskRepository.findById(id).orElseThrow(() -> new TaskNotFoundException(id));
        var bundle = bundleRepository.getBundle(task);

        return fileResponseBuilderFactory.forFile(bundle)
            .setAsAttachment(true)
            .build();
    }

    @GetMapping("/task/{id}/spec")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_VIEW)")
    @Operation(summary = "Übermittlung der STEF-Spezifikation für die Aufgabe als zip-Datei")
    @SubatoApiResponse(TaskNotFoundException.class)
    ResponseEntity<Resource> downloadSpec(@PathVariable Long id) {
        var task = taskRepository.findById(id).orElseThrow(() -> new TaskNotFoundException(id));
        var zipFile = taskContentRepository.getSpecAsZip(task);

        return fileResponseBuilderFactory.forFile(zipFile)
            .setFileName("%s.zip".formatted(task.getSoftId()))
            .setAsAttachment(true)
            .build();
    }

    @GetMapping("/task/{id}/asset")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_VIEW)")
    @Operation(
        summary = "Zugriff auf eine Datei im Text-Ordner in der STEF Spezifikation durch Angabe eines relativen Pfades",
        description = "Ein Aufruf von ```/task/15/assets/images/BTree.png``` würde z.B. auf ```/15SomeTask/text/images/BTree.png``` verweisen"
    )
    @SubatoApiResponse(TaskNotFoundException.class)
    @SubatoApiResponse(AssetNotFoundException.class)
    ResponseEntity<Resource> downloadAsset(@PathVariable Long id,
        @RequestParam("path") String path) {
        var task = taskRepository.findById(id).orElseThrow(() -> new TaskNotFoundException(id));
        var file = downloadAssetUseCase.getAsset(task, path);

        return fileResponseBuilderFactory.forFile(file)
            .setAsAttachment(false)
            .build();
    }

    @DeleteMapping("/task/{id}")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_ARCHIVE)")
    @Operation(summary = "Archiviert eine Aufgabe, sodass sie nicht mehr verfügbar ist aber bestehende Links (von z.B. Lösungen) bestehen bleiben")
    @SubatoApiResponse(TaskNotFoundException.class)
    @SubatoApiResponse(TaskPoolReadOnlyException.class)
    void delete(@PathVariable Long id) {
        var task = taskRepository.findById(id).orElseThrow(() -> new TaskNotFoundException(id));
        deleteTaskUseCase.delete(task);
    }
}
