package de.hsrm.sls.subato.tasks.task.usecase;

import de.hsrm.sls.subato.shared.fs.TmpDirProvider;
import de.hsrm.sls.subato.shared.helper.FileHelper;
import de.hsrm.sls.subato.shared.helper.ZipHelper;
import de.hsrm.sls.subato.tasks.imp.TaskImportException;
import de.hsrm.sls.subato.tasks.stef.spec.FileRef;
import de.hsrm.sls.subato.tasks.stef.spec.FileType;
import de.hsrm.sls.subato.tasks.task.Task;
import de.hsrm.sls.subato.tasks.task.repository.TaskContentRepository;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.FileSystemUtils;

@Service
public class BuildBundleUseCase {

    @Autowired
    private TmpDirProvider tmpDirProvider;

    @Autowired
    private TaskContentRepository taskContentRepository;


    /**
     * generates a zip file, bundling all public source files (excluding the solution) associated
     * with the given task. then stores the zip file in the file system
     * <p>
     * useful for students wanting to import the task into their IDE without explicitly downloading
     * each file manually
     */
    public File buildBundle(Task task) {
        var taskPath = taskContentRepository.getTaskPath(task);
        var spec = task.getSpec();

        var srcFiles = spec.getMeta().getFiles().stream()
            .filter(fileRef -> fileRef.isPublic())
            .filter(fileRef -> fileRef.getType() == FileType.TEMPLATE ||
                fileRef.getType() == FileType.INCLUDE ||
                fileRef.getType() == FileType.TEST)
            .toList();

        var miscFiles = spec.getMeta().getFiles().stream()
            .filter(fileRef -> fileRef.isPublic())
            .filter(fileRef -> fileRef.getType() == FileType.MISC)
            .toList();
        var targetFiles = new ArrayList<FileRef>();
        targetFiles.addAll(srcFiles);
        targetFiles.addAll(miscFiles);

        if (targetFiles.size() == 0) {
            return null;
        }

        var tempDir = tmpDirProvider.createTmpPath().toFile();

        try {
            for (var fileRef : targetFiles) {
                try {
                    var targetPath = srcFiles.contains(fileRef)
                        ? tempDir.toPath().resolve("src").resolve(Path.of(fileRef.getPath()))
                        : tempDir.toPath().resolve("resources").resolve(Path.of(fileRef.getPath()));
                    var targetDirectory = targetPath.getParent();
                    Files.createDirectories(targetDirectory);

                    var bytes = FileHelper.toBytes(fileRef.getFile().getFile(taskPath));
                    Files.write(targetPath, bytes);
                } catch (IOException e) {
                    throw new TaskImportException("bundle could not be built", e);
                }
            }

            return ZipHelper.zip(tempDir, false);
        } finally {
            FileSystemUtils.deleteRecursively(tempDir);
        }
    }
}
