package de.hsrm.sls.subato.tasks.task.usecase;

import de.hsrm.sls.subato.tasks.task.Task;
import de.hsrm.sls.subato.tasks.task.exception.AssetNotFoundException;
import de.hsrm.sls.subato.tasks.task.repository.TaskContentRepository;
import java.io.File;
import java.io.IOException;
import java.nio.file.Path;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class DownloadAssetUseCase {

    @Autowired
    private TaskContentRepository taskContentRepository;

    /**
     * gets a file within the text-folder in the STEF spec.
     * <p>
     * can be used for resolving images referenced in the task content.
     *
     * @param task task from where to get the file
     * @param path relative to the text folder in the spec. example: images/BTree.png refers to
     *             /15SomeTask/text/images/BTree.png
     * @return the file
     * @throws AssetNotFoundException if no file exists in the spec with the given relative path
     */
    public File getAsset(Task task, String path) {
        var file = taskContentRepository.getFile(task, Path.of("text").resolve(path));
        if (!file.exists()) {
            throw new AssetNotFoundException(task.getId(), path, false);
        }

        try {
            // check for malicious attacks, e.g. path = '../files/solution/First.java'
            var textFolder = taskContentRepository.getTaskPath(task).resolve("text");
            if (!file.getCanonicalPath()
                .startsWith(textFolder.toFile().getCanonicalPath() + File.separator)) {
                throw new AssetNotFoundException(task.getId(), path, true);
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        return file;
    }
}
