package de.hsrm.sls.subato.user;


import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.users.UsersAction;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.exercises.exercise.ExerciseNotFoundException;
import de.hsrm.sls.subato.shared.swagger.SubatoApiResponse;
import de.hsrm.sls.subato.user.dto.UpdateProfileDto;
import de.hsrm.sls.subato.user.dto.UserDetailDto;
import de.hsrm.sls.subato.user.dto.UserMapper;
import de.hsrm.sls.subato.user.repository.UserFilter;
import de.hsrm.sls.subato.user.repository.UserQuery;
import de.hsrm.sls.subato.user.repository.UserRepository;
import de.hsrm.sls.subato.user.repository.UserSort;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import java.util.Optional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.web.PageableDefault;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RestController
@Tag(name = "Querschnitt")
public class UserController {

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private PolicyEvaluator policy;

    @Autowired
    private UpdateProfileUseCase updateProfileUseCase;

    @GetMapping("/user/{id}")
    @Operation(summary = "Liefert Informationen zum angegebenen Benutzer")
    @SubatoApiResponse(UserNotFoundException.class)
    UserDetailDto getOne(@PathVariable Long id,
        @AuthenticationPrincipal SubatoPrincipal principal) {
        var user = userRepository.findById(id).orElseThrow(() -> new UserNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(user)
            .setPermission(UsersAction.USER_VIEW).build();
        policy.checkAndRaise(context);

        return userMapper.mapDetail(user);
    }

    @GetMapping("/user")
    Page<UserDetailDto> findAll(
        @PageableDefault Pageable page,
        @RequestParam("q") Optional<String> query,
        @AuthenticationPrincipal SubatoPrincipal principal
    ) {
        var context = new PolicyContext.Builder().setSubject(principal)
            .setPermission(UsersAction.USER_VIEW).build();
        var filter = new UserFilter.Builder().query(query.orElse(null)).build();
        var updatedPage = PageRequest.of(page.getPageNumber(), page.getPageSize(),
            UserSort.Native.byName());

        var userQuery = new UserQuery.Builder()
            .setFilter(filter)
            .setPolicyContext(context)
            .build();

        var users = userRepository.findAll(userQuery, updatedPage);
        return users.map(u -> userMapper.mapDetail(u));
    }

    @PutMapping("/user/{id}/profile")
    @Operation(
        summary = "Aktualisiert das Benutzerprofil"
    )
    @SubatoApiResponse(UserNotFoundException.class)
    UserDetailDto update(
        @PathVariable Long id,
        @RequestBody UpdateProfileDto dto,
        @AuthenticationPrincipal SubatoPrincipal principal
    ) {
        var user = userRepository.findById(id)
            .orElseThrow(() -> new ExerciseNotFoundException(id));

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(user)
            .setPermission(UsersAction.USER_PROFILE_UPDATE).build();
        policy.checkAndRaise(context);

        user = updateProfileUseCase.update(user, dto);
        return userMapper.mapDetail(user);
    }
}
