package de.hsrm.sls.subato.aor.term;

import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertNull;

import de.hsrm.sls.subato.SubatoTest;
import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.aor.term.usecase.ImportTermDataUseCase;
import de.hsrm.sls.subato.courses.course.repository.CourseRepository;
import de.hsrm.sls.subato.modules.term.Term;
import de.hsrm.sls.subato.modules.term.TermRepository;
import de.hsrm.sls.subato.user.repository.UserRepository;
import java.nio.file.Files;
import java.util.List;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ResourceUtils;

/**
 * ad-hoc integration tests, should be refactored someday to test the individual import steps
 * isolated
 */
@SubatoTest
@DirtiesContext(classMode = DirtiesContext.ClassMode.BEFORE_EACH_TEST_METHOD)
@AutoConfigureTestDatabase(replace = AutoConfigureTestDatabase.Replace.ANY)
@Tag(Tags.INTEGRATION)
@Transactional
public class ImportTermDataUseCaseTests {

    @Autowired
    private ImportTermDataUseCase importTermDataUseCase;

    @Autowired
    private CourseRepository courseRepository;

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private TermRepository termRepository;

    private Term getTerm(String name) {
        return termRepository.findAll().stream().filter(t -> t.getName().equals(name))
            .findFirst().orElse(null);
    }

    @Test
    public void shouldCreateTermIfNotExists() throws Exception {
        assertNull(getTerm("SS2022"));

        var sampleImportFile = ResourceUtils.getFile("classpath:testfiles/aor/exportSS22.json");
        importTermDataUseCase.importFromJson(Files.readString(sampleImportFile.toPath()));

        assertNotNull(getTerm("SS2022"));
    }

    @Test
    public void shouldCreateParticipantsAndLecturersIfNotExists() throws Exception {
        var sampleImportFile = ResourceUtils.getFile(
            "classpath:testfiles/aor/exportWS2223_modgroup.json");
        var loginNames = List.of("stud1", "stud2", "stud3", "stud4", "lect1", "lect2");
        for (var loginName : loginNames) {
            assertNull(userRepository.findByLoginNameIgnoreCase(loginName).orElse(null), loginName);
        }

        importTermDataUseCase.importFromJson(Files.readString(sampleImportFile.toPath()));

        for (var loginName : loginNames) {
            assertNotNull(userRepository.findByLoginNameIgnoreCase(loginName).orElse(null),
                loginName);
        }
    }

    @Test
    public void shouldImportSimpleCourse() throws Exception {
        var sampleImportFile = ResourceUtils.getFile(
            "classpath:testfiles/aor/exportWS2223_simple.json");
        importTermDataUseCase.importFromJson(Files.readString(sampleImportFile.toPath()));

        var course = courseRepository.findById(2L).get();
        CourseImportAssert.assertSimpleCourse(course);

        // nothing should change now
        importTermDataUseCase.importFromJson(Files.readString(sampleImportFile.toPath()));
        course = courseRepository.findById(2L).get();
        CourseImportAssert.assertSimpleCourse(course);
    }

    @Test
    public void shouldImportModGroupCourse() throws Exception {
        var sampleImportFile = ResourceUtils.getFile(
            "classpath:testfiles/aor/exportWS2223_modgroup.json");
        importTermDataUseCase.importFromJson(Files.readString(sampleImportFile.toPath()));

        var course = courseRepository.findById(2L).get();
        CourseImportAssert.assertModGroupCourse(course);

        // nothing should change now
        importTermDataUseCase.importFromJson(Files.readString(sampleImportFile.toPath()));
        course = courseRepository.findById(2L).get();
        CourseImportAssert.assertModGroupCourse(course);
    }

    @Test
    public void shouldImportModGroupPartialCourse() throws Exception {
        var sampleImportFile = ResourceUtils.getFile(
            "classpath:testfiles/aor/exportWS2223_modgroup_partial.json");

        importTermDataUseCase.importFromJson(Files.readString(sampleImportFile.toPath()));
        var course = courseRepository.findById(2L).get();
        CourseImportAssert.assertModGroupPartialCourse(course);
    }
}
