package de.hsrm.sls.subato.courses.course;

import static de.hsrm.sls.subato.utils.MockMvcHelper.getJson;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import de.hsrm.sls.subato.SubatoTest;
import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.courses.course.dto.CourseDto;
import de.hsrm.sls.subato.utils.JsonHelper;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.security.test.context.support.WithUserDetails;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

@SubatoTest
@AutoConfigureMockMvc
@Tag(Tags.INTEGRATION)
public class GetOneCourseTest {

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private JsonHelper jsonHelper;

    @Test
    public void shouldAllowGuestsToViewPublicCourses() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/course/1"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        CourseDto courseDto = jsonHelper.jsonAsDto(result.getResponse().getContentAsString());
        assertEquals(1, courseDto.getId());
    }

    @Test
    public void shouldDenyGuestsToViewInternalCourses() throws Exception {
        this.mockMvc.perform(getJson("/course/3"))
            .andDo(print()).andExpect(status().isUnauthorized());
    }

    @Test
    public void shouldDenyGuestsToViewParticipantCourses() throws Exception {
        this.mockMvc.perform(getJson("/course/4"))
            .andDo(print()).andExpect(status().isUnauthorized());
    }

    @Test
    public void shouldDenyGuestsToViewPrivateCourses() throws Exception {
        this.mockMvc.perform(getJson("/course/5"))
            .andDo(print()).andExpect(status().isUnauthorized());
    }

    @Test
    @WithUserDetails("U_STUD")
    public void shouldAllowStudentsToViewPublicCourses() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/course/1"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        CourseDto courseDto = jsonHelper.jsonAsDto(result.getResponse().getContentAsString());
        assertEquals(1, courseDto.getId());
    }

    @Test
    @WithUserDetails("U_STUD")
    public void shouldAllowStudentsToViewInternalCourses() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/course/3"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        CourseDto courseDto = jsonHelper.jsonAsDto(result.getResponse().getContentAsString());
        assertEquals(3, courseDto.getId());
    }

    @Test
    @WithUserDetails("U_STUD")
    public void shouldDenyStudentsToViewParticipantCourses() throws Exception {
        this.mockMvc.perform(getJson("/course/4"))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STUD")
    public void shouldDenyStudentsToViewPrivateCourses() throws Exception {
        this.mockMvc.perform(getJson("/course/5"))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STAFF")
    public void shouldAllowStaffToViewPublicCourses() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/course/1"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        CourseDto courseDto = jsonHelper.jsonAsDto(result.getResponse().getContentAsString());
        assertEquals(1, courseDto.getId());
    }

    @Test
    @WithUserDetails("U_STAFF")
    public void shouldAllowStaffToViewInternalCourses() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/course/3"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        CourseDto courseDto = jsonHelper.jsonAsDto(result.getResponse().getContentAsString());
        assertEquals(3, courseDto.getId());
    }

    @Test
    @WithUserDetails("U_STAFF")
    public void shouldAllowStaffToViewParticipantCourses() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/course/4"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        CourseDto courseDto = jsonHelper.jsonAsDto(result.getResponse().getContentAsString());
        assertEquals(4, courseDto.getId());
    }

    @Test
    @WithUserDetails("U_STAFF")
    public void shouldAllowStaffToViewPrivateCourses() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/course/5"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        CourseDto courseDto = jsonHelper.jsonAsDto(result.getResponse().getContentAsString());
        assertEquals(5, courseDto.getId());
    }

    @Test
    @WithUserDetails("U_STUD_C4P")
    public void shouldAllowCourseParticipantsToViewParticipantCourses() throws Exception {
        MvcResult result = this.mockMvc.perform(getJson("/course/4"))
            .andDo(print()).andExpect(status().isOk())
            .andReturn();
        CourseDto courseDto = jsonHelper.jsonAsDto(result.getResponse().getContentAsString());
        assertEquals(4, courseDto.getId());
    }

    @Test
    @WithUserDetails("U_STUD_C5P")
    public void shouldDenyCourseParticipantsToViewPrivateCourses() throws Exception {
        this.mockMvc.perform(getJson("/course/5"))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_C5A")
    public void shouldAllowCourseAdminToViewOwnPrivateCourses() throws Exception {
        this.mockMvc.perform(getJson("/course/5"))
            .andDo(print()).andExpect(status().isOk());
    }

    @Test
    @WithUserDetails("U_C1A")
    public void shouldDenyCourseAdminToViewOtherPrivateCourses() throws Exception {
        this.mockMvc.perform(getJson("/course/5"))
            .andDo(print()).andExpect(status().isForbidden());
    }
}
