package de.hsrm.sls.subato.courses.course;

import static de.hsrm.sls.subato.utils.MockMvcHelper.putJson;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import de.hsrm.sls.subato.SubatoTest;
import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.courses.course.dto.UpdateCourseDto;
import de.hsrm.sls.subato.courses.course.model.CourseVisibility;
import de.hsrm.sls.subato.courses.course.model.SubmitPolicy;
import de.hsrm.sls.subato.utils.JsonHelper;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.security.test.context.support.WithUserDetails;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.web.servlet.MockMvc;

/**
 * Integration Test für {@link CourseController#update(Long, UpdateCourseDto)}
 */
@SubatoTest
@AutoConfigureMockMvc
@Tag(Tags.INTEGRATION)
public class UpdateCourseTest {

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private JsonHelper jsonHelper;

    private UpdateCourseDto update;

    @BeforeEach
    public void setup() {
        update = new UpdateCourseDto();
        update.setName("Name");
        update.setDescription("Beschreibung");
        update.setShortName("Kurzname");
        update.setVisibility(CourseVisibility.PRIVATE);
        update.setSubmitPolicy(SubmitPolicy.PARTICIPANTS);
    }

    @Test
    @DirtiesContext
    @WithUserDetails("U_C1A")
    public void shouldAllowCourseAdminsForOwnCourse() throws Exception {
        this.mockMvc.perform(putJson("/course/1")
                .content(jsonHelper.asJsonString(update)))
            .andDo(print()).andExpect(status().isOk());
    }


    @Test
    @WithUserDetails("U_C1A")
    public void shouldDenyCourseAdminsForOtherCourse() throws Exception {
        this.mockMvc.perform(putJson("/course/2")
                .content(jsonHelper.asJsonString(update)))
            .andDo(print()).andExpect(status().isForbidden());
    }


    @Test
    @WithUserDetails("U_STUD_C1P")
    public void shouldDenyCourseParticipants() throws Exception {
        this.mockMvc.perform(putJson("/course/1")
                .content(jsonHelper.asJsonString(update)))
            .andDo(print()).andExpect(status().isForbidden());
    }


    @Test
    @WithUserDetails("U_STUD")
    public void shouldDenyStudents() throws Exception {
        this.mockMvc.perform(putJson("/course/1")
                .content(jsonHelper.asJsonString(update)))
            .andDo(print()).andExpect(status().isForbidden());
    }


    @Test
    public void shouldDenyGuests() throws Exception {
        this.mockMvc.perform(putJson("/course/1")
                .content(jsonHelper.asJsonString(update)))
            .andDo(print()).andExpect(status().isUnauthorized());
    }


    @Test
    @WithUserDetails("U_C1T")
    public void shouldDenyCourseTutors() throws Exception {
        this.mockMvc.perform(putJson("/course/1")
                .content(jsonHelper.asJsonString(update)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_C1L")
    public void shouldDenyCourseLecturers() throws Exception {
        this.mockMvc.perform(putJson("/course/1")
                .content(jsonHelper.asJsonString(update)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STAFF")
    public void shouldDenyStaff() throws Exception {
        this.mockMvc.perform(putJson("/course/1")
                .content(jsonHelper.asJsonString(update)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_C6A")
    public void shouldPreventIfFinished() throws Exception {
        this.mockMvc.perform(putJson("/course/6")
                .content(jsonHelper.asJsonString(update)))
            .andDo(print()).andExpect(status().isBadRequest());
    }
}
