package de.hsrm.sls.subato.exercises.submission;

import static de.hsrm.sls.subato.utils.MockMvcHelper.postJson;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import de.hsrm.sls.subato.SubatoTest;
import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.assessment.submission.dto.UpdatePointsDto;
import de.hsrm.sls.subato.utils.JsonHelper;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.security.test.context.support.WithUserDetails;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.web.servlet.MockMvc;

@SubatoTest
@AutoConfigureMockMvc
@Tag(Tags.INTEGRATION)
public class UpdateSubmissionPointsTest {

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private JsonHelper jsonHelper;

    @Test
    @DirtiesContext
    @WithUserDetails("U_C1A")
    public void shouldAllowCourseAdminUpdatePoints() throws Exception {
        var updatePointsDto = new UpdatePointsDto();
        updatePointsDto.setPoints(17);
        updatePointsDto.setTaskId(11L);

        this.mockMvc.perform(postJson("/submission/2/points")
                .content(jsonHelper.asJsonString(updatePointsDto)))
            .andDo(print()).andExpect(status().isOk());
    }

    @Test
    @DirtiesContext
    @WithUserDetails("U_C1L")
    public void shouldAllowCourseLecturerUpdatePoints() throws Exception {
        var updatePointsDto = new UpdatePointsDto();
        updatePointsDto.setPoints(17);
        updatePointsDto.setTaskId(11L);

        this.mockMvc.perform(postJson("/submission/2/points")
                .content(jsonHelper.asJsonString(updatePointsDto)))
            .andDo(print()).andExpect(status().isOk());
    }

    @Test
    public void shouldDenyGuests() throws Exception {
        var updatePointsDto = new UpdatePointsDto();
        updatePointsDto.setPoints(17);
        updatePointsDto.setTaskId(11L);

        this.mockMvc.perform(postJson("/submission/1/points")
                .content(jsonHelper.asJsonString(updatePointsDto)))
            .andDo(print()).andExpect(status().isUnauthorized());
    }

    @Test
    @WithUserDetails("U_C1T")
    public void shouldDenyCourseTutorUpdatePoints() throws Exception {
        var updatePointsDto = new UpdatePointsDto();
        updatePointsDto.setPoints(17);
        updatePointsDto.setTaskId(11L);

        this.mockMvc.perform(postJson("/submission/2/points")
                .content(jsonHelper.asJsonString(updatePointsDto)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STUD_C1P")
    public void shouldDenyCourseParticipantsUpdatePoints() throws Exception {
        var updatePointsDto = new UpdatePointsDto();
        updatePointsDto.setPoints(17);
        updatePointsDto.setTaskId(11L);

        this.mockMvc.perform(postJson("/submission/2/points")
                .content(jsonHelper.asJsonString(updatePointsDto)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_C4A")
    public void shouldDenyOtherCourseAdminUpdatePoints() throws Exception {
        var updatePointsDto = new UpdatePointsDto();
        updatePointsDto.setPoints(17);
        updatePointsDto.setTaskId(11L);

        this.mockMvc.perform(postJson("/submission/2/points")
                .content(jsonHelper.asJsonString(updatePointsDto)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STAFF")
    public void shouldDenyStaffUpdatePoints() throws Exception {
        var updatePointsDto = new UpdatePointsDto();
        updatePointsDto.setPoints(17);
        updatePointsDto.setTaskId(11L);

        this.mockMvc.perform(postJson("/submission/2/points")
                .content(jsonHelper.asJsonString(updatePointsDto)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_STUD")
    public void shouldDenyStudentUpdatePoints() throws Exception {
        var updatePointsDto = new UpdatePointsDto();
        updatePointsDto.setPoints(17);
        updatePointsDto.setTaskId(11L);

        this.mockMvc.perform(postJson("/submission/2/points")
                .content(jsonHelper.asJsonString(updatePointsDto)))
            .andDo(print()).andExpect(status().isForbidden());
    }

    @Test
    @WithUserDetails("U_C6A")
    public void shouldFailIfCourseFinished() throws Exception {
        var updatePointsDto = new UpdatePointsDto();
        updatePointsDto.setPoints(17);
        updatePointsDto.setTaskId(11L);

        this.mockMvc.perform(postJson("/submission/3/points")
                .content(jsonHelper.asJsonString(updatePointsDto)))
            .andDo(print()).andExpect(status().isBadRequest());
    }

    @Test
    @WithUserDetails("U_C1A")
    public void shouldFailIfTaskInstanceDoesNotExist() throws Exception {
        var updatePointsDto = new UpdatePointsDto();
        updatePointsDto.setPoints(17);
        updatePointsDto.setTaskId(1399L);

        this.mockMvc.perform(postJson("/submission/2/points")
                .content(jsonHelper.asJsonString(updatePointsDto)))
            .andDo(print()).andExpect(status().isBadRequest());
    }

    @Test
    @WithUserDetails("U_C1A")
    public void shouldFailIfTaskInstanceDoesNotUsePoints() throws Exception {
        var updatePointsDto = new UpdatePointsDto();
        updatePointsDto.setPoints(17);
        updatePointsDto.setTaskId(11L);

        this.mockMvc.perform(postJson("/submission/1/points")
                .content(jsonHelper.asJsonString(updatePointsDto)))
            .andDo(print()).andExpect(status().isBadRequest());
    }
}
