package de.hsrm.sls.subato.policy;

import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;

import de.hsrm.sls.subato.SubatoTest;
import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.auth.Role;
import de.hsrm.sls.subato.auth.policy.PolicyContext;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.courses.CoursesAction;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipalMapper;
import de.hsrm.sls.subato.courses.course.model.Course;
import de.hsrm.sls.subato.courses.staff.model.StaffGroup;
import de.hsrm.sls.subato.courses.unit.model.CourseUnit;
import de.hsrm.sls.subato.utils.SubatoPrincipalBuilder;
import java.util.List;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * Unit-Test für {@link de.hsrm.sls.subato.auth.policy.courses.CourseUpdatePolicy}
 */
@SubatoTest
@Tag(Tags.UNIT)
public class CourseUpdateAuthorizationTest {

    @Autowired
    SubatoPrincipalMapper mapper;

    @Autowired
    PolicyEvaluator evaluator;

    private Course course;

    @BeforeEach
    public void setup() {
        course = new Course();
        course.setId(1L);
        course.setUnits(List.of(new CourseUnit()));
    }

    @Test
    public void shouldAllowCourseAdmins() {
        var principal = new SubatoPrincipalBuilder().membership(course, StaffGroup.ADMIN)
            .build(mapper);

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(course)
            .setPermission(CoursesAction.COURSE_UPDATE).build();
        assertTrue(evaluator.check(context));
    }

    @Test
    public void shouldDenyCourseLecturers() {
        var principal = new SubatoPrincipalBuilder().membership(course, StaffGroup.LECTURER)
            .build(mapper);

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(course)
            .setPermission(CoursesAction.COURSE_UPDATE).build();
        assertFalse(evaluator.check(context));
    }

    @Test
    public void shouldDenyCourseTutors() {
        var principal = new SubatoPrincipalBuilder().membership(course, StaffGroup.TUTOR)
            .build(mapper);

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(course)
            .setPermission(CoursesAction.COURSE_UPDATE).build();
        assertFalse(evaluator.check(context));
    }

    @Test
    public void shouldDenyParticipants() {
        var principal = new SubatoPrincipalBuilder().participantOf(course).build(mapper);

        var context = new PolicyContext.Builder()
            .setSubject(principal)
            .setObj(course)
            .setPermission(CoursesAction.COURSE_UPDATE).build();
        assertFalse(evaluator.check(context));
    }

    @Test
    public void shouldDenyGuests() {
        var context = new PolicyContext.Builder()
            .setObj(course)
            .setPermission(CoursesAction.COURSE_UPDATE).build();
        assertFalse(evaluator.check(context));
    }

    @Test
    public void shouldDenyAllRoles() {
        for (var role : Role.regularRoles()) {
            var principal = new SubatoPrincipalBuilder().role(role).build(mapper);

            var context = new PolicyContext.Builder()
                .setSubject(principal)
                .setObj(course)
                .setPermission(CoursesAction.COURSE_UPDATE).build();
            assertFalse(evaluator.check(context), "Role %s".formatted(role));
        }
    }
}
